@@ -400,11 +400,11 @@ void ssh_agent_open(SshAgentOpenCallback callback, void *context)
   c->context = context;
 
   /* Connect to the agent socket. */
-#ifndef SSH_WIN_CLIENT
+
   ssh_agenti_connect(ssh_agent_open_complete, FALSE, (void *)c);
-#else
-  ssh_agent_open_complete(NULL, (void *)c);
-#endif /* SSH_WIN_CLIENT */
+
+
+
 }
 
 /* Closes the connection to the authentication agent.  If a command is
