@@ -416,7 +416,7 @@ void hostbased_destructor(void *gdata)
   memset(state, 'F', sizeof(*state));
 }
 
-SshFSMStateMapItem hostbased_states[] =
+SshFSMStateMapItemStruct hostbased_states[] =
 {
   { "hostbased_suspend", "Suspend thread", hostbased_suspend },
   
@@ -616,9 +616,10 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
           break;
         }
 
+      /* Here we continue as parent. */
+
       ssh_packet_wrapper_can_receive(state->wrapper, FALSE);
 
-      /* Here we continue as parent. */
       state->main_thread = ssh_fsm_spawn(state->fsm,
                                          0, "hostbased_send_compat_flags",
                                          NULL, NULL);
