@@ -208,7 +208,7 @@ void ssh_client_auth_pubkey_sign_complete(SshAgentError error,
     }
   else
     {
-      signature_blob = (char *)result;
+      signature_blob = (unsigned char *)result;
       signature_blob_len = len;
       result = NULL;
     }
@@ -933,7 +933,8 @@ void ssh_client_auth_pubkey(SshAuthClientOperation op,
         case SSH_MSG_USERAUTH_PK_OK:
           /* The server is willing to accept this key as authentication. */
           if (ssh_client_auth_pubkey_send_signature(state, user,
-                                                session_id, session_id_len))
+                                                    session_id,
+                                                    session_id_len))
             {
               /* Reading private part of the key was failed. Move to the
                  next candidate.*/
