@@ -2,10 +2,10 @@
 
   auth-hostbased-rhosts.c
 
-  Author: Tatu Ylonen <ylo@cs.hut.fi>
-          Sami Lehtinen <sjl@ssh.fi>
+  Author: Tatu Ylonen <ylo@ssh.com>
+          Sami Lehtinen <sjl@ssh.com>
           
-  Copyright (C) 1997-1999 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Hostbased authentication, server-side. .[sr]hosts and
@@ -26,6 +26,13 @@
 #ifdef HAVE_NETGROUP_H
 #include <netgroup.h>
 #endif /* HAVE_NETGROUP_H */
+#ifdef HAVE_INNETGR
+#ifndef HAVE_PROTO_INNETGR
+/* innetgr's prototype, if it isn't defined already. */
+int innetgr(const char *netgroup, const char *machine, const char *user,
+            const char *domain);
+#endif /* HAVE_PROTO_INNETGR */
+#endif /* HAVE_INNETGR */
 
 #define SSH_DEBUG_MODULE "Ssh2AuthHostBasedRhosts"
 
@@ -299,8 +306,14 @@ Boolean ssh_server_auth_hostbased_rhosts(SshUser user_data,
   if (!rhosts_files[rhosts_file_index] && 
       ssh_userfile_stat(ssh_user_uid(user_data), "/etc/hosts.equiv", &st) < 0 &&
       ssh_userfile_stat(ssh_user_uid(user_data), SSH_HOSTS_EQUIV, &st) < 0)
-    return FALSE; /* The user has no .shosts or .rhosts file and there are no
-                 system-wide files. */
+    {
+      ssh_log_event(config->log_facility, SSH_LOG_WARNING,
+                    "hostbased-authentication (rhosts) refused for "
+                    "%.100s: no .shosts or .rhosts files and no system-wide "
+                    "files (ie: /etc/shosts.equiv)",
+                    ssh_user_name(user_data));      
+      return FALSE;
+    }
   
   /* If not logging in as superuser, try /etc/hosts.equiv and shosts.equiv. */
   if (ssh_user_uid(user_data) != UID_ROOT)
@@ -330,11 +343,11 @@ Boolean ssh_server_auth_hostbased_rhosts(SshUser user_data,
     {
       ssh_log_event(config->log_facility, SSH_LOG_WARNING,
                     "hostbased-authentication (rhosts) refused for " \
-                    "%.100: no home directory %.200s",
+                    "%.100s: no home directory %.200s",
                     ssh_user_name(user_data),
                     ssh_user_dir(user_data));
       SSH_TRACE(2, ("hostbased-authentication (rhosts) refused for " \
-                    "%.100: no home directory %.200s", \
+                    "%.100s: no home directory %.200s", \
                     ssh_user_name(user_data), ssh_user_dir(user_data)));
       return FALSE;
     }
