@@ -2,9 +2,9 @@
 
   auths-hostbased.c
 
-  Author: Sami Lehtinen <sjl@ssh.fi>
+  Author: Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1999 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
                   
   Hostbased authentication, server-side.
@@ -21,30 +21,34 @@
 #include "sshcipherlist.h"
 #include "sshuserfiles.h"
 #include "sshuserfile.h"
+#include "auths-common.h"
 #include "auths-hostbased.h"
+#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthHostBasedServer"
 
-SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
-                                              const char *user,
-                                              SshBuffer *packet,
-                                              const unsigned char *session_id,
-                                              size_t session_id_len,
-                                              void **state_placeholder,
-                                              void **longtime_placeholder,
-                                              void *method_context)
+void ssh_server_auth_hostbased(SshAuthServerOperation op,
+                               const char *user,
+                               SshUser uc,
+                               SshBuffer packet,
+                               const unsigned char *session_id,
+                               size_t session_id_len,
+                               void **state_placeholder,
+                               void **longtime_placeholder,
+                               SshAuthServerCompletionProc completion_proc,
+                               void *completion_context,
+                               void *method_context)
 {
-  SshUser uc = (SshUser)*longtime_placeholder;
   unsigned char *certs, *sig;
   size_t certs_len, sig_len, hostname_len;
   char *client_user_name, *client_hostname, *client_pubkey_alg, *hostname; 
-  SshServer server;
+  char *service;
+  SshServer server = (SshServer) method_context;
+  SshConfig config = server->config;
   SshPublicKey pubkey;
-  SshBuffer *buf;
+  SshBuffer buf;
   Boolean sig_ok;
   
-  server = (SshServer) method_context;
-
   SSH_DEBUG(6, ("auth_hostbased op = %d  user = %s", op, user));
 
   switch(op)
@@ -52,33 +56,27 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
     case SSH_AUTH_SERVER_OP_START:
       /* This is actually the only thing we do in this authentication
          method. */
-      if (uc == NULL)
+      if (ssh_server_auth_check(uc, user, config, server->common,
+                                SSH_AUTH_HOSTBASED))
         {
-          uc = ssh_user_initialize(user, TRUE);
-          if (!uc)
-            {
-              /* If user context allocation failed, the user probably
-                 does not exist. */
-              ssh_log_event(server->config->log_facility,
-                            SSH_LOG_WARNING,
-                            "User %s does not exist. "
-                            "(How did we get here?)", user);
-              return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-            }       
+          (*completion_proc)(SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
+                             packet, completion_context);
+          return;
         }
+
       if(ssh_user_uid(uc) == SSH_UID_ROOT &&
-         server->config->permit_root_login == SSH_ROOTLOGIN_FALSE)
+         config->permit_root_login == SSH_ROOTLOGIN_FALSE)
         {
-            /* XXX Add client addresses etc. */
-            ssh_log_event(server->config->log_facility,
-                          SSH_LOG_WARNING,
-                          "root logins are not permitted.");
-            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
-                          "not permitted."));
-            return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-          }
-      
-      *longtime_placeholder = (void *)uc;
+          /* XXX Add client addresses etc. */
+          ssh_log_event(config->log_facility,
+                        SSH_LOG_WARNING,
+                        "root logins are not permitted.");
+          SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
+                        "not permitted."));
+          (*completion_proc)(SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
+                             packet, completion_context);
+          return;
+        }
 
       if (!ssh_decode_buffer(packet,
                              SSH_FORMAT_UINT32_STR, &client_pubkey_alg, NULL,
@@ -90,9 +88,9 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
         {
           /* Error during decoding. */
           SSH_TRACE(1, ("Error decoding packet."));
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
-                        "Error decoding packet.");
+                        "Error decoding \"hostbased\" packet.");
           goto error;
         }
 
@@ -117,7 +115,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
           /* There shouldn't be anything else in the packet, so this
              packet was invalid. */
           SSH_TRACE(1, ("Invalid packet. (extra data at end)"));
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
                         "Invalid packet. (extra data at end)");
           goto error;
@@ -131,7 +129,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
           SSH_TRACE(1, ("Client's public key algorithms are not " \
                         "supported by us. (client sent '%s')", \
                         client_pubkey_alg));
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
                         "Client's public key algorithms are not " \
                         "supported by us. (client sent '%s')", \
@@ -146,12 +144,12 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
       hostname[hostname_len] = '.';
       hostname[hostname_len + 1] = '\0';
       
-      if (strcmp(client_hostname, hostname))
+      if (strcasecmp(client_hostname, hostname))
         {
           SSH_TRACE(1, ("Client gave us a hostname ('%s') which " \
                         "doesn't match the one we got from DNS ('%s')", \
                         client_hostname, hostname));
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
                         "Client gave us a hostname ('%s') which " \
                         "doesn't match the one we got from DNS ('%s')", 
@@ -170,28 +168,19 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
         unsigned char *key_blob;
         size_t key_blob_len;
         char *candidate;
-        size_t candidate_len;
         Boolean retry = FALSE;
         
       retry_with_global_dir:
         
-        if (server->config->user_known_hosts && !retry)
+        if (config->user_known_hosts && !retry)
           {
             char *user_ssh2_dir;
             
-            user_ssh2_dir = ssh_userdir(uc, server->config, TRUE);
-            candidate_len = strlen(user_ssh2_dir) +
-              strlen(SSH_KNOWNHOSTS_DIR) + 1 + /* '/' */
-              strlen(hostname) +
-              strlen(client_pubkey_alg) + strlen(".pub") + 1; /* that
-                                                                 last one
-                                                                 just to be
-                                                                 sure */
-            candidate = ssh_xcalloc(candidate_len , sizeof(char));
-          
-            snprintf(candidate, candidate_len, "%s%s%c%s%s%s",
-                     user_ssh2_dir, SSH_KNOWNHOSTS_DIR, '/',
-                     hostname, client_pubkey_alg, ".pub");
+            user_ssh2_dir = ssh_userdir(uc, config, TRUE);
+
+            ssh_dsprintf(&candidate, "%s%s%c%s%s%s",
+                         user_ssh2_dir, SSH_KNOWNHOSTS_DIR, '/',
+                         hostname, client_pubkey_alg, ".pub");
           }
         else
           {
@@ -205,18 +194,10 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
               }
             
             tried = TRUE;
-            
-            candidate_len = strlen(SSH_GLOBAL_KNOWNHOSTS_DIR) +  1 /* '/' */
-              + strlen(hostname) +
-              strlen(client_pubkey_alg) + strlen(".pub") + 1; /* that
-                                                                 last one just
-                                                                 to be sure */
-            
-            candidate = ssh_xcalloc(candidate_len , sizeof(char));
-            
-            snprintf(candidate, candidate_len, "%s%c%s%s%s",
-                     SSH_GLOBAL_KNOWNHOSTS_DIR, '/',
-                     hostname, client_pubkey_alg, ".pub");
+
+            ssh_dsprintf(&candidate, "%s%c%s%s%s",
+                         SSH_GLOBAL_KNOWNHOSTS_DIR, '/',
+                         hostname, client_pubkey_alg, ".pub");
           }
 
         /* Try to load pubkey-candidate into memory */
@@ -227,7 +208,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
         ssh_userfile_init(ssh_user_name(uc), ssh_user_uid(uc),
                           ssh_user_gid(uc), NULL, NULL);
         
-        if (ssh2_key_blob_read(uc, candidate, &comment, &key_blob,
+        if (ssh2_key_blob_read(uc, candidate, TRUE, &comment, &key_blob,
                                &key_blob_len, NULL)
             == SSH_KEY_MAGIC_FAIL)
           {
@@ -249,7 +230,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
               {
                 SSH_TRACE(1, ("The public stored in %s and the given " \
                               "by client were different.", candidate));
-                ssh_log_event(server->config->log_facility,
+                ssh_log_event(config->log_facility,
                               SSH_LOG_WARNING,
                               "The public stored in %s and the given " \
                               "by client were different.", candidate);
@@ -260,7 +241,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
           {
             SSH_TRACE(1, ("The public stored in %s and the given " \
                           "by client were different.", candidate));
-            ssh_log_event(server->config->log_facility,
+            ssh_log_event(config->log_facility,
                           SSH_LOG_WARNING,
                           "The public stored in %s and the given " \
                           "by client were different.", candidate);
@@ -277,7 +258,7 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
       if ((pubkey = ssh_decode_pubkeyblob(certs, certs_len)) == NULL)
         {
           SSH_TRACE(1, ("Importing client pubkey failed."));
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
                         "Importing client pubkey failed.");
           goto error;
@@ -288,7 +269,13 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
       /* Contruct a throw-away SSH_MSG_USERAUTH_REQUEST packet.*/
       
       buf = ssh_buffer_allocate();
-      
+
+      if (!(*server->common->compat_flags->
+            hostbased_service_name_draft_incompatibility))
+        service = SSH_CONNECTION_SERVICE;
+      else
+        service = SSH_USERAUTH_SERVICE;
+
       ssh_encode_buffer(buf,
                         SSH_FORMAT_UINT32_STR, session_id, session_id_len,
                         /* byte SSH_MSG_USERAUTH_REQUEST */
@@ -297,9 +284,8 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
                         /* string User name*/
                         SSH_FORMAT_UINT32_STR, ssh_user_name(uc),
                         strlen(ssh_user_name(uc)),
-                        /* string "ssh-userauth" */
-                        SSH_FORMAT_UINT32_STR, SSH_USERAUTH_SERVICE,
-                        strlen(SSH_USERAUTH_SERVICE),
+                        /* string service_name */
+                        SSH_FORMAT_UINT32_STR, service, strlen(service),
                         /* string "hostbased" */
                         SSH_FORMAT_UINT32_STR, SSH_AUTH_HOSTBASED,
                         strlen(SSH_AUTH_HOSTBASED),
@@ -309,7 +295,8 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
                         /* string client's public host key and len. */
                         SSH_FORMAT_UINT32_STR, certs, certs_len,
                         /* string client hostname */
-                        SSH_FORMAT_UINT32_STR, hostname, strlen(hostname),
+                        SSH_FORMAT_UINT32_STR, client_hostname,
+                        strlen(client_hostname),
                         /* string user's user name at client host */
                         SSH_FORMAT_UINT32_STR, client_user_name,
                         strlen(client_user_name),
@@ -319,27 +306,100 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
                         ssh_buffer_ptr(buf), ssh_buffer_len(buf));
       SSH_DEBUG_HEXDUMP(7, ("Signature"), sig, sig_len);
       
-      /* Do the actual verifying. */
-      sig_ok = ssh_public_key_verify_signature(pubkey,
-                                               sig, sig_len,
-                                               ssh_buffer_ptr(buf),
-                                               ssh_buffer_len(buf));   
+      {
+        size_t decoded_len, real_sig_len;
+        char *recv_pubkeytype, *pubkeytype;
+        unsigned char *real_sig;
+
+        if (!*(server->common->compat_flags->
+               malformed_signatures_draft_incompatibility))
+          {
+            pubkeytype =
+              ssh_pubkeyblob_type(certs, certs_len);
+
+            if (!pubkeytype)
+              {
+                ssh_warning("Received malformed pubkey during hostbased "
+                            "authentication.");
+                goto error;
+              }
+          
+            decoded_len =
+              ssh_decode_array(sig, sig_len,
+                               SSH_FORMAT_UINT32_STR,
+                               &recv_pubkeytype, NULL,
+                               SSH_FORMAT_UINT32_STR,
+                               &real_sig, &real_sig_len,
+                               SSH_FORMAT_END);
+
+            if (decoded_len == 0 || decoded_len != sig_len)
+              {
+                ssh_xfree(pubkeytype);
+                SSH_DEBUG(2, ("decoded_len: %ld, sig_len: %ld",
+                              decoded_len, sig_len));
+                ssh_warning("Received malformed signature during hostbased "
+                            "authentication.");
+                goto error;
+              }
+
+            if (strcmp(recv_pubkeytype, pubkeytype) != 0)
+              {
+                ssh_warning("Received malformed signature during hostbased "
+                            "authentication. (public key type doesn't "
+                            "match the one in signature.)");
+                ssh_xfree(recv_pubkeytype);
+                ssh_xfree(pubkeytype);
+                memset(real_sig, 'F', real_sig_len);
+                ssh_xfree(real_sig);
+                goto error;
+              }
+            ssh_xfree(recv_pubkeytype);            
+            ssh_xfree(pubkeytype);
+          }
+        else
+          {
+            real_sig = sig;
+            real_sig_len = sig_len;
+            sig = NULL;
+          }
+        
+        /* Do the actual verifying. */
+        sig_ok = ssh_public_key_verify_signature(pubkey,
+                                                 real_sig, real_sig_len,
+                                                 ssh_buffer_ptr(buf),
+                                                 ssh_buffer_len(buf));   
+
+        if (!*(server->common->compat_flags->
+               malformed_signatures_draft_incompatibility))
+          {
+            memset(real_sig, 'F', real_sig_len);
+            ssh_xfree(real_sig);
+          }        
+      }
+      
       ssh_public_key_free(pubkey);
       ssh_buffer_free(buf);
 
       if (!sig_ok)
         {
-          SSH_TRACE(1, ("Hostbased operation failed for %s.", \
-                        ssh_user_name(uc)));
-          
-          ssh_log_event(server->config->log_facility,
+          ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
-                        "Hostbased operation failed for %s.",
+                        "Hostbased authentication failed for %s.",
                         ssh_user_name(uc));
           goto error;
         }
       
-      return SSH_AUTH_SERVER_ACCEPTED;
+      ssh_log_event(config->log_facility,
+                    SSH_LOG_NOTICE,
+                    "Hostbased authentication for user %s accepted.",
+                    ssh_user_name(uc));
+
+      server->authenticated_client_user_name = client_user_name;
+      client_user_name = NULL;
+      server->authenticated_client_host = ssh_xstrdup(hostname);
+      /* XXX Free memory, pliis. */
+      (*completion_proc)(SSH_AUTH_SERVER_ACCEPTED, packet, completion_context);
+      return;
       break;
     case SSH_AUTH_SERVER_OP_ABORT:
       /* This shouldn't happen as we haven't sent
@@ -364,23 +424,17 @@ SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
       goto error;
       break;
     case SSH_AUTH_SERVER_OP_UNDO_LONGTIME:
-      /* XXX */
-      /* We return this just for fun, as this will be ignored. */
-      return SSH_AUTH_SERVER_REJECTED;
-      break;
     case SSH_AUTH_SERVER_OP_CLEAR_LONGTIME:
-      /* XXX should we do something else here too? */
-      *longtime_placeholder = NULL;
       /* We return this just for fun, as this will be ignored. */
-      return SSH_AUTH_SERVER_REJECTED;
-      break;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
     }
 
  error:
   /* An error occurred, either an internal, or an protocol
      error. Either way, authentication isn't successful. */
-  /* XXX */
-  *longtime_placeholder = NULL;
+  (*completion_proc)(SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED, packet,
+                     completion_context);
+  return;
 
-  return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
 }
