@@ -12,19 +12,21 @@
 #ifndef AUTHS_HOSTBASED_H
 #define AUTHS_HOSTBASED_H
 
-#include "ssh2includes.h"
 #include "sshuser.h"
 
 /*   Hostbased authentication, server-side. */
 
-SshAuthServerResult ssh_server_auth_hostbased(SshAuthServerOperation op,
-                                              const char *user,
-                                              SshBuffer packet,
-                                              const unsigned char *session_id,
-                                              size_t session_id_len,
-                                              void **state_placeholder,
-                                              void **longtime_placeholder,
-                                              void *method_context);
+void ssh_server_auth_hostbased(SshAuthServerOperation op,
+                               const char *user,
+                               SshUser uc,
+                               SshBuffer packet,
+                               const unsigned char *session_id,
+                               size_t session_id_len,
+                               void **state_placeholder,
+                               void **longtime_placeholder,
+                               SshAuthServerCompletionProc completion_proc,
+                               void *completion_context,
+                               void *method_context);
 
 /* Returns TRUE if hostbased authentication can continue (by checking
    /etc/hosts.equiv, /etc/shosts.equiv and $HOME/.[rs]hosts). */
