@@ -2,13 +2,13 @@
 
 auths-passwd.c
 
-Author: Tatu Ylonen <ylo@cs.hut.fi>
+  Author: Tatu Ylonen <ylo@ssh.com>
 
-Copyright (c) 1997 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
-                   All rights reserved
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  All rights reserved.
 
-Password authentication, server-side.  This calls functions in machine-specific
-files to perform the actual authentication.
+  Password authentication, server-side.  This calls functions in 
+  machine-specific files to perform the actual authentication.
 
 */
 
@@ -16,104 +16,145 @@ files to perform the actual authentication.
 #include "sshencode.h"
 #include "sshauth.h"
 #include "sshmsgs.h"
-#include "auths-passwd.h"
 #include "sshuser.h"
 #include "sshserver.h"
 #include "sshconfig.h"
+#include "auths-common.h"
+#include "auths-passwd.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthPasswdServer"
 
+
+/* Checks if the user is root and if root login is disabled.
+   Returns TRUE if user is root and root login is disabled,
+   FALSE otherwise. */
+
+Boolean is_root_and_disabled(SshConfig config, SshUser uc,
+                             SshBuffer packet,
+                             SshAuthServerCompletionProc completion_proc,
+                             void *completion_context)
+{
+  if(ssh_user_uid(uc) == SSH_UID_ROOT &&
+     (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
+      config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
+    {
+      /* XXX Add client addresses etc. */
+      SSH_DEBUG(2, ("root logins are not permitted."));
+      ssh_log_event(config->log_facility,
+                    SSH_LOG_WARNING,
+                    "root login denied for user '%s'.",
+                    ssh_user_name(uc));
+
+      if (config->password_guesses > 0)
+        {
+          (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet,
+                             completion_context);
+          return TRUE;
+        }
+      else
+        {
+          (*completion_proc)
+            (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED, packet,
+             completion_context);
+          return TRUE;
+        }
+    }
+  return FALSE;
+}
+
 /* Password authentication.  This handles all forms of password authentication,
    including local passwords, kerberos, and secure rpc passwords. */
 
-SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
-                                           const char *user,
-                                           SshBuffer *packet,
-                                           const unsigned char *session_id,
-                                           size_t session_id_len,
-                                           void **state_placeholder,
-                                           void **longtime_placeholder,
-                                           void *method_context)
+void ssh_server_auth_passwd(SshAuthServerOperation op,
+                            const char *user,
+                            SshUser uc,
+                            SshBuffer packet,
+                            const unsigned char *session_id,
+                            size_t session_id_len,
+                            void **state_placeholder,
+                            void **longtime_placeholder,
+                            SshAuthServerCompletionProc completion_proc,
+                            void *completion_context,
+                            void *method_context)
 {
   SshServer server = (SshServer)method_context;
   SshConfig config = server->config;
-  SshUser uc = (SshUser)*longtime_placeholder;
   Boolean change_request;
   char *password, *prompt;
   int disable_method = 0;
-
-  SSH_DEBUG(6, ("auth_passwd op = %d  user = %s", op, user));
+  size_t pass_len = 0L;
+  
+  SSH_TRACE(6, ("password auth."));
+  
+  SSH_DEBUG(6, ("op = %d  user = %s", op, user));
   
   switch (op)
     {
     case SSH_AUTH_SERVER_OP_START:
-      if (uc == NULL)
+      if (ssh_server_auth_check(uc, user, config, server->common,
+                                SSH_AUTH_PASSWD))
         {
-          uc = ssh_user_initialize(user, TRUE);
-          if (!uc)
+          config->password_guesses--;
+          if (config->password_guesses <= 0)
             {
-              /* If user context allocation failed, the user probably does not 
-                 exist. */
-              ssh_log_event(config->log_facility,
-                            SSH_LOG_WARNING,
-                            "User %s does not exist. "
-                            "(How did we get here?)", user);
-              return TRUE;
-            }       
+              /* If this attempt is not succesful, disable this method. */
+              disable_method = 1;
+            }
+          goto password_bad;
         }
-      *longtime_placeholder = (void *)uc;
       
       {
-        /* XXX it is possible to get rid of these. Modify
-           sshd2.c/auth_policy_proc*/
-        
         config->password_guesses--;
         if (config->password_guesses <= 0)
           {
             /* If this attempt is not succesful, disable this method. */
             disable_method = 1;
           }
-        else if(ssh_user_uid(uc) == SSH_UID_ROOT &&
-                (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
-                 config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
-          {
-            /* XXX Add client addresses etc. */
-            ssh_log_event(config->log_facility,
-                          SSH_LOG_WARNING,
-                          "root logins are not permitted.");
-            SSH_DEBUG(2, ("ssh_server_auth_passwd: root logins are " \
-                          "not permitted."));
-            return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-          }
+
+
+        if (is_root_and_disabled(config, uc, packet,
+          completion_proc, completion_context))
+          return;
+
       }
-      
+
       /* Parse the password authentication request. */
       if (ssh_decode_buffer(packet,
                             SSH_FORMAT_BOOLEAN, &change_request,
-                            SSH_FORMAT_UINT32_STR, &password, NULL,
+                            SSH_FORMAT_UINT32_STR, &password, &pass_len,
                             SSH_FORMAT_END) == 0)
         {
-          SSH_DEBUG(2, ("ssh_server_auth_passwd: bad packet"));
+          SSH_DEBUG(2, ("bad packet"));
           goto password_bad;
         }
 
+      if (!config->permit_empty_passwords && pass_len == 0L)
+        {
+          ssh_log_event(config->log_facility,
+                        SSH_LOG_WARNING,
+                        "attempt to login as %s with empty password denied.",
+                        ssh_user_name(uc));
+          SSH_DEBUG(2, ("attempted login with empty password denied"));
+          goto password_bad;
+        }
+      
       /* Password changing requests should only be received as continuation
          messages. */
       if (change_request)
         {
-          SSH_DEBUG(2 ,("ssh_server_auth_passwd: changing password " \
-                        "cannot start."));
+          SSH_DEBUG(2 ,("cannot change password before authentication"));
           goto password_bad;
         }
       
       /* Sanity check: do not pass excessively long passwords to system
          functions to avoid buffer overflows in operating system code. */
-      if (strlen(password) > 64)
+      if (strlen(password) > 256)
         {
-          SSH_DEBUG(2, ("ssh_server_auth_passwd: password too long."));
+          SSH_DEBUG(2, ("password too long."));
           ssh_xfree(password);
           goto password_bad;
         }
+      
 
       /* Try SECURE RPC passwords.  We do this first, as this might be
          needed to access disks. */
@@ -133,14 +174,17 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
         {
           ssh_log_event(config->log_facility,
                         SSH_LOG_NOTICE,
-                        "User %s's kerberos password accepted.",
-                        ssh_user_name(uc));
+                        "Kerberos password accepted for user %s (%s).",
+                        ssh_user_name(uc),
+                        ssh_user_kerberos_name(uc));
           SSH_DEBUG(5, ("ssh_server_auth_passwd: accepted by " \
                         "kerberos passwd"));
           goto password_ok;
         }
 
+
       /* Try a local password (either normal or shadow). */
+#ifdef SSHDIST_SESSION_SIA
 #ifdef HAVE_SIA
       if (ssh_user_validate_local_password(uc, 
                                            password,
@@ -148,23 +192,42 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
 #else /* HAVE_SIA */
       if (ssh_user_validate_local_password(uc, password))
 #endif /* HAVE_SIA */
+
+
+#endif /* SSHDIST_SESSION_SIA */
         {
+
+
+
+
+
+
+
+
+
           ssh_log_event(config->log_facility,
                         SSH_LOG_NOTICE,
                         "User %s's local password accepted.",
-                        ssh_user_name(uc));       
+                        ssh_user_name(uc));
+
           SSH_DEBUG(5, ("ssh_server_auth_passwd: accepted by local passwd"));
           goto password_ok;
         }
-      
+
+      ssh_log_event(config->log_facility,
+                    SSH_LOG_WARNING,
+                    "Wrong password given for user '%s'.",
+                    ssh_user_name(uc));
+
       ssh_xfree(password);
       goto password_bad;
 
     password_bad:
-      return (disable_method ?
-              SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED :
-              SSH_AUTH_SERVER_REJECTED);
-
+      (*completion_proc)((disable_method ?
+                          SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED :
+                          SSH_AUTH_SERVER_REJECTED),
+                         packet, completion_context);
+      return;
     password_ok:
       /* Password authentication passed, but we still need to check whether
          the password needs to be changed. */
@@ -184,38 +247,33 @@ SshAuthServerResult ssh_server_auth_passwd(SshAuthServerOperation op,
                             SSH_FORMAT_UINT32_STR, prompt, strlen(prompt),
                             SSH_FORMAT_END);
           ssh_xfree(prompt);
-          return SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK;
+          (*completion_proc)(SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK,
+                             packet, completion_context);
+          return;
         }
 
-      return SSH_AUTH_SERVER_ACCEPTED;
+      (*completion_proc)(SSH_AUTH_SERVER_ACCEPTED, packet, completion_context);
+      return;
 
     case SSH_AUTH_SERVER_OP_ABORT:
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
       
     case SSH_AUTH_SERVER_OP_CONTINUE:
       SSH_DEBUG(1, ("ssh_server_auth_passwd: XXX CONTINUE not yet "\
                     "implemented"));
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
       
     case SSH_AUTH_SERVER_OP_UNDO_LONGTIME:
-      if (uc != NULL)
-        {
-          if (!ssh_user_free(uc, TRUE))
-            {
-              /* XXX failed unto undo everything. Should disconnect, but we
-                 don't yet have the interface for that. */
-              return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
-            }
-        }
-      /* fall down... */
     case SSH_AUTH_SERVER_OP_CLEAR_LONGTIME:
-      *longtime_placeholder = NULL;
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc) (SSH_AUTH_SERVER_REJECTED, packet,
+                          completion_context);
+      return;
       
     default:
       ssh_fatal("ssh_server_auth_passwd: unknown op %d", (int)op);
     }
-  
+ 
   SSH_NOTREACHED;
-  return SSH_AUTH_SERVER_REJECTED;
 }
