@@ -30,17 +30,51 @@
 #include "sshuserfiles.h"
 #include "auths-common.h"
 #include "auths-pubkey.h"
+
 #ifdef WITH_PGP
 #include "ssh2pgp.h"
 #endif /* WITH_PGP */
 
 #define SSH_DEBUG_MODULE "Ssh2AuthPubKeyServer"
 
+/* Uncomment this to put a delay to a successful authentication attempt
+   (to test the authentication protocol). */
+/* #define TEST_ASYNC_SERVER_METHODS */
+
+#ifdef TEST_ASYNC_SERVER_METHODS
+#include "sshtimeouts.h"
+
+typedef struct SshPublicKeyAuthCtxRec
+{
+  SshBuffer packet;
+  SshAuthServerResult result;
+  SshAuthServerCompletionProc completion_proc;
+  void *completion_context;
+} *SshPublicKeyAuthCtx, SshPublicKeyAuthCtxStruct;
+
+void publickey_auth_finalize_callback(void *context)
+{
+  SshPublicKeyAuthCtx publickey_auth_ctx = (SshPublicKeyAuthCtx)context;
+  SSH_PRECOND(publickey_auth_ctx);
+  SSH_PRECOND(publickey_auth_ctx->completion_proc);
+  SSH_PRECOND(publickey_auth_ctx->completion_context);
+
+  SSH_TRACE(2, ("Delay ended."));
+  
+  (*publickey_auth_ctx->completion_proc)
+    (publickey_auth_ctx->result,
+     publickey_auth_ctx->packet,
+     publickey_auth_ctx->completion_context);
+
+  ssh_xfree(publickey_auth_ctx);
+}
+
+#endif /* TEST_ASYNC_SERVER_METHODS */
+
 /* Check whether the key is authorized for login as the specified
    user from specified host.  If check_signatures if FALSE, this is not 
    required to verify signatures on authorization certificates.  This may 
    use a local database or the certificates to determine authorization. */
-
 Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
                                       unsigned char *certs,
                                       size_t certs_len, 
@@ -68,16 +102,19 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
   char *pgp_public_key_file = 
     ssh_xstrdup(server->common->config->pgp_public_key_file);
 #endif /* WITH_PGP */
-#ifndef SSHDIST_WINDOWS
+
   char *userdir, **vars = NULL, **vals = NULL;
+
+
 #ifdef HAVE_SIGNAL
   void (*old_sigchld_handler)(int);
   old_sigchld_handler = signal(SIGCHLD, SIG_DFL);
 #endif /* HAVE_SIGNAL */
+
+
   ssh_userfile_init(ssh_user_name(uc), ssh_user_uid(uc), ssh_user_gid(uc),
                     NULL, NULL);
 
-#endif /* SSHDIST_WINDOWS */
 
   SSH_DEBUG(6, ("auth_pubkey_verify user = %s  check_sig = %s",
                 ssh_user_name(uc), check_signatures ? "yes" : "no"));
@@ -87,7 +124,6 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
   if (certs_len < 16)  /* ever seen a 12-byte public key ? */
     goto exit_false;
 
-#ifndef SSHDIST_WINDOWS
   if ((userdir = ssh_userdir(uc, server->config, FALSE)) == NULL)
     goto exit_false;
 
@@ -95,7 +131,7 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
            server == NULL || server->common == NULL || 
            server->common->config == NULL || 
            server->common->config->authorization_file == NULL ?
-           SSH_AUTHORIZATION_FILE : 
+           SSH_AUTHORIZATION_FILE :
            server->common->config->authorization_file);
 
   n = ssh2_parse_config(uc, remote_ip ? remote_ip : "",
@@ -133,7 +169,7 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
                               ssh_xstrdup(vals[i + 1]);
                           }
                       }
-                    ssh_xfree(tblob);               
+                    ssh_xfree(tblob);
                     goto match;
                   }
               ssh_xfree(tblob);
@@ -274,15 +310,12 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
         }
 #endif /* WITH_PGP */
     }
-#endif /* SSHDIST_WINDOWS */
 
   SSH_DEBUG(6, ("auth_pubkey_verify: the key didn't match."));
 
-#ifndef SSHDIST_WINDOWS
   ssh_xfree(userdir);
 
   ssh_free_varsvals(n, vars, vals);
-#endif /* SSHDIST_WINDOWS */
 
   goto exit_false;
 
@@ -291,11 +324,9 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
 
   SSH_DEBUG(6, ("auth_pubkey_verify: the key matched."));
     
-#ifndef SSHDIST_WINDOWS
   ssh_xfree(userdir);
 
   ssh_free_varsvals(n, vars, vals);
-#endif /* SSHDIST_WINDOWS */
 
   if (!check_signatures)
     goto exit_true;
@@ -401,7 +432,7 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
             goto exit_false;
           }
 
-        if (strcmp(recv_certs_type, certs_type) != 0)
+        if (strcmp(recv_certs_type, (char *)certs_type) != 0)
           {
             ssh_warning("Received malformed signature during public key "
                         "authentication. (public key type doesn't match the "
@@ -443,27 +474,32 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
     }
 
  exit_true:
+
+ ssh_log_event(server->config->log_facility,
+               SSH_LOG_NOTICE,
+               "Public key %s used.",
+               filen);
+
 #ifdef WITH_PGP  
   ssh_xfree(pgp_public_key_file);
 #endif /* WITH_PGP */
-#ifndef SSHDIST_WINDOWS
   ssh_userfile_uninit();
+
 #ifdef HAVE_SIGNAL
   signal(SIGCHLD, old_sigchld_handler);
 #endif /* HAVE_SIGNAL */
-#endif /* SSHDIST_WINDOWS */
   return TRUE;
 
  exit_false:
 #ifdef WITH_PGP  
   ssh_xfree(pgp_public_key_file);
 #endif /* WITH_PGP */
-#ifndef SSHDIST_WINDOWS
+
   ssh_userfile_uninit();
 #ifdef HAVE_SIGNAL
   signal(SIGCHLD, old_sigchld_handler);
 #endif /* HAVE_SIGNAL */
-#endif /* SSHDIST_WINDOWS */
+
   /* if login failed, free memory (possibly) allocated by forced command,
      so that we don't accidentally execute commands with wrong keys. */
   ssh_xfree(server->common->config->forced_command);
@@ -476,16 +512,18 @@ Boolean ssh_server_auth_pubkey_verify(SshUser uc, char *remote_ip,
 /* Public key authentication.  The possession of a private key serves
    as authentication. */
 
-SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
-                                           const char *user,
-                                           SshBuffer packet,
-                                           const unsigned char *session_id,
-                                           size_t session_id_len,
-                                           void **state_placeholder,
-                                           void **longtime_placeholder,
-                                           void *method_context)
+void ssh_server_auth_pubkey(SshAuthServerOperation op,
+                            const char *user,
+                            SshUser uc,
+                            SshBuffer packet,
+                            const unsigned char *session_id,
+                            size_t session_id_len,
+                            void **state_placeholder,
+                            void **longtime_placeholder,
+                            SshAuthServerCompletionProc completion_proc,
+                            void *completion_context,
+                            void *method_context)
 {
-  SshUser uc = (SshUser)*longtime_placeholder;
   unsigned char *certs, *data, *sig, *certs_type = NULL;
   size_t certs_len, sig_len, len, bytes;
   SshServer server = (SshServer) method_context;
@@ -497,13 +535,15 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
   switch (op)
     {
     case SSH_AUTH_SERVER_OP_START:
-      if (ssh_server_auth_check(&uc, user, config, server->common,
+      if (ssh_server_auth_check(uc, user, config, server->common,
                                 SSH_AUTH_PUBKEY))
         {
-          return SSH_AUTH_SERVER_REJECTED;
+          (*completion_proc) (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
+                              packet,
+                              completion_context);
+          return;
         }
       
-      *longtime_placeholder = (void *)uc;
       
       /* Parse the publickey authentication request. */
       
@@ -544,7 +584,9 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
                         "got bad packet when verifying user %s's publickey.",
                         ssh_user_name(uc));
           SSH_DEBUG(2, ("bad packet"));
-          return SSH_AUTH_SERVER_REJECTED;
+          (*completion_proc) (SSH_AUTH_SERVER_REJECTED, packet,
+                              completion_context);
+          return;
         }
 
       if (real_request)
@@ -559,7 +601,9 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
                             "%s's publickey.",
                             ssh_user_name(uc));
               SSH_DEBUG(2, ("bad packet (real request)"));
-              return SSH_AUTH_SERVER_REJECTED;
+              (*completion_proc) (SSH_AUTH_SERVER_REJECTED, packet,
+                                  completion_context);
+              return;
             }
         }
       
@@ -579,8 +623,10 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
           if (sig != NULL)
             ssh_xfree(sig);
           ssh_xfree(certs_type);
-          SSH_DEBUG(6, ("auth_pubkey_verify returned false"));
-          return SSH_AUTH_SERVER_REJECTED;
+          SSH_DEBUG(6, ("auth_pubkey_verify returned FALSE"));
+          (*completion_proc) (SSH_AUTH_SERVER_REJECTED, packet,
+                              completion_context);
+          return;
         }
       
       if (real_request)
@@ -588,8 +634,8 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
           /* Free the signature blob. */
           ssh_xfree(sig);
           ssh_xfree(certs_type);
+
           /* Check for root login and forced commands */
-#ifndef SSHDIST_WINDOWS
           if(ssh_user_uid(uc) == SSH_UID_ROOT &&
              config->permit_root_login == SSH_ROOTLOGIN_FALSE)
             {
@@ -600,21 +646,47 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
                                 SSH_LOG_NOTICE,
                                 "root logins are not permitted.");
                   SSH_TRACE(2, ("root logins are not permitted."));
-                  return SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED;
+                  (*completion_proc)
+                    (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED, packet,
+                     completion_context);
+                  return;
                 }
               else
                 {
                   /* XXX add client address etc. */
-                  ssh_log_event(SSH_LOGFACILITY_AUTH,
+                  ssh_log_event(config->log_facility,
                                 SSH_LOG_NOTICE,
                                 "root login permitted for forced command.");
                 }
             }
-#endif /* SSHDIST_WINDOWS */
 
           /* Because it is an real request, and it has been verified, the
              authorization is granted. */
-          return SSH_AUTH_SERVER_ACCEPTED;
+          ssh_log_event(config->log_facility,
+                        SSH_LOG_NOTICE,
+                        "Public key authentication for user %s accepted.",
+                        ssh_user_name(uc));
+
+#ifdef TEST_ASYNC_SERVER_METHODS
+          {
+            SshPublicKeyAuthCtx finalize_ctx;
+
+            finalize_ctx = ssh_xcalloc(1, sizeof(*finalize_ctx));
+            finalize_ctx->packet = packet;
+            finalize_ctx->result = SSH_AUTH_SERVER_ACCEPTED;
+            finalize_ctx->completion_proc = completion_proc;
+            finalize_ctx->completion_context = completion_context;
+
+            SSH_TRACE(2, ("Delaying success message for two seconds..."));
+            ssh_register_timeout(2L, 0L,
+                                 publickey_auth_finalize_callback,
+                                 finalize_ctx);
+          }
+#else /* TEST_ASYNC_SERVER_METHODS */
+          (*completion_proc)(SSH_AUTH_SERVER_ACCEPTED, packet,
+                             completion_context);
+#endif /* TEST_ASYNC_SERVER_METHODS */
+          return;
         }
 
       /* It was just a probe request, return status now. */
@@ -625,7 +697,8 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
         ssh_encode_buffer(packet,                    
                           SSH_FORMAT_CHAR,
                           (unsigned int) SSH_MSG_USERAUTH_PK_OK,
-                          SSH_FORMAT_UINT32_STR, certs_type, strlen(certs_type),
+                          SSH_FORMAT_UINT32_STR, certs_type,
+                          strlen((char *)certs_type),
                           SSH_FORMAT_UINT32_STR, certs, certs_len,
                           SSH_FORMAT_END);
       
@@ -638,29 +711,27 @@ SshAuthServerResult ssh_server_auth_pubkey(SshAuthServerOperation op,
         
       ssh_xfree(certs_type);
       
-      return SSH_AUTH_SERVER_REJECTED_WITH_PACKET_BACK;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED_WITH_PACKET_BACK, packet,
+                         completion_context);
+      return;
 
     case SSH_AUTH_SERVER_OP_ABORT:
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
       
     case SSH_AUTH_SERVER_OP_CONTINUE:
       SSH_DEBUG(2, ("ssh_server_auth_pubkey: unexpected CONTINUE"));
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
       
     case SSH_AUTH_SERVER_OP_UNDO_LONGTIME:
     case SSH_AUTH_SERVER_OP_CLEAR_LONGTIME:
-      if (uc != NULL)
-        {
-          if (!ssh_user_free(uc, op == SSH_AUTH_SERVER_OP_UNDO_LONGTIME))
-            ssh_fatal("ssh_server_auth_pubkey: undo failed XXX");
-        }
-      *longtime_placeholder = NULL;
-      return SSH_AUTH_SERVER_REJECTED;
+      (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
+      return;
       
     default:
       ssh_fatal("ssh_server_auth_pubkey: unknown op %d", (int)op);
     }
 
   SSH_NOTREACHED;
-  return SSH_AUTH_SERVER_REJECTED; /* let's keep gcc happy */
 }
