@@ -8,7 +8,7 @@
   All rights reserved.
 
   Secure Copy. Designed to transfer files between hosts, using ssh2.
-  
+
 */
 /* TODO:
 
@@ -17,8 +17,11 @@
    - -n parameter isn't yet implemented.
 
 */
-#ifndef WIN32
 #include "sshincludes.h"
+
+
+
+#undef interface
 #include "sshtty.h"
 #include "ssheloop.h"
 #include "sshgetopt.h"
@@ -36,24 +39,23 @@
 #include <sys/ioctl.h>
 #endif /* HAVE_SYS_IOCTL_H */
 
-#else
-#include "ssh2includes.h"
-#include "ssheloop.h"
-#include "sshgetopt.h"
-#include "sshdllist.h"
-#include "sshappcommon.h"
-#include "sshfilecopy.h"
-#include "sshglob.h"
-#include "sshdsprintf.h"
-#include "sshgetcwd.h"
-#include "sshfsm.h"
-#include "sshmiscstring.h"
 
-#define SSH_WIN_SCP2
-#include "win32/commandline/connection.h"
-#include "win32/commandline/dosutil.h"
-#include "win32/commandline/version.h"
-#endif
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 #define SSH_DEBUG_MODULE "Scp2"
 
@@ -82,6 +84,9 @@ typedef struct ScpSessionRec
   /* Set, if user wants debug output. */
   Boolean verbose_mode;
 
+  /* Set, if we should give "-d" instead of "-v" to ssh2. */
+  Boolean debug_mode;
+  
   /* If set, ssh2 is invoked with ``BatchMode yes''. */
   Boolean batch_mode;
   
@@ -102,6 +107,12 @@ typedef struct ScpSessionRec
   /* Optional path to ssh. */
   char *ssh_path;
 
+  /* Additional options for ssh. */
+  SshDlList ssh_options;
+
+  /* Identity file. */
+  char *identity_file;
+  
   /* Optional debug level as a C-string. */
   char *debug_level;
 
@@ -149,13 +160,13 @@ typedef struct ScpSessionRec
      used to abort the operation. */
   SshOperationHandle transfer_op_handle;
 
-#ifdef SSH_WIN_CLIENT
-  /* Parameters that are used only in the Windows version. */
-  Boolean compression;
-  char   *fw_name;
-  char   *fw_port;
-  char   *keys_dir;
-#endif
+
+
+
+
+
+
+
 } *ScpSession;
 
 /* Structure, which holds a connection and the files to be transferred
@@ -244,6 +255,7 @@ void scp_session_initialize(ScpSession session)
   session->original_file_list = ssh_dllist_allocate();
   session->final_file_list = ssh_dllist_allocate();
   session->ssh_path = ssh_xstrdup("ssh2");
+  session->ssh_options = ssh_dllist_allocate();
   session->attrs = ssh_xcalloc(1, sizeof(*session->attrs));
 }
 
@@ -265,8 +277,10 @@ void scp_session_destroy(ScpSession session)
   ssh_xfree(session->ssh_path);
   ssh_xfree(session->debug_level);
   ssh_xfree(session->remote_port);
-
+  ssh_xfree(session->identity_file);
+  
   ssh_app_free_list(session->cipher_list, NULL);
+  ssh_app_free_list(session->ssh_options, NULL);
   ssh_app_free_list(session->original_file_list,
                     scp_file_list_item_destroy);
   if (session->final_file_list)
@@ -362,15 +376,16 @@ void scp_fatal_signal_handler(int signal, void *context)
   ssh_fatal("Received signal %d.", signal);
 }
 
-#ifndef SSH_WIN_CLIENT
+
 void scp_usage(ScpSession session)
 {
   SSH_PRECOND(session);
   
   fprintf(stderr, "usage: %s [-D debug_level_spec] [-d] [-p] [-n] [-u] "
-                  "[-v] [-1]\n", session->scp2_name);
+          "[-v] [-1]\n", session->scp2_name);
   fprintf(stderr, "           [-c cipher] [-S ssh2-path] [-h] "
-                  "[-P ssh2-port]\n");
+          "[-P ssh2-port]\n");
+  fprintf(stderr, "           [-o ssh_option]\n");
   fprintf(stderr, "           [[user@]host[#port]:]file ...\n");
   fprintf(stderr, "           [[user@]host[#port]:]file_or_dir\n");
   fprintf(stderr, "\n");
@@ -378,81 +393,82 @@ void scp_usage(ScpSession session)
   fprintf(stderr, "  -D debug_level_spec  Set debug level.\n");
   fprintf(stderr, "  -d                   Force target to be a directory.\n");
   fprintf(stderr, "  -q                   Make scp quiet (only fatal errors "
-                  "are displayed).\n");
+          "are displayed).\n");
   fprintf(stderr, "  -Q                   Don't show progress indicator.\n");
   fprintf(stderr, "  -p                   Preserve file attributes and "
-                  "timestamps.\n");
-/*  fprintf(stderr, "  -n                   Show what would've been done "
-                  "without actually copying\n");
-  fprintf(stderr, "                       any files.\n");*/
+          "timestamps.\n");
+  /*  fprintf(stderr, "  -n                   Show what would've been done "
+      "without actually copying\n");
+      fprintf(stderr, "                       any files.\n");*/
   fprintf(stderr, "  -u                   Remove source files after "
-                  "copying.\n");
+          "copying.\n");
   fprintf(stderr, "  -B                   Sets batch-mode on.\n");
   fprintf(stderr, "  -r                   Recurse subdirectories.\n");
   fprintf(stderr, "  -v                   Verbose mode; equal to `-D 2'.\n");
   fprintf(stderr, "  -1                   Engage scp1 compatibility.\n");
   fprintf(stderr, "  -c cipher            Select encryption algorithm. "
-                  "Multiple -c options are \n");
+          "Multiple -c options are \n");
   fprintf(stderr, "                       allowed and a single -c flag "
-                  "can have only one cipher.\n");
+          "can have only one cipher.\n");
   fprintf(stderr, "  -S ssh2-path         Tell scp2 where to find ssh2.\n");
   fprintf(stderr, "  -P ssh2-port         Tell scp2 which port sshd2 "
-                  "listens on the remote machine.\n");
-  fprintf(stderr, "  -V                   Display version.\n");
-  fprintf(stderr, "  -h                   Display this help.\n");
-  fprintf(stderr, "\n");
-}
-#else
-void scp_version(ScpSession session)
-{
-  fprintf(stderr, "SSH scp2.exe version: %s\n"
-                  "Copyright (c) 2000 SSH Communications Security Corp\n",
-                  SSH_SCP_VERSION_WITH_BUILD);
-}
-
-void scp_usage(ScpSession session)
-{
-  SSH_PRECOND(session);
-  
-  scp_version(session);
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Usage: %s [-D debug_level] [-d] [-q] [-Q] [-p] [-u] "
-                  "[-r] [-v] [-c cipher]\n", session->scp2_name);
-  fprintf(stderr, "            [-C] [-P ssh2-port] [-f fw-name] [-F fw-port]"
-                  " [-k dir] [-V] [-h]\n");
-  fprintf(stderr, "            [[user@]host[#port]:]file ...\n");
-  fprintf(stderr, "            [[user@]host[#port]:]file_or_dir\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Options:\n");
-  fprintf(stderr, "  -D debug_level_spec  Set debug level. (Syntax is module=level)\n");
-  fprintf(stderr, "  -d                   Force target to be a directory.\n");
-  fprintf(stderr, "  -q                   Make scp quiet (only fatal errors "
-                  "are displayed).\n");
-  fprintf(stderr, "  -Q                   Don't show progress indicator.\n");
-  fprintf(stderr, "  -p                   Preserve file attributes and "
-                  "timestamps.\n");
-  fprintf(stderr, "  -u                   Remove source files after "
-                  "copying.\n");
-  fprintf(stderr, "  -r                   Recurse subdirectories.\n");
-  fprintf(stderr, "  -v                   Verbose mode; equal to `-D 2'.\n");
-  fprintf(stderr, "  -c cipher            Select encryption algorithm. "
-                  "Multiple -c options are \n");
-  fprintf(stderr, "                       allowed and a single -c flag "
-                  "can have only one cipher.\n");
-  fprintf(stderr, "  -C                   Sets compression on. "
-                  "Default is off.\n");
-  fprintf(stderr, "  -P ssh2-port         sshd2 port.\n");
-  fprintf(stderr, "  -f fw-name           Firewall name.\n");
-  fprintf(stderr, "  -F fw-port           Firewall port.\n");
-  fprintf(stderr, "  -k dir               Store host keys and read user keys "
-                  "from this dir \n");
-  fprintf(stderr, "                       instead of the user profile dir.\n");
+          "listens on the remote machine.\n");
+  fprintf(stderr, "  -o                   Specify additional options "
+          "for ssh2.\n");
   fprintf(stderr, "  -V                   Display version.\n");
   fprintf(stderr, "  -h                   Display this help.\n");
   fprintf(stderr, "\n");
 }
 
-#endif /* SSH_WIN_CLIENT */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 void glob_ready_cb(SshFileCopyError error,
                    const char *error_message,
@@ -556,14 +572,14 @@ SSH_FSM_STEP(scp_glob)
 
   ssh_dllist_fw(gdata->original_file_list, 1);
 
-    /* XXX local files shouldn't be globbed, as they are globbed by
-       the shell. If wildcards survive to here, then the user has
-       deliberately escaped them, and we should use them
-       literalily. This remains here for the time being for debugging
-       purposes. */
+  /* XXX local files shouldn't be globbed, as they are globbed by
+     the shell. If wildcards survive to here, then the user has
+     deliberately escaped them, and we should use them
+     literalily. This remains here for the time being for debugging
+     purposes. */
 
-    /* Note: On Windows we require globbing here since the 'shell' doesn't
-       do globbing. */
+  /* Note: On Windows we require globbing here since the 'shell' doesn't
+     do globbing. */
   SSH_FSM_ASYNC_CALL(ssh_file_copy_glob(*(item->connection),    \
                                         item->location,         \
                                         NULL,                   \
@@ -656,7 +672,12 @@ void recurse_nonfatal_error_cb(SshFileCopyError error,
 
 void mapcar_func(SshFileCopyFile file, void *context)
 {
-  ssh_debug("%s", ssh_file_copy_file_get_name(file));
+  char *filename = (char *)ssh_file_copy_file_get_name(file);
+
+  if (filename == NULL)
+    return;
+  
+  ssh_debug("%s", filename);
 }
 
 SSH_FSM_STEP(scp_recurse)
@@ -696,8 +717,8 @@ SSH_FSM_STEP(scp_recurse)
 }
 
 void transfer_ready_cb(SshFileCopyError error,
-                      const char *error_message,
-                      void *context)
+                       const char *error_message,
+                       void *context)
 {
   SshFSMThread thread = (SshFSMThread) context;
   ScpSession session = ssh_fsm_get_gdata(thread);
@@ -858,7 +879,7 @@ void transfer_progress_cb(SshFileCopyConnection source,
   if (!at_end)
     {
       eta_string = stat_eta((SshUInt64)((attrs->size - written_bytes) /
-                            (transfer_rate < 1.0 ? 1.0 : transfer_rate)));
+                                        (transfer_rate < 1.0 ? 1.0 : transfer_rate)));
     }
   else
     {
@@ -874,7 +895,7 @@ void transfer_progress_cb(SshFileCopyConnection source,
       prev_width = session->screen_width;
 
       snprintf(format_string, sizeof(format_string),
-              "\r%%-%d.%ds | %%5sB | %%5.1f kB/s | %%3.3s: %%s | %%3d%%%%",
+               "\r%%-%d.%ds | %%5sB | %%5.1f kB/s | %%3.3s: %%s | %%3d%%%%",
                session->screen_width - 47, session->screen_width - 47);
     }
   
@@ -985,7 +1006,7 @@ void scp_interrupt_signal_handler(int sig, void *context)
   ssh_unregister_signal(SIGINT);
 }
 #endif /* TEST_TRANSFER_ABORT */
-SshFSMStateMapItem scp_states[] =
+SshFSMStateMapItemStruct scp_states[] =
 { { "scp_glob", "Glob locations", scp_glob },
   { "scp_recurse", "Recurse source directories", scp_recurse },
   { "scp_transfer", "Tranfer files", scp_transfer },
@@ -1016,183 +1037,194 @@ int main(int argc, char **argv)
   ssh_debug_register_callbacks(scp_fatal, scp_warning, scp_debug,
                                (void *)(session));
 
-#ifndef SSH_WIN_CLIENT
+
   if ((filename = strrchr(argv[0], '/')) != NULL)
     {
       filename++;
-      session->scp2_name = ssh_xstrdup(filename);
     }
   else
     {
-      session->scp2_name = ssh_xstrdup(argv[0]);
+      filename = argv[0];
     }
-#else
-  session->scp2_name = ssh_xstrdup("scp2");
-  ssh_register_program_name("scp2");
-#endif /* SSH_WIN_CLIENT */
+  session->scp2_name = ssh_xstrdup(filename);
+  ssh_register_program_name(filename);
 
+
+
+
+  
   filename = NULL;
 
-#ifndef SSH_WIN_CLIENT
-  while ((ch = ssh_getopt(argc, argv, "qQdpvuhS:P:c:D:tf1rOVB", NULL)) != -1)
-#else
-  while ((ch = ssh_getopt(argc, argv, "qQdpvuhP:c:D:f:F:k:rVC", NULL)) != -1)
-#endif /* SSH_WIN_CLIENT */
+
+  while ((ch = ssh_getopt(argc, argv, "qQdpvuhS:P:c:D:tf1rVBo:i:", NULL))
+         != -1)
+
+
+
     {
-      if (!ssh_optval)
-        {
-          scp_usage(session);
-          exit(SSH_FC_ERROR);
-        }
-      switch(ch)
-        {
-#ifndef SSH_WIN_CLIENT
-        case 't':
-        case 'f':
-          /* Scp 1 compatibility mode, this is remote server for ssh 1 scp,
-             exec old scp here. */
+        if (!ssh_optval)
           {
-            ssh_warning("Executing scp1 compatibility.");
-            execvp("scp1", argv);
-            ssh_fatal("Executing ssh1 in compatibility mode failed (Check "
-                      "that scp1 is in your PATH).");
+            scp_usage(session);
+            exit(SSH_FC_ERROR);
           }
-          break;
-        case '1':
-          /* Scp 1 compatibility in the client */
+        switch(ch)
           {
-            char **av;
-            int j;
-            int removed_flag = FALSE;
 
-            av = ssh_xcalloc(argc, sizeof(char *));
-            ssh_warning("Executing scp1 compatibility.");
-            for (i = 0, j = 0 ; i < argc ; i++)
-              {
-
-                SSH_DEBUG(3, ("argv[%d] = %s", i, argv[i]));
-
-                /* skip first "-1" argument */
-                if (strcmp("-1", argv[i]) || removed_flag)
-                  {
-                    av[j] = argv[i];
-                    j++;
-                  }
-                else
-                  {
-                    removed_flag = TRUE;
-                  }
-
-              }
-
-            execvp("scp1", av);
-            ssh_fatal("Executing ssh1 in compatibility mode failed (Check "
-                      "that scp1 is in your PATH).");
-          }
-          break;
-#else
-        case 'C':
-          session->compression = TRUE;
-          break;
-
-        case 'f':
-          session->fw_name = ssh_xstrdup(ssh_optarg);
-          break;
-
-        case 'F':
-          session->fw_port = ssh_xstrdup(ssh_optarg);
-          break;
-
-        case 'k':
-          session->keys_dir = ssh_xstrdup(ssh_optarg);
-          break;
-
-#endif /* SSH_WIN_CLIENT */
-        case 'p':
-          session->attrs->preserve_attributes = TRUE;
-          break;
-        case 'r':
-          session->attrs->recurse_dirs = TRUE;
-          break;
-        case 'P':
-          session->remote_port = ssh_xstrdup(ssh_optarg);
-          
-          {
-            int port = atoi(session->remote_port);
-            
-            if ((port <= 0) || (port > 65535))
-              {
-                ssh_warning("Port specification '%s' is invalid or out of "
-                            "range.",
-                            session->remote_port);
-                scp_usage(session);
-                exit(SSH_FC_ERROR);
-              }
-          }
-          
-          break;
-        case 'c':
-          if ( ssh_dllist_add_item(session->cipher_list,
-                                   (void *)ssh_xstrdup(ssh_optarg),
-                                   SSH_DLLIST_END) != SSH_DLLIST_OK)
+          case 't':
+          case 'f':
+            /* Scp 1 compatibility mode, this is remote server for ssh 1 scp,
+               exec old scp here. */
             {
-              ssh_fatal("Error in cipher list handling.");
+              ssh_warning("Executing scp1 compatibility.");
+              execvp("scp1", argv);
+              ssh_fatal("Executing ssh1 in compatibility mode failed (Check "
+                        "that scp1 is in your PATH).");
             }
-          break;
-        case 'S':
-          ssh_xfree(session->ssh_path);
-          session->ssh_path = ssh_xstrdup(ssh_optarg);
-          break;
-        case 'd':
-          session->attrs->destination_must_be_dir = TRUE;
-          break;
-        case 'B':
-          session->batch_mode = TRUE;
-          break;
-        case 'D':
-          session->debug_level = ssh_xstrdup(ssh_optarg);
-          ssh_debug_set_level_string(session->debug_level);
-          session->verbose_mode = TRUE;
-          break;
-        case 'v':
-          session->debug_level = ssh_xstrdup("2");
-          ssh_debug_set_level_string(session->debug_level);
-          session->verbose_mode = TRUE;
-          break;
-        case 'q':
-          session->quiet_mode = TRUE;
-          break;
-        case 'Q':
-          session->no_statistics = TRUE;
-          break;
-        case 'u':
-          session->attrs->unlink_source = TRUE;
-          break;
-        case 'n':
-          session->do_not_copy = TRUE;
-          break;
-        case 'h':
-          scp_usage(session);
-          exit(SSH_FC_OK);
-          break;
-        case 'V':
-#ifndef SSH_WIN_CLIENT
-          ssh2_version(argv[0]);
-#else
-          scp_version(session);
-#endif /* SSH_WIN_CLIENT */
-          exit(SSH_FC_OK);
-          break;
-        default:
-          scp_usage(session);
-          exit(SSH_FC_ERROR);
-          break;
-        }
-    }
+            break;
+          case '1':
+            /* Scp 1 compatibility in the client */
+            {
+              char **av;
+              int j;
+              int removed_flag = FALSE;
+
+              av = ssh_xcalloc(argc, sizeof(char *));
+              ssh_warning("Executing scp1 compatibility.");
+              for (i = 0, j = 0 ; i < argc ; i++)
+                {
+
+                  SSH_DEBUG(3, ("argv[%d] = %s", i, argv[i]));
+
+                  /* skip first "-1" argument */
+                  if (strcmp("-1", argv[i]) || removed_flag)
+                    {
+                      av[j] = argv[i];
+                      j++;
+                    }
+                  else
+                    {
+                      removed_flag = TRUE;
+                    }
+
+                }
+
+              execvp("scp1", av);
+              ssh_fatal("Executing ssh1 in compatibility mode failed (Check "
+                        "that scp1 is in your PATH).");
+            }
+            break;
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+          case 'p':
+            session->attrs->preserve_attributes = TRUE;
+            break;
+          case 'r':
+            session->attrs->recurse_dirs = TRUE;
+            break;
+          case 'P':
+            session->remote_port = ssh_xstrdup(ssh_optarg);
+          
+            {
+              int port = atoi(session->remote_port);
+            
+              if ((port <= 0) || (port > 65535))
+                {
+                  ssh_warning("Port specification '%s' is invalid or out of "
+                              "range.",
+                              session->remote_port);
+                  scp_usage(session);
+                  exit(SSH_FC_ERROR);
+                }
+            }
+          
+            break;
+          case 'c':
+            if ( ssh_dllist_add_item(session->cipher_list,
+                                     (void *)ssh_xstrdup(ssh_optarg),
+                                     SSH_DLLIST_END) != SSH_DLLIST_OK)
+              {
+                ssh_fatal("Error in cipher list handling.");
+              }
+            break;
+          case 'S':
+            ssh_xfree(session->ssh_path);
+            session->ssh_path = ssh_xstrdup(ssh_optarg);
+            break;
+          case 'd':
+            session->attrs->destination_must_be_dir = TRUE;
+            break;
+          case 'B':
+            session->batch_mode = TRUE;
+            break;
+          case 'D':
+            session->debug_level = ssh_xstrdup(ssh_optarg);
+            ssh_debug_set_level_string(session->debug_level);
+            session->verbose_mode = TRUE;
+            session->debug_mode = TRUE;
+            break;
+          case 'v':
+            session->debug_level = ssh_xstrdup("2");
+            ssh_debug_set_level_string(session->debug_level);
+            session->verbose_mode = TRUE;
+            break;
+          case 'q':
+            session->quiet_mode = TRUE;
+            break;
+          case 'Q':
+            session->no_statistics = TRUE;
+            break;
+          case 'u':
+            session->attrs->unlink_source = TRUE;
+            break;
+          case 'n':
+            session->do_not_copy = TRUE;
+            break;
+          case 'h':
+            scp_usage(session);
+            exit(SSH_FC_OK);
+            break;
+          case 'i':
+            session->identity_file = ssh_xstrdup(ssh_optarg);
+            break;
+          case 'V':
+
+            ssh2_version(NULL);
+
+
+
+            exit(SSH_FC_OK);
+            break;
+          case 'o':
+            SSH_VERIFY(ssh_dllist_add_item(session->ssh_options,
+                                           ssh_xstrdup(ssh_optarg),
+                                           SSH_DLLIST_END) == SSH_DLLIST_OK);
+            break;
+          default:
+            scp_usage(session);
+            exit(SSH_FC_ERROR);
+            break;
+          }
+      }
+
 
-#ifndef SSH_WIN_CLIENT
   ssh_register_signal(SIGHUP, scp_fatal_signal_handler, (void *)session);
-#endif
+
 #ifdef SIGIO
   ssh_register_signal(SIGIO, NULL, NULL);
   /* This has to be done, because for example in NetBSD, the system
@@ -1245,13 +1277,13 @@ int main(int argc, char **argv)
       exit(SSH_FC_ERROR);
     }
 
-#ifndef SSH_WIN_CLIENT
+
   /* This is because in some cases, where ssh2 dies (ie. it is killed
      etc.) scp2 receives SIGPIPE, which kills it, too. This is not
      good, as sshfilecopy has an internal error recovery system. */
   ssh_register_signal(SIGPIPE, NULL, NULL);
   ssh_sigchld_initialize();
-#endif /* SSH_WIN_CLIENT */
+
 
   /* Parse source locations */
   for (i = ssh_optind; i < argc - 1; i++)
@@ -1277,18 +1309,18 @@ int main(int argc, char **argv)
          should be preceded by current working directory. */
       if ((*source_connection)->host == NULL)
         {
-#ifndef SSH_WIN_CLIENT
+
           if (*filename != '/')
-#else
-          if (*filename != '/' && *filename != '\\' && filename[1] != ':')
-#endif /* SSH_WIN_CLIENT */
+
+
+
             {
               char *new_filename, *cwd;
-
+            
               cwd = ssh_getcwd();
-
+            
               ssh_dsprintf(&new_filename, "%s/%s", cwd, filename);
-
+            
               ssh_xfree(cwd);
               ssh_xfree(filename);
               filename = new_filename;
@@ -1341,25 +1373,25 @@ int main(int argc, char **argv)
 
   if ((*destination->connection)->host == NULL)
     {
-#ifndef SSH_WIN_CLIENT
-      if (*filename != '/')
-#else
-      if (*filename != '/' && *filename != '\\' && filename[1] != ':')
-#endif /* SSH_WIN_CLIENT */
-        {
-          char *new_filename, *cwd;
-          
-          cwd = ssh_getcwd();
 
-          if (cwd == NULL)
-            ssh_fatal("Couldn't get path of current working directory");
+      if (*filename != '/')
+
+
+
+          {
+            char *new_filename, *cwd;
           
-          ssh_dsprintf(&new_filename, "%s/%s", cwd, filename);
+            cwd = ssh_getcwd();
+
+            if (cwd == NULL)
+              ssh_fatal("Couldn't get path of current working directory");
           
-          ssh_xfree(cwd);
-          ssh_xfree(filename);
-          filename = new_filename;
-        }
+            ssh_dsprintf(&new_filename, "%s/%s", cwd, filename);
+          
+            ssh_xfree(cwd);
+            ssh_xfree(filename);
+            filename = new_filename;
+          }
 
     }
 
@@ -1394,16 +1426,15 @@ int main(int argc, char **argv)
 
   ssh_event_loop_uninitialize();
   
-#ifdef SSH_WIN_CLIENT
-  /* Note: The random pool is loaded in SshConnect(). */
-  SshSaveRandomPool();
-#endif
+
+
+
+
 
   return exit_value;
 }
 
 
-#ifndef SSH_WIN_CLIENT
 void new_stream_callback(SshStreamNotification notification, void *context)
 {
   ScpConnectionCtx ctx = (ScpConnectionCtx) context;
@@ -1415,17 +1446,20 @@ void new_stream_callback(SshStreamNotification notification, void *context)
     {
     case SSH_STREAM_INPUT_AVAILABLE:
 
-      while ((ret = ssh_stream_read(ctx->stream,
-                                    &(ctx->buffer[ctx->read_bytes]), 1)) > 0 &&
+      while ((ret =
+              ssh_stream_read(ctx->stream,
+                              (unsigned char *)&(ctx->buffer[ctx->read_bytes]),
+                              1)) > 0 &&
              ctx->read_bytes < sizeof(ctx->buffer) - 1)
         {
           ctx->read_bytes += ret;
 
-          SSH_TRACE(4, ("read char: %c", ctx->buffer[strlen(ctx->buffer) - 1]));
+          SSH_TRACE(4, ("read char: %c",
+                        ctx->buffer[strlen(ctx->buffer) - 1]));
           SSH_TRACE(4, ("read_bytes: %d, buffer len: %d",       \
                         ctx->read_bytes, strlen(ctx->buffer)));
           SSH_DEBUG_HEXDUMP(4, ("received message:"),           \
-                            ctx->buffer, ctx->read_bytes);
+                            (unsigned char *)ctx->buffer, ctx->read_bytes);
           fflush(stderr);
           if (ctx->buffer[strlen(ctx->buffer) - 1] == '\n')
             {
@@ -1438,7 +1472,8 @@ void new_stream_callback(SshStreamNotification notification, void *context)
               else
                 {
                   SSH_DEBUG_HEXDUMP(3, ("received message:"),           \
-                                    ctx->buffer, ctx->read_bytes);
+                                    (unsigned char *)ctx->buffer,
+                                    ctx->read_bytes);
                   ssh_warning("ssh2 client failed to authenticate. (or you "
                               "have too old ssh2 installed, check "
                               "with ssh2 -V)");
@@ -1470,20 +1505,20 @@ void new_stream_callback(SshStreamNotification notification, void *context)
   ssh_stream_destroy(ctx->stream);
   (*ctx->completion_cb)(NULL, ctx->completion_context);
 }
-#else /* SSH_WIN_CLIENT */
-void new_stream_callback(Boolean ok, SshStream stream, void *context)
-{
-  ScpConnectionCtx ctx = (ScpConnectionCtx) context;
-  if (!ok)
-  {
-    ssh_warning("Failed to authenticate.");
-  }
-  else
-  {
-    (*ctx->completion_cb)(stream, ctx->completion_context);
-  }
-}
-#endif /* SSH_WIN_CLIENT */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 void connect_callback(SshFileCopyConnection connection, void *context,
                       SshFileCopyStreamReturnCB completion_cb,
@@ -1492,12 +1527,38 @@ void connect_callback(SshFileCopyConnection connection, void *context,
   ScpSession session = (ScpSession) context;
   ScpConnectionCtx new_ctx;
 
-#ifndef SSH_WIN_CLIENT
-#define SSH_ARGV_SIZE   64
+#define SSH_ARGV_SIZE   128
   SshStream client_stream;
-  char *ssh_argv[SSH_ARGV_SIZE];
+  char *ssh_argv[SSH_ARGV_SIZE], *option_str = NULL;
   int i;
-  
+
+  char *strcipherlist = NULL, *strnewlist = NULL, *strcipher = NULL;
+
+  new_ctx = ssh_xcalloc(1, sizeof(*new_ctx));
+  new_ctx->session = session;
+  new_ctx->completion_cb = completion_cb;
+  new_ctx->completion_context = completion_context;
+
+  /* Build a string of list of ciphers */
+  ssh_dllist_rewind(session->cipher_list);
+  strcipherlist = NULL;
+  while ((strcipher = ssh_dllist_current(session->cipher_list)) != NULL)
+    {
+      if (strcipherlist != NULL)
+        {
+          ssh_dsprintf(&strnewlist, "%s,%s", strcipherlist, strcipher);
+          ssh_xfree(strcipherlist);
+        }
+      else
+        {
+          strnewlist = ssh_xstrdup(strcipher);
+        }
+      
+      strcipherlist = strnewlist;
+      ssh_dllist_fw(session->cipher_list, 1);
+    }
+
+
   i = 0;
 
   ssh_argv[i++] = session->ssh_path;
@@ -1517,26 +1578,61 @@ void connect_callback(SshFileCopyConnection connection, void *context,
       ssh_argv[i++] = "-p";
       ssh_argv[i++] = session->remote_port;
     }
-  
-  if (session->verbose_mode)
-    ssh_argv[i++] = "-v";
 
+  if (session->debug_mode)
+    {
+      ssh_argv[i++] = "-d";
+      ssh_argv[i++] = session->debug_level;
+    }
+  else if (session->verbose_mode)
+    {
+      ssh_argv[i++] = "-v";
+    }
+  
   if (session->batch_mode)
     {
       ssh_argv[i++] = "-o";
       ssh_argv[i++] = "batchmode yes";
     }
-  
+
+  if (session->identity_file)
+    {
+      ssh_argv[i++] = "-i";
+      ssh_argv[i++] = session->identity_file;
+    }
+
   ssh_argv[i++] = "-x";
   ssh_argv[i++] = "-a";
-  
+
+  ssh_argv[i++] = "-o";
+  ssh_argv[i++] = "clearallforwardings yes";
+
   ssh_argv[i++] = "-o";
   ssh_argv[i++] = "passwordprompt %U@%H's password: ";
   ssh_argv[i++] = "-o";
   ssh_argv[i++] = "nodelay yes";
   ssh_argv[i++] = "-o";
   ssh_argv[i++] = "authenticationnotify yes";
-  
+  if (strcipherlist)
+    {
+      ssh_argv[i++] = "-o";
+      ssh_dsprintf(&strnewlist, "ciphers %s", strcipherlist);
+      ssh_xfree(strcipherlist);
+      strcipherlist = strnewlist;
+      ssh_argv[i++] = strcipherlist;
+    }
+
+  ssh_dllist_rewind(session->ssh_options);
+  while ((option_str = ssh_dllist_current(session->ssh_options)) != NULL)
+    {
+      if (i >= SSH_ARGV_SIZE - 2 - 4 - 1)
+        ssh_fatal("Too many arguments for %s (at least %d given).",
+                  session->ssh_path, i + 2 + 1); /* Starts from zero... */
+      ssh_argv[i++] = "-o";
+      ssh_argv[i++] = option_str;
+      ssh_dllist_fw(session->ssh_options, 1);
+    }
+
   ssh_argv[i++] = connection->host;
 
   ssh_argv[i++] = "-s";
@@ -1560,6 +1656,7 @@ void connect_callback(SshFileCopyConnection connection, void *context,
       ssh_fatal("ssh_pipe_create_and_fork() failed");
       
     case SSH_PIPE_PARENT_OK:      
+      ssh_xfree(strcipherlist);
       new_ctx = ssh_xcalloc(1, sizeof(*new_ctx));
       new_ctx->child_pid = ssh_pipe_get_pid(client_stream);
       new_ctx->stream = client_stream;
@@ -1588,56 +1685,30 @@ void connect_callback(SshFileCopyConnection connection, void *context,
       exit(-1);
     }
   SSH_NOTREACHED;
-#else /* SSH_WIN_CLIENT */
-  char *strcipherlist;
-  char *strnewlist;
-  char *strcipher;
 
-  new_ctx = ssh_xcalloc(1, sizeof(*new_ctx));
-  new_ctx->session = session;
-  new_ctx->completion_cb = completion_cb;
-  new_ctx->completion_context = completion_context;
 
-  /* Build a string of list of ciphers */
-  ssh_dllist_rewind(session->cipher_list);
-  strcipherlist = NULL;
-  while (strcipher = ssh_dllist_current(session->cipher_list))
-  {
-    if (strcipherlist != NULL)
-    {
-      strnewlist = ssh_string_concat_3(strcipherlist, ",", strcipher);
-      ssh_xfree(strcipherlist);
-    }
-    else
-      strnewlist = ssh_xstrdup(strcipher);
 
-    strcipherlist = strnewlist;
-    ssh_dllist_fw(session->cipher_list, 1);
-  }
 
-  /* Try to connect to the remote host. new_stream_callback() will be called
-     once the connection is up. */
-  SshConnect(connection->user,
-    connection->host,
-    session->remote_port,
-    strcipherlist,
-    session->compression,
-    session->fw_name,
-    session->fw_port,
-    session->keys_dir,
-    new_stream_callback,
-    new_ctx);
 
-  ssh_xfree(strcipherlist);
-#endif /* SSH_WIN_CLIENT */
+
+
+
+
+
+
+
+
+
+
+
 }
 
 Boolean recurse_func(const char *filename,
-                      const char *long_name,
-                      SshFileAttributes
-                      file_attributes,
-                      SshFileCopyRecurseAttrs attrs,
-                      void *context)
+                     const char *long_name,
+                     SshFileAttributes
+                     file_attributes,
+                     SshFileCopyRecurseAttrs attrs,
+                     void *context)
 {
 #if 0
   fprintf(stderr, "%s -- %s,perms 0%o\n", filename, long_name,
@@ -1717,20 +1788,20 @@ Boolean scp_location_parse(SshFileCopyConnection connection,
       return TRUE;
     }
   
-#ifndef SSH_WIN_CLIENT
+
   string = ssh_xstrdup(orig_string);
 
   /* Separate filename and [[user@]host[#port] */
   if ((filename = ssh_glob_next_unescaped_char(string, ':')) != NULL)
-#else
-  /* Replace "*.*" with single "*". */
-  string = ssh_replace_in_string(orig_string, "*.*", "*");
-  
-  /* File names on Windows may have form "C:\foo\readme.txt". The first ':'
-     should not be parsed as host/file name separator. */
-  if (orig_string[1] != ':' &&
-    (filename = ssh_glob_next_unescaped_char(string, ':')) != NULL)
-#endif /* SSH_WIN_CLIENT */
+
+
+
+
+
+
+
+
+
     {
       *filename = '\0';
       filename++;
@@ -1815,8 +1886,8 @@ Boolean scp_location_parse(SshFileCopyConnection connection,
 
 void scp_add_raw_file(SshFileCopyLocation location, char *filename)
 {
-#ifdef SSH_WIN_CLIENT
-  replace_backslashes(filename);
-#endif /* SSH_WIN_CLIENT */
+
+
+
   ssh_file_copy_location_add_raw_file(location, filename);
 }
