@@ -3,20 +3,20 @@
 ssh-add.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1999 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Adds an identity to the authentication server, or removes an identity.
+  Adds an identity to the authentication server, or removes an identity.
 
 */
 
 /*
- * $Id: ssh-add2.c,v 1.23 1999/05/04 02:14:41 kivinen Exp $
+ * $Id: ssh-add2.c,v 1.32.2.1 2000/12/04 20:23:52 sjl Exp $
  * $Log: ssh-add2.c,v $
  * $EndLog$
  */
@@ -29,7 +29,7 @@ Adds an identity to the authentication server, or removes an identity.
 #include "sshuser.h"
 #include "readpass.h"
 #include "sshuserfiles.h"
-#include "sshunixeloop.h"
+#include "ssheloop.h"
 #include "sshgetopt.h"
 #include "sshmiscstring.h"
 #ifdef WITH_PGP
@@ -38,7 +38,9 @@ Adds an identity to the authentication server, or removes an identity.
 #include "sshpgp.h"
 #include "ssh2pgp.h"
 #include "ssh2pubkeyencode.h"
+#include "ssh1keydecode.h"
 #endif /* WITH_PGP */
+#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "SshAdd"
 
@@ -95,6 +97,10 @@ SshTime key_timeout = 0;
 Boolean have_attrs = FALSE;
 SshUInt32 use_limit = 0xffffffff;
 
+unsigned char *user_cert = NULL;
+size_t user_cert_len = 0;
+#define SSH_ADD_USER_CERT_MAX_SIZE 0x8000
+
 /* Force to read passphrases from stdin. */
 int use_stdin = FALSE;
 
@@ -105,10 +111,10 @@ void agent_completion(SshAgentError result, void *context);
 
 void add_file(SshAgent agent, const char *filename)
 {
-  SshPrivateKey key;
+  SshPrivateKey key = NULL;
   char *saved_comment, *comment = NULL, *pass;
   int query_cnt;
-  unsigned char *certs;
+  unsigned char *certs = NULL;
   size_t certs_len;
   char privname[500], pubname[500];
   unsigned long magic;
@@ -176,6 +182,7 @@ void add_file(SshAgent agent, const char *filename)
       /* Read the public key blob. */
       magic = ssh2_key_blob_read(user, 
                                  pubname,
+                                 TRUE,
                                  &saved_comment,
                                  &certs, 
                                  &certs_len,
@@ -183,6 +190,7 @@ void add_file(SshAgent agent, const char *filename)
       if (magic != SSH_KEY_MAGIC_PUBLIC)
         {
           printf("Bad public key file %s\n", pubname);
+          ssh_xfree(certs);
           (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
           return;
         }
@@ -238,10 +246,26 @@ void add_file(SshAgent agent, const char *filename)
               else
                 {
                   if (query_cnt <= 1)
-                    printf("Need passphrase for %s (%s).\n", 
-                           privname, saved_comment);
+                    {
+                      if ((strcmp(privname, saved_comment) == 0) ||
+                          (((strlen(privname) + 4) == strlen(saved_comment)) &&
+                           (strncmp(privname, 
+                                    saved_comment, 
+                                    strlen(privname)) == 0)))
+                        {
+                          printf("Need passphrase for %s.\n", 
+                                 privname);
+                        }
+                      else
+                        {
+                          printf("Need passphrase for %s (%s).\n", 
+                                 privname, saved_comment);
+                        }
+                    }
                   else
-                    printf("Bad passphrase.\n");
+                    {
+                      printf("Bad passphrase.\n");
+                    }
                   pass = ssh_read_passphrase("Enter passphrase: ", use_stdin);
                   if (pass == NULL || strcmp(pass, "") == 0)
                     {
@@ -280,20 +304,36 @@ void add_file(SshAgent agent, const char *filename)
         {
           /* Send the key to the authentication agent. */
           if (have_attrs)
-            ssh_agent_add_with_attrs(agent, key, certs, certs_len, 
-                                     saved_comment, path_limit, 
-                                     path_constraint, use_limit,
-                                     forbid_compat, key_timeout,
-                                     agent_completion, (void *)agent);
+            ssh_agent_add_with_attrs(agent,
+                                     key, 
+                                     (user_cert ? user_cert : certs), 
+                                     (user_cert ? user_cert_len : certs_len),
+                                     saved_comment,
+                                     path_limit, 
+                                     path_constraint,
+                                     use_limit,
+                                     forbid_compat, 
+                                     key_timeout,
+                                     agent_completion,
+                                     (void *)agent);
           else
-            ssh_agent_add(agent, key, certs, certs_len, saved_comment,
-                          agent_completion, (void *)agent);
+            ssh_agent_add(agent, 
+                          key, 
+                          (user_cert ? user_cert : certs), 
+                          (user_cert ? user_cert_len : certs_len),
+                          saved_comment,
+                          agent_completion, 
+                          (void *)agent);
           ssh_private_key_free(key);
         }
       else if (action == DELETE)
         {
-          ssh_agent_delete(agent, certs, certs_len, saved_comment,
-                           agent_completion, (void *)agent);
+          ssh_agent_delete(agent, 
+                           (user_cert ? user_cert : certs), 
+                           (user_cert ? user_cert_len : certs_len),
+                           saved_comment,
+                           agent_completion, 
+                           (void *)agent);
         }
       ssh_xfree(saved_comment);
     }
@@ -302,7 +342,7 @@ void add_file(SshAgent agent, const char *filename)
     {
       unsigned char *blob, *public_blob;
       size_t blob_len, public_blob_len;
-      Boolean found;
+      Boolean found = FALSE;
       unsigned long id;
       char *endptr;
       SshPgpSecretKey pgp_key;
@@ -708,12 +748,20 @@ void usage()
 
 int main(int ac, char **av)
 {
-  int opt, i, len;
+  int opt, i;
   DIR *ssh2dir = NULL;
-  char *ssh2dirname;
+  char *ssh2dirname = NULL;
   Boolean dynamic_array = FALSE;
   struct dirent * cand;
 
+  /* ptrs for the list of private keys, and a config to open ident file */
+  struct SshConfigPrivateKey **priv_key_list;
+  struct SshConfigPrivateKey **tmp_priv_key_list;
+  struct SshConfigPrivateKey *tmp_key_ptr;
+  SshConfig config;
+  char temp_s[1024];
+
+
   /* Save program name. */
   if (strchr(av[0], '/'))
     av0 = strrchr(av[0], '/') + 1;
@@ -727,7 +775,7 @@ int main(int ac, char **av)
 #endif /* WITH_PGP */
 
   action = ADD;
-  while ((opt = ssh_getopt(ac, av, "ldDput:f:F:1LUNPI", NULL)) != EOF)
+  while ((opt = ssh_getopt(ac, av, "ldDput:f:F:1LUNPIC:", NULL)) != EOF)
     {
       if (!ssh_optval)
         {
@@ -781,6 +829,41 @@ int main(int ac, char **av)
           use_stdin = TRUE;
           break;
 
+        case 'C':
+          {
+            FILE *f;
+
+            f = fopen(ssh_optarg, "r");
+            if (f == NULL)
+              {
+                fprintf(stderr, "%s: Can't open certificate file \"%s\".\n", 
+                        ssh_optarg,
+                        av0);
+                exit(EXIT_STATUS_ERROR);
+              }
+            user_cert = ssh_xmalloc(SSH_ADD_USER_CERT_MAX_SIZE + 1);
+            user_cert_len = fread(user_cert, 
+                                  1,
+                                  SSH_ADD_USER_CERT_MAX_SIZE + 1, 
+                                  f);
+            fclose(f);
+            if (user_cert_len == 0) 
+              {
+                fprintf(stderr, "%s: Can't read certificate file \"%s\".\n", 
+                        ssh_optarg,
+                        av0);
+                exit(EXIT_STATUS_ERROR);
+              }
+            if (user_cert_len > SSH_ADD_USER_CERT_MAX_SIZE)
+              {
+                fprintf(stderr, "%s: Certificate file \"%s\" too big.\n", 
+                        ssh_optarg,
+                        av0);
+                exit(EXIT_STATUS_ERROR);
+              }
+          }
+          break;
+
         case 'd':
           if (action == ADD_URL)
             action = DELETE_URL;
@@ -865,28 +948,122 @@ int main(int ac, char **av)
   files = &av[ssh_optind];
   num_files = ac - ssh_optind;
 
-  /* Fetch default from ~/.ssh2/id_* (the first that we happen to get) */
 
-#define ID_PREFIX "id_"
-  
-#ifdef WITH_PGP
-  if ((ac == 1) && (num_files == 0) && (pgp_mode != PGP_KEY_NONE))
+  /* no key supplied on command line so look in identification file */
+  if (num_files == 0 && action != LIST && action != DELETE_ALL &&
+      action != LOCK && action != UNLOCK)
     {
-      fprintf(stderr, "%s: Nothing to do!",  av0);
-      exit(EXIT_STATUS_ERROR);
-    }
+
+#ifdef WITH_PGP
+      if (pgp_mode != PGP_KEY_NONE)
+        {
+          fprintf(stderr, "%s: Nothing to do!\n",  av0);
+          exit(EXIT_STATUS_ERROR);
+        }
 #endif /* WITH_PGP */
 
-  if (ac == 1 && num_files == 0)
+      config = NULL;
+      config = ssh_xcalloc( 1, sizeof(*config));
+
+      if (config == NULL)
+        {
+          ssh_fatal("Failed to allocate enough memory\n");
+        }
+
+      config = ssh_client_create_config();
+
+      /* Try to read the global configuration file */
+      ssh_config_read_file(user, config, NULL,
+                           SSH_CLIENT_GLOBAL_CONFIG_FILE, NULL);
+
+
+      /* Try to read in the user configuration file. */
+      ssh2dirname = ssh_userdir(user, config, TRUE);
+      if (ssh2dirname == NULL)
+        {
+          ssh_fatal("Failed to create user ssh directory.");
+        }
+
+      snprintf(temp_s, sizeof (temp_s), "%s/%s",
+               ssh2dirname, SSH_CLIENT_CONFIG_FILE);
+      ssh_xfree(ssh2dirname);
+
+      ssh_config_read_file(user, config, config->host_to_connect,
+                           temp_s, NULL);
+
+      /* get list of private keys from identification file */
+      priv_key_list = NULL;
+      priv_key_list = ssh_privkey_list( user, NULL, config );
+
+      num_files = 0;
+
+      if (priv_key_list != NULL)
+        {
+
+          /* count the number of private key files */
+
+          tmp_priv_key_list = priv_key_list;
+          while ((*tmp_priv_key_list) != NULL)
+            {
+              num_files++;
+              tmp_priv_key_list++;
+            }
+ 
+          /* put the file names into the files array
+             and free the ptr list at the same time */
+
+          files = ssh_xcalloc( num_files + 1, sizeof(char*) );
+          dynamic_array = TRUE;
+ 
+          for (i = 0; i < num_files; i++)
+            {
+              files[i] = ssh_xstrdup((*priv_key_list)->keyfile);
+
+              /* we've copied the file name, now delete the
+                 SshConfigPrivateKey struct */
+
+              ssh_xfree((*priv_key_list)->keyfile);
+
+#ifdef WITH_PGP
+             if ((*priv_key_list)->pgp_keyring)
+               ssh_xfree((*priv_key_list)->pgp_keyring);
+
+             if ((*priv_key_list)->pgp_name)
+               ssh_xfree((*priv_key_list)->pgp_name);
+
+             if ((*priv_key_list)->pgp_fingerprint)
+               ssh_xfree((*priv_key_list)->pgp_fingerprint);
+#endif /* WITH_PGP */
+
+              tmp_key_ptr = *priv_key_list;
+              priv_key_list++;
+              ssh_xfree(tmp_key_ptr);
+            }
+          }
+
+      ssh_xfree(config);
+
+    }
+
+  /* Fetch default from ~/.ssh2/id_* (the first that we happen to get)
+     if no key supplied from command line or none in identification file */
+
+#define ID_PREFIX "id"
+  
+  if (num_files == 0 && action != LIST && action != DELETE_ALL &&
+      action != LOCK && action != UNLOCK)
     {
-      /* len includes '/' and '\0'*/
-      len = strlen(ssh_user_dir(user)) + strlen(SSH_USER_DIR) + 2;
-      ssh2dirname = ssh_xcalloc(len, sizeof(char));
-      snprintf(ssh2dirname, len, "%s/%s", ssh_user_dir(user), SSH_USER_DIR);
-      
+#ifdef WITH_PGP
+      if (pgp_mode != PGP_KEY_NONE)
+        {
+          fprintf(stderr, "%s: Nothing to do!\n",  av0);
+          exit(EXIT_STATUS_ERROR);
+        }
+#endif /* WITH_PGP */
+      ssh_dsprintf(&ssh2dirname, "%s/%s", ssh_user_dir(user), SSH_USER_DIR);
       ssh2dir = opendir(ssh2dirname);
 
-      if (! ssh2dir)
+      if (!ssh2dir)
         {
           fprintf(stderr, "%s: Can't open directory \"%s\"", av0, ssh2dirname);
           exit(EXIT_STATUS_ERROR);
@@ -894,16 +1071,21 @@ int main(int ac, char **av)
           
       while ((cand = readdir(ssh2dir)) != NULL)
         {
-          if (strlen(cand->d_name) >= strlen(ID_PREFIX) &&
-              strncmp(cand->d_name, ID_PREFIX, strlen(ID_PREFIX)) == 0)
+          if ((strlen(cand->d_name) > strlen(ID_PREFIX)) &&
+              (strncmp(cand->d_name, ID_PREFIX, strlen(ID_PREFIX)) == 0) &&
+              ((strlen(cand->d_name) < 4) ||
+               (strcmp(cand->d_name + strlen(cand->d_name) - 4,
+                       ".pub") != 0)) &&
+              ((((cand->d_name)[strlen(ID_PREFIX)]) == '_') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '-') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '.') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '(') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '[') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '<') ||
+               (((cand->d_name)[strlen(ID_PREFIX)]) == '>')))
             {
               files = ssh_xcalloc(2, sizeof(char *));
-
-              /* len includes '/' and '\0'*/
-              len = strlen(ssh2dirname) + strlen(cand->d_name) + 2; 
-                                                                      
-              files[0] = ssh_xcalloc(len, sizeof(char));
-              snprintf(files[0],len, "%s/%s", ssh2dirname, cand->d_name);
+              ssh_dsprintf(&files[0], "%s/%s", ssh2dirname, cand->d_name);
               ssh_xfree(ssh2dirname);
               num_files++;
               dynamic_array = TRUE;
@@ -932,5 +1114,5 @@ int main(int ac, char **av)
     }
   
   ssh_user_free(user, FALSE);
-  exit(EXIT_STATUS_OK);
+  return EXIT_STATUS_OK;
 }
