@@ -1,18 +1,21 @@
 /*
 
-ssh-askpass.c
+  ssh-askpass.c
 
-Copyright (c) 1997 SSH Communications Security, Finland
-                   All rights reserved
+  Author: Antti Huima <huima@ssh.com>
+
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  All rights reserved.
 
 */
 
+#define SSH_ALLOW_CPLUSPLUS_KEYWORDS
 #include "sshincludes.h"
 #include "sshgetopt.h"
 
 #ifdef time
 #undef time
-#endif 
+#endif
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -22,6 +25,7 @@ Copyright (c) 1997 SSH Communications Security, Finland
 #include <X11/keysym.h>
 #include <X11/cursorfont.h>
 #include "sshmalloc.h"
+#include <stdlib.h>
 
 #define SSH_DEBUG_MODULE "SshAskPass"
 
@@ -99,7 +103,7 @@ void fatal(char *fmt, ...)
 
 /* Open connection to the X display. */
 
-void open_display()
+void open_display(void)
 {
   char *display_name;
 
@@ -115,7 +119,7 @@ void open_display()
 
 /* Close the display after freeing data structures. */
 
-void close_display()
+void close_display(void)
 {
   XFreeModifiermap(modifiers);
   XUnloadFont(display, font);
@@ -125,7 +129,7 @@ void close_display()
 
 /* Create the graphics contexts. Color version. */
 
-void create_GCs()
+void create_GCs(void)
 {
   int i;
   XGCValues values;
@@ -144,7 +148,7 @@ void create_GCs()
 
 /* Create the graphics contexts. B/W version. */
 
-void create_GCs_bw()
+void create_GCs_bw(void)
 {
   int i;
   XGCValues values;
@@ -166,7 +170,7 @@ void create_GCs_bw()
 /* Try to allocate the colors we would like to use.
    If any of the allocations fails, revert to black'n'white mode. */
 
-void allocate_colors()
+void allocate_colors(void)
 {
   XColor xcolor, exact_color;
   int red, green, blue;
@@ -181,39 +185,39 @@ void allocate_colors()
   red = exact_color.red;
   blue = exact_color.blue;
   green = exact_color.green;
-  
+
   red += 16384;
   if (red > 65535) red = 65535;
   blue += 16384;
   if (blue > 65535) blue = 65535;
   green += 16384;
   if (green > 65535) green = 65535;
-  
+
   exact_color.red = red;
   exact_color.blue = blue;
   exact_color.green = green;
-  
+
   if(!XAllocColor(display, map, &exact_color))
     goto revert;
 
   color[hilight] = exact_color.pixel;
-  
+
   red -= 32768;
   if (red < 0) red = 0;
   blue -= 32768;
   if (blue < 0) blue = 0;
   green -= 32768;
   if (green < 0) green = 0;
-  
+
   exact_color.red = red;
   exact_color.blue = blue;
   exact_color.green = green;
-  
+
   if (!XAllocColor(display, map, &exact_color))
     goto revert;
 
   color[shadow] = exact_color.pixel;
-  
+
   if (!XAllocNamedColor(display, map, foreground_color_name,
                    &xcolor, &exact_color))
     goto revert;
@@ -258,7 +262,7 @@ void button_dimensions(char *string, int *width, int *height)
 
 /* Open windows (the dialog box and the cancel button). */
 
-void open_windows()
+void open_windows(void)
 {
   int i; int scrap;
   XSetWindowAttributes attributes;
@@ -297,7 +301,7 @@ void open_windows()
                                 CopyFromParent, CWCursor |
                                 CWBackingStore,
                                 &attributes);
-  
+
   leds = (w_width - relief * 2 - margin * 2 + (led_width - led_i_width)) /
     led_width;
   scrap = (w_width - relief * 2 - margin * 2 + (led_width - led_i_width)) %
@@ -306,7 +310,7 @@ void open_windows()
 
   led_state = ssh_xmalloc(sizeof(int) * leds);
   for (i = 0; i < leds; i++) led_state[i] = 0;
-  
+
   attributes.cursor = XCreateFontCursor(display, XC_top_left_arrow);
   XChangeWindowAttributes(display, cancel_button, CWCursor | CWBackingStore,
                           &attributes);
@@ -314,7 +318,7 @@ void open_windows()
 
 /* Map the windows. */
 
-void map_windows()
+void map_windows(void)
 {
   XMapRaised(display, main_window);
   XMapWindow(display, cancel_button);
@@ -322,7 +326,7 @@ void map_windows()
 
 /* Load the font wanted. */
 
-void get_font()
+void get_font(void)
 {
   font_struct = XLoadQueryFont(display, font_spec);
   if (!font_struct)
@@ -336,9 +340,8 @@ void get_font()
 
 /* Calculate correct geometry for the main window. */
 
-void compute_dimensions()
+void compute_dimensions(void)
 {
-  Status status;
   Window root;
   int dummy, d_width, d_height;
   int dir, asc, desc;
@@ -356,10 +359,10 @@ void compute_dimensions()
   w_height = asc + desc - 1 + margin * 2 + b_height + led_height;
 
   w_height += relief * 2 + margin * 2;
-  status = XGetGeometry(display, DefaultRootWindow(display),
-                        &root, &dummy, &dummy, (unsigned int *)&d_width,
-                        (unsigned int *)&d_height, (unsigned int *)&dummy, 
-                        (unsigned int *)&dummy);
+  (void) XGetGeometry(display, DefaultRootWindow(display),
+                      &root, &dummy, &dummy, (unsigned int *)&d_width,
+                      (unsigned int *)&d_height, (unsigned int *)&dummy,
+                      (unsigned int *)&dummy);
   w_x = (d_width - w_width) / 2;
   w_y = (d_height - w_height) / 2;
 }
@@ -376,7 +379,7 @@ void get_resource(char *name, char *class, char **where)
 
 /* Read and get some resources. */
 
-void read_resources()
+void read_resources(void)
 {
   Status err;
   unsigned char *prop_return;
@@ -404,7 +407,7 @@ void read_resources()
     {
       if (prop_return != NULL)
         {
-          database = XrmGetStringDatabase(prop_return);
+          database = XrmGetStringDatabase((char *) prop_return);
           XFree(prop_return);
         }
     }
@@ -469,7 +472,7 @@ void draw_led(int no, int on)
 {
   relief_box(leds_x + led_width * no, leds_y,
              leds_x + led_width * no + led_i_width,
-             leds_y + led_height, shadow, hilight, 2, 
+             leds_y + led_height, shadow, hilight, 2,
              on ? light_led : dark_led, main_window);
 }
 
@@ -496,7 +499,7 @@ void draw_button(char *string, Window window, int width, int height,
 
 /* Draw the dialog box and the cancel button. */
 
-void draw_it()
+void draw_it(void)
 {
   int y; int i;
   relief_box(0, 0, w_width - 1, w_height - 1, hilight, shadow, relief,
@@ -539,18 +542,18 @@ void button_release(XButtonEvent *event)
     }
 }
 
-void advance_leds()
+void advance_leds(void)
 {
   led_state[next_led] ^= 1;
   draw_led(next_led, led_state[next_led]);
   next_led = (next_led + 1) % leds;
 }
 
-void backward_leds()
+void backward_leds(void)
 {
   next_led--; if (next_led == -1) next_led += leds;
   led_state[next_led] ^= 1;
-  draw_led(next_led, led_state[next_led]);  
+  draw_led(next_led, led_state[next_led]);
 }
 
 /* Handle key press. */
@@ -586,7 +589,7 @@ void key_press(XKeyEvent *event)
           if (ppointer != phrase)
             {
               ppointer--;
-              backward_leds();        
+              backward_leds();
             }
           break;
           /* These cause the event loop to terminate. */
@@ -604,7 +607,7 @@ void key_press(XKeyEvent *event)
 }
 
 /* Read the modifiers mapping. */
-void check_keyboard()
+void check_keyboard(void)
 {
   modifiers = XGetModifierMapping(display);
 }
@@ -616,12 +619,12 @@ void timeout(int sig) /*ARGUSED*/
 }
 
 /* Event loop. */
-void event_loop()
+void event_loop(void)
 {
   int focused = 0;
   XEvent event;
   XSelectInput(display, main_window, ButtonPressMask |
-               VisibilityChangeMask | StructureNotifyMask |            
+               VisibilityChangeMask | StructureNotifyMask |
                ExposureMask | ButtonReleaseMask | KeyPressMask);
 
   /* Windows must be mapped not before XSelectInput, so that the
@@ -656,7 +659,7 @@ void event_loop()
               focused = 1;
             }
           }
-          break;                     
+          break;
         }
     }
   switch (exiting)
@@ -664,9 +667,9 @@ void event_loop()
     case 1:
       *ppointer = 0;
       printf("%s\n", phrase);
-      break;     
+      break;
     default:
-      break;      
+      break;
     }
 }
 
