@@ -23,23 +23,23 @@ sshauthmethods.h
 #include "sshdllist.h"
 
 /* Array of authentication methods for the server. */
-extern const SshAuthServerMethod *ssh_server_authentication_methods;
+extern const SshAuthServerMethod ssh_server_authentication_methods;
 
 /* Array of authentication methods for the client. */
-extern const SshAuthClientMethod *ssh_client_authentication_methods;
+extern const SshAuthClientMethod ssh_client_authentication_methods;
 
 /* Initializes the authentication methods array for the server. */
-SshAuthServerMethod *
+SshAuthServerMethod 
 ssh_server_authentication_initialize(SshDlList conf_methods);
 
 /* Frees the returned authentication method array. */
-void ssh_server_authentication_uninitialize(SshAuthServerMethod *methods);
+void ssh_server_authentication_uninitialize(SshAuthServerMethod methods);
 
 /* Initializes the authentication methods array for the client. */
-SshAuthClientMethod *
+SshAuthClientMethod 
 ssh_client_authentication_initialize(SshDlList conf_methods);
 
 /* Frees the returned authentication method array. */
-void ssh_client_authentication_uninitialize(SshAuthClientMethod *methods);
+void ssh_client_authentication_uninitialize(SshAuthClientMethod methods);
 
 #endif /* SSHAUTHMETHODS_H */
