@@ -23,12 +23,15 @@ sshchtcpfwd.c
 #include "sshconn.h"
 #include "sshuser.h"
 #include "sshcommon.h"
-#ifndef SSH_WIN_CLIENT
+
+
+
+
 #include <syslog.h>
 #ifdef NEED_SYS_SYSLOG_H
 #include <sys/syslog.h>
 #endif /* NEED_SYS_SYSLOG_H */
-#endif /* SSH_WIN_CLIENT */
+
 
 #ifdef SSH_CHANNEL_TCPFWD
 #include "sshchtcpfwd.h"
@@ -50,33 +53,38 @@ typedef struct SshChannelTcpFwdConnectRec
   SshRemoteTcpForward fwd;
   int channel_id;
   SshConnOpenCompletionProc completion;
+  char *originator_ip;
   void *completion_context;
 } *SshChannelTcpFwdConnect;
 
-#ifndef SSH_WIN_CLIENT
 
+#ifndef SSH_WIN_CLIENT
 /* returns FALSE if user is allowed to forward. */
 Boolean is_tcp_forwarding_denied_for_user(SshCommon common)
 {
   SshUser uc = NULL;
   uid_t uid;
-  gid_t gid;
-  const char *group, *user;
+  const char *user;
   Boolean ret = FALSE;
   SshRegexContext rex_ctx = NULL;
   SshConfig config = common->config;
 
+  SshGroup *groups;
+
+  int i = 0;
+
   SSH_PRECOND(common);
   SSH_PRECOND(config);
 
   user = common->user;
-  uc = ssh_user_initialize(user, TRUE);
-  if (!uc)
-    return TRUE;
+  uc = common->user_data;
   
+  SSH_ASSERT(uc);
+  
+
+  groups = ssh_user_get_groups(uc);
+
   uid = ssh_user_uid(uc);
-  gid = ssh_user_gid(uc);
-  group = ssh_group_name(uc);
   rex_ctx = ssh_regex_create_context();
   
   if (config->denied_tcp_forwarding_users)
@@ -101,35 +109,41 @@ Boolean is_tcp_forwarding_denied_for_user(SshCommon common)
       goto free_and_return;
     }
   
+
   if (config->denied_tcp_forwarding_groups)
-    if (!ssh_match_string_or_number_in_list
-        (group, gid,
-         config->denied_tcp_forwarding_groups,
-         rex_ctx))
-      {
-        ret = TRUE;
-        goto free_and_return;
-      }
+    for (i = 0; groups[i]; i++)
+      if (!ssh_match_string_or_number_in_list
+          (ssh_group_get_name(groups[i]),
+           ssh_group_get_gid(groups[i]),
+           config->denied_tcp_forwarding_groups,
+           rex_ctx))
+        {
+          ret = TRUE;
+          goto free_and_return;
+        }
   
   if (config->allowed_tcp_forwarding_groups)
     {
       ret = TRUE;
-      if (!ssh_match_string_or_number_in_list
-          (group, gid,
-           config->allowed_tcp_forwarding_groups,
-           rex_ctx))
-        ret = FALSE;
-      
-      goto free_and_return;
+      for (i = 0; groups[i]; i++)
+        if (!ssh_match_string_or_number_in_list
+            (ssh_group_get_name(groups[i]),
+             ssh_group_get_gid(groups[i]),
+             config->allowed_tcp_forwarding_groups,
+             rex_ctx))
+          {
+            ret = FALSE;
+            break;
+          }      
     }
 
- free_and_return:
-  ssh_user_free(uc, FALSE);
-  ssh_regex_free_context(rex_ctx);
-return ret;
-}
 
-#endif /* SSH_WIN_CLIENT */
+ free_and_return:
+  ssh_regex_free_context(rex_ctx);
+  return ret;
+}
+#endif /* !SSH_WIN_CLIENT */
+
 
 /***********************************************************************
  * Glue functions for creating/destroying channel type and session
@@ -265,6 +279,7 @@ void ssh_channel_ftcp_open_connected(SshIpError error,
                                      void *context)
 {
   SshChannelTcpFwdConnect c = (SshChannelTcpFwdConnect)context;
+  SshTcpForwardActive active_fwd_context;
 
   if (error != SSH_IP_OK)
     {
@@ -285,6 +300,32 @@ void ssh_channel_ftcp_open_connected(SshIpError error,
         {
           ssh_warning("forwarding protocol \"http\" defaults to \"tcp\"");
         }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
       else 
         {
           ssh_warning("forwarding protocol \"%s\" defaults to \"tcp\"",
@@ -299,12 +340,29 @@ void ssh_channel_ftcp_open_connected(SshIpError error,
 
   /* Record that we now have a new channel. */
   ssh_common_new_channel(c->fwd->common);
-  
+
+  /* init the new tcp forward struct */
+  active_fwd_context = ssh_xmalloc(sizeof(*active_fwd_context));
+
+  active_fwd_context->connect_from_host = ssh_xstrdup(c->originator_ip);
+  active_fwd_context->port = ssh_xstrdup(c->fwd->port);
+  active_fwd_context->connect_to_port = ssh_xstrdup(c->fwd->connect_to_port);
+  active_fwd_context->connect_to_host = ssh_xstrdup(c->fwd->connect_to_host);
+  active_fwd_context->common = c->fwd->common;
+  active_fwd_context->channel_type = SSH_TFACT_REMOTE;
+
+  /* add the channel to the active list in the common object */
+  ssh_common_add_active_forward(active_fwd_context);
+
   /* Call the completion procedure to indicate that we are done. */
   (*c->completion)(SSH_OPEN_OK,
                    stream, TRUE, TRUE, SSH_TCPIP_WINDOW, NULL, 0,
-                   NULL, ssh_channel_tcp_connection_destroy,
-                   (void *)c->fwd->common, c->completion_context);
+                   NULL, ssh_common_delete_active_forward,
+                   (void *)active_fwd_context, c->completion_context);
+
+  if (c->originator_ip != NULL)
+    ssh_xfree(c->originator_ip);
+
   memset(c, 'F', sizeof(*c));
   ssh_xfree(c);
 }
@@ -357,10 +415,10 @@ void ssh_channel_ftcp_open_request(const char *type, int channel_id,
         c->channel_id = channel_id;
         c->completion = completion;
         c->completion_context = completion_context;
-
-        ssh_tcp_connect_with_socks(fwd->connect_to_host, fwd->connect_to_port,
-                           NULL, 1, ssh_channel_ftcp_open_connected,
-                           (void *)c);
+        c->originator_ip = ssh_xstrdup(originator_ip);
+        ssh_tcp_connect(fwd->connect_to_host, fwd->connect_to_port,
+                        NULL, ssh_channel_ftcp_open_connected,
+                        (void *)c);
 
         ssh_xfree(address_to_bind);
         ssh_xfree(originator_ip);
@@ -390,6 +448,102 @@ typedef struct {
   void *completion_context;
 } *SshDirectTcp;
 
+#if 0
+/* Returns FALSE if forwarding definition matches pattern, TRUE
+   otherwise. */
+Boolean match_forward_with_mask(SshUser uc,
+                                const char *to_host,
+                                const char *to_port,
+                                const char *from_ip,
+                                const char *from_port,
+                                const char *mask)
+{
+  char *from_mask = NULL, *to_mask = NULL;
+  Boolean ret_val = FALSE, match_found = FALSE;
+  
+  SSH_PRECOND(mask && from_port && from_ip && to_port && to_host && user);
+  
+  from_mask = ssh_xstrdup(mask);
+
+  /* Separate "from" and "to" masks. ({<from>}{<to>}) */
+  if (*from_mask != '{')
+    SCREWUP("from-part doesn't begin with '{'");
+  from_mask++;
+  to_mask = ssh_glob_next_unescaped_char(from_mask, '}');
+  if (!to_mask)
+    SCREWUP("from-part doesn't end with '}'");
+  *to_mask = '\0';
+  to_mask++;
+  to_mask = ssh_xstrdup(to_mask); /* Note: test with {XXX}\0 */
+  if (*to_mask != '{')
+    SCREWUP("to-part doesn't begin with '{'");
+  to_mask++;
+  tmp = ssh_glob_next_unescaped_char(from_mask, '}');
+  if (!tmp)
+    SCREWUP("to-part doesn't end with '}'");
+  *tmp = '\0';
+
+  /* Check if from-part or user/group definition matches to "from" part. */
+  do {
+    next_def = ssh_glob_next_unescaped_char(from_mask, '|');
+    if (next_def)
+      {
+        *next_def = '\0';
+        next_def++;
+        next_def = ssh_xstrdup(next_def);
+      }
+    cur_def = ssh_xstrdup(from_mask);
+    ssh_xfree(from_mask);
+    from_mask = next_def;
+    next_def = NULL;
+    
+    switch (match_from_def(cur_def, &rest))
+      {
+      case FROM_USER:
+        MATCH(ssh_user_name(uc), rest);
+        MATCH_NUMBER(ssh_user_uid(uc), rest);
+        goto clean_up;
+      case FROM_GROUP:
+        {
+          SshGroup *groups = ssh_user_get_groups(uc);
+
+          for (i = 0; groups[i]; i++)
+            {
+              MATCH(groups[i].name, rest);
+              MATCH_NUMBER(groups[i].gid, rest);
+            }
+          if (!groups[i])
+            /* No match. */
+            goto clean_up;
+        }
+        break;
+      case FROM_ORIGINATOR_IP:
+        MATCH(from_ip, rest);
+        goto clean_up;
+      case FROM_ORIGINATOR_PORT:
+        MATCH_PORT(from_port, rest);
+        goto clean_up;
+      default:
+        SCREWUP("invalid 'from' definition.");
+      }
+    
+    ssh_xfree(cur_def);
+    cur_def = NULL;    
+  } while (from_mask);
+
+  /* If we get this far, all of the "from" validators have matched. */
+  
+ clean_up:
+  ssh_xfree(from_mask);
+  ssh_xfree(to_mask);
+  ssh_xfree(tmp);
+  ssh_xfree(cur_def);  
+  
+  return ret_val;
+}
+#endif
+                                     
+                                     
 /* Called when connecting to the real destination port is complete. */
 
 void ssh_channel_dtcp_connected(SshIpError error,
@@ -494,7 +648,30 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
       return;
     }
   
+  /* XXX Add checkin of configured forwarding "mask" here.
 
+   Example: You could deny all users in group "users" from specifying
+   forwardings to other than localhost, or some specified port(s).
+
+   from:
+   user
+   group
+   orig_ip
+   orig_port
+
+   to:
+   host:SshRegex
+   port:x-y+z+a-b+c-+-d
+
+   ex1:
+   
+   {user:sjl,kahvi}{host:wonder[:isascii:]##|port:-1024+1300}
+
+   allows users sjl and kahvi to make a local forwarding to hosts
+   wonder[a-zA-Z]*, to ports below 1024 (inclusive) and to port 1300.
+   
+  */
+  
   /* Convert port number to string. */
   snprintf(connect_to_port, sizeof(connect_to_port), "%ld", (long) port);
 
@@ -510,16 +687,16 @@ void ssh_channel_dtcp_open_request(const char *type, int channel_id,
   tcp->completion_context = completion_context;
 
   /* Connect to the given host/port. */
-  ssh_tcp_connect_with_socks(connect_to_host, connect_to_port, 
-                             NULL, 1, ssh_channel_dtcp_connected, 
-                             (void *)tcp);
+  ssh_tcp_connect(connect_to_host, connect_to_port, 
+                  NULL, ssh_channel_dtcp_connected, 
+                  (void *)tcp);
 
   /* Free dynamically allocated data. */
   ssh_xfree(originator_ip);
   ssh_xfree(connect_to_host);
 }
 
-#endif /* !SSH_WIN_CLIENT */
+#endif /* SSH_WIN_CLIENT */
 
 /***********************************************************************
  * Processing an incoming connection to a remotely forwarded socket.
@@ -561,8 +738,8 @@ void ssh_channel_ftcp_incoming_connection(SshIpError error, SshStream stream,
     struct request_info req;
     struct servent *serv;
     char fwdportname[32];
-    void *old_handler;
-    
+    static RETSIGTYPE (*old_handler)(int sig) = NULL;
+
     old_handler = signal(SIGCHLD, SIG_DFL);
 
     if (old_handler == SIG_ERR)
@@ -746,6 +923,7 @@ Boolean ssh_channel_remote_tcp_forward_request(const char *type,
   /* Create a socket listener. */
   fwd = ssh_xcalloc(1, sizeof(*fwd));
   fwd->listener = ssh_tcp_make_listener(address_to_bind, port_string,
+                                        NULL, 
                                         ssh_channel_ftcp_incoming_connection,
                                         (void *)fwd);
   if (fwd->listener == NULL)
@@ -780,10 +958,6 @@ Boolean ssh_channel_remote_tcp_forward_request(const char *type,
                 (unsigned long) port);
   return TRUE;
 }  
-#endif /* !SSH_WIN_CLIENT */
-
-
-#ifndef SSH_WIN_CLIENT
 
 /* Processes a received request to cancel remote TCP/IP forwarding. */
 
@@ -846,7 +1020,7 @@ Boolean ssh_channel_tcp_forward_cancel(const char *type,
   return FALSE;
 }
 
-#endif /* !SSH_WIN_CLIENT */
+#endif /* SSH_WIN_CLIENT */
 
 /***********************************************************************
  * Sending a request to start remote TCP/IP forwarding.
@@ -898,6 +1072,34 @@ void ssh_channel_start_remote_tcp_forward(SshCommon common,
   ssh_buffer_uninit(&buffer);
 }
 
+/* Requests cancellation of the given remote TCP/IP port forwarding.
+   If the completion procedure is non-NULL, it will be called when
+   done. */
+
+void ssh_channel_cancel_remote_tcp_forward(SshCommon common,
+                                           const char *address_to_bind,
+                                           const char *port,
+                                           void (*completion)(Boolean ok,
+                                                              void *context),
+                                           void *context)
+{
+  SshBufferStruct buffer;
+
+  SSH_DEBUG(5, ("cancelling remote forwarding for port %s", port));
+
+  /* Send a forwarding request to the remote side. */
+  ssh_buffer_init(&buffer);
+  ssh_encode_buffer(&buffer,
+                    SSH_FORMAT_UINT32_STR,
+                      address_to_bind, strlen(address_to_bind),
+                    SSH_FORMAT_UINT32, (SshUInt32) atol(port),
+                    SSH_FORMAT_END);
+  ssh_conn_send_global_request(common->conn, "cancel-tcpip-forward",
+                               ssh_buffer_ptr(&buffer), ssh_buffer_len(&buffer),
+                               completion, context);
+  ssh_buffer_uninit(&buffer);
+}
+
 /***********************************************************************
  * Handling incoming connections to a locally forwarded port
  ***********************************************************************/
@@ -910,6 +1112,7 @@ void ssh_channel_dtcp_incoming_connection(SshIpError op,
 {
   SshLocalTcpForward fwd = (SshLocalTcpForward)context;
   char ip[20], port[20];
+  SshTcpForwardActive active_fwd_context;
 
   /* We should only receive new connection notifications. */
   if (op != SSH_IP_NEW_CONNECTION)
@@ -926,8 +1129,8 @@ void ssh_channel_dtcp_incoming_connection(SshIpError op,
     struct request_info req;
     struct servent *serv;
     char fwdportname[32];
-    void *old_handler;
-    
+    static RETSIGTYPE (*old_handler)(int sig) = NULL;
+
     old_handler = signal(SIGCHLD, SIG_DFL);
     
     if (old_handler == SIG_ERR)
@@ -986,6 +1189,32 @@ void ssh_channel_dtcp_incoming_connection(SshIpError op,
         {
           ssh_warning("forwarding protocol \"http\" defaults to \"tcp\"");
         }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
       else 
         {
           ssh_warning("forwarding protocol \"%s\" defaults to \"tcp\"",
@@ -998,6 +1227,24 @@ void ssh_channel_dtcp_incoming_connection(SshIpError op,
                 fwd->connect_to_port,
                 fwd->protocol));
 
+  /* Initialize the tcp forward context. This provides us with
+     identity of the channel when the connection is closed.  We have
+     to add this to the list now, since in dtcp_open_to_remote we lose
+     some information (the local port the connection is going
+     through). */
+
+  active_fwd_context = ssh_xmalloc(sizeof(*active_fwd_context));
+
+  active_fwd_context->connect_from_host = ssh_xstrdup(ip);
+  active_fwd_context->port = ssh_xstrdup(fwd->port);
+  active_fwd_context->connect_to_host = ssh_xstrdup(fwd->connect_to_host);
+  active_fwd_context->connect_to_port = ssh_xstrdup(fwd->connect_to_port);
+  active_fwd_context->common = fwd->common;
+  active_fwd_context->channel_type = SSH_TFACT_LOCAL;
+
+  /* add active channel to the common object's active channel list */
+  ssh_common_add_active_forward(active_fwd_context);
+
   /* Send a request to open a channel and connect it to the given port. */
   ssh_channel_dtcp_open_to_remote(fwd->common, stream,
                                   fwd->connect_to_host,
@@ -1022,12 +1269,15 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
   SshLocalTcpForward fwd;
   SshChannelTypeTcpDirect ct;
   long portnumber;
+#ifndef SSH_WIN_CLIENT
   SshUser user;
-  
+#endif /* SSH_WIN_CLIENT */
+
   SSH_DEBUG(5, ("requesting local forwarding for port %s to %s:%s",
                 port, connect_to_host, connect_to_port));
 
   portnumber = atol(port);
+
 #ifndef SSH_WIN_CLIENT
   user = ssh_user_initialize(NULL, FALSE);
     /* If user is not logged in as a privileged user, don't allow
@@ -1043,6 +1293,7 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
         }
     }
 #endif /* SSH_WIN_CLIENT */
+
   if (portnumber >= 65536)
     {
       ssh_warning("Tried to forward " \
@@ -1056,6 +1307,7 @@ Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
   fwd = ssh_xcalloc(1, sizeof(*fwd));
   fwd->common = common;
   fwd->listener = ssh_tcp_make_listener(address_to_bind, port,
+                                        NULL, 
                                         ssh_channel_dtcp_incoming_connection,
                                         (void *)fwd);
   if (!fwd->listener)
@@ -1095,6 +1347,7 @@ void ssh_channel_dtcp_open_to_remote(SshCommon common, SshStream stream,
                                      const char *originator_port)
 {
   SshBufferStruct buffer;
+  void* active_forward;
 
   SSH_DEBUG(5, ("opening direct TCP/IP connection to %s:%s originator %s:%s",
                 connect_to_host, connect_to_port,
@@ -1113,6 +1366,14 @@ void ssh_channel_dtcp_open_to_remote(SshCommon common, SshStream stream,
                       originator_ip, strlen(originator_ip),
                     SSH_FORMAT_UINT32, (SshUInt32) atol(originator_port),
                     SSH_FORMAT_END);
+
+  /* Since we've just added the active forward to the (front) of the
+     active forward list (in dtcp_incoming_connection)
+     we take this struct to pass to the callback function */
+
+  /* copy the first item for the callback */
+  ssh_dllist_rewind(common->active_local_forwards);
+  active_forward = ssh_dllist_current(common->active_local_forwards);
   
   /* Send the channel open request. */
   ssh_conn_send_channel_open(common->conn, "direct-tcpip",
@@ -1120,8 +1381,8 @@ void ssh_channel_dtcp_open_to_remote(SshCommon common, SshStream stream,
                              SSH_TCPIP_PACKET_SIZE,
                              ssh_buffer_ptr(&buffer), ssh_buffer_len(&buffer),
                              NULL, 
-                             ssh_channel_tcp_connection_destroy,
-                             (void *)common, NULL, NULL);
+                             ssh_common_delete_active_forward,
+                             (void *)active_forward, NULL, NULL);
 
   ssh_buffer_uninit(&buffer);
 }
