@@ -3,15 +3,15 @@
 sshchtcpfwd.h
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
-Public interface to TCP/IP forwarding channels.
+  Public interface to TCP/IP forwarding channels.
 
 */
 
@@ -36,6 +36,10 @@ typedef struct SshLocalTcpForwardRec
   /* Port number to listen. */
   char *port;
 
+  /* Protocol name.  This is "tcp" on transparent forwards.
+     Only client knows this. */
+  char *protocol;
+
   /* Listener for the port. */
   SshTcpListener listener;
 } *SshLocalTcpForward;
@@ -60,6 +64,10 @@ typedef struct SshRemoteTcpForwardRec
   char *connect_to_host;
   char *connect_to_port;
 
+  /* Protocol name.  This is "tcp" on transparent forwards. 
+     Only client knows this. */
+  char *protocol;
+
   /* Socket listener.  This is only used at the remote end. */
   SshTcpListener listener;
 } *SshRemoteTcpForward;
@@ -94,6 +102,10 @@ void *ssh_channel_ftcp_create(SshCommon common);
    channels and listeners and free the context. */
 void ssh_channel_ftcp_destroy(void *context);
 
+/* Returns the channel type context from the SshCommon object. */
+
+SshChannelTypeTcpForward ssh_channel_ftcp_ct(SshCommon common);
+
 #if 0
 /* This function is called whenever a protocol request is received to
    set up a remote TCP/IP forwarding. */
@@ -109,10 +121,9 @@ Boolean ssh_channel_ftcp_global_cancel_forward(const char *type,
                                                size_t len,
                                                void *context);
 #endif
-/* Returns the channel type context from the SshCommon object. */
 
-SshChannelTypeTcpForward ssh_channel_ftcp_ct(SshCommon common);
 
+#ifndef SSH_WIN_CLIENT
 /* This function is called whenever an open request is received for a
    locally forwarded tcp/ip channel. */
 void ssh_channel_dtcp_open_request(const char *type,
@@ -123,6 +134,8 @@ void ssh_channel_dtcp_open_request(const char *type,
                                    void *completion_context,
                                    void *context);
 
+#endif /* !SSH_WIN_CLIENT */
+
 /* This function is called once when a SshCommon object is created. */
 void *ssh_channel_dtcp_create(SshCommon common);
 
@@ -135,6 +148,8 @@ void ssh_channel_dtcp_destroy(void *context);
 
 SshChannelTypeTcpDirect ssh_channel_dtcp_ct(SshCommon common);
 
+
+#ifndef SSH_WIN_CLIENT
 /* Processes a received request to set up remote TCP/IP forwarding. */
 Boolean ssh_channel_remote_tcp_forward_request(const char *type,
                                                const unsigned char *data,
@@ -146,6 +161,7 @@ Boolean ssh_channel_tcp_forward_cancel(const char *type,
                                        const unsigned char *data,
                                        size_t len,
                                        void *context);
+#endif /* !SSH_WIN_CLIENT */
 
 /* Requests forwarding of the given remote TCP/IP port.  If the completion
    procedure is non-NULL, it will be called when done. */
@@ -154,17 +170,29 @@ void ssh_channel_start_remote_tcp_forward(SshCommon common,
                                           const char *port,
                                           const char *connect_to_host,
                                           const char *connect_to_port,
+                                          const char *protocol,
                                           void (*completion)(Boolean ok,
                                                              void *context),
                                           void *context);
 
+/* Requests cancellation of the given remote TCP/IP port forwarding.
+   If the completion procedure is non-NULL, it will be called when
+   done. */
+void ssh_channel_cancel_remote_tcp_forward(SshCommon common,
+                                           const char *address_to_bind,
+                                           const char *port,
+                                           void (*completion)(Boolean ok,
+                                                              void *context),
+                                           void *context);
+
 /* Requests forwarding of the given local TCP/IP port.  Returns TRUE if
    forwarding was successfully started, FALSE otherwise. */
 Boolean ssh_channel_start_local_tcp_forward(SshCommon common,
                                         const char *address_to_bind,
                                         const char *port,
                                         const char *connect_to_host,
-                                        const char *connect_to_port);
+                                        const char *connect_to_port,
+                                        const char *protocol);
 
 /* Opens a direct connection to the given TCP/IP port at the remote side.
    The originator values should be set to useful values and are passed
