@@ -26,6 +26,7 @@ sshchx11.c
 #include "sshcommon.h"
 #include "sshfdstream.h"
 #include "sshdsprintf.h"
+#include <stdlib.h>
 
 #ifdef SSH_CHANNEL_X11
 
@@ -54,9 +55,9 @@ sshchx11.c
 #endif /* HAVE_SYS_UTSNAME_H */
 #endif /* XAUTH_PATH || HPSUX_NONSTANDARD_X11_KLUDGE */
 
-#ifndef SSH_WIN_CLIENT
+
 #include <syslog.h>
-#endif /* SSH_WIN_CLIENT */
+
 
 #ifdef NEED_SYS_SYSLOG_H
 #include <sys/syslog.h>
@@ -213,7 +214,7 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
   /* Find the channel type context. */
   ct = ssh_channel_x11_ct(common);
 
-#ifndef SSH_WIN_CLIENT
+
   /* If DISPLAY isn't set, no sense in requesting X11 forwarding. */
   if (getenv("DISPLAY") == NULL)
     {
@@ -235,9 +236,9 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
     screen_number = atoi(cp + 1);
   else
     screen_number = 0;
-#else /* SSH_WIN_CLIENT */
-    screen_number = 0;
-#endif /* SSH_WIN_CLIENT */
+
+
+
 
   /* Generate a fake cookie only if this is the first X11 request in this
      connection. */
@@ -247,12 +248,19 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
       /* Try to get Xauthority information for the display. */
       snprintf(line, sizeof(line), "%s list %s 2>/dev/null",
                XAUTH_PATH, getenv("DISPLAY"));
+
+      /* XXX async, please! */
       f = popen(line, "r");
-      if (f && fgets(line, sizeof(line), f) &&
-          sscanf(line, "%*s %s %s", proto, data) == 2)
-        got_data = TRUE;
-      else
+      if (f)
+        {     
+          while(!(cp = fgets(line, sizeof(line), f)) && errno == EINTR)
+            ;
+          if (cp && sscanf(line, "%*s %s %s", proto, data) == 2)
+            got_data = TRUE;
+        }
+      if (!got_data)
         ssh_debug("Failed to get local xauth data.");
+      
       if (f)
         pclose(f);
 #else /* XAUTH_PATH */
@@ -342,7 +350,7 @@ void ssh_channel_x11_connection_destroy(void *context)
   ssh_common_destroy_channel(common);
 }
 
-#ifndef SSH_WIN_CLIENT
+
 
 /* This function is called whenever a new X11 connection is received. */
 
@@ -386,7 +394,7 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
 #ifdef HAVE_LIBWRAP
   {
     struct request_info req;
-    void *old_handler;
+    static RETSIGTYPE (*old_handler)(int sig) = NULL;
 
     old_handler = signal(SIGCHLD, SIG_DFL);
 
@@ -541,7 +549,8 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
 
       /* Open a TCP/IP socket for listening the port. */
       session->x11_listener =
-        ssh_tcp_make_listener("0.0.0.0", buf, ssh_channel_x11_connection,
+        ssh_tcp_make_listener("0.0.0.0", buf,
+                              NULL, ssh_channel_x11_connection,
                               (void *)session);
 
       /* Break if we got the display number. */
@@ -595,7 +604,7 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
   session->forward_x11 = TRUE;
   return TRUE;
 }
-#endif /* !SSH_WIN_CLIENT */
+
 
 
 /***********************************************************************
@@ -743,6 +752,7 @@ void ssh_channel_x11_open_connected(SshStream stream,
                                     void *context)
 {
   SshX11Connection x = (SshX11Connection)context;
+  SshTcpForwardActive active_context;
 
   if (stream == NULL)
     {
@@ -761,6 +771,16 @@ void ssh_channel_x11_open_connected(SshStream stream,
   /* Increment the number of channels. */
   ssh_common_new_channel(x->common);
 
+  active_context = ssh_xcalloc(1, sizeof(*active_context));
+  active_context->connect_from_host = ssh_xstrdup(x->originator_ip);
+  active_context->port = ssh_xstrdup("");
+  active_context->connect_to_host = ssh_xstrdup("");
+  active_context->connect_to_port = ssh_xstrdup("");
+  active_context->channel_type = SSH_TFACT_X11;
+  active_context->common = x->common;
+
+  ssh_common_add_active_forward(active_context);
+
   /* Wrap the stream to the X server into a filter stream that is used
      to replace the fake authentication cookie with the real one. */
   stream = ssh_stream_filter_create(stream, X11_MAX_AUTH_PACKET_SIZE,
@@ -771,8 +791,8 @@ void ssh_channel_x11_open_connected(SshStream stream,
   /* Call the open completion procedure. */
   (*x->completion)(SSH_OPEN_OK,
                    stream, TRUE, TRUE, X11_WINDOW_SIZE, NULL, 0,
-                   NULL, ssh_channel_x11_connection_destroy,
-                   (void *)x->common, x->context);
+                   NULL, ssh_common_delete_active_forward,
+                   (void *)active_context, x->context);
 }
 
 /* This function is called whenever connecting to a TCP/IP X11 display
@@ -873,7 +893,7 @@ void ssh_channel_x11_open(const char *type, int channel_id,
     x->originator_string = ssh_xstrdup(originator_ip);
   
   /* Try to open a socket for the local X server. */
-#ifndef SSH_WIN_CLIENT
+
   display = getenv("DISPLAY");
   if (!display)
     ssh_fatal("ssh_channel_x11_open: DISPLAY not set.");
@@ -937,9 +957,9 @@ void ssh_channel_x11_open(const char *type, int channel_id,
       ssh_local_connect(buf, ssh_channel_x11_open_connected, (void *)x);
       return;
     }
-#else /* SSH_WIN_CLIENT */
-  display = "localhost:0";
-#endif /* SSH_WIN_CLIENT */
+
+
+
 
   /* Connect to an inet socket.  The DISPLAY value is supposedly
      hostname:d[.s], where hostname may also be numeric IP address. */
@@ -969,12 +989,11 @@ void ssh_channel_x11_open(const char *type, int channel_id,
   /* Host name is now in ``buf''.  Format port number in ``port''. */
   snprintf(port, sizeof(port), "%d", 6000 + display_number);
 
-  ssh_tcp_connect_with_socks(buf, port, NULL, 1,
-                             ssh_channel_x11_open_connected_tcp,
-                             (void *)x);
+  ssh_tcp_connect(buf, port, NULL,
+                  ssh_channel_x11_open_connected_tcp,
+                  (void *)x);
 }
 
-#ifndef SSH_WIN_CLIENT
 
 /* Returns the value of DISPLAY in the server. */
 
@@ -1002,7 +1021,6 @@ const char *ssh_channel_x11_get_auth_cookie(void *session_placeholder)
 
   return session->auth_cookie;
 }
-#endif /* !SSH_WIN_CLIENT */
 
 
 #endif /* SSH_CHANNEL_X11 */
