@@ -17,6 +17,9 @@
 */
 
 #include "ssh2includes.h"
+
+
+
 #include "sshtrans.h"
 #include "sshauth.h"
 #include "sshconn.h"
@@ -135,7 +138,7 @@ SSH_FSM_STEP(keycheck_change_key);
 SSH_FSM_STEP(keycheck_change_key_confirmation);
 SSH_FSM_STEP(keycheck_make_fingerprint);
 
-SshFSMStateMapItem keycheck_states[] =
+SshFSMStateMapItemStruct keycheck_states[] =
 {
   /* the prefix "ckc" stands for "client keycheck". */
   { "ckc_begin", "Begin keycheck", keycheck_begin },
@@ -254,7 +257,11 @@ void ssh_client_key_check(const char *server_name,
   
   if (stat(filen, &st) < 0)
     {
+
+
+
       if (mkdir(filen, 0700) < 0)
+
         {
           ssh_warning("ssh_userdir: could not create user's ssh hostkey" 
                       "directory %s", filen);
@@ -407,6 +414,7 @@ SSH_FSM_STEP(keycheck_check_fd)
   SSH_FSM_GDATA(SshClientKeyCheckContext);
   SSH_PRECOND(gdata->next_state);
 
+
   if (isatty(fileno(stdin)))
     fi = stdin;
   else
@@ -439,6 +447,7 @@ SSH_FSM_STEP(keycheck_check_fd)
 
   gdata->readfd = fileno(fi);
   gdata->writefd = fileno(fo);
+
   SSH_FSM_SET_NEXT(gdata->next_state);
   return SSH_FSM_CONTINUE;
 }
@@ -470,8 +479,9 @@ void readline_cb(const char *line, void *context)
     }
   else
     {
-      ssh_readline_eloop("Please answer 'yes' or 'no': ",
-                         "", gdata->client->rl, readline_cb, thread);
+      ssh_readline_eloop((unsigned char *)"Please answer 'yes' or 'no': ",
+                         (unsigned char *)"", gdata->client->rl,
+                         readline_cb, thread);
       return;
     }
 
@@ -518,9 +528,10 @@ SSH_FSM_STEP(keycheck_key_not_found)
   
   gdata->next_state = "ckc_continue_confirmation";
   
-  SSH_FSM_ASYNC_CALL(ssh_readline_eloop("Are you sure you want to continue " \
-                                        "connecting (yes/no)? ",             \
-                                        "", gdata->client->rl,               \
+  SSH_FSM_ASYNC_CALL(ssh_readline_eloop((unsigned char *)                       \
+                                        "Are you sure you want to continue "    \
+                                        "connecting (yes/no)? ",                \
+                                        (unsigned char *)"", gdata->client->rl, \
                                         readline_cb, thread));
 }
 
@@ -601,7 +612,17 @@ SSH_FSM_STEP(keycheck_key_different)
   ssh_informational("You can get a public key's fingerprint by running\r\n");
   ssh_informational("%% ssh-keygen -F publickey.pub\r\n");
   ssh_informational("on the keyfile.\r\n");
-  
+
+
+  if (gdata->client->common->doing_rekey)
+    {
+      /* since we're in the middle of rekeying, we can't ask the user
+         for confirmation if they want to continue, since the
+         gdata->client->rl is invalid (NULL). So all they get is a warning */
+      SSH_FSM_SET_NEXT("ckc_success");
+      return SSH_FSM_CONTINUE;
+    }
+ 
   if (gdata->client->config->strict_host_key_checking ==
       SSH_STRICT_HOSTKEY_CHECKING_YES)
     {
@@ -647,9 +668,10 @@ SSH_FSM_STEP(keycheck_key_different)
   
   gdata->next_state = "ckc_warning_confirmation";
   
-  SSH_FSM_ASYNC_CALL(ssh_readline_eloop("Are you sure you want to continue " \
-                                        "connecting (yes/no)? ",             \
-                                        "", gdata->client->rl,               \
+  SSH_FSM_ASYNC_CALL(ssh_readline_eloop((unsigned char *)                       \
+                                        "Are you sure you want to continue "    \
+                                        "connecting (yes/no)? ",                \
+                                        (unsigned char *)"", gdata->client->rl, \
                                         readline_cb, thread));
 }
 
@@ -670,9 +692,10 @@ SSH_FSM_STEP(keycheck_change_key)
       return SSH_FSM_CONTINUE;
     }
   
-  SSH_FSM_ASYNC_CALL(ssh_readline_eloop("Do you want to change the host key "   \
+  SSH_FSM_ASYNC_CALL(ssh_readline_eloop((unsigned char *)                       \
+                                        "Do you want to change the host key "   \
                                         "on disk (yes/no)? ",                   \
-                                        "", gdata->client->rl,                  \
+                                        (unsigned char *)"", gdata->client->rl, \
                                         readline_cb, thread));
 }
 
@@ -825,13 +848,29 @@ Boolean ssh_client_version_check(const char *version,
   extern char **environ;
   char *host = NULL;
   int option;
-  
-  ssh_debug("Remote version: %s", version);
 
+  client->remote_version = ssh_xstrdup(version);
+  
   if (strncmp(version, "SSH-1.", 6) == 0 &&
       strncmp(version, "SSH-1.99", 8) != 0 &&
       (client->config->ssh1compatibility == TRUE))
     {
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
       if ((client->config->ssh1_path == NULL) ||
           (client->config->ssh1_argv == NULL))
         return TRUE; /* Can't use exec ssh1 compatibility. */
@@ -956,6 +995,7 @@ Boolean ssh_client_version_check(const char *version,
       /* Use ssh1 to connect. */
       execve(client->config->ssh1_path, args, environ);
       ssh_fatal("Executing ssh1 in compatibility mode failed.");
+
     }
   return TRUE;
 }
@@ -1070,6 +1110,10 @@ SshClient ssh_client_wrap(SshStream stream, SshConfig config,
   client->debug = debug;
   client->authentication_notify = authentication_notify;
 
+
+
+
+
   
   /* Create a transport layer protocol object. */
   ssh_debug("ssh_client_wrap: creating transport protocol");
@@ -1080,11 +1124,12 @@ SshClient ssh_client_wrap(SshStream stream, SshConfig config,
                                     server_host_name,
                                     ssh_client_key_check,
                                     (void *)client,
-                                    (config->ssh1_path &&
-                                     config->ssh1compatibility) ?
-                                      ssh_client_version_check : NULL,
-                                    (void *)client);
-
+                                    ssh_client_version_check,
+                                    (void *)client,
+                                    ssh_common_rekey_notify_cb,
+                                    /* SshCommon object is allocated a
+                                       below. */
+                                    (void *)&(client->common));
   
   ssh_transport_get_compatibility_flags(trans, &compat_flags);
   client->tr_context = ssh_stream_get_context(trans);
@@ -1119,7 +1164,7 @@ SshClient ssh_client_wrap(SshStream stream, SshConfig config,
 
 void auth_uninit_timeout(void *context)
 {
-  SshAuthClientMethod *methods = (SshAuthClientMethod *)context;
+  SshAuthClientMethod methods = (SshAuthClientMethod)context;
   SSH_PRECOND(methods);
 
   ssh_client_authentication_uninitialize(methods);  
@@ -1135,12 +1180,14 @@ void ssh_client_destroy(SshClient client)
   client->being_destroyed = TRUE;
   ssh_common_destroy(client->common);
   ssh_register_timeout(0L, 0L, auth_uninit_timeout, (void *)client->methods);
+
   if (client->rl)
     {
       ssh_readline_eloop_unitialize(client->rl);
       client->rl = NULL;
     }
-  
+
+
   if (!client->session_requested)
     {
       /* If session has been requested, then the lower layer destroys
@@ -1157,7 +1204,8 @@ void ssh_client_destroy(SshClient client)
           client->stderr_stream = NULL;
         }
     }
-  
+
+  ssh_xfree(client->remote_version);
   memset(client, 'F', sizeof(*client));
   ssh_xfree(client);
 }
@@ -1287,7 +1335,7 @@ void ssh_client_parent_notify_authentication(Boolean successful,
   ssh_dsprintf(&buffer, "AUTHENTICATED %s\n", successful ? "YES" : "NO");
 
   /* XXX What if the write blocks? */
-  ssh_stream_write(stream, buffer, strlen(buffer));
+  ssh_stream_write(stream, (unsigned char *)buffer, strlen(buffer));
 
   ssh_xfree(buffer);
 }
