@@ -51,7 +51,7 @@ struct SshClientRec
   SshConfig config;
   
   /* Authentication methods. */
-  SshAuthClientMethod *methods;
+  SshAuthClientMethod methods;
 
   /* Data for the user at the client end. */
   SshUser user_data;
@@ -90,6 +90,9 @@ struct SshClientRec
   const char *terminal_type;
   void (*close_notify)(SshUInt32 exit_status, void *context);
 
+  /* Remote version string. */
+  char *remote_version;
+  
   /* These are wrappers for the same calls from the SshCommon
      level. These are needed, so that we can give a SshClient object
      to SshCommon, because we need to call registered
@@ -100,6 +103,12 @@ struct SshClientRec
   SshCommonAuthenticationNotify authentication_notify;
 
 
+
+
+
+
+
+
   void *context;
 };
 
