@@ -24,9 +24,10 @@
 #include "sshcrypt.h"
 #include "sshconn.h"
 #include "sshtrans.h"
+#include "sshdllist.h"
+
 
 
-#ifndef SSH_WIN_CLIENT
 #  ifndef DISABLE_PORT_FORWARDING
 #    define SSH_CHANNEL_AGENT
 #    define SSH_CHANNEL_SSH1_AGENT
@@ -37,9 +38,9 @@
 #      define SSH_CHANNEL_X11
 #    endif /* DISABLE_X11_FORWARDING */
 #  endif /* X_DISPLAY_MISSING */
-#else /* !SSH_WIN_CLIENT */
-#  define SSH_CHANNEL_TCPFWD
-#endif /* !SSH_WIN_CLIENT */
+
+
+
 
 #define SSH_CHANNEL_SESSION
 
@@ -57,6 +58,27 @@ typedef void (*SshCommonAuthenticationNotify)(const char *user,
    when a SshCommon object is being destroyed. */
 typedef void (*SshCommonDestroyNotify)(void *context);
 
+typedef enum
+{
+  SSH_TFACT_LOCAL = 0,
+  SSH_TFACT_REMOTE,
+  SSH_TFACT_X11
+} SshTcpForwardActiveChannelType;
+
+/* struct to keep a list of active tcp forwards in common object */
+typedef struct SshTcpForwardActiveRec
+{
+  char* connect_from_host;
+  char* port;
+  char* connect_to_host;
+  char* connect_to_port;
+
+  SshTcpForwardActiveChannelType channel_type;
+
+  SshCommon common;
+} *SshTcpForwardActive;
+
+
 struct SshCommonRec
 {
   /* TRUE if we are a client, FALSE if we are a server. */
@@ -71,6 +93,18 @@ struct SshCommonRec
   /* Number of active channels on the common. */
   unsigned int num_channels;
 
+  /* List of active local port forwardings. */
+  SshDlList active_local_forwards;
+  /* List of active remote port forwardings. */
+  SshDlList active_remote_forwards;
+  /* List of active X11 port forwardings. */
+  SshDlList active_x11_forwards;
+  
+
+
+
+
+
   /* Function to call when a disconnect message is received.  This function
      is supposed to destroy the object. */
   SshConnDisconnectProc disconnect;
@@ -124,12 +158,22 @@ struct SshCommonRec
   char *last_login_from_host;
   unsigned int sizeof_last_login_from_host;
   
-  /* Authenticated host name, or empty if none. */
-  char *authenticated_client_host;
-
   /* TRUE if authentication has been successful, FALSE if not. */
   Boolean authenticated;
+
+  /* TRUE, if transport layer is in the process of rekeying.*/
+  Boolean doing_rekey;
   
+  /* Relevant parts of a SSH_CROSS_ALGORITHMS packet. Used in rekey
+     etc.*/
+  char *chosen_host_key_alg;
+  char *chosen_cipher_c_to_s;
+  char *chosen_cipher_s_to_c;
+  char *chosen_mac_c_to_s;
+  char *chosen_mac_s_to_c;
+  char *chosen_compression_c_to_s;
+  char *chosen_compression_s_to_c;
+
   /* An array of contexts returned by the type create function for each
      channel type. */
   void **type_contexts;
@@ -137,15 +181,20 @@ struct SshCommonRec
   /* Dynamically built array of global requests.  This array is allocated
      using ssh_xmalloc. */
   unsigned int num_global_requests;
-  SshConnGlobalRequest *global_requests;
+  SshConnGlobalRequest global_requests;
 
   /* Dynamically built array of channel opens.  This array is allocated
      using ssh_xmalloc. */
   unsigned int num_channel_opens;
-  SshConnChannelOpen *channel_opens;
+  SshConnChannelOpen channel_opens;
 
   /* TRUE if context is being destroyed. */
   Boolean being_destroyed;
+
+  /* Pointer to a location that will receive a SshUser object for the
+     authenticated user before SSH_CROSS_AUTHENTICATED is received.
+     This can also be NULL. */
+  SshUser *ucp;
 };
 
 /* Creates the common processing object for the SSH server/client connection.
@@ -216,4 +265,27 @@ void ssh_common_new_channel(SshCommon common);
    done in a channel destroy function. */
 void ssh_common_destroy_channel(SshCommon common);
 
+/* Requests immediate rekey from transport layer. This function must
+   not be used before first KEX_INIT, as there are no chosen
+   algorithms. */
+void ssh_common_request_rekey(SshCommon common);
+
+/* A callback function, which should be passed to
+   ssh_transport_{server_client}_wrap(), respectively, to notify
+   SshCommon of rekeys. This is because transport layer doesn't like
+   it when a rekey is being requested when already doing one. The
+   context should a pointer to SshCommon (type SshCommon *), because
+   ssh_common_wrap() is usually called later than
+   ssh_transport_{server_client}_wrap(). The `context' has to be
+   valid, but it can point to NULL, in which case this function just
+   returns. */
+void ssh_common_rekey_notify_cb(void *context);
+
+/* Adds the newly formed channel to the active channel list. */
+void ssh_common_add_active_forward(SshTcpForwardActive fwd_active);
+
+/* Deletes the active channel from the list and calls
+   ssh_common_destroy_channel to delete the channel. */
+void ssh_common_delete_active_forward(void* context);
+
 #endif /* SSHCOMMON_H */
