@@ -13,19 +13,27 @@ sshconfig.c
 
   Processing configuration data in SSH (both client and server).
 
+  Note: sshunixconfig.c has been eliminated and its functions have been
+  moved here.
 */
 
 #include "ssh2includes.h"
 #include "sshconfig.h"
 #include "sshuser.h"
+#include "sshuserfile.h"
 #include "sshuserfiles.h"
+#include "sshmatch.h"
 #include "sshcipherlist.h"
+#include "ssh2pubkeyencode.h"
 #include "namelist.h"
 #include "sshdllist.h"
 #include "sshappcommon.h"
 #include "sshglob.h"
 #include "sshdsprintf.h"
 
+#include "sshbuffer.h"
+
+
 #define SSH_DEBUG_MODULE "SshConfig"
 
 /* Free the "vars" and "vals" arrays */
@@ -40,7 +48,7 @@ void ssh_free_varsvals(int n, char **vars, char **vals)
       ssh_xfree(vals[i]);
     }
   ssh_xfree(vars);
-  ssh_xfree(vals); 
+  ssh_xfree(vals);
 }
 
 /* Parses a given comma-separated list to tokens, which are stored in
@@ -126,6 +134,9 @@ SshDlList ssh_config_parse_list(char *string,
 SshConfig ssh_config_init(Boolean client)
 {
   SshConfig config;
+
+
+
   config = ssh_xcalloc(1, sizeof(*config));
   config->client = client;
 
@@ -158,7 +169,21 @@ SshConfig ssh_config_init(Boolean client)
 
   config->user_known_hosts = TRUE;
   config->port = ssh_xstrdup(SSH_DEFAULT_PORT);
+
+
   config->ciphers = NULL;
+
+
+
+
+
+
+
+
+
+
+
+
   config->macs = NULL;
   config->user_conf_dir = ssh_xstrdup(SSH_USER_CONFIG_DIRECTORY);
   config->identity_file = ssh_xstrdup(SSH_IDENTIFICATION_FILE);
@@ -166,7 +191,11 @@ SshConfig ssh_config_init(Boolean client)
 
   config->password_prompt = ssh_xstrdup("%U's password: ");
   config->password_guesses = 3;
+  config->number_of_password_prompts = 3;
 
+  config->rekey_interval_bytes = 0L;
+  config->rekey_interval_seconds = 3600L;
+  
 #ifndef SSH2_MAX_CONNECTIONS
   config->max_connections = 0;
 #else /* ! SSH2_MAX_CONNECTIONS */
@@ -241,18 +270,47 @@ SshConfig ssh_config_init(Boolean client)
 
 #ifdef SSH1_COMPATIBILITY
   config->ssh1_path = ssh_xstrdup(client ? SSH1_PATH : SSHD1_PATH);
+
+
+
+
+
   config->ssh1compatibility = TRUE;
 #else /* SSH1_COMPATIBILITY */
   config->ssh1_path = NULL;
+
+
+
+
+
   config->ssh1compatibility = FALSE;  
 #endif /* SSH1_COMPATIBILITY */
   config->ssh_agent_compat = SSH_AGENT_COMPAT_NONE;
-  
+  config->ssh_pam_client_path = ssh_xstrdup(SSH_PAM_CLIENT_PATH);
+
+#ifdef SSH_SERVER_WITH_SECURID
+  config->securid_guesses = 3;
+#endif /* SSH_SERVER_WITH_SECURID */
+
   config->signer_path = ssh_xstrdup(SSH_SIGNER_PATH);
   config->default_domain = NULL;
+
+  config->banner_message_file_path = ssh_xstrdup(SSH_BANNER_MSG_FILE);
+  config->banner_msg = NULL;
+
+  config->idle_timeout = SSH_SERVER_DEFAULT_IDLE_TIMEOUT;
+
+
+
+
+
+
+
+
   return config;
 }
 
+
 /* This should be called after initializing the config-struct
    (ie. after command-line variables have been parsed. This checks,
    that some required members are initialized properly.*/
@@ -262,9 +320,14 @@ void ssh_config_init_finalize(SshConfig config)
   /* "hostbased"-authentication method is not enabled by default. */
   if (!config->allowed_authentications)
     config->allowed_authentications =
-      ssh_config_parse_list((char *)SSH_AUTH_PUBKEY "," SSH_AUTH_PASSWD,
+      ssh_config_parse_list((char *)
+#ifdef KERBEROS
+                            SSH_AUTH_KERBEROS_TGT ","
+                            SSH_AUTH_KERBEROS ","
+#endif /* KERBEROS */
+                            SSH_AUTH_PUBKEY "," SSH_AUTH_PASSWD,
                             NULL, NULL);
-  
+
   /* Client. */
   if (config->client)
     {
@@ -273,13 +336,42 @@ void ssh_config_init_finalize(SshConfig config)
   /* Server. */
   else
     {
+      SshUserFile f;
+      
       /* If IgnoreRootRhosts isn't defined at this stage, assign it to
          the same as IgnoreRhosts. */
       if (config->ignore_root_rhosts == -1)
         config->ignore_root_rhosts = config->ignore_rhosts;
-    }  
+      
+      if ((f = ssh_userfile_open(getuid(), config->banner_message_file_path,
+                                 O_RDONLY, 0755)) != NULL)
+        {
+          SshBuffer buf;
+          char linebuf[1024];
+
+          buf = ssh_buffer_allocate();
+
+          while (ssh_userfile_gets(linebuf, sizeof(linebuf) - 1, f) != NULL)
+            {
+              ssh_buffer_append(buf, linebuf, strlen(linebuf));
+            }
+
+          if (ssh_buffer_len(buf) > 0)
+            {
+              config->banner_msg = ssh_xcalloc(ssh_buffer_len(buf) + 1,
+                                               sizeof(char));
+              memcpy(config->banner_msg, ssh_buffer_ptr(buf),
+                     ssh_buffer_len(buf));
+            }
+          
+          ssh_buffer_free(buf);
+          
+          ssh_userfile_close(f);
+        }
+    }
 }
 
+
 /* Helper function, that combines two lists. dst will receive nodes
    from src, and src will be freed. (so the pointer isn't valid after
    this call.) If one or both arguments are NULL, does nothing. */
@@ -363,8 +455,18 @@ void ssh_config_free(SshConfig config)
       ssh_xfree(config->public_host_key_blob);
     }
 
+  ssh_xfree(config->ssh_pam_client_path);
+  
   ssh_xfree(config->signer_path);
   ssh_xfree(config->default_domain);
+
+  ssh_xfree(config->banner_message_file_path);
+  ssh_xfree(config->banner_msg);
+
+
+
+
+
   memset(config, 0, sizeof(*config));
   ssh_xfree(config);
 }
@@ -415,9 +517,25 @@ Boolean auth_param_validity(const char *param, void *context)
   if (strcasecmp(param, SSH_AUTH_PASSWD) == 0)
     return FALSE;
 
+#if defined (SSH_SERVER_WITH_SECURID) || defined(SSH_CLIENT_WITH_SECURID)
+  if (strcasecmp(param, SSH_AUTH_SECURID) == 0)
+    return FALSE;
+#endif /* SSH_SERVER_WITH_SECURID || SSH_CLIENT_WITH_SECURID */
+
+
+  if (strcasecmp(param, SSH_AUTH_PAM) == 0)
+    return FALSE;
+
   if (strcasecmp(param, SSH_AUTH_HOSTBASED) == 0)
     return FALSE;
 
+  if (strcasecmp(param, SSH_AUTH_KERBEROS) == 0)
+    return FALSE;
+
+  if (strcasecmp(param, SSH_AUTH_KERBEROS_TGT) == 0)
+    return FALSE;
+
+
   return TRUE;
 }
 
@@ -429,6 +547,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   unsigned int i;
   int num;
   SshSubsystem ss;
+  SshForward current_forward;
   
   switch (val[0])
     {
@@ -543,7 +662,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   
   if (strcmp(var, "ciphers") == 0)
     {
-      SSH_DEBUG(3, ("Got config cipherlist \"%s\"", val));
+      SSH_DEBUG(5, ("Got config cipherlist \"%s\"", val));
       ssh_xfree(config->ciphers);
       if (strcasecmp(val, "any") == 0)
         {
@@ -596,13 +715,13 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
         {
           config->ciphers = ssh_cipher_list_canonicalize(val);
         }
-      SSH_DEBUG(3, ("Final cipherlist \"%s\"", config->ciphers));
+      SSH_DEBUG(5, ("Final cipherlist \"%s\"", config->ciphers));
       return FALSE;
     }
   
   if (strcmp(var, "macs") == 0)
     {
-      SSH_DEBUG(3, ("Got config maclist \"%s\"", val));
+      SSH_DEBUG(5, ("Got config maclist \"%s\"", val));
       ssh_xfree(config->macs);
       if (strcasecmp(val, "any") == 0)
         {
@@ -655,7 +774,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
         {
           config->macs = ssh_hash_list_canonicalize(val);
         }
-      SSH_DEBUG(3, ("Final maclist \"%s\"", config->macs));
+      SSH_DEBUG(5, ("Final maclist \"%s\"", config->macs));
       return FALSE;
     }
 
@@ -744,7 +863,20 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       config->ssh1compatibility = bool;
       return FALSE;
     }
-  
+
+  /* XXX not yet implemented */
+  if (strcmp(var, "rekeyintervalbytes") == 0)
+    {
+      config->rekey_interval_bytes = strtoul(val, NULL, 0);
+      return FALSE;
+    }
+
+  if (strcmp(var, "rekeyintervalseconds") == 0)
+    {
+      config->rekey_interval_seconds = strtoul(val, NULL, 0);
+      return FALSE;
+    }
+
   /* for client only */
 
   if (config->client == TRUE)
@@ -857,6 +989,11 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
 
       if (strcmp(var, "ssh1internalemulation") == 0)
         {
+
+
+
+
+
           return FALSE;
         }
 
@@ -885,7 +1022,6 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
             }
         }
 
-#ifndef SSH_WIN_CLIENT
       if (strcmp(var, "localforward") == 0)
         {
           if(ssh_parse_forward(&(config->local_forwards), val))
@@ -904,8 +1040,32 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
               return TRUE;
             }
           return FALSE;
-        }  
-#endif /* !SSH_WIN_CLIENT */
+        }
+
+      if (strcmp(var, "clearallforwardings") == 0)
+        {
+          if (bool == TRUE)
+            {
+              while (config->remote_forwards != NULL)
+                {
+                  current_forward = config->remote_forwards;
+                  config->remote_forwards = config->remote_forwards->next;
+                  ssh_xfree(current_forward);
+                }
+
+              while (config->local_forwards != NULL)
+                {
+                  current_forward = config->local_forwards;
+                  config->local_forwards = config->local_forwards->next;
+                  ssh_xfree(current_forward);
+                }
+            }
+
+          return FALSE; 
+
+        }
+
+
       if (strcmp(var, "forwardx11") == 0)
         {
           config->forward_x11 = bool;
@@ -955,7 +1115,14 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
         {
           config->auth_success_msg = bool;
           return FALSE;
-        }      
+        }
+
+      if (strcmp(var, "numberofpasswordprompts") == 0)
+        {
+          config->number_of_password_prompts = num;
+          return FALSE;
+        }
+
     }
   else
     {
@@ -1464,6 +1631,14 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           return FALSE;
         }
 
+#ifdef SSH_SERVER_WITH_SECURID
+      if (strcmp(var, "securidguesses") == 0)
+        {
+          config->securid_guesses = num;
+          return FALSE;
+        }
+#endif /* SSH_SERVER_WITH_SECURID */
+
       if (strcmp(var, "maxconnections") == 0)
         {
 #ifndef SSH2_MAX_CONNECTIONS
@@ -1499,6 +1674,20 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           config->ssh1_path = ssh_xstrdup(val);
           return FALSE;
         }
+
+      if (strcmp(var, "sshpamclientpath") == 0)
+        {
+          ssh_xfree(config->ssh_pam_client_path);
+          config->ssh_pam_client_path = ssh_xstrdup(val);
+          return FALSE;
+        }
+      
+      if (strcmp(var, "bannermessagefile") == 0)
+        {
+          ssh_xfree(config->banner_message_file_path);
+          config->banner_message_file_path = ssh_xstrdup(val);
+          return FALSE;
+        }
       
       /* Parse subsystem definitions */
       if (strncmp(var, SUBSYSTEM_PREFIX, SUBSYSTEM_PREFIX_LEN) == 0)
@@ -1540,11 +1729,502 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
           ss->name = ssh_xstrdup(&var[SUBSYSTEM_PREFIX_LEN]);
           ss->path = ssh_xstrdup(val);
           config->subsystems[config->no_subsystems++] = ss;
-                          
+
           return FALSE;
         }
-      
+
+      if (strcmp(var, "idletimeout") == 0)
+        {
+          config->idle_timeout = num;
+          return FALSE;
+        }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
     }
+
   ssh_warning("Unrecognized configuration parameter %s", var);
   return TRUE;
 }
+
+
+
+/* These functions were moved here from sshunixconfig.c, which was
+   discarded. These functions are also used on Windows. */
+char *ssh2_config_line_heading_separator(char *linebuf)
+{
+  char *hlp = linebuf;
+  int inquotes = 0;
+
+  while (*hlp) 
+    {
+      if ((*hlp == ':') && (inquotes == 0) && isspace(*(hlp+1)))
+        return hlp;
+      else if (*hlp == '"')
+        inquotes = !inquotes;
+      hlp++;
+    }
+  return NULL;
+}
+
+void ssh2_config_remove_quotes(char *str)
+{
+  char *hlp1, *hlp2;
+  int in_quotes = 0;
+  int quoted = 0;
+
+  hlp1 = hlp2 = str;
+
+  while (*hlp1)
+    {
+      switch (*hlp1)
+        {
+        case '"':
+          if (quoted)
+            {
+              *hlp2 = *hlp1;
+              hlp2++;
+            }
+          in_quotes = !in_quotes;
+          break;
+
+
+        case '\\':
+          if ((!in_quotes) || quoted)
+            {
+              *hlp2 = *hlp1;
+              hlp2++;
+              quoted = 0;
+            }
+          else
+            {
+              quoted = 1;
+            }
+          break;
+
+
+        default:
+          *hlp2 = *hlp1;
+          hlp2++;
+          quoted = 0;
+        }
+      hlp1++;
+    }
+  *hlp2 = '\0';
+}
+
+/* Parse a configuration/authorization file into an array of 
+   variable name <-> value pairs. Return the number of variables or -1 on 
+   error. Pointers to tables of pointers to null-terminated strings are
+   placed at *variables and *values. */
+int ssh2_parse_config(SshUser user, const char *instance, const char *path, 
+                      char ***variables, char ***values, void *context)
+{
+  SshUserFile f;
+  char **vars, **vals, *varpos, *valpos, *hlp;
+  char linebuf[1024];
+  size_t n;
+  int i, j;
+  int line, ch;
+  Boolean matching;
+
+
+  if (user == NULL)
+    {
+      if ((f = ssh_userfile_open(getuid(), path, O_RDONLY, 0755)) == NULL)
+        {
+          ssh_debug("Unable to open %s", path);
+          return -1;
+        }
+    }
+  else
+
+    {
+      if ((f = ssh_userfile_open(ssh_user_uid(user), path, O_RDONLY, 0755)) ==
+          NULL)
+        {
+          ssh_debug("Unable to open %s", path);
+          return -1;
+        }
+    }
+
+  line = 0;
+  i = 0;
+  n = 16;
+  matching = TRUE;
+  vars = ssh_xcalloc(n, sizeof(char *));
+  vals = ssh_xcalloc(n, sizeof(char *));
+
+  while (ssh_userfile_gets(linebuf, sizeof(linebuf) - 1, f) != NULL)
+    {
+      line++;
+
+      /* skip the starting white spaces and comment lines */
+
+      for (j = 0;; j++)
+        {
+          ch = linebuf[j];
+          if (ch == '\0' || ch == '#')
+            goto skip;
+          if (!isspace(ch))
+            break;
+        }
+      
+      /* see if this is a heading or not.. */
+      hlp = ssh2_config_line_heading_separator(linebuf);
+      if (hlp != NULL)
+        {
+          *hlp = '\0';
+          matching = ssh_match_pattern(instance, &linebuf[j]);
+          continue;
+        }
+
+      /* ok, it must be a variable definition */
+
+      if (!matching)
+        goto skip;
+
+      varpos = &linebuf[j];
+
+      /* convert the variable name to lower case */
+
+      for (j = 0; varpos[j] && (isalnum(varpos[j]) || varpos[j] == '-'); j++)
+        varpos[j] = tolower(varpos[j]);
+
+      if (!varpos[j] || !isspace(varpos[j]))
+        {
+          ssh_warning("%s: invalid definition in line %d.", 
+                      path, line);
+          goto skip;
+        }
+      varpos[j++] = 0;
+
+      for (; varpos[j] && isspace(varpos[j]); j++)
+        ;
+      valpos = &varpos[j];
+
+      /* remove spaces from the tail */
+
+      for (j = strlen(valpos) - 1; j > 0 && isspace(valpos[j]); j--)
+        ;
+      valpos[j + 1] = 0;
+
+      vars[i] = ssh_xstrdup(varpos);
+      vals[i] = ssh_xstrdup(valpos);
+      ssh2_config_remove_quotes(vals[i]);
+      i++;
+
+      /* get more space if needed */
+
+      if (i >= n)
+        {
+          n = 2 * n;
+          vars = ssh_xrealloc(vars, n * sizeof(char *));
+          vals = ssh_xrealloc(vals, n * sizeof(char *));
+        }
+    skip:
+      ;
+    }
+
+  ssh_userfile_close(f);
+  *variables = vars;
+  *values = vals;
+
+  return i;
+}
+
+
+Boolean ssh_server_load_host_key(SshConfig config,
+                                 SshPrivateKey *private_host_key,
+                                 unsigned char **public_host_key_blob,
+                                 size_t *public_host_key_blob_len,
+                                 void *context)
+{
+  SshUser user;
+  char *userdir = NULL, *comment;
+  char hostkeyfile[256];
+  SshPrivateKey privkey;
+
+  if ((user = ssh_user_initialize(NULL, TRUE)) == NULL)
+    ssh_fatal("ssh_server_load_host_key: ssh_user_initialize failed");
+
+  /* load the host key from (typically) /etc/ssh2/hostkey */
+  if (config->host_key_file[0] != '/')
+    {
+      if (ssh_user_uid(user) == 0 )
+        {
+          userdir = ssh_xstrdup(SSH_SERVER_DIR);
+        }
+      else
+        {
+          if ((userdir = ssh_userdir(user, config, TRUE)) == NULL)
+            ssh_fatal("ssh_server_load_host_key: no ssh2 user directory");
+        }
+      
+      snprintf(hostkeyfile, sizeof(hostkeyfile), "%s/%s",
+               userdir, config->host_key_file);
+    }
+  else
+    {
+      snprintf(hostkeyfile, sizeof(hostkeyfile), "%s",
+               config->host_key_file);
+    }
+
+  ssh_debug("Reading private host key from %s", hostkeyfile);
+  if ((privkey = ssh_privkey_read(user, hostkeyfile, "", &comment,
+                                  NULL)) == NULL)
+    ssh_fatal("ssh_privkey_read from %s failed.", hostkeyfile);
+
+  /* print the comment just for fun.. */
+  if (comment != NULL)
+    {
+      if (strlen(comment) > 0)
+        ssh_debug("Key comment: %s", comment);
+      ssh_xfree(comment);
+    }
+  *private_host_key = privkey;
+
+  /* ok, now read the public host key blob */
+  if (config->public_host_key_file[0] != '/')
+    {
+      snprintf(hostkeyfile, sizeof(hostkeyfile), "%s/%s", 
+               userdir, config->public_host_key_file);
+    }
+  else
+    {
+      snprintf(hostkeyfile, sizeof(hostkeyfile), "%s", 
+               config->public_host_key_file);
+    }
+
+  SSH_TRACE(1, ("Reading public host key from: %s", hostkeyfile));
+
+  if (ssh2_key_blob_read(user, hostkeyfile, TRUE, NULL,
+                        public_host_key_blob,
+                        public_host_key_blob_len, NULL) 
+      != SSH_KEY_MAGIC_PUBLIC)
+    ssh_fatal("Unable to load public host key from %s.", hostkeyfile);
+
+  /* check keytype */
+
+  if ((config->public_key_algorithm =
+       ssh_pubkeyblob_type(*public_host_key_blob,
+                           *public_host_key_blob_len))
+      == NULL)
+    {
+      SSH_TRACE(0, ("Unable to get public key type from %s.", hostkeyfile));
+      goto error;
+    }
+  
+
+  ssh_xfree(userdir);
+  ssh_user_free(user, FALSE);
+
+  return TRUE;
+
+ error: 
+  ssh_xfree(userdir);
+  ssh_user_free(user, FALSE);
+  return FALSE;
+}
+
+
+
+/* Parse a line of input */
+
+Boolean ssh_config_parse_line(SshConfig config, char *line)
+{
+  int j, k;
+  char var[1024], val[1024];
+
+  /* skip over spaces in the beginning */
+
+  for (j = 0; line[j] != '\0' && isspace(line[j]); j++)
+    ;
+  if (line[j] == '\0')
+    return TRUE;
+  
+  /* convert the variable name to lower case while copying it */
+  
+  k = 0;
+  for (k = 0; k < (sizeof(var)-2) && isalnum(line[j]); j++)
+    var[k++] = tolower(line[j]);
+  var[k++] = '\0';
+  if (!isspace(line[j]))
+    return TRUE;
+  
+  /* skip the spaces in the middle */
+  for (; isspace(line[j]); j++)
+    ;
+
+  /* determine the actual length of the value */
+  for (k = strlen(line); isspace(line[k]); k--)
+    ;
+  if ((k - j) >= (sizeof(val) - 1) || k == j)
+    return TRUE;
+  
+  /* make a copy of the value */
+  memcpy(val, &line[j], k - j);
+  val[k - j] = '\0';
+  
+  /* now we're ready to call ssh_config_set_parameter() */
+  
+  if (!ssh_config_set_parameter(config, var, val))
+    return TRUE;
+  
+  return FALSE;
+}
+
+/* Reads config data from the given file.  Returns FALSE if an error
+   occurs (displays error messages with ssh_warning.) */
+
+Boolean ssh_config_read_file(SshUser user, SshConfig config,
+                             char *instance, const char *filename,
+                             void *context)
+{
+  SshUser user_data;
+  char **vars, **vals;
+  int i, n;
+  
+  if (filename == NULL || strlen(filename) == 0)
+    return FALSE;
+  
+  if (user == NULL)
+    user_data = ssh_user_initialize(NULL, FALSE);
+  else
+    user_data = user;
+
+  /* try to read in the file */
+  instance = (instance ? instance : "");
+
+  n = ssh2_parse_config(user_data, instance, filename, &vars, &vals, NULL);
+
+  if (n < 0)
+    {
+      if (user_data != user)
+        ssh_user_free(user_data, FALSE);
+      return FALSE;
+    }
+
+  /* ok, now fill in the fields */
+
+  for (i = 0; i < n; i++)
+    ssh_config_set_parameter(config, vars[i], vals[i]);
+
+  ssh_free_varsvals(n, vars, vals);
+
+  if (user_data != user)
+    ssh_user_free(user_data, FALSE);
+
+  return TRUE;
+}
+
+/* Parse forwarding definitions. Format is 
+   [protocol/]port:remotehost:remoteport */
+Boolean ssh_parse_forward(SshForward *forwards, char *spec_orig)
+{
+  SshForward fwd;
+  char *local_port, *host, *port, *proto, *spec, *to_be_deleted;
+  char *hlp1, *hlp2;
+
+  spec = ssh_xstrdup(spec_orig);
+  to_be_deleted = spec;
+  
+  if ((hlp1 = strchr(spec, '/')) &&
+      (hlp2 = strchr(spec, ':')) &&
+      (hlp1 < hlp2))
+    {
+      *hlp1 = '\0';
+      proto = ssh_xstrdup(spec);
+      spec = hlp1 + 1;
+    }
+  else
+    {
+      proto = ssh_xstrdup("tcp");
+    }
+
+  local_port = strtok(spec, ":");
+  if (local_port == NULL)
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
+  host = strtok(NULL, ":");
+  if (host == NULL)
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
+  port = strtok(NULL, ":");
+  if (port == NULL)
+    {
+      ssh_xfree(proto);
+      return TRUE;
+    }
+  fwd = ssh_xcalloc(1, sizeof(*fwd));
+  fwd->local_addr = ssh_xstrdup("0.0.0.0");
+  fwd->port = ssh_xstrdup(local_port);
+  fwd->connect_to_host = ssh_xstrdup(host);
+  fwd->connect_to_port = ssh_xstrdup(port);
+  fwd->protocol = proto; /* This is already mallocated. */
+  fwd->next = *forwards;
+  *forwards = fwd;
+  ssh_xfree(to_be_deleted);
+  
+  return FALSE;
+}
