@@ -33,6 +33,7 @@
 #include "auths-common.h"
 #include "sshencode.h"
 #include "sshdsprintf.h"
+#include "sshappcommon.h"
 #include <syslog.h>
 #ifdef NEED_SYS_SYSLOG_H
 #include <sys/syslog.h>
@@ -43,6 +44,11 @@ int allow_severity = SSH_LOG_INFORMATIONAL;
 int deny_severity = SSH_LOG_WARNING;
 #endif /* HAVE_LIBWRAP */
 
+#ifdef SSHDIST_SESSION_SIA
+#ifdef HAVE_SIA
+#include "sshsia.h"
+#endif /* HAVE_SIA */
+#endif /* SSHDIST_SESSION_SIA */
 
 #ifdef HAVE_SCO_ETC_SHADOW
 #include <sys/types.h>
@@ -257,16 +263,14 @@ Boolean ssh_server_version_check(const char *version, void *context)
   int i, arg;
   extern char **environ;
 
-  ssh_debug("Remote version: %s\n", version);
-
   if (strncmp(version, "SSH-1.", 6) == 0 &&
       strncmp(version, "SSH-1.99", 8) != 0 &&
       c->server->config->ssh1compatibility == TRUE &&
       c->server->config->ssh1_path != NULL &&
       c->server->config->ssh1_argv != NULL)
     {
-      ssh_debug("Executing %s for ssh1 compatibility.",
-                c->server->config->ssh1_path);
+      SSH_TRACE(0, ("Executing %s for ssh1 compatibility.",
+                    c->server->config->ssh1_path));
 
       arg = 0;
       args[arg++] = "sshd";
@@ -619,11 +623,12 @@ void new_connection_callback(SshIpError error, SshStream stream,
                        (data->config->ssh1compatibility ? 
                         "SSH-1.99-%.200s" : "SSH-2.0-%.200s"),
                        SSH2_PROTOCOL_VERSION_STRING);
-          ssh_buffer_append(buffer2, version_string, strlen(version_string));
+          ssh_buffer_append(buffer2, (unsigned char *)version_string,
+                            strlen(version_string));
           /* No CR in ssh1 compatible mode. */
           if (! data->config->ssh1compatibility)
-            ssh_buffer_append(buffer2, "\r", 1);
-          ssh_buffer_append(buffer2, "\n", 1);
+            ssh_buffer_append(buffer2, (unsigned char *)"\r", 1);
+          ssh_buffer_append(buffer2, (unsigned char *)"\n", 1);
           ssh_xfree(version_string);
           /* Assume granularity that is enough for server. */
           pad_len = (ssh_buffer_len(buffer1) + 4 + 7) % 16;
@@ -691,7 +696,7 @@ void new_connection_callback(SshIpError error, SshStream stream,
 #ifdef HAVE_LIBWRAP
       {
         struct request_info req;
-        void *old_handler;
+        static RETSIGTYPE (*old_handler)(int sig) = NULL;
 
         old_handler = signal(SIGCHLD, SIG_DFL);
         /* XXX SIGALRM, SIGHUP */
@@ -1087,13 +1092,20 @@ int main(int argc, char **argv)
   else
     av0 = argv[0];
 
+  ssh2_version(av0);
+  
   /* Initializations */
-  restart = FALSE;
-
+  restart = FALSE;  
+  
 #ifdef HAVE_SCO_ETC_SHADOW
   set_auth_parameters(argc, argv);
 #endif /* HAVE_SCO_ETC_SHADOW */
 
+#ifdef SSHDIST_SESSION_SIA
+#ifdef HAVE_SIA
+  ssh_sia_initialize(argc, argv);
+#endif /* HAVE_SIA */
+#endif /* SSHDIST_SESSION_SIA */
 
   data = ssh_xcalloc(1, sizeof(*data));
   user = ssh_user_initialize(NULL, TRUE);
@@ -1225,7 +1237,7 @@ int main(int argc, char **argv)
           if (!ssh_optval)
             ssh_fatal("%s: Illegal -g parameter.", av0);
           data->config->login_grace_time = atoi(ssh_optarg);
-          if (data->config->login_grace_time < 1)
+          if (data->config->login_grace_time < 0)
             ssh_fatal("%s: Illegal login grace time %s seconds",
                       av0, ssh_optarg);
           i++;
@@ -1335,6 +1347,7 @@ int main(int argc, char **argv)
       ssh_debug("Creating listener");
       data->listener = ssh_tcp_make_listener(data->config->listen_address,
                                              data->config->port,
+                                             NULL, 
                                              new_connection_callback,
                                              (void *)data);
       if (data->listener == NULL)
@@ -1428,60 +1441,61 @@ int main(int argc, char **argv)
   ssh_log_event(data->config->log_facility,
                 SSH_LOG_WARNING,
                 "Daemon is running.");
-
+  
   ssh_debug("Running event loop");
   ssh_event_loop_run();
 
   ssh_signals_reset();
 
-  ssh_debug("Exiting event loop");
-  ssh_event_loop_uninitialize();
-
   if (data->listener)
     {
       remove(pidfile);
 
-      if (restart)
+      ssh_tcp_destroy_listener(data->listener);
+      data->listener = NULL;
+
+      if (data->broadcast_listener)
+        ssh_udp_destroy_listener(data->broadcast_listener);
+      data->broadcast_listener = NULL;      
+    }
+  else if (restart)
+    {
+      restart = FALSE;
+    }
+
+  ssh_debug("Exiting event loop");
+  ssh_event_loop_uninitialize();
+
+  if (restart)
+    {
+      int ret;
+
+      ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
+                    "restarting...");
+      ret = fork();
+
+      if (ret == 0)
         {
-          int ret;
-
-          if (data->listener)
-            ssh_tcp_destroy_listener(data->listener);
-          data->listener = NULL;
-          if (data->broadcast_listener)
-            ssh_udp_destroy_listener(data->broadcast_listener);
-          data->broadcast_listener = NULL;
-
-          SSH_DEBUG(0, ("restarting..."));
-          ssh_log_event(data->config->log_facility, SSH_LOG_WARNING,
-                        "restarting...");
-
-          ret = fork();
-
-          if (ret == 0)
+          /* Child */
+          execvp(argv[0], argv);
+          ssh_fatal("Restart (exec) failed on SIGHUP. "
+                    "(error message \"%s\")",
+                    strerror(errno));
+        }
+      else
+        {
+          /* Parent */
+          if (ret == -1)
             {
-              /* Child */
-              execv(argv[0], argv);
-              ssh_fatal("Restart (exec) failed on SIGHUP. "
-                        "(error message \"%s\")",
-                        strerror(errno));
+              /* Fork failed */
+              ssh_fatal("Restart (fork) failed on SIGHUP.");
             }
           else
             {
-              /* Parent */
-              if (ret == -1)
-                {
-                  /* Fork failed */
-                  ssh_fatal("Restart (fork) failed on SIGHUP.");
-                }
-              else
-                {
-                  exit(0);
-                }
+              exit(0);
             }
         }
     }
 
   return 0;
 }
-
