@@ -14,10 +14,11 @@
 #include "sshfilecopyi.h"
 
 
-#ifdef WIN32
-/* There is no symbolic link on Windows */
-#define S_IFLNK 0
-#endif
+
+
+
+
+
 
 
 #define SSH_DEBUG_MODULE "SshFCRecurse"
@@ -694,9 +695,10 @@ void recurse_lstat_cb(SshFileClientError error,
 
 SSH_FSM_STEP(recurse_lstat_file)
 {
+#if 0
   SshFileCopyFileListItem item;
   SshFileCopyFile file;
-  
+#endif
   SSH_FSM_GDATA(SshFileCopyRecurseContext);
 
   if (gdata->conn_flags == SSH_FCC_DOWN)
@@ -706,7 +708,7 @@ SSH_FSM_STEP(recurse_lstat_file)
     }
 
   SSH_FSM_SET_NEXT("fcr_next_file");
-
+#if 0
   SSH_ASSERT(ssh_dllist_is_current_valid(gdata->file_list));
   
   item = (SshFileCopyFileListItem) ssh_dllist_current(gdata->file_list);
@@ -714,7 +716,7 @@ SSH_FSM_STEP(recurse_lstat_file)
   SSH_ASSERT(ssh_dllist_is_current_valid(item->files->file_list));
 
   file = (SshFileCopyFile) ssh_dllist_current(item->files->file_list);
-
+#endif
   SSH_FSM_ASYNC_CALL(ssh_file_client_lstat(gdata->source->client,
                                            gdata->filename_to_be_statted,
                                            recurse_lstat_cb,
@@ -1289,7 +1291,7 @@ SSH_FSM_STEP(recurse_done)
   return SSH_FSM_FINISH;
 }
 
-SshFSMStateMapItem recurse_states_array[] =
+SshFSMStateMapItemStruct recurse_states_array[] =
 { { "fcr_next_file", "Set next file", recurse_next_file },
   { "fcr_lstat", "lstat file", recurse_lstat_file },
   { "fcr_parse_raw", "Parse \"raw\" file", recurse_parse_raw },
