@@ -576,13 +576,13 @@ SshFileCopyFileListItem ssh_file_copy_file_list_item_allocate(void)
   return temp_list;
 }
 
-unsigned char *
+char *
 ssh_file_copy_file_list_item_get_original_name(SshFileCopyFileListItem item)
 {
   return item->original_filename;
 }
 
-unsigned char *
+char *
 ssh_file_copy_file_list_item_get_directory_name(SshFileCopyFileListItem item)
 {
   return item->files->file->name;
@@ -910,6 +910,7 @@ ssh_file_copy_connect(SshFileCopyConnection connection,
 
       /* Make local connection. */
       ssh_stream_pair_create(&stream1, &stream2);
+      /* XXX local_server should be freed. */
       local_server = ssh_file_server_wrap(stream1);
       local_client = ssh_file_client_wrap(stream2);
 
