@@ -12,9 +12,11 @@
 */
 
 #include "sshincludes.h"
-#ifdef WIN32
-#include <direct.h> /* header for getcwd() */
-#endif /* WIN32 */
+
+
+
+
+
 
 #define SSH_DEBUG_MODULE "SshGetCwd"
 
