@@ -133,8 +133,11 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
                                     config->public_host_key_blob,
                                     config->public_host_key_blob_len,
                                     version_check,
-                                    (void *)context);
-
+                                    (void *)context,
+                                    ssh_common_rekey_notify_cb,
+                                    /* SshCommon object is allocated a
+                                       below. */
+                                    (void *)&(server->common));
   
   ssh_transport_get_compatibility_flags(trans, &compat_flags);
   
@@ -146,7 +149,9 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
   ssh_debug("ssh_server_wrap: creating userauth protocol");
 
   auth = ssh_auth_server_wrap(trans, auth_policy_proc, (void *)server,
-                              server->methods, (void *)server);
+                              server->methods, (void *)server,
+                              config->banner_msg,
+                              &server->uc);
 
   /* Create the common part of client/server objects. */
   server->common = ssh_common_wrap(stream, auth, FALSE, config, 
@@ -161,11 +166,16 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
       ssh_xfree(server);
       return NULL;
     }
+
+  /* Set common->ucp to point to the location that will receive the "SshUser"
+     object for the authenticated user before SSH_CROSS_AUTHENTICATED is
+     delivered. */
+  server->common->ucp = &server->uc;
   
   return server;
 }
 
-/* External declaration. (this is defined is sshd2.c.)*/
+/* External declaration. (this is defined is sshd2.c.) */
 extern void ssh_login_grace_time_exceeded(void *context);
 
 /* Forcibly destroys the given server. */
@@ -180,6 +190,8 @@ void ssh_server_destroy(SshServer server)
      lib/sshproto/sshauths.c after this call. */
   ssh_server_authentication_uninitialize(server->methods);
 #endif
+  ssh_xfree(server->authenticated_client_user_name);
+  ssh_xfree(server->authenticated_client_host);
   memset(server, 'F', sizeof(*server));
   ssh_xfree(server);
 }
