@@ -14,6 +14,9 @@
 */
 
 #include "ssh2includes.h"
+
+
+
 #include "sshfilterstream.h"
 #include "sshtcp.h"
 #include "sshcommon.h"
@@ -21,6 +24,9 @@
 #include "sshtty.h"
 #include "sshappcommon.h"
 #include "sshclient.h"
+#include "sshencode.h"
+#include "sshcross.h"
+#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "SshStdIOFilter"
 
@@ -68,10 +74,6 @@ void ssh_escape_char_help(int esc_char)
   fprintf(stderr,
           "  %c-  - disable escape character uncancellably\n",
           esc_char);
-  fprintf(stderr,
-          "  %c&  - background ssh (when waiting for "
-          "connections to terminate)\n",
-          esc_char);
   fprintf(stderr,
           "  %c?  - this message\n",
           esc_char);
@@ -83,6 +85,10 @@ void ssh_escape_char_help(int esc_char)
   fprintf(stderr,
           "  %cs   - dump statistics and connection information to stderr\n",
           esc_char);
+  fprintf(stderr,
+          "  %cr   - initiate rekey immediately with old algorithms (ie. "
+          "changes\n"
+          "         encryption and integrity keys)\n", esc_char);
   fprintf(stderr,
           "  (Note that escapes are only recognized immediately "
           "after newline.)\n");
@@ -124,16 +130,6 @@ void ssh_escape_char_quit(int esc_char)
   exit(0);
 }
 
-void ssh_escape_char_background(int esc_char)
-{
-  clearerr(stderr);        /* XXX */
-  ssh_leave_raw_mode(-1);
-  ssh_cancel_nonblocking();
-  fprintf(stderr, "\nBackgrounding not yet supported.\n");
-  ssh_set_nonblocking();
-  ssh_enter_raw_mode(-1, FALSE);
-}
-
 void ssh_escape_char_suspend(int esc_char)
 {
 #ifdef SIGWINCH
@@ -173,12 +169,80 @@ void ssh_escape_char_suspend(int esc_char)
 #endif /* SIGWINCH */
 }
 
-void ssh_escape_char_list_connections(int esc_char)
+void ssh_escape_char_list_connections(int esc_char, SshClient client)
 {
+  SshTcpForwardActive tmp_active_ptr;
+  SshCommon common;
+
   clearerr(stderr);        /* XXX */
   ssh_leave_raw_mode(-1);
   ssh_cancel_nonblocking();
-  fprintf(stderr, "\nConnection listing not yet supported.\n");
+
+  common = client->common;
+
+  if (!ssh_dllist_is_empty(common->active_local_forwards))
+    {      
+      fprintf(stderr, "\nActive local forwards:\n");
+      ssh_dllist_rewind(common->active_local_forwards);
+  
+      while ((tmp_active_ptr =
+              ssh_dllist_current(common->active_local_forwards)) != NULL)
+        {
+          fprintf(stderr, "%s : %s : %s from %s\n",
+                  tmp_active_ptr->port,
+                  tmp_active_ptr->connect_to_host,
+                  tmp_active_ptr->connect_to_port,
+                  tmp_active_ptr->connect_from_host);
+
+          ssh_dllist_fw(common->active_local_forwards, 1);
+        }
+    }
+  else
+    {
+      fprintf(stderr, "\nNo active local forwards.\n");
+    }
+  
+  if (!ssh_dllist_is_empty(common->active_remote_forwards))
+    {
+      fprintf(stderr, "\nActive remote forwards:\n");
+      ssh_dllist_rewind(common->active_remote_forwards);
+
+      while ((tmp_active_ptr =
+              ssh_dllist_current(common->active_remote_forwards)) != NULL)
+        {
+          fprintf(stderr, "%s : %s : %s from %s\r\n",
+                  tmp_active_ptr->port,
+                  tmp_active_ptr->connect_to_host,
+                  tmp_active_ptr->connect_to_port,
+                  tmp_active_ptr->connect_from_host);
+
+          ssh_dllist_fw(common->active_remote_forwards, 1);
+        }
+    }
+  else
+    {
+      fprintf(stderr, "No active remote forwards.\n");
+    }
+  
+  if (!ssh_dllist_is_empty(common->active_x11_forwards))
+    {
+      fprintf(stderr, "\nActive X11 forwards:\n");
+      ssh_dllist_rewind(common->active_x11_forwards);
+
+      while ((tmp_active_ptr = ssh_dllist_current(common->active_x11_forwards))
+             != NULL)
+        {
+          fprintf(stderr, "X11 forward from %s\r\n",
+                  tmp_active_ptr->connect_from_host);
+      
+          ssh_dllist_fw(common->active_x11_forwards, 1);
+        }
+    }
+  else
+    {
+      fprintf(stderr, "No active X11 forwards.\n");
+    }
+  
   ssh_set_nonblocking();
   ssh_enter_raw_mode(-1, FALSE);
 }
@@ -194,13 +258,50 @@ void ssh_escape_char_dump_version(int esc_char)
 
 void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
 {
-  SshTransportStatistics stats;
+  SshTransportStatisticsStruct stats;
+  char *local_version_string = NULL;
+  /* XXX this should be somewhere else. */
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 256
+#endif /* MAXHOSTNAMELEN */
+  char local_host_name[MAXHOSTNAMELEN + 1];
+  
+  SSH_PRECOND(client && client->config && client->common);
   clearerr(stderr);        /* XXX */
   ssh_leave_raw_mode(-1);
   fprintf(stderr, "\n");
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   ssh_transport_get_statistics_tr(client->tr_context, &stats);
 
+
+  ssh_tcp_get_host_name(local_host_name, MAXHOSTNAMELEN);
+  
+  if (client->config->host_to_connect)
+    fprintf(stderr, "remote host: %s\n", client->config->host_to_connect);
+  if (local_host_name)
+    fprintf(stderr, "local host: %s\n", local_host_name);
+  
+  ssh_dsprintf(&local_version_string,
+               (client->config->ssh1compatibility ? 
+                "SSH-1.99-%.200s" : "SSH-2.0-%.200s"),
+               SSH2_PROTOCOL_VERSION_STRING);
+  fprintf(stderr, "remote version: %s\n", client->remote_version);
+  fprintf(stderr, "local version:  %s\n", local_version_string);
+  ssh_xfree(local_version_string);
   fprintf(stderr, "compressed bytes in: %ld\n",
           stats.compressed_incoming_bytes);
   fprintf(stderr, "uncompressed bytes in: %ld\n",
@@ -213,6 +314,8 @@ void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
           stats.incoming_packets);
   fprintf(stderr, "packets out: %ld\n",
           stats.outgoing_packets);
+  fprintf(stderr, "rekeys: %ld\n", stats.rekeys);
+  
   fprintf(stderr, "\nAlgorithms:\n");
 
   fprintf(stderr, "Chosen key exchange algorithm: %s\n", stats.kex_name);
@@ -230,6 +333,17 @@ void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
   ssh_enter_raw_mode(-1, FALSE);
 }
 
+void ssh_escape_char_request_rekey(int esc_char, SshClient client)
+{
+  SSH_PRECOND(client && client->common);
+  clearerr(stderr);        /* XXX */
+  ssh_leave_raw_mode(-1);
+  fprintf(stderr, "\n");
+  ssh_common_request_rekey(client->common);
+  fprintf(stderr, "Rekeying.\n");
+  ssh_enter_raw_mode(-1, FALSE);
+}
+
 int ssh_stdio_output_filter(SshBuffer data,
                             size_t offset,
                             Boolean eof_received,
@@ -295,12 +409,6 @@ int ssh_stdio_input_filter(SshBuffer data,
           i--;            /* Re-exam current position */
           ssh_escape_char_quit(e);
           break;
-        case '&':
-          ssh_buffer_consume_middle(data, offset + i, 1);
-          received_len--; /* Buffer is now shorter    */
-          i--;            /* Re-exam current position */
-          ssh_escape_char_background(e);
-          break;
         case 26: /* ^Z */
           ssh_buffer_consume_middle(data, offset + i, 1);
           received_len--; /* Buffer is now shorter    */
@@ -311,7 +419,7 @@ int ssh_stdio_input_filter(SshBuffer data,
           ssh_buffer_consume_middle(data, offset + i, 1);
           received_len--; /* Buffer is now shorter    */
           i--;            /* Re-exam current position */
-          ssh_escape_char_list_connections(e);
+          ssh_escape_char_list_connections(e, client);
           break;
         case '-':
           ssh_buffer_consume_middle(data, offset + i, 1);
@@ -330,6 +438,12 @@ int ssh_stdio_input_filter(SshBuffer data,
           i--;            /* Re-exam current position */
           ssh_escape_char_dump_statistics(e, client);
           break;
+        case 'r':
+          ssh_buffer_consume_middle(data, offset + i, 1);
+          received_len--; /* Buffer is now shorter    */
+          i--;            /* Re-exam current position */
+          ssh_escape_char_request_rekey(e, client);
+          break;
         default:
           if (c != e)
             {
