@@ -2,15 +2,18 @@
 
   sshttymodesi.h
   
-  Author: Tatu Ylonen <ylo@cs.hut.fi>
-          SGTTY stuff contributed by Janne Snabb <snabb@niksula.hut.fi>
-          Sami Lehtinen <sjl@ssh.fi>
+  Author: Tatu Ylonen <ylo@ssh.com>
+          SGTTY stuff contributed by Janne Snabb <snabb@ssh.com>
+          Sami Lehtinen <sjl@ssh.com>
   
+  Copyright (C) 1995-2000 SSH Communications Security Corp, Helsinki, Finland
+  All rights reserved.
+
   Based on ttymodes.h from ssh-1.2.26.
 */
 
 /*
- * $Id: sshttyflagsi.h,v 1.1 1999/02/16 12:42:07 sjl Exp $
+ * $Id: sshttyflagsi.h,v 1.4 2000/09/14 23:49:51 sjl Exp $
  * $Log: sshttyflagsi.h,v $
  * $EndLog$
  */
