@@ -3,12 +3,12 @@
   sshunixuserfiles.c
 
   Authors:
-        Tatu Ylonen <ylo@ssh.fi>
-        Markku-Juhani Saarinen <mjos@ssh.fi>
-        Timo J. Rinne <tri@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+        Tatu Ylonen <ylo@ssh.com>
+        Markku-Juhani Saarinen <mjos@ssh.com>
+        Timo J. Rinne <tri@ssh.com>
+        Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-1998 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Simple functions that update user's files. These are unix-spesific.
@@ -29,120 +29,174 @@
 #include "sshconfig.h"
 #include "sshmiscstring.h"
 
+
+
+
+
 #define SSH_DEBUG_MODULE "SshUnixUserFiles"
 
-/* Return a pointer to user's ssh2 directory.
-   The directory is created if `create_if_needed' is TRUE. 
-   Return NULL on failure. The caller is responsible for freeing the returned
-   value with ssh_xfree when no longer needed. */
-
-char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed)
-{
-  char *sshdir;
-  struct stat st;
-
-  /* create the .ssh2 directory name */
-
-  sshdir = ssh_user_conf_dir(config, user);
-
-  if (stat(sshdir, &st) < 0)
-    {
-      if (create_if_needed)
-        {
-          if (mkdir(sshdir, 0755) < 0)
-            {
-              SSH_DEBUG(2, ("could not create user's ssh directory %s", 
-                            sshdir));
-              ssh_xfree(sshdir);
-              return NULL;
-            }  
-        }
-      else
-        {
-          ssh_xfree(sshdir);
-          return NULL;
-        }
-    }
-
-  return sshdir;
-}
 
 
 /* Make sure that the random seed file exists and return a pointer to it. 
    return NULL on failure. The file name is found from `config'. 
-   If `config' is NULL, use the standard SSH_RANDSEED_FILE.
+   If `config->random_seed_file' is NULL, use the standard SSH_RANDSEED_FILE.
 
    The caller is responsible for freeing the returned value with ssh_xfree 
    when no longer needed. */
 
-char *ssh_randseed_file(SshUser user, SshConfig config)
+char *ssh_randseed_file(SshUser user, SshConfig config, Boolean create)
 {
   SshUserFile f;
   char *sshdir, *sshseed;
   size_t sshseedlen;
   struct stat st;
-  
-  /* XXX config is not used */
+  int flags = O_RDWR | O_CREAT;
+  char *random_seed_file;
+  Boolean client;
 
-  /* See if the random seed directory exists */
-  
-  if ((sshdir = ssh_userdir(user, config, TRUE)) == NULL)
-    return NULL;
-  sshseedlen = sizeof(SSH_RANDSEED_FILE) + strlen(sshdir) + 4;
-  sshseed = ssh_xmalloc(sshseedlen);
-  snprintf(sshseed, sshseedlen, "%s/%s", sshdir, SSH_RANDSEED_FILE);
 
-  /* If it doesn't exist, create it. */
 
-  if (ssh_userfile_stat(ssh_user_uid(user), sshseed, &st) < 0)
+
+
+  /* get the rand seed file name if available */
+  if (config == NULL)
+    {
+      /* config is NULL, so assume we're the client and default filename.
+         this situation occurs when using ssh-keygen */
+      random_seed_file = ssh_xstrdup(SSH_RANDSEED_FILE);
+      client = TRUE;
+    }
+  else if (config->random_seed_file == NULL)
+    {
+      /* normally shouldn't happen, since file defaults to SSH_RANDSEED_FILE */
+      random_seed_file = ssh_xstrdup(SSH_RANDSEED_FILE);
+      client = config->client;
+    }
+  else if (strlen(config->random_seed_file) <= 0)
+    {
+      /* may as well be safe and make sure the file name the
+         user gave is long enough */
+      random_seed_file = ssh_xstrdup(SSH_RANDSEED_FILE);
+      client = config->client;
+    }
+  else
+    {
+      /* config struct ok, so use the those values*/
+      random_seed_file = ssh_xstrdup(config->random_seed_file);
+      client = config->client;
+    }
+
+  /* random seed file now defined.. we can continue */
+
+
+  if (random_seed_file[0] == '/')
+
+
+
+    {
+      /* the random seed is an absolute path so use that without
+         modification. */
+      sshseed = ssh_xstrdup(random_seed_file);
+    }
+  else if (client == FALSE)
+    {
+      /* the random seed file name is without an absolute path, and
+         the server called this, so the random_seed should be in
+         SSH_SERVER_DIR*/
+
+
+      sshseedlen = strlen(random_seed_file) +
+                   sizeof(SSH_SERVER_DIR) + 4;
+      sshseed = ssh_xmalloc(sshseedlen);
+      snprintf(sshseed, sshseedlen, "%s/%s",
+               SSH_SERVER_DIR, random_seed_file);
+
+
+
+
+
+
+
+
+
+
+
+    }
+  else
+    {
+      /* the client called this so the random_seed should be in $HOME/.ssh2
+         since there isn't a absolute path in the file name */
+      /* See if the random seed directory exists */
+
+      if ((sshdir = ssh_userdir(user, config, TRUE)) == NULL)
+        {
+          ssh_xfree(random_seed_file);
+          return NULL;
+        }
+
+      sshseedlen = strlen(random_seed_file) + strlen(sshdir) + 4;
+      sshseed = ssh_xmalloc(sshseedlen);
+      snprintf(sshseed, sshseedlen, "%s/%s", sshdir, random_seed_file);
+
+      ssh_xfree(sshdir);
+    }
+
+  /* the sshseed variable is now set, so we don't
+     need random seed file anymore */
+   ssh_xfree(random_seed_file);
+
+  /* If seed doesn't exist, create it. */
+  if (create && ssh_userfile_stat(ssh_user_uid(user), sshseed, &st) < 0)
     {
       if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, 
-                                 O_RDWR | O_CREAT, 0600)) == NULL)
+                                 flags, 0600)) == NULL)
         {
           SSH_DEBUG(2, ("Could not create random seed file %s.", sshseed));
-          ssh_xfree(sshdir);
           ssh_xfree(sshseed);
           return NULL;
         }
       ssh_userfile_close(f);
     }
-  
-  ssh_xfree(sshdir);
 
   return sshseed;
+
 }
 
 /* Get the random state from the file.  This loads and merges any data
    in the seed file into the generator. */
 
-void ssh_randseed_load(SshUser user, SshRandomState random_state,
-                       SshConfig config)
+void ssh_randseed_load(SshUser user, SshConfig config)
 {
   int i;
   SshUserFile f;
   unsigned char randbuf[16];
   char *sshseed;
   size_t nbytes;
+  int flags = O_RDONLY;
+
+
+
+
 
   /* Stir a bit.  This will add a couple of bits of new randomness to the 
      pool. */
   for (i = 0; i < 3; i++)
-    ssh_random_stir(random_state);
+    ssh_random_stir();
   
   /* Stir the seed file in, if possible. */
-  sshseed = ssh_randseed_file(user, config);
-  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, O_RDONLY, 0)) != 
+  sshseed = ssh_randseed_file(user, config, TRUE);
+  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, flags, 0)) != 
       NULL)
     {
       while ((nbytes = ssh_userfile_read(f, randbuf, sizeof(randbuf))) > 0)
-        ssh_random_add_noise(random_state, randbuf, nbytes);
+        ssh_random_add_noise(randbuf, nbytes);
       ssh_userfile_close(f);
     }
   ssh_xfree(sshseed); 
   
   /* Stir a bit.  This will add a few bits of new randomness to the pool. */
   for (i = 0; i < 3; i++)
-    ssh_random_stir(random_state);
+    ssh_random_stir();
 }
 
 /* Updates the random seed file with information from the random
@@ -151,26 +205,31 @@ void ssh_randseed_load(SshUser user, SshRandomState random_state,
    contain traces of both the generator state and the old seed
    file. */
 
-void ssh_randseed_update(SshUser user, SshRandomState rs, SshConfig config)
+void ssh_randseed_update(SshUser user, SshConfig config)
 {
   size_t i;
   SshUserFile f;
   char *sshseed;
   unsigned char seed[SSH_RANDSEED_LEN];
+  int flags = O_CREAT | O_WRONLY;
+
+
+
+
 
   /* Load the old random seed file and mix it into the generator. */
-  ssh_randseed_load(user, rs, config);
+  ssh_randseed_load(user, config);
   
   /* Write data from the generator into the random seed file. */
-  sshseed = ssh_randseed_file(user, config);
-  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, O_CREAT | O_WRONLY, 
+  sshseed = ssh_randseed_file(user, config, TRUE);
+  if ((f = ssh_userfile_open(ssh_user_uid(user), sshseed, flags, 
                          0600)) == NULL)
     {
       SSH_DEBUG(2, ("unable to write the random seed file!"));
       goto error;
     }
   for (i = 0; i < SSH_RANDSEED_LEN; i++)
-    seed[i] = ssh_random_get_byte(rs);
+    seed[i] = ssh_random_get_byte();
   if (ssh_userfile_write(f, seed, SSH_RANDSEED_LEN) != SSH_RANDSEED_LEN)
     ssh_warning("unable to write to the random seed file %s.", sshseed);
 
@@ -192,6 +251,11 @@ Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob,
   unsigned char *data;
   struct stat st;
   size_t datalen;
+  int flags = O_RDONLY;
+
+
+
+
 
   *bloblen = 0;
   *blob = NULL;
@@ -205,7 +269,7 @@ Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob,
   datalen = st.st_size;
   data = ssh_xmalloc(datalen);
 
-  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, O_RDONLY, 0)) == NULL) 
+  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, flags, 0)) == NULL)
     {
       SSH_DEBUG(2, ("Could not open %s.", fname));
       ssh_xfree(data);
@@ -236,8 +300,13 @@ Boolean ssh_blob_write(SshUser user, const char *fname, mode_t mode,
                        void *context)
 {
   SshUserFile f;
+  int flags = O_WRONLY | O_CREAT;
 
-  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, O_WRONLY | O_CREAT, 
+
+
+
+
+  if ((f = ssh_userfile_open(ssh_user_uid(user), fname, flags, 
                          mode)) == NULL)
     {
       SSH_DEBUG(2, ("could not open %s.", fname));
@@ -279,9 +348,14 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
 
   /* read and sort the names */
 
-  snprintf(buf, sizeof(buf)-1, "%s/%s", udir, 
-           config == NULL || config->identity_file == NULL ?
-           SSH_IDENTIFICATION_FILE : config->identity_file);
+  if (config != NULL && config->identity_file != NULL &&
+      *(config->identity_file) == '/')
+    snprintf(buf, sizeof(buf)-1, "%s", config->identity_file);
+  else
+    snprintf(buf, sizeof(buf)-1, "%s/%s", udir, 
+             config == NULL || config->identity_file == NULL ?
+             SSH_IDENTIFICATION_FILE : config->identity_file);
+  
   n = ssh2_parse_config(user, host, buf, &vars, &vals, NULL);
 
   if (n < 0)
@@ -303,8 +377,12 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
     {
       if (strcmp(vars[i], "idkey") == 0)
         {
-          snprintf(buf, sizeof(buf), "%s/%s",
-                   udir, vals[i]);
+          if (*vals[i] == '/')
+            snprintf(buf, sizeof(buf), "%s", vals[i]);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, vals[i]);
+              
           prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
           prkey[j]->keyfile = ssh_xstrdup(buf);
           j++;
@@ -323,8 +401,12 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
           id = strtoul(vals[i], &endptr, 0);
           if (((*(vals[0])) != '\0') && ((*endptr) == '\0'))
             {
-              snprintf(buf, sizeof(buf), "%s/%s",
-                       udir, pgp_secret_key_file);
+              if (*pgp_secret_key_file == '/')
+                snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+              else
+                snprintf(buf, sizeof(buf), "%s/%s",
+                         udir, pgp_secret_key_file);
+              
               prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
               prkey[j]->pgp_keyring = ssh_xstrdup(buf);
               prkey[j]->pgp_id = id;
@@ -337,8 +419,12 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
         }
       else if (strcmp(vars[i], "idpgpkeyname") == 0)
         {
-          snprintf(buf, sizeof(buf), "%s/%s",
-                   udir, pgp_secret_key_file);
+          if (*pgp_secret_key_file == '/')
+            snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, pgp_secret_key_file);
+              
           prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
           prkey[j]->pgp_keyring = ssh_xstrdup(buf);
           prkey[j]->pgp_name = ssh_xstrdup(vals[i]);
@@ -346,8 +432,12 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
         }
       else if (strcmp(vars[i], "idpgpkeyfingerprint") == 0)
         {
-          snprintf(buf, sizeof(buf), "%s/%s",
-                   udir, pgp_secret_key_file);
+          if (*pgp_secret_key_file == '/')
+            snprintf(buf, sizeof(buf), "%s", pgp_secret_key_file);
+          else
+            snprintf(buf, sizeof(buf), "%s/%s",
+                     udir, pgp_secret_key_file);
+              
           prkey[j] = ssh_xcalloc(1, sizeof (struct SshConfigPrivateKey));
           prkey[j]->pgp_keyring = ssh_xstrdup(buf);
           prkey[j]->pgp_fingerprint = ssh_xstrdup(vals[i]);
@@ -365,47 +455,3 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
   return prkey;
 }
 
-char *ssh_user_conf_dir(SshConfig config, SshUser user)
-{
-  const char *user_str;
-  char *conf_dir;
-  unsigned long x;
-  char buf[32];
-  char *tmp;
-
-  /* Get template from config data */
-  conf_dir = ssh_xstrdup((config && config->user_conf_dir) ? 
-                         config->user_conf_dir : SSH_USER_CONFIG_DIRECTORY);
-
-  /* Replace %D with home directory */
-  user_str = ssh_user_dir(user);
-  tmp = ssh_replace_in_string(conf_dir, 
-                              "%D", (user_str != NULL) ? user_str : "");
-  ssh_xfree(conf_dir);
-  conf_dir = tmp;
-
-  /* Ssh_Replace %U with user login name */
-  user_str = ssh_user_name(user);
-  tmp = ssh_replace_in_string(conf_dir, 
-                              "%U", (user_str != NULL) ? user_str : "");
-  ssh_xfree(conf_dir);
-  conf_dir = tmp;
-
-  /* Ssh_Replace %IU with user ID number */
-  x = (unsigned long)ssh_user_uid(user);
-  snprintf(buf, sizeof (buf), "%lu", x);
-  tmp = ssh_replace_in_string(conf_dir, 
-                              "%IU", (user_str != NULL) ? user_str : "");
-  ssh_xfree(conf_dir);
-  conf_dir = tmp;
-
-  /* Ssh_Replace %IG with group ID number */
-  x = (unsigned long)ssh_user_gid(user);
-  snprintf(buf, sizeof (buf), "%lu", x);
-  tmp = ssh_replace_in_string(conf_dir, 
-                              "%IG", (user_str != NULL) ? user_str : "");
-  ssh_xfree(conf_dir);
-  conf_dir = tmp;
-
-  return conf_dir;
-}
