@@ -1,4 +1,4 @@
- /*
+/*
 
   sshuserfiles.c
 
@@ -25,6 +25,9 @@
 #include "sshmiscstring.h"
 #include "ssh1keydecode.h"
 
+
+
+
 #define SSH_DEBUG_MODULE "SshUserFiles"
 
 /* List of identifier strings for public key blobs. */
@@ -660,7 +663,7 @@ SshPublicKey ssh_pubkey_read(SshUser user, const char *fname, char **comment,
   unsigned char *pkeybuf;
   size_t pkeylen;
   unsigned long magic;
-  SshPublicKey pubkey;
+  SshPublicKey pubkey = NULL;
 
   pkeybuf = NULL;
   magic = ssh2_key_blob_read(user, 
@@ -905,3 +908,92 @@ char *ssh_generate_name_from_blob(char *name,
   return buf;
 }
 
+/* Return a pointer to user's ssh2 directory.
+   The directory is created if `create_if_needed' is TRUE. 
+   Return NULL on failure. The caller is responsible for freeing the returned
+   value with ssh_xfree when no longer needed. */
+
+char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed)
+{
+  char *sshdir;
+  struct stat st;
+
+  /* create the .ssh2 directory name */
+  sshdir = ssh_user_conf_dir(config, user);
+
+  if (stat(sshdir, &st) < 0)
+    {
+      if (create_if_needed)
+        {
+
+
+
+          if (mkdir(sshdir, 0755) < 0)
+
+            {
+              SSH_DEBUG(2, ("could not create user's ssh directory %s", 
+                            sshdir));
+              ssh_xfree(sshdir);
+              return NULL;
+            }  
+        }
+      else
+        {
+          ssh_xfree(sshdir);
+          return NULL;
+        }
+    }
+
+  return sshdir;
+}
+
+char *ssh_user_conf_dir(SshConfig config, SshUser user)
+{
+  /* Get template from config data */
+  const char *conf_dir = (config && config->user_conf_dir) ? 
+                         config->user_conf_dir : SSH_USER_CONFIG_DIRECTORY;
+  return ssh_user_build_dir(conf_dir, user);
+}
+
+char *ssh_user_build_dir(const char *path_template, SshUser user)
+{
+  const char *user_str;
+  char *conf_dir;
+  unsigned long x;
+  char buf[32];
+  char *tmp;
+
+  /* Replace %D with home directory */
+  user_str = ssh_user_dir(user);
+  conf_dir = ssh_replace_in_string(path_template, 
+                              "%D", (user_str != NULL) ? user_str : "");
+
+  /* Ssh_Replace %U with user login name */
+
+  user_str = ssh_user_name(user);
+
+
+
+  tmp = ssh_replace_in_string(conf_dir, 
+                              "%U", (user_str != NULL) ? user_str : "");
+  ssh_xfree(conf_dir);
+  conf_dir = tmp;
+
+  /* Ssh_Replace %IU with user ID number */
+  x = (unsigned long)ssh_user_uid(user);
+  snprintf(buf, sizeof (buf), "%lu", x);
+  tmp = ssh_replace_in_string(conf_dir, 
+                              "%IU", (user_str != NULL) ? user_str : "");
+  ssh_xfree(conf_dir);
+  conf_dir = tmp;
+
+  /* Ssh_Replace %IG with group ID number */
+  x = (unsigned long)ssh_user_gid(user);
+  snprintf(buf, sizeof (buf), "%lu", x);
+  tmp = ssh_replace_in_string(conf_dir, 
+                              "%IG", (user_str != NULL) ? user_str : "");
+  ssh_xfree(conf_dir);
+  conf_dir = tmp;
+
+  return conf_dir;
+}
