@@ -27,26 +27,45 @@
 #include "sshuser.h"
 #include "sshconfig.h"
 
-/* the name of the random seed directory under the user's home directory. */
+
+
 
 #ifndef SSH_USER_DIR
 #define SSH_USER_DIR ".ssh2"
 #endif /* SSH_USER_DIR */
 
 /* the random seed file */
-
 #ifndef SSH_RANDSEED_FILE
-#ifndef SSHDIST_WINDOWS
-#  define SSH_RANDSEED_FILE "random_seed"
-#endif /* SSHDIST_WINDOWS */
+#define SSH_RANDSEED_FILE "random_seed"
 #endif /* SSH_RANDSEED_FILE */
 
 /* the size of the random seed file (in bytes) */
-
 #ifndef SSH_RANDSEED_LEN
 #define SSH_RANDSEED_LEN 512
 #endif /* SSH_RANDSEED_LEN */
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /* Default names for pgp keyring files */
 #ifndef SSH_PGP_PUBLIC_KEY_FILE
 #define SSH_PGP_PUBLIC_KEY_FILE "pubring.pgp"
@@ -85,14 +104,21 @@
 #endif /* SSH_IDENTIFICATION_FILE */
 
 /* the standard "hostkey" file */
-#ifndef SSHDIST_WINDOWS
+
 #  ifndef SSH_HOSTKEY_FILE
 #    define SSH_HOSTKEY_FILE "hostkey"
 #  endif /* SSH_HOSTKEY_FILE */
 #  ifndef SSH_PUBLIC_HOSTKEY
 #    define SSH_PUBLIC_HOSTKEY "hostkey.pub"
 #  endif
-#endif /* SSHDIST_WINDOWS */
+
+
+
+
+
+
+
+
 /* server directory */
 
 #ifndef ETCDIR
@@ -119,6 +145,12 @@
 #define SSH_GLOBAL_KNOWNHOSTS_DIR SSH_SERVER_DIR SSH_KNOWNHOSTS_DIR
 #endif /* SSH_GLOBAL_KNOWNHOSTS_DIR */
 
+/* File, from which a possible banner message is sent to the client
+   before authentication. */
+#ifndef SSH_BANNER_MSG_FILE
+#define SSH_BANNER_MSG_FILE SSH_SERVER_DIR "/ssh_banner_message"
+#endif /* SSH_BANNER_MSG_FILE */
+
 /* global configuration file for the client */
 
 #ifndef SSH_CLIENT_GLOBAL_CONFIG_FILE
@@ -137,7 +169,6 @@
 #define SSH_SERVER_CONFIG_FILE "sshd2_config"
 #endif /* SSH_SERVER_CONFIG_FILE */
 
-
 /* Ssh-only version of /etc/hosts.equiv. */
 #ifndef SSH_HOSTS_EQUIV
 #define SSH_HOSTS_EQUIV ETCDIR "/shosts.equiv"
@@ -153,7 +184,7 @@
 #define SSH_KEY_MAGIC_SSH1_PRIVATE              0x73733102
 #define SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED    0x73733103
 
-#ifndef SSHDIST_WINDOWS
+
 /* Return a pointer to user's ssh2 directory.  The directory is
    created if `create_if_needed' is TRUE.  Return NULL on failure.
    The returned value has been allocated with ssh_xmalloc, and the
@@ -161,20 +192,22 @@
    needed. */
 char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed);
 
-/* Make sure that the random seed file exists and return a pointer to it. 
-   return NULL on failure. The file name is found from `config'. 
+/* Return a pointer to the user's random seed file name.
+   Return NULL on failure. The file name is found from `config'. 
    If `config' is NULL, use the standard SSH_RANDSEED_FILE.
 
+   'create' specifies whether an empty seed file is created if the file
+   does not exist.
+
    The caller is responsible for freeing the returned value with ssh_xfree 
    when no longer needed. */
-char *ssh_randseed_file(SshUser user, SshConfig config);
+char *ssh_randseed_file(SshUser user, SshConfig config, Boolean create);
 
 /* Get the random state from the file.  This loads and merges any data
    in the seed file into the generator. */
 
 void ssh_randseed_load(SshUser user,SshConfig config);
 
-#endif /* SSHDIST_WINDOWS */
 
 /* Reads a blob into a buffer. Return TRUE on failure.  The caller must free
    `*blob' with ssh_xfree when no longer needed. */
@@ -255,13 +288,6 @@ struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
                                               char *host, 
                                               SshConfig config);
 
-#if 0 /* XXX */
-/* definition for the ssh2 log facility. */
-#ifndef SSH_LOGFACILITY
-#define SSH_LOGFACILITY SSH_LOGFACILITY_AUTH
-#endif /* SSH_LOGFACILITY */
-#endif
-
 /* Generate a name string from any blob.  String consists of
    caller given string and space and sha1 hash of the blob in hex. 
    String is allocated with ssh_xmalloc. */
@@ -274,4 +300,9 @@ char *ssh_generate_name_from_blob(char *name,
    directory. */
 char *ssh_user_conf_dir(SshConfig config, SshUser user);
 
+/* Builds a directory name according to the dir template and user info.
+   Extracts %D, %U, %IU and %IG in the template to the values
+   specified by user. */
+char *ssh_user_build_dir(const char *path_template, SshUser user);
+
 #endif /* SSHUSERFILES_H */
