@@ -12,7 +12,7 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 */
 
 /*
- * $Id: sshuserfile.c,v 1.4 2000/05/28 16:20:27 kivinen Exp $
+ * $Id: sshuserfile.c,v 1.5 2000/09/14 23:38:17 sjl Exp $
  * $Log: sshuserfile.c,v $
  * $EndLog$
  */
@@ -142,11 +142,20 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 #include <rpc/rpc.h>
 #endif
 
-
 #if defined (__FreeBSD__) && defined(HAVE_LOGIN_CAP_H)
 #include <login_cap.h>
 #endif
 
+#ifdef HAVE_GRP_H
+#include <grp.h>
+#endif /* HAVE_GRP_H */
+
+#ifdef HAVE_INITGROUPS
+#ifndef HAVE_PROTO_INITGROUPS
+int initgroups (char *, gid_t);
+#endif /* !HAVE_PROTO_INITGROUPS */
+#endif /* HAVE_INITGROUPS */
+
 /* Protocol message types. */
 #define USERFILE_OPEN           1
 #define USERFILE_OPEN_REPLY     2
