@@ -15,7 +15,7 @@ Code for using SshBuffer routines through SshUserFile mechanism.
 */
 
 /*
- * $Id: sshuserfilebuf.c,v 1.1 1999/04/06 17:38:58 tri Exp $
+ * $Id: sshuserfilebuf.c,v 1.2 2000/06/14 00:41:23 kivinen Exp $
  * $Log: sshuserfilebuf.c,v $
  * $EndLog$
  */
@@ -44,7 +44,7 @@ size_t ssh_file_buffer_userfile_read_cb(unsigned char *buf,
 }
 
 /* Attach an userfile to a file buffer. */
-Boolean ssh_file_buffer_attach_userfile(SshFileBuffer *buf, 
+Boolean ssh_file_buffer_attach_userfile(SshFileBuffer buf, 
                                         SshUserFile userfile)
 {
   return
