@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: dlfix.h,v 1.3 1999/04/29 13:37:53 huima Exp $
+ * $Id: dlfix.h,v 1.4 2000/06/14 00:41:54 kivinen Exp $
  * $Log: dlfix.h,v $
  * $EndLog$
  */
@@ -25,6 +25,6 @@
 /* Search a parameter set of name "name". Returns TRUE if found. */
 
 Boolean ssh_dlp_set_param(const char *name, const char **outname,
-                          SshInt *p, SshInt *q, SshInt *g);
+                          SshMPInt p, SshMPInt q, SshMPInt g);
 
 #endif /* DLFIX_H */
