@@ -9,7 +9,7 @@
     */
 
 /*
- * $Id: genciph.c,v 1.53 2000/05/28 16:20:34 kivinen Exp $
+ * $Id: genciph.c,v 1.55 2000/07/26 10:02:49 mtr Exp $
  * $Log: genciph.c,v $
  * $EndLog$
  */
@@ -34,6 +34,14 @@
 #include "blowfish.h"
 #endif /* SSHDIST_CRYPT_BLOWFISH */
 
+#ifdef SSHDIST_CRYPT_CAST
+#include "cast.h"
+#endif /* SSHDIST_CRYPT_CAST */
+
+
+
+
+
 
 
 #ifdef SSHDIST_CRYPT_TWOFISH
@@ -43,6 +51,24 @@
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -59,27 +85,19 @@
 
 
 
+
+
+
+
+
+
+
+
+
 #endif /* !KERNEL */
 
 #define SSH_DEBUG_MODULE "SshGenCiph"
 
-/* Some definitions to make memory allocation more readable between
-   the user- and kernel-mode differences. Yes, when we are in usermode
-   we don't need to check the return value of allocators, but, we need
-   to do that when in kernel mode. So, we opt to lowest common
-   denominator and always check the return value */
-#ifdef _KERNEL
-#define MEM_ALLOC(S)            ssh_kmalloc(S)
-#define MEM_FREE(P)             ssh_kfree(P)
-#define MEM_REALLOC(P,O,S)      ssh_krealloc(P,O,S)
-#define MEM_STRDUP(S)           ssh_kstrdup(S)
-#else /* _KERNEL */
-#define MEM_ALLOC(S)            ssh_xmalloc(S)
-#define MEM_FREE(P)             ssh_xfree(P)
-#define MEM_REALLOC(P,O,S)      ssh_xrealloc(P,S)
-#define MEM_STRDUP(S)           ssh_xstrdup(S)
-#endif /* _KERNEL */
-
 /* Algorithm definitions */
 
 static const SshCipherDef ssh_cipher_algorithms[] =
@@ -95,6 +113,26 @@ static const SshCipherDef ssh_cipher_algorithms[] =
     ssh_des3_init_with_key_check, ssh_des3_ofb },
 #endif /* SSHDIST_CRYPT_DES */
 
+#ifdef SSHDIST_CRYPT_CAST
+  /* CAST is patented by Nortel, but they've said it can be freely used. */
+  { "cast128-ecb", 8, 16,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_ecb },
+  { "cast128-cbc", 8, 16,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_cbc },
+  { "cast128-cfb", 8, 16,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_cfb },
+  { "cast128-ofb", 8, 16,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_ofb },
+  /* CAST is patented by Nortel, but they've said it can be freely used. */
+  { "cast128-12-ecb", 8, 0,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_12_ecb },
+  { "cast128-12-cbc", 8, 0,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_12_cbc },
+  { "cast128-12-cfb", 8, 0,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_12_cfb },
+  { "cast128-12-ofb", 8, 0,
+    ssh_cast_ctxsize, ssh_cast_init, ssh_cast_init, ssh_cast_12_ofb },
+#endif /* SSHDIST_CRYPT_CAST */
 
 #ifdef SSHDIST_CRYPT_BLOWFISH
   { "blowfish-ecb", 8, 0,
@@ -123,6 +161,18 @@ static const SshCipherDef ssh_cipher_algorithms[] =
 #endif /* SSHDIST_CRYPT_DES */
 
 
+
+
+
+
+
+
+
+
+
+
+
+
 #ifdef SSHDIST_CRYPT_TWOFISH
   { "twofish-ecb", 16, 0,
     ssh_twofish_ctxsize, ssh_twofish_init, ssh_twofish_init, ssh_twofish_ecb },
@@ -140,11 +190,135 @@ static const SshCipherDef ssh_cipher_algorithms[] =
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #ifndef KERNEL
   /* The ciphers below can only be used in user-level code.  See
      the comments above for adding ciphers to the kernel. */
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #ifdef SSHDIST_CRYPT_ARCFOUR
   { "arcfour", 1, 0, ssh_arcfour_ctxsize, ssh_arcfour_init, ssh_arcfour_init,
     ssh_arcfour_transform },
@@ -153,6 +327,20 @@ static const SshCipherDef ssh_cipher_algorithms[] =
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* !KERNEL */
 
   { "none", 1, 0, NULL, NULL, NULL, ssh_none_cipher },
@@ -175,12 +363,39 @@ const struct SshCipherAliasRec ssh_cipher_aliases[] =
 #ifdef SSHDIST_CRYPT_DES
   { "3des", "3des-cbc" },
 #endif /* SSHDIST_CRYPT_DES */
+#ifdef SSHDIST_CRYPT_CAST
+  { "cast", "cast128-cbc" },
+#endif /* SSHDIST_CRYPT_CAST */
 #ifdef SSHDIST_CRYPT_BLOWFISH
   { "blowfish", "blowfish-cbc" },
 #endif /* SSHDIST_CRYPT_BLOWFISH */
+
+
+
+
+
+
 #ifdef SSHDIST_CRYPT_TWOFISH
   { "twofish", "twofish-cbc" },
 #endif /* SSHDIST_CRYPT_TWOFISH */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   { NULL, NULL }
 };
 
@@ -234,7 +449,7 @@ ssh_cipher_get_native_name(const char *name)
   if (cipher_def == NULL)
     return NULL;
 
-  return MEM_STRDUP(cipher_def->name);
+  return SSH_MEM_STRDUP(cipher_def->name);
 }
 
 /* Check if given cipher name belongs to the set of supported ciphers
@@ -291,7 +506,7 @@ ssh_cipher_get_supported_native(void)
           newsize *= 2;
 
           /* XXX: this leaks memory if in kernel */
-          if ((list = MEM_REALLOC(list, list_len, newsize)) == NULL)
+          if ((list = SSH_MEM_REALLOC(list, list_len, newsize)) == NULL)
             return NULL;
 
           list_len = newsize;
@@ -355,7 +570,7 @@ ssh_cipher_get_supported(void)
           newsize *= 2;
 
           /* XXX: This leaks memory if in kernel */
-          if ((list = MEM_REALLOC(list, list_len, newsize)) == NULL)
+          if ((list = SSH_MEM_REALLOC(list, list_len, newsize)) == NULL)
             return NULL;
 
           list_len = newsize;
@@ -431,7 +646,7 @@ ssh_cipher_allocate_internal(const char *name,
         expanded_key_len = SSH_CIPHER_MINIMAL_KEY_LENGTH;
 
       /* XXX: Is this proper error code to return on memory allocation failure? */
-      if (!(expanded_key = MEM_ALLOC(expanded_key_len)))        
+      if (!(expanded_key = SSH_MEM_ALLOC(expanded_key_len)))
         return SSH_CRYPTO_OPERATION_FAILED;
 
       ssh_hash_expand_key_internal(expanded_key, expanded_key_len,
@@ -464,9 +679,9 @@ ssh_cipher_allocate_internal(const char *name,
 
   /* Initialize the cipher. */
 
-  if (!(*cipher = MEM_ALLOC(sizeof(**cipher))))
+  if (!(*cipher = SSH_MEM_ALLOC(sizeof(**cipher))))
     {
-      MEM_FREE(expanded_key);
+      SSH_MEM_FREE(expanded_key);
 
       /* XXX: Correct return value on memory allocation failure? */
       return SSH_CRYPTO_OPERATION_FAILED;
@@ -484,12 +699,12 @@ ssh_cipher_allocate_internal(const char *name,
   if (cipher_def->ctxsize)
     {
       /* Allocate the context of the cipher. */
-      if (!((*cipher)->context = MEM_ALLOC((*cipher_def->ctxsize)())))
+      if (!((*cipher)->context = SSH_MEM_ALLOC((*cipher_def->ctxsize)())))
         {
-          MEM_FREE(*cipher);
+          SSH_MEM_FREE(*cipher);
 
           if (expand)
-            MEM_FREE(expanded_key);
+            SSH_MEM_FREE(expanded_key);
 
           /* XXX: again, correct code? */
           return SSH_CRYPTO_OPERATION_FAILED;
@@ -521,12 +736,12 @@ ssh_cipher_allocate_internal(const char *name,
 
   /* Free memory of the expanded key if necessary. */
   if (expand)
-    MEM_FREE(expanded_key);
+    SSH_MEM_FREE(expanded_key);
 
   if (rv == FALSE)
     {
-      MEM_FREE((*cipher)->context);
-      MEM_FREE(*cipher);
+      SSH_MEM_FREE((*cipher)->context);
+      SSH_MEM_FREE(*cipher);
       *cipher = NULL;
       return SSH_CRYPTO_OPERATION_FAILED;
     }
@@ -572,14 +787,14 @@ ssh_cipher_allocate_and_test_weak_keys(const char *name,
 void
 ssh_cipher_free(SshCipher cipher)
 {
-  MEM_FREE(cipher->context);
-  MEM_FREE(cipher);
+  SSH_MEM_FREE(cipher->context);
+  SSH_MEM_FREE(cipher);
 }
 
 char *
 ssh_cipher_get_name(SshCipher cipher)
 {
-  return MEM_STRDUP(cipher->ops->name);
+  return SSH_MEM_STRDUP(cipher->ops->name);
 }
 
 size_t
