@@ -1,20 +1,20 @@
 /*
 
   genmac.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Thu Jan  9 12:22:52 1997 [mkojo]
 
-  Message authentication code calculation routines. 
+  Message authentication code calculation routines.
 
   */
 
 /*
- * $Id: genmac.c,v 1.29 1999/01/13 19:30:55 ylo Exp $
+ * $Id: genmac.c,v 1.39 2000/07/26 10:02:51 mtr Exp $
  * $Log: genmac.c,v $
  * $EndLog$
  */
@@ -24,14 +24,27 @@
 #include "sshcrypti.h"
 #include "sshbuffer.h"
 
+#ifdef SSHDIST_CRYPT_SSHMACS
 #include "macs.h"
+#endif /* SSHDIST_CRYPT_SSHMACS */
+#ifdef SSHDIST_CRYPT_HMAC
 #include "hmac.h"
 
+
+
+#endif /* SSHDIST_CRYPT_HMAC */
+
+#ifdef SSHDIST_CRYPT_MD5
 #include "md5.h"
+#endif /* SSHDIST_CRYPT_MD5 */
 
+#ifdef SSHDIST_CRYPT_SHA
 #include "sha.h"
+#endif /* SSHDIST_CRYPT_SHA */
 
+#ifdef SSHDIST_CRYPT_RIPEMD160
 #include "ripemd160.h"
+#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
 #ifndef KERNEL
 /* These MACs/hashes can only be used in user-mode code.  To add a
@@ -41,12 +54,21 @@
 
 
 
+
+
+
+
+
+
 #endif /* !KERNEL */
 
 /* Control structure. */
-  
+
 static const SshMacDef ssh_mac_algorithms[] =
 {
+#ifdef SSHDIST_CRYPT_HMAC
+
+#ifdef SSHDIST_CRYPT_MD5
   { "hmac-md5", 16, FALSE,
     &ssh_hash_md5_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
@@ -57,8 +79,10 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
     ssh_hmac_96_of_buffer },
+#endif /* SSHDIST_CRYPT_MD5 */
+#ifdef SSHDIST_CRYPT_SHA
   { "hmac-sha1", 20, FALSE,
-    &ssh_hash_sha_def, 
+    &ssh_hash_sha_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_final,
     ssh_hmac_of_buffer },
@@ -67,6 +91,8 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
     ssh_hmac_96_of_buffer },
+#endif /* SSHDIST_CRYPT_SHA */
+#ifdef SSHDIST_CRYPT_RIPEMD160
   { "hmac-ripemd160", 20, FALSE,
     &ssh_hash_ripemd160_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
@@ -77,6 +103,7 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
     ssh_hmac_96_of_buffer },
+#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
 #ifndef KERNEL
   /* The macs below can only be used in user-mode code.  See comments
@@ -84,10 +111,74 @@ static const SshMacDef ssh_mac_algorithms[] =
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* !KERNEL */
 
+#endif /* SSHDIST_CRYPT_HMAC */
 
+#ifdef SSHDIST_CRYPT_SSHMACS
 
+#ifdef SSHDIST_CRYPT_SHA
   { "sha1-8", 8, TRUE,
     &ssh_hash_sha_def,
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
@@ -98,6 +189,8 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
     ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
     ssh_kdk_mac_of_buffer },
+#endif /* SSHDIST_CRYPT_SHA */
+#ifdef SSHDIST_CRYPT_MD5
   { "md5-8", 8, TRUE,
     &ssh_hash_md5_def,
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
@@ -108,6 +201,8 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
     ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
     ssh_kdk_mac_of_buffer },
+#endif /* SSHDIST_CRYPT_MD5 */
+#ifdef SSHDIST_CRYPT_RIPEMD160
   { "ripemd160-8", 8, TRUE,
     &ssh_hash_ripemd160_def,
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
@@ -118,14 +213,45 @@ static const SshMacDef ssh_mac_algorithms[] =
     ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
     ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
     ssh_kdk_mac_of_buffer },
+#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
 #ifndef KERNEL
   /* These MACs can only be used in user-mode code.  See comments
      above for more information. */
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* !KERNEL */
-  
+
+#endif /* SSHDIST_CRYPT_SSHMACS */
   { "none", 0, FALSE, NULL },
   { NULL }
 };
@@ -140,12 +266,39 @@ struct SshMacRec
 /* Returns a comma-separated list of supported mac types.  The caller
    must return the list with ssh_xfree(). */
 
-DLLEXPORT char * DLLCALLCONV
+char *
 ssh_mac_get_supported(void)
 {
   int i;
-  SshBuffer buf;
   char *list;
+  size_t offset, list_len;
+
+  list = NULL;
+  offset = list_len = 0;
+
+  for (i = 0; ssh_mac_algorithms[i].name != NULL; i++)
+    {
+      size_t newsize;
+      newsize = offset + 1 + !!offset + strlen(ssh_mac_algorithms[i].name);
+
+      if (list_len < newsize)
+        {
+          newsize *= 2;
+
+          /* XXX: This leaks memory if in kernel */
+          if ((list = SSH_MEM_REALLOC(list, list_len, newsize)) == NULL)
+            return NULL;
+
+          list_len = newsize;
+        }
+
+      offset += snprintf(list + offset, list_len - offset, "%s%s",
+                         offset ? "," : "",
+                         ssh_mac_algorithms[i].name);
+
+    }
+#if 0
+  SshBufferStruct buf;
 
   ssh_buffer_init(&buf);
   for (i = 0; ssh_mac_algorithms[i].name != NULL; i++)
@@ -158,19 +311,20 @@ ssh_mac_get_supported(void)
   ssh_buffer_append(&buf, (unsigned char *) "\0", 1);
   list = ssh_xstrdup(ssh_buffer_ptr(&buf));
   ssh_buffer_uninit(&buf);
+#endif /* _KERNEL */
   return list;
 }
 
 /* Check if given mac name belongs to the set of supported ciphers. */
 
-DLLEXPORT Boolean DLLCALLCONV
+Boolean
 ssh_mac_supported(const char *name)
 {
   unsigned int i;
 
   if (name == NULL)
     return FALSE;
-  
+
   for (i = 0; ssh_mac_algorithms[i].name != NULL; i++)
     if (strcmp(ssh_mac_algorithms[i].name, name) == 0)
       return TRUE;
@@ -179,7 +333,7 @@ ssh_mac_supported(const char *name)
 
 /* Allocate mac for use in session. */
 
-DLLEXPORT SshCryptoStatus DLLCALLCONV
+SshCryptoStatus
 ssh_mac_allocate(const char *type,
                  const unsigned char *key, size_t keylen,
                  SshMac *mac_return)
@@ -193,22 +347,30 @@ ssh_mac_allocate(const char *type,
       if (strcmp(ssh_mac_algorithms[i].name, type) == 0)
         {
           /* Found the specified mac type.  Initialize the data structure. */
-          mac = ssh_xmalloc(sizeof(*mac));
+          if (!(mac = SSH_MEM_ALLOC(sizeof(*mac))))
+            return SSH_CRYPTO_OPERATION_FAILED;
           mac->ops = &ssh_mac_algorithms[i];
           mac->ops_allocated = FALSE;
-          
+
           if (mac->ops->ctxsize)
             {
-              mac->context = ssh_xmalloc((*mac->ops->ctxsize)
-                                         (ssh_mac_algorithms[i].hash_def) +
-                                         (mac->ops->allocate_key ==
-                                          TRUE ? keylen : 0));
+              mac->context = SSH_MEM_ALLOC((*mac->ops->ctxsize)
+                                           (ssh_mac_algorithms[i].hash_def) +
+                                           (mac->ops->allocate_key == TRUE
+                                            ? keylen : 0));
+
+              if (!mac->context)
+                {
+                  SSH_MEM_FREE(mac);
+                  return SSH_CRYPTO_OPERATION_FAILED;
+                }
+
               (*mac->ops->init)(mac->context, key, keylen,
                                 ssh_mac_algorithms[i].hash_def);
             }
           else
             mac->context = NULL;
-              
+
           /* Return the MAC context. */
           *mac_return = mac;
           return SSH_CRYPTO_OK;
@@ -217,14 +379,14 @@ ssh_mac_allocate(const char *type,
   return SSH_CRYPTO_UNSUPPORTED;
 }
 
-DLLEXPORT void * DLLCALLCONV
+void *
 ssh_mac_info_derive_from_hash(SshHash hash,
                               SshMacType type)
 {
   const SshHashDef *hash_def;
   SshMacDef  *mac_def;
   char buffer[128], *tmp;
-  
+
   if (hash == NULL)
     return NULL;
   hash_def = ssh_hash_get_definition_internal(hash);
@@ -233,11 +395,13 @@ ssh_mac_info_derive_from_hash(SshHash hash,
   if (hash_def->name == NULL)
     return NULL;
 
-  mac_def = ssh_xmalloc(sizeof(SshMacDef));
+  if (!(mac_def = SSH_MEM_ALLOC(sizeof(SshMacDef))))
+    return NULL;
   mac_def->hash_def = hash_def;
 
   switch (type)
     {
+#ifdef SSHDIST_CRYPT_HMAC
     case SSH_MAC_TYPE_HMAC:
       mac_def->digest_length = hash_def->digest_length;
       mac_def->allocate_key  = FALSE;
@@ -250,11 +414,17 @@ ssh_mac_info_derive_from_hash(SshHash hash,
 
       /* Build a name for it. */
       snprintf(buffer, 128, "hmac-%s", hash_def->name);
-      tmp = ssh_xmalloc(strlen(buffer) + 1);
+      if (!(tmp = SSH_MEM_ALLOC(strlen(buffer) + 1)))
+        {
+          SSH_MEM_FREE(mac_def);
+          return NULL;
+        }
       memcpy(tmp, buffer, strlen(buffer) + 1);
       mac_def->name = tmp;
       break;
+#endif /* SSHDIST_CRYPT_HMAC */
 
+#ifdef SSHDIST_CRYPT_SSHMACS
     case SSH_MAC_TYPE_KDK:
       mac_def->allocate_key  = TRUE;
       mac_def->init          = ssh_kdk_mac_init;
@@ -265,29 +435,34 @@ ssh_mac_info_derive_from_hash(SshHash hash,
 
       /* Build a name for it. */
       snprintf(buffer, 128, "kdk-mac-%s", hash_def->name);
-      tmp = ssh_xmalloc(strlen(buffer) + 1);
+      if (!(tmp = SSH_MEM_ALLOC(strlen(buffer) + 1)))
+        {
+          SSH_MEM_FREE(mac_def);
+          return NULL;
+        }
       memcpy(tmp, buffer, strlen(buffer) + 1);
       mac_def->name = tmp;
       break;
-      
+#endif /* SSHDIST_CRYPT_SSHMACS */
+
     default:
-      ssh_xfree(mac_def);
+      SSH_MEM_FREE(mac_def);
       return NULL;
     }
   return (void *)mac_def;
 }
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_mac_info_free(void *mac_info)
 {
   SshMacDef *mac_def = mac_info;
-  ssh_xfree((char *)mac_def->name);
-  ssh_xfree(mac_info);
+  SSH_MEM_FREE((char *)mac_def->name);
+  SSH_MEM_FREE(mac_info);
 }
 
 /* Derive a mac from a hash. */
-DLLEXPORT SshMac DLLCALLCONV
-ssh_mac_allocate_with_info(const void *mac_info, 
+SshMac
+ssh_mac_allocate_with_info(const void *mac_info,
                            unsigned char *key,
                            size_t keylen)
 {
@@ -297,45 +472,52 @@ ssh_mac_allocate_with_info(const void *mac_info,
     return NULL;
 
   /* Found the specified mac type.  Initialize the data structure. */
-  mac      = ssh_xmalloc(sizeof(*mac));
+  if (!(mac = SSH_MEM_ALLOC(sizeof(*mac))))
+    return NULL;
   mac->ops = (SshMacDef *)mac_info;
   mac->ops_allocated = TRUE;
-  
+
   if (mac->ops->ctxsize)
     {
-      mac->context = ssh_xmalloc((*mac->ops->ctxsize)
-                                 (mac->ops->hash_def) +
-                                 (mac->ops->allocate_key ==
-                                  TRUE ? keylen : 0));
+      mac->context = SSH_MEM_ALLOC((*mac->ops->ctxsize)
+                                   (mac->ops->hash_def) +
+                                   (mac->ops->allocate_key == TRUE
+                                    ? keylen : 0));
+
+      if (!mac->context)
+        {
+          SSH_MEM_FREE(mac);
+          return NULL;
+        }
       (*mac->ops->init)(mac->context, key, keylen,
                         mac->ops->hash_def);
     }
   else
     {
-      ssh_xfree(mac);
+      SSH_MEM_FREE(mac);
       return NULL;
     }
-  
+
   return mac;
 }
 
 /* Free the mac. */
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_mac_free(SshMac mac)
 {
   if (mac->ops_allocated)
     {
-      ssh_xfree((char *)mac->ops->name);
-      ssh_xfree((SshMacDef *)mac->ops);
+      SSH_MEM_FREE((char *)mac->ops->name);
+      SSH_MEM_FREE((SshMacDef *)mac->ops);
     }
-  ssh_xfree(mac->context);
-  ssh_xfree(mac);
+  SSH_MEM_FREE(mac->context);
+  SSH_MEM_FREE(mac);
 }
 
 /* Get the lenght of mac digest */
 
-DLLEXPORT size_t DLLCALLCONV
+size_t
 ssh_mac_length(SshMac mac)
 {
   if (mac->ops)
@@ -346,7 +528,7 @@ ssh_mac_length(SshMac mac)
 
 /* Reset the mac to its initial state. This should be called before
    processing a new packet/message. */
-DLLEXPORT void DLLCALLCONV
+void
 ssh_mac_start(SshMac mac)
 {
   if (mac->ops)
@@ -354,7 +536,7 @@ ssh_mac_start(SshMac mac)
       (*mac->ops->start)(mac->context);
 }
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_mac_update(SshMac mac, const unsigned char *data, size_t len)
 {
   if (mac->ops)
@@ -362,7 +544,7 @@ ssh_mac_update(SshMac mac, const unsigned char *data, size_t len)
       (*mac->ops->update)(mac->context, data, len);
 }
 
-DLLEXPORT void DLLCALLCONV
+void
 ssh_mac_final(SshMac mac, unsigned char *digest)
 {
   if (mac->ops)
