@@ -21,7 +21,7 @@
   */
 
 /*
- * $Id: genmp.h,v 1.24 1999/08/26 00:59:33 kivinen Exp $
+ * $Id: genmp.h,v 1.25 2000/06/14 00:41:58 kivinen Exp $
  * $Log: genmp.h,v $
  * $EndLog$
  */
@@ -42,7 +42,7 @@ extern const unsigned int ssh_prime_table[SSH_MAX_PRIMES_IN_TABLE + 1];
 
 /* Generates a random integer of the desired number of bits. */
 
-void ssh_mp_random_integer(SshInt *ret, unsigned int bits);
+void ssh_mp_random_integer(SshMPInt ret, unsigned int bits);
 
 /* Makes and returns a random pseudo prime of the desired number of bits.
    Note that the random number generator must be initialized properly
@@ -54,20 +54,20 @@ void ssh_mp_random_integer(SshInt *ret, unsigned int bits);
    Primality is tested with Miller-Rabin test, ret thus having
    probability about 1 - 2^(-50) (or more) of being a true prime.
    */
-void ssh_mp_random_prime(SshInt *ret,unsigned int bits);
+void ssh_mp_random_prime(SshMPInt ret,unsigned int bits);
 
 /* Generate a random prime within the [min, max] interval. We observe that
    the process can just choose a random number modulo (max - min) and
    then start from there. If it goes beyond max-1 then it cycles.
    */
-void ssh_mp_random_prime_within_interval(SshInt *ret,
-                                         SshInt *min, SshInt *max);
+void ssh_mp_random_prime_within_interval(SshMPInt ret,
+                                         SshMPInt min, SshMPInt max);
 
 #if 0
 /* Find next prime from start. Useful when in need of field that contains
    element start. */
 
-void ssh_mp_next_prime(SshInt *ret, SshInt *start);
+void ssh_mp_next_prime(SshMPInt ret, SshMPInt start);
 #endif
 
 /* Similar to the ssh_mp_random_prime, except that the 'strong' pseudo
@@ -81,7 +81,7 @@ void ssh_mp_next_prime(SshInt *ret, SshInt *start);
    neccessary.
    */
 
-void ssh_mp_strong_p1363_random_prime(SshInt *prime, SshInt *div,
+void ssh_mp_strong_p1363_random_prime(SshMPInt prime, SshMPInt div,
                                       int big_bits, int small_bits);
 
 /* Generate a strong random prime, where 'prime' = 'order' * u + 1. Similar
@@ -89,53 +89,53 @@ void ssh_mp_strong_p1363_random_prime(SshInt *prime, SshInt *div,
    P1363 strong primes satisfy some other facts, but in practice these
    primes seem as good with discrete log based cryptosystems. */
 
-void ssh_mp_random_strong_prime(SshInt *prime,
-                                SshInt *order,
+void ssh_mp_random_strong_prime(SshMPInt prime,
+                                SshMPInt order,
                                 int prime_bits, int order_bits);
 
 /* Modular invert with positive results. */
 
-int ssh_mp_mod_invert(SshInt *op_dest, const SshInt *op_src,
-                      const SshInt *modulo);
+int ssh_mp_mod_invert(SshMPInt op_dest, const SshMPInt op_src,
+                      const SshMPInt modulo);
 
 /* Random number with special modulus */
 
-void ssh_mp_mod_random(SshInt *op, const SshInt *modulo);
+void ssh_mp_mod_random(SshMPInt op, const SshMPInt modulo);
 
 /* Generate a random integer with entropy at most _bits_ bits. The atmost,
    means that the actual number of bits depends whether the modulus is
    smaller in bits than the _bits_.  */
-void ssh_mp_mod_random_entropy(SshInt *op, const SshInt *modulo,
+void ssh_mp_mod_random_entropy(SshMPInt op, const SshMPInt modulo,
                                unsigned int bits);
                                
 
 #if 0
 /* Lucas functions */
 
-void ssh_mp_reduced_lucas(SshInt *op_dest, const SshInt *op_e,
-                          const SshInt *op_p, const SshInt *op_n);
+void ssh_mp_reduced_lucas(SshMPInt op_dest, const SshMPInt op_e,
+                          const SshMPInt op_p, const SshMPInt op_n);
 
-void ssh_mp_lucas(SshInt *op_dest, const SshInt *op_src1,
-                  const SshInt *op_src2,
-                  const SshInt *k, const SshInt *modulo);
+void ssh_mp_lucas(SshMPInt op_dest, const SshMPInt op_src1,
+                  const SshMPInt op_src2,
+                  const SshMPInt k, const SshMPInt modulo);
 
 /* Modular square root */
 
-int ssh_mp_mod_sqrt(SshInt *op_dest, const SshInt *op_src,
-                    const SshInt *modulo);
+int ssh_mp_mod_sqrt(SshMPInt op_dest, const SshMPInt op_src,
+                    const SshMPInt modulo);
 #endif
 
 /* Check the MOV condition, for elliptic curves */
 
-Boolean ssh_mp_mov_condition(const SshInt *b,
-                             const SshInt *q, const SshInt *r);
+Boolean ssh_mp_mov_condition(const SshMPInt b,
+                             const SshMPInt q, const SshMPInt r);
 
 /* Check whether op_src is of order op_ord mod modulo */
 
-int ssh_mp_is_order(const SshInt *op_ord, const SshInt *op_src,
-                    const SshInt *modulo);
+int ssh_mp_is_order(const SshMPInt op_ord, const SshMPInt op_src,
+                    const SshMPInt modulo);
 
 /* Find a random generator of order 'order' modulo 'modulo'. */
 
-Boolean ssh_mp_random_generator(SshInt *g, SshInt *order, SshInt *modulo);
+Boolean ssh_mp_random_generator(SshMPInt g, SshMPInt order, SshMPInt modulo);
 #endif /* GENMP_H */
