@@ -10,7 +10,7 @@ Copyright (C) 1996-2000 SSH Security Communications Oy, Espoo, Finland
 */
 
 /*
- * $Id: genrand.c,v 1.18 2000/03/10 18:34:12 mkojo Exp $
+ * $Id: genrand.c,v 1.21 2000/09/14 10:33:53 dima Exp $
  * $Log: genrand.c,v $
  * $EndLog$
  */
@@ -80,6 +80,9 @@ void ssh_random_acquire_light_environmental_noise(void)
 
   state = ssh_random_get_global();
 
+#ifdef VXWORKS
+  /* XXX - where to get noise from? */
+#else /* VXWORKS */
 #if !defined(WINDOWS) && !defined(DOS) && !defined(macintosh)
   {
     int f;
@@ -178,6 +181,7 @@ void ssh_random_acquire_light_environmental_noise(void)
 #if defined(CLK_TCK) && !defined(WINDOWS) && !defined(DOS)
   ssh_random_xor_noise(14, (SshUInt32)(CLK_TCK << 16));
 #endif /* CLK_TCK && !WINDOWS */
+#endif /* VXWORKS */
 }
 
 /* Mixes the bytes from the buffer into the pool.  The pool should be stirred
