@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: namelist.c,v 1.21 2000/05/28 16:20:37 kivinen Exp $
+ * $Id: namelist.c,v 1.22 2000/06/14 00:42:00 kivinen Exp $
  * $Log: namelist.c,v $
  * $EndLog$
  */
@@ -607,7 +607,7 @@ char *ssh_ntree_transform_list_to_tree(char *namelist,
 {
   SshNameTree tree, list;
   SshNameNode node, temp, level, tmp;
-  SshDStack *stack;
+  SshDStack stack;
   char *ret;
   
   /* We assume that the given parameter is actually a list, but then you
@@ -727,7 +727,7 @@ char *ssh_ntree_transform_tree_to_list(char *nametree)
   SshBufferStruct buffer;
   SshNameTree tree;
   SshNameNode node, temp;
-  SshDStack *stack = NULL;
+  SshDStack stack = NULL;
   unsigned int count;
   char *ret;
   
