@@ -34,8 +34,13 @@
 size_t ssh_encode_pubkeyblob(SshPublicKey pubkey, unsigned char **blob)
 {
 #ifdef SSHDIST_CRYPT_DSA
-  SshIntC p, q, g, y;  /* DSS public parameters */
+  SshMPIntC p, q, g, y;  /* DSS public parameters */
 #endif /* SSHDIST_CRYPT_DSA */
+
+
+
+
+
   SshBuffer buf;
   size_t len;
   char *keytype;
@@ -120,6 +125,61 @@ size_t ssh_encode_pubkeyblob(SshPublicKey pubkey, unsigned char **blob)
 #endif /* SSHDIST_CRYPT_DSA */
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   ssh_debug("ssh_encode_pubkeyblob: unrecognized key type %s", keytype);
   ssh_xfree(keytype);
   return 0;
@@ -133,8 +193,13 @@ SshPublicKey ssh_decode_pubkeyblob(const unsigned char *blob, size_t bloblen)
   unsigned char *keytype;
   SshPublicKey pubkey;
 #ifdef SSHDIST_CRYPT_DSA
-  SshIntC p, q, g, y;  /* DSS public parameters */
+  SshMPIntC p, q, g, y;  /* DSS public parameters */
 #endif /* SSHDIST_CRYPT_DSA */
+
+
+
+
+
   SshCryptoStatus code;
   SshBuffer buf;
 
@@ -210,6 +275,50 @@ SshPublicKey ssh_decode_pubkeyblob(const unsigned char *blob, size_t bloblen)
 #endif /* SSHDIST_CRYPT_DSA */
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   /* could not identify key type */
 
   ssh_debug("ssh_decode_pubkeyblob: unrecognized key type %s", 
