@@ -619,10 +619,10 @@ ssh_rgf_hash_free(SshRGFHash hash);
    is very special behaviour and should be performed only when all the
    power of the interface is to be used.
 
-   Note: most input will be given in SshInt's because it is reasonably
+   Note: most input will be given in SshMPInt's because it is reasonably
          convenient way of handling large strings of bits. However,
          not all values are integers, nor natural numbers. You should
-         not assume that any SshInt you get can be used for reasonable
+         not assume that any SshMPInt you get can be used for reasonable
          computations without knowledge of the meaning of the actual
          value.
    */
@@ -786,13 +786,13 @@ typedef enum
 
        dl-modp:
 
-          generation:  SshInt *
-          reading:     SshInt *
+          generation:  SshMPInt
+          reading:     SshMPInt
 
        ec-modp:
 
-          generation:  SshInt *, SshInt *
-          reading:     SshInt *, SshInt *
+          generation:  SshMPInt, SshMPInt
+          reading:     SshMPInt, SshMPInt
 
           The comma denotes that we mean a pair or values. E.g. the first
           one is the x co-ordinate and the second y co-ordinate. We
@@ -800,8 +800,8 @@ typedef enum
 
        ec-gf2n:
 
-          generation: SshInt *, SshInt *
-          reading:    SshInt *, SshInt *
+          generation: SshMPInt, SshMPInt
+          reading:    SshMPInt, SshMPInt
 
      */
 
@@ -811,18 +811,18 @@ typedef enum
 
        dl-modp:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt
+         reading:      SshMPInt
 
        ec-modp:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
        ec-gf2n:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
      */
 
@@ -836,26 +836,26 @@ typedef enum
 
        if-modn:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
      For the discrete logarithm problem based methods this identifier
      means the integer field modulus.
 
        dl-modp:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
        ec-modp:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
        ec-gf2n:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
      */
 
@@ -868,8 +868,8 @@ typedef enum
 
        ec-gf2n:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
     */
 
@@ -883,26 +883,26 @@ typedef enum
 
        if-modn:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
      For the discrete logarithm problem bases system this identifier
      means the order of the group in which computation occurs.
 
        dl-modp:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
        ec-modp:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
        ec-gf2n:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
      */
 
@@ -912,13 +912,13 @@ typedef enum
 
        dl-modp:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
        ec-modp:
 
-         generation:   SshInt *, SshInt *
-         reading:      SshInt *, SshInt *
+         generation:   SshMPInt , SshMPInt 
+         reading:      SshMPInt , SshMPInt 
 
        Here we denote by comma the fact that you are supposed to give
        two values, a pair, as input. The first component will be x, and
@@ -926,8 +926,8 @@ typedef enum
 
        ec-gf2n:
 
-         generation:   SshInt *, SshInt *
-         reading:      SshInt *, SshInt *
+         generation:   SshMPInt , SshMPInt 
+         reading:      SshMPInt , SshMPInt 
 
 
      In general generator is a value which generates the set of numbers
@@ -943,8 +943,8 @@ typedef enum
 
        if-modn:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
      Currently implemented is the RSA which used n = pq and e = d^1 (mod n).
 
@@ -957,8 +957,8 @@ typedef enum
 
        if-modn:
 
-         generation:    SshInt *
-         reading:       SshInt *
+         generation:    SshMPInt 
+         reading:       SshMPInt 
 
        In context of RSA this value will set the public exponent
        explicitly to some value or make sure that the value set is the
@@ -974,8 +974,8 @@ typedef enum
 
        if-modn:
 
-         generation:     SshInt *
-         reading:        SshInt *
+         generation:     SshMPInt 
+         reading:        SshMPInt 
 
        In RSA this value has priority over public exponent. Given this
        value and primes p and q, all the RSA parameters can be
@@ -988,8 +988,8 @@ typedef enum
 
        if-modn:
 
-         generation:     SshInt *
-         reading:        SshInt *
+         generation:     SshMPInt 
+         reading:        SshMPInt 
 
        This value can be excluded because it is automatically computed
        within the parameter making utility. Although, if given among all
@@ -1020,13 +1020,13 @@ typedef enum
 
   /*   ec-modp:
 
-         generation:       SshInt *
-         reading:          SshInt *
+         generation:       SshMPInt 
+         reading:          SshMPInt 
 
        ec-gf2n:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
        The number of points that lie of elliptic curve your are defining,
        or reading. This is not important for the working of the system,
@@ -1038,13 +1038,13 @@ typedef enum
 
   /*   ec-modp:
 
-         generation:       SshInt *
-         reading:          SshInt *
+         generation:       SshMPInt 
+         reading:          SshMPInt 
 
        ec-gf2n:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt 
 
        The another number which is used to define the elliptic curve.
 
@@ -1052,13 +1052,13 @@ typedef enum
   SSH_PKF_CURVE_A,
   /*   ec-modp:
 
-         generation:       SshInt *
-         reading:          SshInt *
+         generation:       SshMPInt 
+         reading:          SshMPInt 
 
        ec-gf2n:
 
-         generation:   SshInt *
-         reading:      SshInt *
+         generation:   SshMPInt 
+         reading:      SshMPInt
 
        The another number which is used to define the elliptic curve.
 
@@ -2015,4 +2015,141 @@ ssh_pk_group_unified_diffie_hellman_agree_async(SshPublicKey public_key,
                                                 void *context);
 #endif /* SSHDIST_CRYPT_GENPKCS */
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* SSHCRYPT_H */
