@@ -1,10 +1,10 @@
 /*
 
   t-cryptest.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1996,1997 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996,1997, 2000 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Wed Dec  3 16:05:36 1997 [mkojo]
@@ -15,18 +15,15 @@
   */
 
 /*
- * $Id: t-cryptest.c,v 1.7 1999/04/29 13:38:22 huima Exp $
+ * $Id: t-cryptest.c,v 1.13 2000/11/27 09:43:16 tmo Exp $
  * $Log: t-cryptest.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshcrypt.h"
+#include "sshmp.h"
 #include "sshpsystem.h"
-#include "timeit.h"
-#include "sshmp.h" /* was "gmp.h" */
-
-#include "testsrcpath.h"
 
 /* The PSystem definition. */
 
@@ -38,8 +35,8 @@ enum {
   TCR_RANDOM,
   TCR_PKCS,
   TCR_SECSHA,
-  TCR_TIMING, 
-  
+  TCR_TIMING,
+
   /* Variables. */
   TCR_NAME,
   TCR_KEY,
@@ -62,7 +59,6 @@ typedef struct
 typedef struct
 {
   FILE *fp;
-  SshRandomState state;
 } Passed;
 
 void fhexdump(FILE *fp, char *str, unsigned char *cp, size_t len)
@@ -125,7 +121,7 @@ typedef struct
 /* Public Key */
 typedef struct
 {
-  int dummy;  
+  int dummy;
 } PublicKeyTest;
 
 /* Secret Sharing */
@@ -136,7 +132,7 @@ typedef struct
 
 /* Definition of TimingInfo */
 
-SshPSystemVar var_timing_info[] =
+SshPSystemVarStruct var_timing_info[] =
 {
   { "Loops", TCR_LOOPS, SSH_PSYSTEM_INTEGER },
   { NULL },
@@ -166,7 +162,7 @@ SSH_PSYSTEM_HANDLER(timing_info)
       switch (aptype)
         {
         case TCR_LOOPS:
-          t->loops = ssh_mp_get_ui((SshInt*)data);
+          t->loops = ssh_mp_get_ui((SshMPInt)data);
           return TRUE;
         default:
           break;
@@ -178,7 +174,7 @@ SSH_PSYSTEM_HANDLER(timing_info)
   return FALSE;
 }
 
-SshPSystemEnv env_common[] =
+SshPSystemEnvStruct env_common[] =
 {
   { "TimingInfo", TCR_TIMING,
     timing_info_handler,
@@ -188,7 +184,7 @@ SshPSystemEnv env_common[] =
 
 /* The Hash case. */
 
-SshPSystemVar var_hash_test[] =
+SshPSystemVarStruct var_hash_test[] =
 {
   { "Name",   TCR_NAME,   SSH_PSYSTEM_NAME },
   { "Data",   TCR_DATA,   SSH_PSYSTEM_STRING },
@@ -212,7 +208,7 @@ Boolean hash_test(void *t)
   unsigned char *buf;
   size_t len;
   Boolean rv = FALSE;
-  
+
   if (ssh_hash_allocate(h->name, &hash) != SSH_CRYPTO_OK)
     {
       fprintf(h->s->fp, "hash_test: hash %s is not supported\n",
@@ -229,7 +225,7 @@ Boolean hash_test(void *t)
       ssh_hash_free(hash);
       goto failed;
     }
-  
+
   ssh_hash_reset(hash);
   ssh_hash_update(hash, h->data, h->data_len);
   buf = ssh_xmalloc(len);
@@ -237,7 +233,8 @@ Boolean hash_test(void *t)
 
   if (memcmp(buf, h->digest, len) != 0)
     {
-      fprintf(h->s->fp, "hash_test: digest did not match!\n");
+      fprintf(h->s->fp, "hash_test: digest did not match! (%s)\n",
+              h->name);
       fhexdump(h->s->fp, "computed", buf, len);
       fhexdump(h->s->fp, " correct", h->digest, len);
       ssh_hash_free(hash);
@@ -284,14 +281,17 @@ SSH_PSYSTEM_HANDLER(hash_test)
         {
         case TCR_NAME:
           h->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DATA:
           h->data = data;
           h->data_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DIGEST:
           h->digest = data;
           h->digest_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_TIMING:
           h->t = data;
@@ -306,7 +306,7 @@ SSH_PSYSTEM_HANDLER(hash_test)
   return FALSE;
 }
 
-SshPSystemVar var_mac_test[] =
+SshPSystemVarStruct var_mac_test[] =
 {
   { "Name", TCR_NAME, SSH_PSYSTEM_NAME },
   { "Key",  TCR_KEY,  SSH_PSYSTEM_STRING },
@@ -332,7 +332,7 @@ Boolean mac_test(void *t)
   Boolean rv = FALSE;
   unsigned char *buf;
   size_t len;
-  
+
   if (ssh_mac_allocate(h->name, h->key, h->key_len, &mac) != SSH_CRYPTO_OK)
     {
       fprintf(h->s->fp, "mac_test: mac %s not supported.\n",
@@ -348,17 +348,18 @@ Boolean mac_test(void *t)
       ssh_mac_free(mac);
       goto failed;
     }
-  
+
   ssh_mac_start(mac);
   ssh_mac_update(mac, h->data, h->data_len);
 
   buf = ssh_xmalloc(len);
-  
+
   ssh_mac_final(mac, buf);
 
   if (memcmp(buf, h->digest, len) != 0)
     {
-      fprintf(h->s->fp, "mac_test: digests did not match!\n");
+      fprintf(h->s->fp, "mac_test: digests did not match! (%s)\n",
+              h->name);
       fhexdump(h->s->fp, "computed", buf, len);
       fhexdump(h->s->fp, " correct", h->digest, len);
       ssh_mac_free(mac);
@@ -372,7 +373,7 @@ Boolean mac_test(void *t)
 
   ssh_mac_free(mac);
   rv = TRUE;
-  
+
 failed:
   mac_test_free(t);
   return rv;
@@ -407,18 +408,22 @@ SSH_PSYSTEM_HANDLER(mac_test)
         {
         case TCR_NAME:
           h->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_KEY:
           h->key = data;
           h->key_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DATA:
           h->data = data;
           h->data_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_DIGEST:
           h->digest = data;
           h->digest_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_TIMING:
           h->t = data;
@@ -430,10 +435,10 @@ SSH_PSYSTEM_HANDLER(mac_test)
     default:
       break;
     }
-  return FALSE;  
+  return FALSE;
 }
 
-SshPSystemVar var_cipher_test[] =
+SshPSystemVarStruct var_cipher_test[] =
 {
   { "Name",   TCR_NAME,   SSH_PSYSTEM_NAME },
   { "Key",    TCR_KEY,    SSH_PSYSTEM_STRING },
@@ -476,15 +481,25 @@ Boolean cipher_test(void *t)
     {
       if (c->iv)
         {
-          if (len != c->input_len || len != c->output_len || len != c->iv_len)
+          if (len != c->input_len || len != c->output_len)
             {
               fprintf(c->s->fp,
-                      "cipher_test: input = %u output = %u iv = %u\n"
+                      "cipher_test: cipher '%s'\n"
+                      "             input = %u output = %u iv = %u\n"
                       "             assumed all to be %u bytes.\n",
+                      c->name,
                       c->input_len, c->output_len, c->iv_len, len);
               ssh_cipher_free(cipher);
               goto failed;
             }
+          if (c->iv_len < c->input_len)
+            {
+              unsigned char *tmp;
+              tmp = ssh_xcalloc(1, c->input_len);
+              memcpy(tmp, c->iv, c->iv_len);
+              ssh_xfree(c->iv);
+              c->iv = tmp;
+            }
           ssh_cipher_set_iv(cipher, c->iv);
         }
       else
@@ -492,9 +507,10 @@ Boolean cipher_test(void *t)
           if (len != c->input_len || len != c->output_len)
             {
               fprintf(c->s->fp,
-                      "cipher_test: input = %u output = %u\n"
+                      "cipher_test: cipher '%s'\n"
+                      "             input = %u output = %u\n"
                       "             assumed both to be %u bytes.\n",
-                      c->input_len, c->output_len, len);
+                      c->name, c->input_len, c->output_len, len);
               ssh_cipher_free(cipher);
               goto failed;
             }
@@ -513,7 +529,7 @@ Boolean cipher_test(void *t)
 
   if (memcmp(buf, c->output, len) != 0)
     {
-      fprintf(c->s->fp, "cipher_test: matching failed!\n");
+      fprintf(c->s->fp, "cipher_test: matching failed! (%s)\n", c->name);
       fhexdump(c->s->fp, "computed", buf, len);
       fhexdump(c->s->fp, " correct", c->output, len);
       ssh_cipher_free(cipher);
@@ -563,22 +579,27 @@ SSH_PSYSTEM_HANDLER(cipher_test)
         {
         case TCR_NAME:
           c->name = data;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_KEY:
           c->key = data;
           c->key_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_IV:
           c->iv = data;
           c->iv_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_INPUT:
           c->input = data;
           c->input_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         case TCR_OUTPUT:
           c->output = data;
           c->output_len = data_len;
+          SSH_PSYSTEM_DATA_TAKEN;
           return TRUE;
         default:
           break;
@@ -590,7 +611,7 @@ SSH_PSYSTEM_HANDLER(cipher_test)
   return FALSE;
 }
 
-SshPSystemEnv env_root[] =
+SshPSystemEnvStruct env_root[] =
 {
   { "HashFunctionTest", TCR_HASH,
     hash_test_handler,
@@ -640,9 +661,9 @@ SSH_PSYSTEM_HANDLER(root)
       break;
     }
   return FALSE;
-}     
+}
 
-SshPSystemEnv root[] =
+SshPSystemEnvStruct root[] =
 {
   { "root", 0,
     root_handler,
@@ -654,7 +675,7 @@ int my_more(void *context, unsigned char **buf, size_t *buf_len)
 {
   unsigned char *tmp = ssh_xmalloc(256);
   size_t bytes;
-  
+
   bytes = fread(tmp, 1, 256, (FILE *)context);
   if (bytes == 0)
     {
@@ -670,32 +691,33 @@ int main(int ac, char *av[])
 {
   FILE *fp;
   void *ret;
-  SshPSystemDef def;  
+  SshPSystemDefStruct def;
   Passed passed;
-  SshPSystemError error;
-  char *default_str = "./cryptest.desc";
-  char *default_str_2 = TEST_SRC_PATH "/cryptest.desc";
-  
-  if (ac > 1) 
+  SshPSystemErrorStruct error;
+  char default_str[128], *srcpath;
+
+  if ((srcpath = getenv("srcdir")) == NULL)
+    {
+      ssh_warning("$srcdir not set, defaulting script file location to `.'");
+      srcpath = ".";
+    }
+  snprintf(default_str, sizeof(default_str), "%s/cryptest.desc", srcpath);
+
+  if (ac > 1)
     {
       fp = fopen(av[1], "r");
     }
   else
     {
       fp = fopen(default_str, "r");
-      if (fp == NULL)
-        {
-          fp = fopen(default_str_2, "r");
-        }
     }
   if (fp == NULL)
     {
       fprintf(stderr, "no cryptest.desc to read from.\n");
       exit(1);
     }
-  
+
   passed.fp = stderr;
-  passed.state = ssh_random_allocate();
   def.root = root;
   def.feeding = &passed;
   def.more_context = fp;
@@ -703,7 +725,6 @@ int main(int ac, char *av[])
   def.assign_operator = "=";
   ret = ssh_psystem_parse(&def, &error);
   fclose(fp);
-  ssh_random_free(passed.state);
   if (error.status == SSH_PSYSTEM_OK)
     return 0;
 
