@@ -8,7 +8,7 @@
   Copyright (c) 1998 SSH Communications Security, Finland
                      All rights reserved
 
-  Generic file transfer module, client side.
+  Generic file transfer module, server side.
 
 */
 
@@ -18,35 +18,46 @@
 #include "sshfilexfer.h"
 #include "sshfilexferi.h"
 
-#ifdef WIN32
-#include <windows.h>
-#include <direct.h>
-#include <sys/utime.h>
 
-#define rmdir _rmdir
 
-/* There is no lstat on Windows; just use stat. */
-#define lstat stat
-#define HAVE_LSTAT
-#define HAVE_UTIME
 
-#define S_IFLNK 0
-#define S_IXUSR 0
-#define S_IXGRP 0
-#define S_IXOTH 0
-#define S_IFBLK 0
-#define S_IFIFO 0
-#define S_IWUSR 0
-#define S_IRUSR 0
-#define S_ISUID 0
-#define S_IRGRP 0
-#define S_IWGRP 0
-#define S_ISGID 0
-#define S_IWGRP 0
-#define S_ISGID 0
-#define S_IROTH 0
-#define S_IWOTH 0
-#endif /* WIN32 */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+#define SSH_DEBUG_MODULE "Ssh2SftpSrv"
+
 
 /* Data structure for a server-side file handle.  This definition is
    private to the implementation. */
@@ -73,16 +84,19 @@ typedef struct SshServerHandleRec
   /* This is the name of the file / directory */  
   char *name;
 
-#ifdef WIN32
-  /* Handle to a directory listing, as returned by FindFirstFile. */
-  HANDLE dir;
-  WIN32_FIND_DATA dirdata;
-  Boolean dirdata_valid;
-#else /* WIN32 */
+
+
+
+
+
+
+
+
+
   /* If the handle refers to a directory, this is a directory pointer for
      the directory as returned by opendir. */
   DIR *dir;
-#endif /* WIN32 */
+
 } *SshServerHandle;
 
 /* Data structure for the file transfer server. */
@@ -96,6 +110,7 @@ struct SshFileServerRec
   SshServerHandle handles;
 };
 
+
 /* Create a new file handle and add it to the list of handles.  This
    returns the new handle. */
 
@@ -123,6 +138,7 @@ SshServerHandle ssh_file_server_new_handle(SshFileServer server,
 #ifdef WIN32
       handle->dir = (HANDLE)fd;
       handle->dirdata_valid = FALSE;
+      handle->root_drives = NULL;
 #else /* WIN32 */
       handle->dir = (DIR *)fd;
 #endif /* WIN32 */
@@ -177,10 +193,14 @@ void ssh_file_server_free_handle(SshFileServer server, SshServerHandle handle)
         *handlep = handle->next;
         ssh_xfree(handle->value);
         ssh_xfree(handle->name);
+#ifdef WIN32
+        if (handle->is_directory && handle->root_drives)
+          ssh_xfree(handle->root_drives);
+#endif /* WIN32 */
         ssh_xfree(handle);
         return;
       }
-          
+
   ssh_warning("ssh_file_server_free_handle: handle not found");
 }
 
@@ -257,15 +277,15 @@ void ssh_file_server_receive_proc(SshPacketType type,
   int fd;
   SshFileClientError error;
   struct stat st;
-#ifdef WIN32
-  HANDLE dir;
-  WIN32_FIND_DATA dirdata;
-  int  lenstr;
-  char dirname[MAXPATHLEN];
-#else /* WIN32 */
+
+
+
+
+
+
   DIR *dir;
   struct dirent *dp;
-#endif /* WIN32 */
+
   const char *dirent_name;
   SshBufferStruct buffer;
   int i;
@@ -294,7 +314,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
   char   group_name[32];  
   char   date_string[32];
   char   name_ext[128];
-  char   long_name[256];
+  char   long_name[MAXPATHLEN];
   static const char *month_name[12] = 
   {
     "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
@@ -386,6 +406,15 @@ void ssh_file_server_receive_proc(SshPacketType type,
       flags |= _O_BINARY;
 #endif /* WIN32 */
 
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_OPEN %s", name));
+
+
+
+
+
+
+
       /* Try to open the file. */
       fd = open(name, flags,
                 (attrs->flags & SSH_FILEXFER_ATTR_PERMISSIONS) ?
@@ -424,6 +453,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
       break;
 
     case SSH_FXP_CLOSE:
+      SSH_DEBUG(3, ("SSH_FXP_CLOSE"));
       /* Parse the CLOSE message. */
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
@@ -448,7 +478,10 @@ void ssh_file_server_receive_proc(SshPacketType type,
          return an error e.g. on NFS file systems. */
       if (handle->is_directory)
 #ifdef WIN32
-        ret = FindClose(handle->dir) ? 0 : -1;
+        if (handle->root_drives)
+          ret = 0;
+        else
+          ret = FindClose(handle->dir) ? 0 : -1;
 #else /* WIN32 */
         ret = closedir(handle->dir);
 #endif /* WIN32 */
@@ -480,6 +513,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
           goto return_bad_status;
         }
 
+      SSH_DEBUG(6, ("SSH_FXP_READ: len %d, offset %d", iolen, offset));
       /* Look up the handle.  If not found, return error status. */
       handle = ssh_file_server_find_handle(server, value, valuelen);
       if (!handle || handle->is_directory)
@@ -536,6 +570,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
           goto return_bad_status;
         }
 
+      SSH_DEBUG(6, ("SSH_FXP_WRITE: len %d, offset %d", iodatalen, offset));
       /* Look up the handle.  If not found, return error status. */
       handle = ssh_file_server_find_handle(server, value, valuelen);
       if (!handle || handle->is_directory)
@@ -568,9 +603,12 @@ void ssh_file_server_receive_proc(SshPacketType type,
           goto return_bad_status;
         }
 
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_STAT %s", name));
+
       /* Stat the given file. */
 #ifdef WIN32
-      strcpy(dirname, name);
+      strncpy(dirname, name, sizeof(dirname));
       len = strlen(dirname);
       if (dirname[len-1] == ':')
       {
@@ -603,17 +641,24 @@ void ssh_file_server_receive_proc(SshPacketType type,
       attrs->flags |= SSH_FILEXFER_ATTR_SIZE;
       attrs->size = st.st_size;
       
+#ifndef WIN32
       /* Owner... */
       attrs->flags |= SSH_FILEXFER_ATTR_UIDGID;
       attrs->uid = st.st_uid;
       attrs->gid = st.st_gid;
-      
+#endif /* WIN32 */
+
       /* Mode flags... */
       /* XXX modify these to use a common, internal format for
          sshfilexfer. Modify client end to recognize these.*/
       attrs->flags |= SSH_FILEXFER_ATTR_PERMISSIONS;
+#ifdef WIN32
+      /* By default, no permission are given for group or other. */
+      attrs->permissions = st.st_mode & 0xFFFFFFC0;
+#else
       attrs->permissions = st.st_mode;
-      
+#endif /* WIN32 */
+
       /* Access and modify time flags... */
       attrs->flags |= SSH_FILEXFER_ATTR_ACMODTIME;
       attrs->atime = (SshTime)st.st_atime;
@@ -635,6 +680,8 @@ void ssh_file_server_receive_proc(SshPacketType type,
           goto return_bad_status;
         }
 
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_LSTAT %s", name));
 #ifdef HAVE_LSTAT
       /* Stat the given file. */
       if (lstat(name, &st) < 0)
@@ -658,6 +705,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
 #endif /* HAVE_LSTAT */
 
     case SSH_FXP_FSTAT:
+      SSH_DEBUG(3, ("SSH_FXP_FSTAT"));
       /* Parse the FSTAT message. */
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
@@ -698,6 +746,8 @@ void ssh_file_server_receive_proc(SshPacketType type,
                                SSH_FORMAT_UINT32, &id,
                                SSH_FORMAT_UINT32_STR, &name, NULL,
                                SSH_FORMAT_END);
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_SETSTAT %s", name));
       if (bytes)
         {
           ssh_xfree(attrs);
@@ -786,6 +836,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
       break;
       
     case SSH_FXP_FSETSTAT:
+      SSH_DEBUG(3, ("SSH_FXP_FSETSTAT"));
       /* Parse the FSETSTAT message. */
       bytes = ssh_decode_array(data, len,
                                SSH_FORMAT_UINT32, &id,
@@ -899,39 +950,63 @@ void ssh_file_server_receive_proc(SshPacketType type,
           ssh_warning("ssh_file_server_receive_proc: bad OPENDIR");
           break;
         }
+      SSH_DEBUG(6, ("SSH_FXP_OPENDIR undecoded %s", name));
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_OPENDIR %s", name));
 
       /* Open the specified directory. */
-#ifdef WIN32
-      /* We have a dir name to search for. For the FindFirstFile(), the dirname
-         must contain wildcard "*.*" to search all the files in the directory. */
-      dirname[0] = 0;
-      lenstr = strlen(name);
-      if (lenstr <= 0)
-      {
-        /* we shouldn't really come here - this is not guaranteed to work */
-        strcpy(dirname, "*.*");
-      }
-      else
-      {
-        strcpy(dirname, name);
-        if (dirname[lenstr-1] != '/' && dirname[lenstr-1] != '\\')
-          strcat(dirname, "/");
 
-        strcat(dirname, "*.*");
-      }
 
-      dir = FindFirstFile(dirname, &dirdata);
-#else /* WIN32 */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
       dir = opendir(strlen(name) == 0 ? "." : name);
-#endif /* WIN32 */
-      
+
+
       /* Send error to the client if opening the directory failed. */
 #ifdef WIN32
       if (dir == INVALID_HANDLE_VALUE)
+        {
+          if (GetLastError() == ERROR_ACCESS_DENIED)
+            errno = EPERM;
+          else
+            errno = -1;
 #else /* WIN32 */
       if (!dir)
-#endif /* WIN32 */
         {
+#endif /* WIN32 */
           ssh_file_server_send_status(server, id,
                                       ssh_file_server_errno_to_error(errno));
           break;
@@ -941,14 +1016,25 @@ void ssh_file_server_receive_proc(SshPacketType type,
          handle object. */
       handle = ssh_file_server_new_handle(server, TRUE, name, (void *)dir);
 
-#ifdef WIN32
-      /* Save the data for the first found file. */
-      handle->dirdata = dirdata;
-      handle->dirdata_valid = TRUE;
-#endif /* WIN32 */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
       /* Free the directory name. */
-      ssh_xfree(name);
+      if (name)
+        ssh_xfree(name);
       
       /* Send a handle message to the client. */
       ssh_file_server_send(server, SSH_FXP_HANDLE, NULL,
@@ -958,6 +1044,7 @@ void ssh_file_server_receive_proc(SshPacketType type,
       break;
       
     case SSH_FXP_READDIR:
+      SSH_DEBUG(3, ("SSH_FXP_READDIR"));
       /* Parse the READDIR message. */
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
@@ -988,35 +1075,52 @@ void ssh_file_server_receive_proc(SshPacketType type,
       ssh_buffer_init(&buffer);
       for (i = 0; i < 100; i++)
         {
-#ifdef WIN32
-          if (handle->dirdata_valid)
-            {
-              dirdata = handle->dirdata;
-              handle->dirdata_valid = FALSE;
-              dirent_name = dirdata.cFileName;
-            }
-          else
-            {
-              if (FindNextFile(handle->dir, &dirdata))
-                dirent_name = dirdata.cFileName;
-              else
-                break;
-            }
-#else /* WIN32 */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
           dp = readdir(handle->dir);
           if (!dp)
             break;
           dirent_name = dp->d_name;
-#endif /* WIN32 */
-          
-#ifndef NO_LONG_NAMES    
 
-          if (handle->name == NULL || strlen(handle->name) == 0)          
-            strncpy(long_name, dirent_name, sizeof(long_name));
-          else      
-            snprintf(long_name, sizeof(long_name), "%s/%s", 
-                     handle->name, dirent_name);
           
+#ifndef NO_LONG_NAMES
+
+          if (handle->name == NULL || strlen(handle->name) == 0)
+            strncpy(long_name, dirent_name, sizeof(long_name));
+          else
+          {
+            if (strlen(handle->name) == 1)
+              snprintf(long_name, sizeof(long_name), "/%s",
+                       dirent_name);
+            else
+              snprintf(long_name, sizeof(long_name), "%s/%s",
+                       handle->name, dirent_name);
+          }
+
 #ifdef HAVE_LSTAT
           if (lstat(long_name, &st))
             goto no_long_name;
@@ -1039,7 +1143,12 @@ void ssh_file_server_receive_proc(SshPacketType type,
 
           /* Mode flags... */
           attrs->flags |= SSH_FILEXFER_ATTR_PERMISSIONS;
+#ifdef WIN32
+         /* By default, no permission are given for group or other. */
+          attrs->permissions = st.st_mode & 0xFFFFFFC0; /*0xFFFFFFE4;*/
+#else
           attrs->permissions = st.st_mode;
+#endif /* WIN32 */
 
           /* Access and modify time flags... */
           attrs->flags |= SSH_FILEXFER_ATTR_ACMODTIME;
@@ -1070,9 +1179,6 @@ void ssh_file_server_receive_proc(SshPacketType type,
           snprintf(group_name, sizeof(group_name), "%d", (int) st.st_gid);
 #endif /* HAVE_GETGRGID */
 
-          /*
-           * tm = localtime(&st.st_mtimespec.tv_sec);
-           */
           ssh_calendar_time(st.st_mtime, tm, TRUE);
           
           /* Print time if modified this year, otherwise print year */
@@ -1100,8 +1206,17 @@ void ssh_file_server_receive_proc(SshPacketType type,
                            sizeof(name_ext) - 4) == -1)
                 strncpy(&name_ext[4], "???", sizeof(name_ext) - 4);
             }
+#else
+          if (!handle->root_drives && (st.st_mode & S_IFMT) == S_IFDIR)
+          {
+            /* It's a directory. Pass all directory names in lowercase,
+               except drive names. */
+            strncpy(dirname, dirent_name, sizeof(dirname));
+            strlwr(dirname);
+            dirent_name = dirname;
+          }
 #endif /* WIN32 */
-                                
+
           if ((st.st_mode & S_IFMT) == S_IFREG &&
               (st.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) != 0)
             strncpy(name_ext, "*", sizeof(name_ext));
@@ -1216,6 +1331,8 @@ void ssh_file_server_receive_proc(SshPacketType type,
           ssh_warning("ssh_file_server_receive_proc: bad REMOVE");
           goto return_bad_status;
         }
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_REMOVE %s", name));
 
       /* Remove the file and send response. */
       if (remove(name) < 0)
@@ -1245,6 +1362,8 @@ void ssh_file_server_receive_proc(SshPacketType type,
           ssh_warning("ssh_file_server_receive_proc: bad MKDIR");
           goto return_bad_status;
         }
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_MKDIR %s", name));
 
       /* Create the directory. */
 #ifdef WIN32
@@ -1273,6 +1392,8 @@ void ssh_file_server_receive_proc(SshPacketType type,
           ssh_warning("ssh_file_server_receive_proc: bad RMDIR");
           goto return_bad_status;
         }
+      name = decode_file_path(name);
+      SSH_DEBUG(3, ("SSH_FXP_RMDIR %s", name));
 
       /* Remove the directory and send response. */
       if (rmdir(name) < 0)
@@ -1298,9 +1419,17 @@ void ssh_file_server_receive_proc(SshPacketType type,
         }
       
       if (ssh_realpath(name, resolved) == NULL)
+      {
+        SSH_DEBUG(1, ("SSH_FXP_REALPATH: '%s' failed", name));
+
         ssh_file_server_send_status(server, id, 
                                     ssh_file_server_errno_to_error(errno));
-      
+        ssh_xfree(name);
+        break;
+      }
+
+      SSH_DEBUG(2, ("SSH_FXP_REALPATH: '%s' resolved to '%s'", name, resolved));
+
       /* Construct a SSH_FXP_NAME consisting only of one name and
          a dummy attributes value */
       
@@ -1328,6 +1457,10 @@ void ssh_file_server_receive_proc(SshPacketType type,
           goto return_bad_status;
         }
 
+      oldpath = decode_file_path(oldpath);
+      newpath = decode_file_path(newpath);
+      SSH_DEBUG(3, ("SSH_FXP_RENAME %s to %s", oldpath, newpath));
+
       /* rename() and send response. */
       if (rename(oldpath, newpath) < 0)
         ssh_file_server_send_status(server, id,
@@ -1371,6 +1504,7 @@ void ssh_file_server_eof_proc(void *context)
       if (handle->is_directory)
         {
 #ifdef WIN32
+        if (handle->root_drives == NULL)
           FindClose(handle->dir);
 #else /* WIN32 */
           closedir(handle->dir);
