@@ -18,13 +18,16 @@
    other representations of integer "exponents". */
 
 /* The Morain-Olivos signed digit {-1,0,1} NAF. */
-unsigned int ssh_mp_transform_mo(const SshInt *k, char **transform_table);
+unsigned int ssh_mp_transform_mo(const SshMPIntStruct *k,
+                                 signed char **transform_table);
 
 /* Standard binary expansion. */
-unsigned int ssh_mp_transform_binary(const SshInt *k, char **transform_table);
+unsigned int ssh_mp_transform_binary(const SshMPIntStruct *k,
+                                     signed char **transform_table);
 
 /* KMOV NAF-expansions. */
-unsigned int ssh_mp_transform_kmov(const SshInt *k, char **transform_table);
+unsigned int ssh_mp_transform_kmov(const SshMPIntStruct *k,
+                                   signed char **transform_table);
 
 
 #endif /* NAF_H */
