@@ -175,7 +175,7 @@ int ssh_mpk_count_leading_zeros(SshWord x)
   SSH_MPK_CLZ_HELP(SSH_MPK_7TH_OCTET, 8);
   SSH_MPK_CLZ_HELP_LAST();
 #else /* Some other word size... */
-  int i;
+  int i, count;
 
   count = 0; 
   for (i = 0; i < (SSH_WORD_BITS/8) - 1; i++)
