@@ -217,6 +217,9 @@ __asm__("divl %4"                  \
    seems to be more suitable for word size multiplication than
    Karatsuba's idea. We describe the details of the algorithm.
 
+   Note also that Henri Cohen discusses the generalization of this method
+   in his book. 
+
    Basic idea here is to compute w = n1*n0, 0 < n1, n0 <= W - 1, in parts as
 
      n0 = c + W*d
@@ -378,9 +381,9 @@ __asm__("divl %4"                  \
    i.e. `n1' contains the half-words `h3' and `h2', and `n0' contains
    the half-words `h1' and `h0'. Let `d' = `(d1, d0)' similarly.
                    
-   We know n1 < d because q fits into a single word.  Standard long
-   division would first divide (h3, h2, h1) by (d1, d0) and obtain the
-   higher word of the quotient:
+   We may assume n1 < d because q must fit into a single word.
+   Standard long division would first divide (h3, h2, h1) by (d1, d0)
+   and obtain the higher word of the quotient:
 
    :                         q1
    :                 _______________
@@ -492,13 +495,54 @@ __asm__("divl %4"                  \
    because q1' was obtained by dividing (h3, h2, 0) by (d1, 0).)
 
                                           -- Huima
+
+   Remark. It is often possible to select the highest word of the
+   divisor in such a way that it is of some special form. Here is a
+   trivial one:
+
+     d = 2^w - 1 (all bits are ones)
+
+   now
+
+     (n1*2^w + n0) = b1*(2^w - 1) + b0  
+     
+     (n1 - b1)*2^w + (n0 + b1) = b0 < (2^w-1).
+
+   We may write b0' = n0 + n1, and then b1' = n1. Iff 0 <= b0' < 2^w - 1,
+   we're done, otherwise b0' must be too large.
+
+   At this point we have b0' + b1'*(2^w-1) = n1*2^w + n0, as
+   we have b0' - b1' + b1'*2^w = n1*2^w + n0. Thus it is sufficient to
+   compute (b0' + b1'(2^w - 1)) / (2^w - 1) = b0' / (2^w - 1) + b1'.
+
+   Hence, we can just continue with b0' with the same algorithm
+   recursively. Namely the algorithm could be
+
+     Input: n1*2^w + n0 and 2^w - 1
+     Output: (c1,c0)
+
+     Step 1. Write i = 0, c1_i = c1, c0_i = c0.
+     Step 2. Write b1_i = b1_i + c1_i, b0_i = c1_i + c0_i.
+     Step 4. If b0_i < 2^w - 1, return (b1_i,b0_i). If b0_i = 2^w - 1,
+             return (b1_i+1, 0).
+     Step 3. Write c1_{i+1}*2^w + c0_{i+1} = b0_i, i = i + 1, goto Step 1.
+
+   It can be extended to divisors of form (2^w +- c), where c is reasonably
+   small. However, this has very little use in computations with
+   single word divisors.
+
+   In our application it can be shown that 2 iterations are required at
+   most. This code will be implement, however, later.
+
    */
 
 #ifndef SSH_MPK_LONG_DIV
 #define SSH_MPK_LONG_DIV_FIX_REMAINDER(b,c,d,e,g)  \
-do{e=g-c*b;if(e>g){if(e+d>e){e+=d<<1;c-=2;}else{e+=d;c--;}}}while(0)
+  do{e=g-c*b;if(e>g){if(e+d>e){e+=d<<1;c-=2;}else{e+=d;c--;}}}while(0)
+/* Remark. The expression n-c*a can be replaced by n%a, but the
+   former is usually faster. */
 #define SSH_MPK_LONG_DIV_STEP(a,n,c,g,extr,n0)  \
-c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0))
+do{c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
 
 #define SSH_MPK_LONG_DIV(q, r, n1, n0, d)  \
 { \
@@ -531,8 +575,10 @@ c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0))
    Algorithms by Plumb suggests only squaring, algorithm by Saarinen's
    suggests only squaring, algorithm by Karatsuba suggests multiplication,
    and algorithms by Montgomery suggest both squaring and multiplication.
-   The selection of Montgomery can be taken with any of the others.   
-      
+   The selection of Montgomery can be taken with any of the others.
+
+   Note that the algorithm of Montgomery here presented is given in the
+   book by Henri Cohen also. 
    */
 #undef    SSH_MPK_USE_PLUMBS_ALGORITHM
 #undef    SSH_MPK_USE_SAARINENS_ALGORITHM
