@@ -24,7 +24,7 @@
   */
 
 /*
- * $Id: factor.c,v 1.4 2000/01/14 19:55:51 mkojo Exp $
+ * $Id: factor.c,v 1.6 2000/11/13 04:17:56 mkojo Exp $
  * $Log: factor.c,v $
  * $EndLog$
  */
@@ -33,6 +33,7 @@
 #include "sshmp.h"
 #include "sieve.h"
 #include "pollard.h"
+#include "sshbuffer.h"
 
 
 /* ECM code. */
@@ -45,18 +46,18 @@
 
 typedef struct SshECMPointRec
 {
-  SshIntModQ x, z;
+  SshMPIntModQStruct x, z;
   /* These are here just to speed things up in addition. However,
      the speed-up is so small it is unlikely that it matters. */
-  SshIntModQ xpz, xmz;
+  SshMPIntModQStruct xpz, xmz;
 } SshECMPoint;
 
 typedef struct SshECMCurveRec
 {
-  SshIntModuli mont_q;
-  SshInt q;
-  SshIntModQ a, A, B;
-  SshIntModQ t1, t2, t3, t4;
+  SshMPIntModuliStruct mont_q;
+  SshMPIntStruct q;
+  SshMPIntModQStruct a, A, B;
+  SshMPIntModQStruct t1, t2, t3, t4;
   SshECMPoint q1, q2;
 } SshECMCurve;
 
@@ -84,7 +85,7 @@ void ssh_ecm_copy_point(SshECMPoint *a, const SshECMPoint *b)
   ssh_mpm_set(&a->xmz, &b->xmz);
 }
 
-void ssh_ecm_init_curve(SshECMCurve *E, const SshInt *q)
+void ssh_ecm_init_curve(SshECMCurve *E, const SshMPInt q)
 {
   ssh_mpm_init_m(&E->mont_q, q);
   ssh_mpm_init(&E->a, &E->mont_q);
@@ -179,7 +180,7 @@ Boolean ssh_ecm_correct(const SshECMPoint *n, SshECMCurve *E)
 {
   int i;
   Boolean rv = TRUE;
-  SshInt t1, t2, t3, t4;
+  SshMPIntStruct t1, t2, t3, t4;
   
   /* This is used to verify that no error has occurred, which is
      something we'd like to know when doing very long runs.
@@ -316,11 +317,11 @@ void ssh_ecm_factorization_mul_ui(SshECMPoint *n, SshECMPoint *m,
    in spirit of Richard P. Brent. */
 
 void
-ssh_ecm_generate_curve_with_known_subgroup_12_brent(SshInt *factor,
+ssh_ecm_generate_curve_with_known_subgroup_12_brent(SshMPInt factor,
                                                     SshECMCurve *E,
                                                     SshECMPoint *n)
 {
-  SshInt theta, u, v, x, y, z, t1, t2, t3, t4;
+  SshMPIntStruct theta, u, v, x, y, z, t1, t2, t3, t4;
 
   ssh_mp_init(&theta);
   ssh_mp_init(&u);
@@ -496,7 +497,7 @@ finish:
    converting back to affine. */
 Boolean ssh_ecm_same(SshECMPoint *a, SshECMPoint *b, SshECMCurve *E)
 {
-  SshIntModQ inv1, inv2;
+  SshMPIntModQStruct inv1, inv2;
 
   ssh_mpm_init(&inv1, &E->mont_q);
   ssh_mpm_init(&inv2, &E->mont_q);
@@ -532,12 +533,12 @@ failed:
    moment many things are just fixed so and so.
    */
 
-Boolean ssh_ecm(SshInt *factor, const SshInt *composite,
-                SshSieve *sieve,
+Boolean ssh_ecm(SshMPInt factor, const SshMPInt composite,
+                SshSieve sieve,
                 SshWord b1, SshWord b2)
 {
-  SshInt b, c, t1, t2, t3, t4;
-  SshIntModQ g, prod, *xz;
+  SshMPIntStruct b, c, t1, t2, t3, t4;
+  SshMPIntModQStruct g, prod, *xz;
   SshECMCurve E;
   unsigned int keep_going;
   unsigned int passes, max_passes, maxi, ik, tempi, l;
@@ -588,7 +589,7 @@ Boolean ssh_ecm(SshInt *factor, const SshInt *composite,
   step_size = 30030;
 
   /* Allocating space. */
-  xz = ssh_xmalloc(sizeof(SshIntModQ) * table_size);
+  xz = ssh_xmalloc(sizeof(SshMPIntModQStruct) * table_size);
   t = ssh_xmalloc(sizeof(SshECMPoint) * table_size);
 
   ssh_mp_set_ui(&t1, 0);
@@ -1026,10 +1027,10 @@ factor_found:
   return rv;
 }
 
-void sieve_allocate_mp(SshSieve *sieve, SshInt *c,
+void sieve_allocate_mp(SshSieve sieve, SshMPInt c,
                        unsigned int max_memory)
 {
-  SshInt t;
+  SshMPIntStruct t;
   unsigned int max;
 
   ssh_mp_init(&t);
@@ -1064,14 +1065,14 @@ void initialize_random(char *str)
 void usage(void)
 {
   fprintf(stderr,
-          "usage: ecm -b1 bound -b2 bound -id unique-id [large composite number]\n");
+          "usage: ecm -b1 bound -b2 bound -id unique-id [-f bigfile] [large composite number]\n");
 }
 
 int main(int ac, char *av[])
 {
-  SshInt c, t, factor;
-  SshSieve sieve;
-  char *unique;
+  SshMPIntStruct c, t, factor;
+  SshSieveStruct sieve;
+  char *unique, *bigfile;
   unsigned long i, e, round, pos, b1, b2, max_memory;
   Boolean show_usage = TRUE;
 
@@ -1080,6 +1081,7 @@ int main(int ac, char *av[])
   b1 = 0;
   b2 = 0;
   max_memory = 1024*1024;
+  bigfile = NULL;
   
   /* Read the input. */
   
@@ -1116,11 +1118,56 @@ int main(int ac, char *av[])
           pos++;
           continue;
         }
+      if (strcmp(av[pos], "-f") == 0)
+        {
+          bigfile = av[pos+1];
+          pos++;
+          show_usage = FALSE;
+          continue;
+        }
+      
       ssh_mp_set_str(&c, av[pos], 0);
       show_usage = FALSE;
       break;
     }
 
+  if (bigfile)
+    {
+      FILE *fp;
+      SshBufferStruct buffer;
+      unsigned char blob[256];
+      fp = fopen(bigfile, "r");
+      if (fp == NULL)
+        {
+          printf("error: could not open the file %s.\n",
+                 bigfile);
+          exit(1);
+        }
+      ssh_buffer_init(&buffer);
+      while (fgets(blob, 255, fp) != NULL)
+        {
+          int i, pos, len = strlen(blob);
+          /* Move to the front... no really, just remove whitespace. */
+          for (i = 0, pos = 0; i < len; i++)
+            {
+              if (isspace(blob[i]))
+                continue;
+              blob[pos] = blob[i];
+              pos++;
+            }
+          /* Make the end. */
+          blob[pos] = '\0';
+          /* Append the buffer. */
+          ssh_buffer_append(&buffer, blob, strlen(blob));
+        }
+      if (ssh_mp_set_str(&c, ssh_buffer_ptr(&buffer), 0) == 0)
+        {
+          printf("error: could not parse file %s.\n", bigfile);
+          exit(1);
+        }
+      ssh_buffer_clear(&buffer);
+    }
+  
   if (show_usage)
     {
       ssh_mp_clear(&c);
