@@ -10,11 +10,11 @@
  *        $Author: kivinen $
  *
  *        Creation          : 18:00 Jul 21 1998 kivinen
- *        Last Modification : 04:11 May  4 1999 kivinen
- *        Last check in     : $Date: 1999/05/04 02:19:53 $
- *        Revision number   : $Revision: 1.4 $
+ *        Last Modification : 02:30 Jun 14 2000 kivinen
+ *        Last check in     : $Date: 2000/06/14 00:42:41 $
+ *        Revision number   : $Revision: 1.10 $
  *        State             : $State: Exp $
- *        Version           : 1.70
+ *        Version           : 1.100
  *
  *        Description       : Test math library speed.
  *
@@ -26,16 +26,16 @@
 #include "sshincludes.h"
 #include "sshmath-types.h"
 #include "sshmp.h"
-#include "timeit.h"
+#include "sshtimemeasure.h"
 
 
 void speed_test(int bits)
 {
-  SshInt a, b, c, g, r, q;
-  SshIntModQ am, bm, gm, rm, qm;
-  SshIntModuli m;
+  SshMPIntStruct a, b, c, g, r, q;
+  SshMPIntModQStruct am, bm, gm, rm, qm;
+  SshMPIntModuliStruct m;
   int i, cnt;
-  TimeIt tmit;
+  struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
 
   ssh_mp_init(&a);
   ssh_mp_init(&b);
@@ -75,42 +75,47 @@ void speed_test(int bits)
   cnt = init_count; \
 label_name: \
   fflush(stdout); \
-  start_timing(&tmit); \
+  ssh_time_measure_reset(&tmit); \
+  ssh_time_measure_start(&tmit); \
   for (i = 0; i < cnt; i++) \
     { \
       operation; \
     } \
-  check_timing(&tmit); \
-  if (tmit.process_secs < 2.0) \
+  ssh_time_measure_stop(&tmit); \
+  if (ssh_time_measure_get(&tmit, SSH_TIME_GRANULARITY_SECOND) < 1.0) \
     { \
-      cnt *= 5; \
+      if (ssh_time_measure_get(&tmit, SSH_TIME_GRANULARITY_MILLISECOND) < 10) \
+        cnt *= 128; \
+      else \
+        cnt = (int) (cnt * 1400.0 / \
+              ssh_time_measure_get(&tmit, SSH_TIME_GRANULARITY_MILLISECOND)); \
       printf("%d...", cnt); \
       goto label_name; \
     } \
   printf("done, %s speed = %f us\n", (test_name), \
-         tmit.process_secs / cnt * 1000 * 1000);
+         (double) ssh_time_measure_get(&tmit, SSH_TIME_GRANULARITY_MICROSECOND) / cnt);
 
-  TEST_IT("Addition", add_label, ssh_mp_add(&r, &a, &b), 100000);
-  TEST_IT("Subraction", sub_label, ssh_mp_sub(&r, &a, &b), 100000);
-  TEST_IT("Multiplication", mul_label, ssh_mp_mul(&r, &a, &b), 100000);
-  TEST_IT("Square", sqr_label, ssh_mp_square(&r, &a), 100000);
-  TEST_IT("Division", div_label, ssh_mp_div(&q, &r, &c, &a), 100000);
-  TEST_IT("Division q", div_q_label, ssh_mp_div_q(&q, &c, &a), 100000);
-  TEST_IT("Modulo", mod_label, ssh_mp_mod(&r, &c, &b), 100000);
-  TEST_IT("Gcd", gcd_label, ssh_mp_gcd(&r, &a, &b), 100);
+  TEST_IT("Addition", add_label, ssh_mp_add(&r, &a, &b), 10000);
+  TEST_IT("Subraction", sub_label, ssh_mp_sub(&r, &a, &b), 10000);
+  TEST_IT("Multiplication", mul_label, ssh_mp_mul(&r, &a, &b), 500);
+  TEST_IT("Square", sqr_label, ssh_mp_square(&r, &a), 500);
+  TEST_IT("Division", div_label, ssh_mp_div(&q, &r, &c, &a), 500);
+  TEST_IT("Division q", div_q_label, ssh_mp_div_q(&q, &c, &a), 500);
+  TEST_IT("Modulo", mod_label, ssh_mp_mod(&r, &c, &b), 500);
+  TEST_IT("Gcd", gcd_label, ssh_mp_gcd(&r, &a, &b), 10);
   TEST_IT("Powm naive", powm_naive_label,
-          ssh_mp_powm_naive(&r, &g, &a, &b), 100);
-  TEST_IT("Powm bsw", powm_bsw_label, ssh_mp_powm_bsw(&r, &g, &a, &b), 100);
+          ssh_mp_powm_naive(&r, &g, &a, &b), 1);
+  TEST_IT("Powm bsw", powm_bsw_label, ssh_mp_powm_bsw(&r, &g, &a, &b), 1);
   TEST_IT("Powm naive mont", powm_naive_mont_label,
-          ssh_mp_powm_naive_mont(&r, &g, &a, &b), 100);
+          ssh_mp_powm_naive_mont(&r, &g, &a, &b), 1);
   TEST_IT("Powm bsw mont", powm_bsw_mont_label,
-          ssh_mp_powm_bsw_mont(&r, &g, &a, &b), 100);
+          ssh_mp_powm_bsw_mont(&r, &g, &a, &b), 1);
   /* TEST_IT("Pow", pow_label, ssh_mp_pow(&r, &a, &b), 1); */
 
-  TEST_IT("Mod add", madd_label, ssh_mpm_add(&rm, &am, &bm), 100000);
-  TEST_IT("Mod sub", msub_label, ssh_mpm_sub(&rm, &am, &bm), 100000);
-  TEST_IT("Mod mul", mmul_label, ssh_mpm_mul(&rm, &am, &bm), 100000);
-  TEST_IT("Mod square", msqr_label, ssh_mpm_square(&rm, &am), 100000);
+  TEST_IT("Mod add", madd_label, ssh_mpm_add(&rm, &am, &bm), 10000);
+  TEST_IT("Mod sub", msub_label, ssh_mpm_sub(&rm, &am, &bm), 10000);
+  TEST_IT("Mod mul", mmul_label, ssh_mpm_mul(&rm, &am, &bm), 1000);
+  TEST_IT("Mod square", msqr_label, ssh_mpm_square(&rm, &am), 1000);
 
   ssh_mpm_clear(&am);
   ssh_mpm_clear(&bm);
@@ -137,7 +142,7 @@ int main(int argc, char **argv)
   int bits;
   
   /* Randomize the random number generator. */
-  srandom(ssh_time());
+  srandom((SshUInt32) ssh_time());
 
   bits = 1024;
 
