@@ -21,13 +21,19 @@
   */
 
 /*
- * $Id: t-mathtest.c,v 1.30 2000/03/10 18:33:29 mkojo Exp $
+ * $Id: t-mathtest.c,v 1.32 2000/11/10 15:43:01 mkojo Exp $
  * $Log: t-mathtest.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshmp.h"
+
+
+
+
+
+
 #include "sshtimemeasure.h"
 #include "sieve.h"
 
@@ -35,7 +41,7 @@
    trying to figure out what was wrong. And also, sometimes to compare
    results with other systems. */
 
-void print_int(char *str, SshInt *op)
+void print_int(char *str, SshMPInt op)
 {
   char *mstr;
 
@@ -44,10 +50,10 @@ void print_int(char *str, SshInt *op)
   ssh_xfree(mstr);
 }
 
-void print_mont(char *str, SshIntModQ *op)
+void print_mont(char *str, SshMPIntModQ op)
 {
   char *mstr;
-  SshInt a;
+  SshMPIntStruct a;
 
   ssh_mp_init(&a);
   ssh_mp_set_mpm(&a, op);
@@ -59,9 +65,9 @@ void print_mont(char *str, SshIntModQ *op)
   ssh_mp_clear(&a);
 }
 
-int check_mod(SshIntModQ *b, SshInt *a)
+int check_mod(SshMPIntModQ b, SshMPInt a)
 {
-  SshInt t;
+  SshMPIntStruct t;
   int rv;
 
   ssh_mp_init(&t);
@@ -71,14 +77,14 @@ int check_mod(SshIntModQ *b, SshInt *a)
   return rv;
 }
 
-void my_rand_mod(SshIntModQ *a, SshInt *b, int bits)
+void my_rand_mod(SshMPIntModQ a, SshMPInt b, int bits)
 {
   int n = random() % bits;
   ssh_mp_rand(b, n);
   ssh_mpm_set_mp(a, b);
 }
 
-void true_rand(SshInt *op, int bits)
+void true_rand(SshMPInt op, int bits)
 {
   ssh_mp_rand(op, random() % bits);
 
@@ -89,7 +95,7 @@ void true_rand(SshInt *op, int bits)
 
 void test_int(int flag, int bits)
 {
-  SshInt a, b, c, d, e, f, g;
+  SshMPIntStruct a, b, c, d, e, f, g;
   int j, k, i, l;
 
   ssh_mp_init(&a);
@@ -286,9 +292,6 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp_ui(&a, 3) < 0)
         continue;
 
-      if ((ssh_mp_get_ui(&a) & 0x1) == 0)
-        ssh_mp_add_ui(&a, &a, 1);
-
       k = random();
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
@@ -322,8 +325,6 @@ void test_int(int flag, int bits)
           true_rand(&a, bits);
           if (ssh_mp_cmp_ui(&a, 3) < 0)
             continue;
-          if ((ssh_mp_get_ui(&a) & 0x1) == 0)
-            ssh_mp_add_ui(&a, &a, 1);
 
           true_rand(&b, bits);
           ssh_mp_abs(&b, &b);
@@ -352,9 +353,6 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp_ui(&a, 3) < 0)
         continue;
 
-      if ((ssh_mp_get_ui(&a) & 0x1) == 0)
-        ssh_mp_add_ui(&a, &a, 1);
-
       k = random();
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
@@ -391,9 +389,6 @@ void test_int(int flag, int bits)
           if (ssh_mp_cmp_ui(&a, 3) < 0)
             continue;
 
-          if ((ssh_mp_get_ui(&a) & 0x1) == 0)
-            ssh_mp_add_ui(&a, &a, 1);
-
           true_rand(&b, bits);
           ssh_mp_mod(&b, &b, &a);
           true_rand(&c, bits);
@@ -524,7 +519,7 @@ void test_int(int flag, int bits)
       printf(" * full exponentiation test with base\n");
       for (j = 0; j < 10; j++)
         {
-          SshMpPowmBase base;
+          SshMPPowmBaseStruct base;
           true_rand(&a, bits);
           ssh_mp_abs(&a, &a);
 
@@ -780,9 +775,9 @@ void test_int(int flag, int bits)
 void test_mod(int flag, int bits)
 {
   /* Montgomery testing. */
-  SshIntModQ a0, b0, c0;
-  SshInt  a1, b1, c1, m1, d;
-  SshIntModuli m0;
+  SshMPIntModQStruct a0, b0, c0;
+  SshMPIntStruct  a1, b1, c1, m1, d;
+  SshMPIntModuliStruct m0;
   int i;
   Boolean rv1, rv2;
 
@@ -1011,14 +1006,1282 @@ void test_mod(int flag, int bits)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /* Speed tests of some sort. */
 
 void timing_int(int bits)
 {
-  SshInt a, b, c, d, e, f[100];
+  SshMPIntStruct a, b, c, d, e, f[100];
   struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
   unsigned int i, j;
-  SshMpPowmBase base;
+  SshMPPowmBaseStruct base;
 
   ssh_mp_init(&a);
   ssh_mp_init(&b);
@@ -1243,9 +2506,9 @@ void timing_int(int bits)
 
 void timing_modular(int bits)
 {
-  SshIntModQ b, c, d, e, f[100];
-  SshIntModuli m;
-  SshInt a;
+  SshMPIntModQStruct b, c, d, e, f[100];
+  SshMPIntModuliStruct m;
+  SshMPIntStruct a;
   int i, j;
   struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
 
@@ -1323,6 +2586,255 @@ void timing_modular(int bits)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 /* Routines for handling the arguments etc. */
 
 typedef struct CommandRec
@@ -1379,6 +2891,15 @@ const Command commands[] =
   { "--modular", C_MOD, 1 },
 
 
+
+
+
+
+
+
+
+
+
   { NULL }
 };
 
@@ -1409,6 +2930,14 @@ void usage(void)
          "advanced options: \n"
          " --integer [on|off] sets the integer arithmetic testing on/off.\n"
          " --modular [on|off] sets the (mod p) arithmetic testing on/off.\n"
+
+
+
+
+
+
+
+
          );
   exit(1);
 }
@@ -1447,6 +2976,13 @@ int main(int ac, char *av[])
          "Features: \n"
          "  - integer arithmetic\n"
          "  - finite field arithmetic (mod p)\n"
+
+
+
+
+
+
+
          "\n");
 
   /* Randomize the random number generator. */
@@ -1499,6 +3035,32 @@ int main(int ac, char *av[])
           i++;
           break;
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
         case C_BITS:
           bits = atoi(av[i + 1]);
           i++;
@@ -1558,6 +3120,44 @@ int main(int ac, char *av[])
           if (timing)
             timing_modular(bits);
         }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
     }
 
   return 0;
