@@ -11,7 +11,7 @@ File handling for OpenPGP files.
 
 */
 /*
- * $Id: pgp_file.c,v 1.5 1999/05/03 12:54:58 tri Exp $
+ * $Id: pgp_file.c,v 1.8 2000/06/14 00:42:43 kivinen Exp $
  * $Log: pgp_file.c,v $
  * $EndLog$
  */
@@ -25,16 +25,16 @@ File handling for OpenPGP files.
 
 #define SSH_DEBUG_MODULE "SshPgpFile"
 
-Boolean ssh_pgp_read_packet(SshFileBuffer *filebuf, SshPgpPacket *packet)
+Boolean ssh_pgp_read_packet(SshFileBuffer filebuf, SshPgpPacket *packet)
 {
   unsigned char type_id;
-  int packet_type;
+  int packet_type = 0;
   int i;
   size_t l;
   Boolean fr;
   Boolean partial_body;
   SshPgpPacket newpacket;
-  SshBuffer partial_buf;
+  SshBufferStruct partial_buf;
   Boolean partial_buf_init;
 
   partial_buf_init = FALSE;
@@ -172,7 +172,7 @@ Boolean ssh_pgp_read_packet(SshFileBuffer *filebuf, SshPgpPacket *packet)
   return FALSE;
 }
 
-Boolean ssh_pgp_next_packet_type(SshFileBuffer *filebuf, int *type)
+Boolean ssh_pgp_next_packet_type(SshFileBuffer filebuf, int *type)
 {
   unsigned char type_id;
   Boolean fr;
