@@ -11,7 +11,7 @@ Parse pgp keyblobs.
 
 */
 /*
- * $Id: pgp_key.c,v 1.8 1999/05/04 08:54:03 tri Exp $
+ * $Id: pgp_key.c,v 1.14 2000/06/14 00:42:43 kivinen Exp $
  * $Log: pgp_key.c,v $
  * $EndLog$
  */
@@ -28,17 +28,17 @@ Parse pgp keyblobs.
 /* Functions to build private keys compatible with SSH Crypto Library.
    Keys are derived from public keys and components of the secret key. */   
 SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
-                                            SshInt *d,
-                                            SshInt *p,
-                                            SshInt *q,
-                                            SshInt *u);
+                                            SshMPInt d,
+                                            SshMPInt p,
+                                            SshMPInt q,
+                                            SshMPInt u);
 SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
-                                            SshInt *x);
+                                            SshMPInt x);
 SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
-                                            SshInt *x);
+                                            SshMPInt x);
 
 /* Decode pgp encoded integer. */
-size_t ssh_pgp_mpz_decode(const unsigned char *data, int len, SshInt *z);
+size_t ssh_pgp_mpz_decode(const unsigned char *data, int len, SshMPInt z);
 
 static size_t bitbytes(size_t bits);
 
@@ -54,7 +54,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
   int version, key_type, pk_algorithm_byte;
   SshUInt32 generation_time, validity_time;
   int i;
-  SshInt z[4];
+  SshMPIntStruct z[4];
   size_t skiptot;
   int skip;
   SshPgpPublicKey r = NULL;
@@ -136,7 +136,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           unsigned char digest[16];
           char print[64];
           int l1, l2;
-          SshInt x1;
+          SshMPIntStruct x1;
 
           /* Hash the payload of the public key material */
           l1 = bitbytes(((((int)(pubkey_data[0])) << 8) |
@@ -275,7 +275,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
       cr = ssh_hash_allocate("sha1", &hash);
       if (cr == SSH_CRYPTO_OK) 
         {
-          char buf[9];
+          unsigned char buf[9];
           unsigned char digest[20];
           char print[64];
 
@@ -441,13 +441,13 @@ size_t ssh_pgp_secret_key_decode_with_passphrase(const unsigned char *data,
   SshPgpSecretKey r = NULL;
   SshPgpCipher cipher = NULL;
   SshCryptoStatus cs;
-  SshInt y[4];
+  SshMPIntStruct y[4];
   size_t skiptot = 0, bits, bytes, mp_buf_origlen;
-  int skip;
-  int s2k_conv;
-  int s2k_type;
-  int s2k_hash;
-  int s2k_count;
+  int skip = 0;
+  int s2k_conv = 0;
+  int s2k_type = 0;
+  int s2k_hash = 0;
+  int s2k_count = 0;
   unsigned char s2k_salt[8];
   int key_enc_method;
   unsigned char iv[8];
@@ -633,6 +633,8 @@ size_t ssh_pgp_secret_key_decode_with_passphrase(const unsigned char *data,
             {
               for (i = 0; i < 4; i++)
                 {
+                  if (len < 2)
+                    goto packet_parsing_failed;
                   bits = (((int)data[0]) << 8) | ((int)data[1]);
                   bytes = bitbytes(bits);
                   len -= 2;
@@ -726,7 +728,8 @@ size_t ssh_pgp_secret_key_decode_with_passphrase(const unsigned char *data,
                   if (skip < 1) 
                     {
                       /* Unexpected end of data */
-                      ssh_xfree(mp_buf);
+                      memset(mp_buf_orig, 'F', mp_buf_origlen);
+                      ssh_xfree(mp_buf_orig);
                       if (key_enc_method == 0)
                         goto packet_parsing_failed;
                       else
@@ -747,7 +750,8 @@ size_t ssh_pgp_secret_key_decode_with_passphrase(const unsigned char *data,
               if (skip < 1) 
                 {
                   /* Unexpected end of data */
-                  ssh_xfree(mp_buf);
+                  memset(mp_buf_orig, 'F', mp_buf_origlen);
+                  ssh_xfree(mp_buf_orig);
                   if (key_enc_method == 0)
                     goto packet_parsing_failed;
                   else
@@ -764,12 +768,13 @@ size_t ssh_pgp_secret_key_decode_with_passphrase(const unsigned char *data,
             default:
               /* If this occures, someone has implemented new public key
                  types but didn't implement secret key parsing. */
-              memset(mp_buf, 'F', len);
-              ssh_xfree(mp_buf);
+              memset(mp_buf_orig, 'F', mp_buf_origlen);
+              ssh_xfree(mp_buf_orig);
               goto key_decryption_failed;
             }
           if (len < 2)
             {
+              memset(mp_buf_orig, 'F', mp_buf_origlen);
               ssh_xfree(mp_buf_orig);
               goto packet_parsing_failed;
             }
@@ -881,7 +886,7 @@ void ssh_pgp_secret_key_free(SshPgpSecretKey key)
 }
 
 /* Decode pgp encoded integer. */
-size_t ssh_pgp_mpz_decode(const unsigned char *data, int len, SshInt *z)
+size_t ssh_pgp_mpz_decode(const unsigned char *data, int len, SshMPInt z)
 {
   int bits, bytes;
   int i;
@@ -907,15 +912,14 @@ size_t ssh_pgp_mpz_decode(const unsigned char *data, int len, SshInt *z)
 }
 
 SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
-                                            SshInt *d,
-                                            SshInt *p,
-                                            SshInt *q,
-                                            SshInt *u)
+                                            SshMPInt d,
+                                            SshMPInt p,
+                                            SshMPInt q,
+                                            SshMPInt u)
 {
-  SshInt e, n;
+  SshMPIntStruct e, n;
   SshPrivateKey key = NULL;
   SshCryptoStatus cs;
-  SshRandomState dummy_rnd_state;
   
   if (public_key == NULL)
     return NULL;
@@ -925,7 +929,6 @@ SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
   SSH_ASSERT(u != NULL);
   ssh_mp_init(&e);
   ssh_mp_init(&n);
-  dummy_rnd_state = ssh_random_allocate();
 
   cs = ssh_public_key_get_info(public_key,
                                SSH_PKF_MODULO_N, &n,
@@ -933,8 +936,7 @@ SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
                                SSH_PKF_END);
   if (cs != SSH_CRYPTO_OK)
     goto failed;
-  cs = ssh_private_key_generate(dummy_rnd_state,
-                                &key,
+  cs = ssh_private_key_generate(&key,
                                 SSH_PGP_CANONICAL_RSA_NAME,
                                 SSH_PKF_MODULO_N, &n,
                                 SSH_PKF_PUBLIC_E, &e,
@@ -952,17 +954,15 @@ SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
  failed:
   ssh_mp_clear(&e);
   ssh_mp_clear(&n);
-  ssh_random_free(dummy_rnd_state);
   return key;
 }
 
 SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
-                                                SshInt *x)
+                                                SshMPInt x)
 {
-  SshInt p, g, y;
+  SshMPIntStruct p, g, y;
   SshPrivateKey key = NULL;
   SshCryptoStatus cs;
-  SshRandomState dummy_rnd_state;
 
   if (public_key == NULL)
     return NULL;
@@ -971,7 +971,6 @@ SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
   ssh_mp_init(&p);
   ssh_mp_init(&g);
   ssh_mp_init(&y);
-  dummy_rnd_state = ssh_random_allocate();
 
   cs = ssh_public_key_get_info(public_key,
                                SSH_PKF_PRIME_P, &p,
@@ -981,8 +980,7 @@ SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
   if (cs != SSH_CRYPTO_OK)
     goto failed;
 
-  cs = ssh_private_key_generate(dummy_rnd_state,
-                                &key,
+  cs = ssh_private_key_generate(&key,
                                 SSH_PGP_CANONICAL_ELGAMAL_NAME,
                                 SSH_PKF_PRIME_P, &p,
                                 SSH_PKF_GENERATOR_G, &g,
@@ -999,17 +997,15 @@ SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
   ssh_mp_clear(&p);
   ssh_mp_clear(&g);
   ssh_mp_clear(&y);
-  ssh_random_free(dummy_rnd_state);
   return key;
 }
 
 SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
-                                            SshInt *x)
+                                            SshMPInt x)
 {
-  SshInt p, q, g, y;
+  SshMPIntStruct p, q, g, y;
   SshPrivateKey key = NULL;
   SshCryptoStatus cs;
-  SshRandomState dummy_rnd_state;
 
   if (public_key == NULL)
     return NULL;
@@ -1019,7 +1015,6 @@ SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
   ssh_mp_init(&q);
   ssh_mp_init(&g);
   ssh_mp_init(&y);
-  dummy_rnd_state = ssh_random_allocate();
 
   cs = ssh_public_key_get_info(public_key,
                                SSH_PKF_PRIME_P, &p,
@@ -1030,8 +1025,7 @@ SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
   if (cs != SSH_CRYPTO_OK)
     goto failed;
 
-  cs = ssh_private_key_generate(dummy_rnd_state,
-                                &key,
+  cs = ssh_private_key_generate(&key,
                                 SSH_PGP_CANONICAL_DSA_NAME,
                                 SSH_PKF_PRIME_P, &p,
                                 SSH_PKF_PRIME_Q, &q,
@@ -1050,7 +1044,6 @@ SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
   ssh_mp_clear(&q);
   ssh_mp_clear(&g);
   ssh_mp_clear(&y);
-  ssh_random_free(dummy_rnd_state);
   return key;
 }
 
