@@ -11,7 +11,7 @@ Dump contents of pgp file.
 
 */
 /*
- * $Id: pgpkeysearch.c,v 1.3 1999/09/21 07:00:59 tri Exp $
+ * $Id: pgpkeysearch.c,v 1.4 2000/06/14 00:42:45 kivinen Exp $
  * $Log: pgpkeysearch.c,v $
  * $EndLog$
  */
@@ -47,7 +47,7 @@ int main(int argc, char **argv)
   char *id_str;
   Boolean failed = FALSE, secret = FALSE, found;
   PgpKeyIdType id_type = KEY_ID_NAME;
-  SshFileBuffer filebuf;
+  SshFileBufferStruct filebuf;
   SshPgpPacket packet;
   SshPgpPublicKey public_key;
   SshPgpSecretKey secret_key;
