@@ -16,7 +16,7 @@
 */
 
 /*
- * $Id: sshauth.h,v 1.13 2000/05/28 16:21:02 kivinen Exp $
+ * $Id: sshauth.h,v 1.16.2.1 2000/11/29 15:21:44 sjl Exp $
  * $Log: sshauth.h,v $
  * $EndLog$
  */
@@ -27,6 +27,7 @@
 #define SSHAUTH_H
 
 #include "sshstream.h"
+#include "sshuser.h"
 #include "sshcross.h"
 
 #define SSH_USERAUTH_SERVICE  "ssh-userauth"
@@ -140,27 +141,51 @@ typedef enum {
   SSH_AUTH_SERVER_OP_CLEAR_LONGTIME
 } SshAuthServerOperation;
 
+/* A function of this type is passed as argument to the server-side
+   authentication method.  The method should call this function when
+   it is done for now.  See the definition of result codes for more info.
+      `result'    result code
+      `packet'    method-specific part of server's response,
+                  or NULL if empty
+      `completion_context'  context from the call
+   This does not free the packet; data is copied from there to internal
+   buffers during the call. */
+typedef void (*SshAuthServerCompletionProc)(SshAuthServerResult result,
+                                            SshBuffer packet,
+                                            void *completion_context);
+
 /* Function used to represent an authentication method.  This function
    performs all processing by the authentication method.  An authentication
    method is policy-independent.
      `user'         user name from the (original) authentication request
+     `uc'           user context.
      `packet'       method-specific remaining part of the packet
+                    (this should be passed to the completion_proc or
+                    freed by the authentication method. In the latter case,
+                    this should be NULL.)
      `session_id'   session identifier
      `session_id_len' length of session identifier
      `state_placeholder' place to store context data between packets
      `longtime_placeholder'  can hold data between authentications
-                    (this is per-method)
+                             (this is per-method)
+     `completion_proc'  This callback MUST be called by the authentication
+                        method when it has finished. If it is not called
+                        the transport layer won't continue or exit.
+     `completion_context' context for the `completion_proc'
      `method_context'   passed to the method function */
-typedef SshAuthServerResult (*SshAuthServerMethodProc)(
-                                        SshAuthServerOperation op,
-                                        const char *user,
-                                        SshBuffer packet,
-                                        const unsigned char *session_id,
-                                        size_t session_id_len,
-                                        void **state_placeholder,
-                                        void **longtime_placeholder,
-                                        void *method_context);
-
+typedef void (*SshAuthServerMethodProc)
+     (SshAuthServerOperation op,
+      const char *user,
+      SshUser uc,
+      SshBuffer packet,
+      const unsigned char *session_id,
+      size_t session_id_len,
+      void **state_placeholder,
+      void **longtime_placeholder,
+      SshAuthServerCompletionProc completion_proc,
+      void *completion_context, 
+      void *method_context);
+  
 /* This structure completely describes an authentication method.  All methods
    supported by an implementation are described by an array of these.
    The "none" method should not be listed in the array.  The array is
@@ -168,7 +193,7 @@ typedef SshAuthServerResult (*SshAuthServerMethodProc)(
 typedef struct SshAuthServerMethodRec {
   const char *name;             /* method name, e.g. "password" */
   SshAuthServerMethodProc proc; /* function to perform authentication */
-} SshAuthServerMethod;
+} *SshAuthServerMethod, SshAuthServerMethodStruct;
 
 /* Wraps the transport layer stream into an authentication stream.  This will
    use the given policy function to decide which authentication methods are
@@ -211,12 +236,21 @@ typedef struct SshAuthServerMethodRec {
      `methods'              array of supported methods, terminated by
                             an element with NULL name.  This needs to stay
                             valid until the stream is destroyed.
-     `method_context'       context to pass to methods (normally NULL) */
+     `method_context'       context to pass to methods (normally NULL)
+     `banner'               If non-NULL, will be sent in an
+                            SSH_MSG_USERAUTH_BANNER to the client.
+     `uc_return'            if non-NULL, this will be set to the SshUser
+                            object for the authenticated user upon successful
+                            authentication.  If set, it will be the
+                            application's responsibility to free this object
+                            by calling ssh_user_free. */
 SshStream ssh_auth_server_wrap(SshStream transport_stream,
                                SshAuthPolicyProc policy_proc,
                                void *policy_context,
-                               SshAuthServerMethod methods[],
-                               void *method_context);
+                               SshAuthServerMethodStruct methods[],
+                               void *method_context,
+                               const char *banner,
+                               SshUser *uc_return);
 
 /************************** client side ****************************/
 
@@ -311,9 +345,14 @@ typedef enum {
      and an authentication request packet should be sent to the other side. */
   SSH_AUTH_CLIENT_SEND,
 
-  /* Like above, but the method expects to receive additional packets back
-     from the server.  Those packets will be directed to the same method
-     with the SSH_AUTH_CLIENT_OP_CONTINUE operation. */
+  /* Like above, but the method expects to receive additional packets
+     back from the server.  Those packets will be directed to the same
+     method with the SSH_AUTH_CLIENT_OP_CONTINUE operation.
+
+     XXX Currently, if the server sends SSH_MSG_USERAUTH_SUCCESS, the
+     success (and ending) of the authentication isn't told to the
+     authentication method. This has led to some rather ugly
+     workarounds, see authc-pam.c .*/
   SSH_AUTH_CLIENT_SEND_AND_CONTINUE,
 
   /* Like above, but the method does not terminate upon 
@@ -376,7 +415,7 @@ typedef void (*SshAuthClientMethodProc)(
 typedef struct SshAuthClientMethodRec {
   const char *name;             /* method name, e.g. "password" */
   SshAuthClientMethodProc proc; /* function to perform authentication */
-} SshAuthClientMethod;
+} *SshAuthClientMethod, SshAuthClientMethodStruct;
 
 /* Wraps the transport layer stream into an authentication stream.  This
    will automatically handle the entire authentication dialog, and will
@@ -407,7 +446,7 @@ typedef struct SshAuthClientMethodRec {
 SshStream ssh_auth_client_wrap(SshStream transport_stream,
                                const char *initial_user,
                                const char *service,
-                               SshAuthClientMethod methods[],
+                               SshAuthClientMethodStruct methods[],
                                void *method_context);
 
 #endif /* SSHTRANS_H */
