@@ -101,7 +101,7 @@ typedef struct SshAuthClientRec {
 
   /* The available authentication methods, and their number.  The array points
      directly to user-supplied data. */
-  SshAuthClientMethod *methods;
+  SshAuthClientMethod methods;
   unsigned int num_methods;
 
   /* Context argument to be passed to all methods. */
@@ -141,7 +141,7 @@ void ssh_authc_start_next_method(SshAuthClient auth);
 /* Construct a string of the usable methods (ie. those not disabled)
    in the methods-table. */
 char *ssh_authc_usable_methods(int *num_methods,
-                               const SshAuthClientMethod methods[],
+                               const SshAuthClientMethodStruct methods[],
                                void *context);
 
 /* Sends a disconnect message both up and down.  This will presumably cause
@@ -305,7 +305,7 @@ void ssh_authc_completion_proc(SshAuthClientResult result,
     case SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD:      
       SSH_ASSERT(auth->active_method_index != -1);
       
-      SSH_TRACE(2, ("Method '%s' disabled.",                            \
+      SSH_TRACE(2, ("Method '%s' disabled.",
                     auth->methods[auth->active_method_index].name));      
 
       auth->methods[auth->active_method_index].proc = NULL;
@@ -315,7 +315,7 @@ void ssh_authc_completion_proc(SshAuthClientResult result,
 
       /* Reconstruct usable methods string. */
       ssh_xfree(auth->methods_string);
-      auth->methods_string = ssh_authc_usable_methods(&(auth->num_methods),
+      auth->methods_string = ssh_authc_usable_methods((int *)&(auth->num_methods),
                                                       auth->methods, auth);
       
       /* Send a "none" request to reconstruct productive
@@ -1040,7 +1040,7 @@ void ssh_authc_down_received_eof(void *context)
    string is ssh_xmalloced, so it should be freed after it is not used
    anymore. */
 char *ssh_authc_usable_methods(int *num_methods,
-                               const SshAuthClientMethod methods[],
+                               const SshAuthClientMethodStruct methods[],
                                void *context)
 {
   SshBufferStruct buffer;
@@ -1103,7 +1103,7 @@ char *ssh_authc_usable_methods(int *num_methods,
 SshStream ssh_auth_client_wrap(SshStream transport,
                                const char *initial_user,
                                const char *service,
-                               SshAuthClientMethod methods[],
+                               SshAuthClientMethodStruct methods[],
                                void *method_context)
 {
   SshAuthClient auth;
@@ -1122,7 +1122,7 @@ SshStream ssh_auth_client_wrap(SshStream transport,
   auth->service = ssh_xstrdup(service);
 
   /* Construct the methods string, and save the number of methods. */
-  auth->methods_string = ssh_authc_usable_methods(&(auth->num_methods),
+  auth->methods_string = ssh_authc_usable_methods((int *)&(auth->num_methods),
                                                   methods, auth);
 
   /* Save the pointer to the methods array. */
