@@ -23,6 +23,7 @@
 #include "sshtimeouts.h"
 #include "sshencode.h"
 #include "sshdsprintf.h"
+#include "sshuser.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthServer"
 
@@ -44,7 +45,7 @@ typedef struct {
   unsigned int num_methods;
 
   /* Array defining the supported authentication methods. */
-  SshAuthServerMethod *methods;
+  SshAuthServerMethod methods;
 
   /* Context argument to pass to the authentication method functions. */
   void *method_context;
@@ -74,7 +75,8 @@ typedef struct {
   /* Length of the session id. */
   size_t session_id_len;
 
-  /* The session id, from the startup packet.  This is allocated by ssh_xmalloc. */
+  /* The session id, from the startup packet.  This is allocated by
+     ssh_xmalloc. */
   unsigned char *session_id;
 
   /* Copy of the startup packet received from the transport layer, or NULL
@@ -89,6 +91,9 @@ typedef struct {
      This is allocated by ssh_xmalloc. */
   char *requested_user;
 
+  /* SshUser object for the requested user. */
+  SshUser uc;
+  
   /* The requested service name, or NULL if no requests have yet been
      received.  This is allocated by ssh_xmalloc. */
   char *requested_service;
@@ -107,6 +112,21 @@ typedef struct {
      continue authentication.  This is the list sent to the client in
      failure messages.  This is allocated by ssh_xmalloc. */
   char *continuations;
+
+  /* If non-NULL, will be sent in an SSH_MSG_USERAUTH_BANNER message
+     to the client to display to the user. (this is needed in some
+     jurisdictions for legal reasons.)*/
+  char *banner_msg;
+
+  /* Helper variable to indicate when to send banner. */
+  Boolean first_request;
+  
+  /* The uc_return argument specified to ssh_auth_server_wrap when it was
+     called.  This will be set to point to the SshUser object for the
+     authenticated user before sending a SSH_CROSS_AUTHENTICATED packet
+     (unless this is NULL).  If set, it will be the caller's responsibility
+     to free the SshUser object when no longer needed. */
+  SshUser *uc_return;
 } *SshAuthServer;
 
 
@@ -235,6 +255,15 @@ void ssh_auths_success(SshAuthServer auth)
                              (unsigned int) SSH_MSG_USERAUTH_SUCCESS,
                              SSH_FORMAT_END);
 
+  /* Either free the SshUser object for the authenticated user, or pass it
+     to the application (in which case the application will take responsibility
+     for freeing it when no longer needed). */
+  if (auth->uc_return)
+    *auth->uc_return = auth->uc;
+  else
+    ssh_user_free(auth->uc, TRUE);
+  auth->uc = NULL;
+
   /* Send up the buffered STARTUP packet. */
   ssh_cross_up_send(auth->up, SSH_CROSS_STARTUP,
                     ssh_buffer_ptr(auth->startup_packet),
@@ -244,7 +273,7 @@ void ssh_auths_success(SshAuthServer auth)
   ssh_cross_up_send(auth->up, SSH_CROSS_ALGORITHMS,
                     ssh_buffer_ptr(auth->algorithms_packet),
                     ssh_buffer_len(auth->algorithms_packet));
-
+  
   /* Send up an AUTHENTICATED packet. */
   ssh_cross_up_send_encode(auth->up, SSH_CROSS_AUTHENTICATED,
                            SSH_FORMAT_UINT32_STR,
@@ -263,38 +292,35 @@ void ssh_auths_success(SshAuthServer auth)
   /* After this, none of our callbacks except destroy should get called. */
 }
 
-/* Calls the currently active authentication method with the current
-   parameters, and updates state according to the return value from
-   the authentication method.. */
-
-void ssh_auths_call_method(SshAuthServer auth,
-                           SshAuthServerOperation op,
-                           const unsigned char *data, size_t len)
+typedef struct SshAuthServerCompletionCtxRec
 {
+  SshAuthServer auth;
+  /* Current op. */
+  SshAuthServerOperation op;
+  /* Active method index for this method. */
+  int method_index;
+  
+} *SshAuthServerCompletionCtx, SshAuthServerCompletionCtxStruct;
+
+void ssh_auths_completion_proc(SshAuthServerResult result,
+                               SshBuffer packet,
+                               void *context)
+{
+  SshAuthServerOperation op;
+  SshAuthServer auth;
   int i;
   unsigned int packet_type;
-  SshAuthServerResult result;
-  SshBuffer buffer;
+   
+  SshAuthServerCompletionCtx completion_context =
+    (SshAuthServerCompletionCtx) context;
 
-  i = auth->active_method_index;
-
-  /* Sanity checks... */
-  SSH_ASSERT(i >= 0 && i < auth->num_methods);
-  SSH_ASSERT(auth->requested_user != NULL);
-
-  /* Save the data in a buffer. */
-  buffer = ssh_buffer_allocate();
-  ssh_buffer_append(buffer, data, len);
-
-  if (auth->methods[i].proc == NULL)
-    return;
+  SSH_PRECOND(completion_context);  
+  op = completion_context->op;
+  auth = completion_context->auth;
+  SSH_ASSERT(auth);
+  i = completion_context->method_index;  
   
-  /* Call the authentication method. */
-  result = (*auth->methods[i].proc)(op, auth->requested_user, buffer,
-                                    auth->session_id, auth->session_id_len,
-                                    &auth->state_placeholders[i],
-                                    &auth->longtime_placeholders[i],
-                                    auth->method_context);
+  ssh_xfree(completion_context);  
 
   /* Process the return value depending on the operation. */
   switch (op)
@@ -318,7 +344,7 @@ void ssh_auths_call_method(SshAuthServer auth,
           else
             {
               /* More authentications are needed. */
-              SSH_DEBUG(6, ("call_method: More authentications needed."));
+              SSH_DEBUG(6, ("More authentications needed."));
               auth->waiting_continuation = FALSE;
               ssh_auths_send_failure(auth, TRUE);
             }
@@ -340,12 +366,12 @@ void ssh_auths_call_method(SshAuthServer auth,
             char *to_be_deleted;
             SSH_ASSERT(auth->state_placeholders[i] == NULL);
             auth->waiting_continuation = FALSE;
-            auth->methods[auth->active_method_index].proc = NULL;
+            auth->methods[i].proc = NULL;
             to_be_deleted = auth->disabled_authentications;
             ssh_dsprintf(&auth->disabled_authentications, "%s%s%s",
                          auth->disabled_authentications,
                          auth->disabled_authentications[0] != '\0' ? "," : "",
-                         auth->methods[auth->active_method_index].name);
+                         auth->methods[i].name);
             ssh_xfree(to_be_deleted);
 
             /* reset continuations */
@@ -368,22 +394,23 @@ void ssh_auths_call_method(SshAuthServer auth,
           /* The authentication method wants to send out a packet, and then
              continue authentication with the same method. */
           auth->waiting_continuation = FALSE;
-          /* Sanity check the outgoing packet. */
 
-          if (ssh_decode_array(ssh_buffer_ptr(buffer), ssh_buffer_len(buffer),
+          /* Sanity check the outgoing packet. */
+          SSH_ASSERT(packet);          
+          if (ssh_decode_array(ssh_buffer_ptr(packet), ssh_buffer_len(packet),
                                SSH_FORMAT_CHAR, &packet_type,
                                SSH_FORMAT_END) == 0)
-            ssh_fatal("ssh_auths_call_method: unable to decode the packet "
+            ssh_fatal("ssh_auths_completion_proc unable to decode the packet "
                       "type");
 
           if (packet_type < SSH_FIRST_USERAUTH_METHOD_PACKET ||
               packet_type > SSH_LAST_USERAUTH_METHOD_PACKET)
-            ssh_fatal("ssh_auths_call_method: method sending bad packet "
+            ssh_fatal("ssh_auths_completion_proc: method sending bad packet "
                       "(1) %d",
                       (int)packet_type);
           /* Send the packet down. */
           ssh_cross_down_send(auth->down, SSH_CROSS_PACKET,
-                              ssh_buffer_ptr(buffer), ssh_buffer_len(buffer));
+                              ssh_buffer_ptr(packet), ssh_buffer_len(packet));
           break;
 
         case SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK:
@@ -392,22 +419,24 @@ void ssh_auths_call_method(SshAuthServer auth,
           auth->waiting_continuation = TRUE;
 
           /* Sanity check the outgoing packet. */
-          if (ssh_decode_array(ssh_buffer_ptr(buffer), ssh_buffer_len(buffer),
+          SSH_ASSERT(packet);
+          if (ssh_decode_array(ssh_buffer_ptr(packet), ssh_buffer_len(packet),
                                SSH_FORMAT_CHAR, &packet_type,
                                SSH_FORMAT_END) == 0 ||
               packet_type < SSH_FIRST_USERAUTH_METHOD_PACKET ||
               packet_type > SSH_LAST_USERAUTH_METHOD_PACKET)
-            ssh_fatal("ssh_auths_call_method: method sending bad packet"
+            ssh_fatal("ssh_auths_completion_proc: method sending bad packet"
                       "(2) %d",
                       (int)packet_type);
           /* Send the packet down. */
           ssh_cross_down_send(auth->down, SSH_CROSS_PACKET,
-                              ssh_buffer_ptr(buffer), ssh_buffer_len(buffer));
+                              ssh_buffer_ptr(packet), ssh_buffer_len(packet));
           break;
 
         default:
           /* Unexpected result from method function. */
-          ssh_fatal("ssh_auths_call_method: unknown result %d", (int)result);
+          ssh_fatal("ssh_auths_completion_proc: unknown result %d",
+                    (int)result);
         }
       break;
 
@@ -427,11 +456,58 @@ void ssh_auths_call_method(SshAuthServer auth,
       break;
     default:
       /* Unknown operation. */
-      ssh_fatal("ssh_auths_call_method: unknown op %d", (int)op);
+      ssh_fatal("ssh_auths_completion_proc: unknown op %d", (int)op);
     }
 
   /* Free the packet buffer. */
-  ssh_buffer_free(buffer);
+  if (packet)
+    ssh_buffer_free(packet);
+}
+
+/* Calls the currently active authentication method with the current
+   parameters, and updates state according to the return value from
+   the authentication method.. */
+
+void ssh_auths_call_method(SshAuthServer auth,
+                           SshAuthServerOperation op,
+                           const unsigned char *data, size_t len)
+{
+  int i;
+  SshBuffer buffer;
+  SshAuthServerCompletionCtx completion_context;
+  
+  i = auth->active_method_index;
+
+  /* Sanity checks... */
+  SSH_ASSERT(i >= 0 && i < auth->num_methods);
+  SSH_ASSERT(auth->requested_user != NULL);
+  
+  if (auth->methods[i].proc == NULL)
+    {
+      SSH_DEBUG(3, ("Authentication method proc is NULL"));
+      return;
+    }
+
+  /* Save the data in a buffer. */
+  buffer = ssh_buffer_allocate();
+  ssh_buffer_append(buffer, data, len);
+
+  /* Allocate context for the completion proc. This has to be freed
+     later. */
+  completion_context = ssh_xcalloc(1, sizeof(*completion_context));
+  completion_context->auth = auth;
+  completion_context->op = op;
+  completion_context->method_index = i;
+  
+  /* Call the authentication method. */
+  /* Call the authentication method. */
+  (*auth->methods[i].proc)(op, auth->requested_user, auth->uc, buffer,
+                           auth->session_id, auth->session_id_len,
+                           &auth->state_placeholders[i],
+                           &auth->longtime_placeholders[i],
+                           ssh_auths_completion_proc,
+                           completion_context,
+                           auth->method_context);
 }
 
 /* Processes an incoming packet from the transport layer stream.  `packet'
@@ -486,10 +562,17 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       ch = user[i];
       if (isalnum(ch))
         continue;
-      if (ch == '-' || ch == '+' || ch == '_' || ch == '.')
+      if (ch == '-' || ch == '+' || ch == '_' || ch == '.' || ch == '%' ||
+          ch == '@')
         continue;
       if (ch >= 128) /* Assume 8-bit iso-latin characters. */
         continue;
+
+
+
+
+
+
       ssh_xfree(user);
       ssh_xfree(service);
       ssh_xfree(method);
@@ -501,6 +584,24 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
   SSH_DEBUG(6, ("process_request: user %.20s service %.20s method %.20s",
                 user, service, method));
 
+  if (auth->first_request)
+    {
+      auth->first_request = FALSE;
+      if (auth->banner_msg)
+        {
+          char *lang = "en"; /* XXX lang tag */
+          
+          ssh_cross_down_send_encode(auth->down, SSH_CROSS_PACKET,
+                                     SSH_FORMAT_CHAR,
+                                     (unsigned int) SSH_MSG_USERAUTH_BANNER,
+                                     SSH_FORMAT_UINT32_STR, auth->banner_msg,
+                                     strlen(auth->banner_msg),
+                                     SSH_FORMAT_UINT32_STR, lang,
+                                     strlen(lang),
+                                     SSH_FORMAT_END);
+        }
+    }
+  
   /* If the possible continuations haven't been computed yet, do it now. */
   if (auth->continuations == NULL)
     auth->continuations = (*auth->policy_proc)(user, service,
@@ -536,6 +637,9 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
           auth->requested_user = NULL;
           auth->requested_service = NULL;
         }
+      if (auth->uc)
+        ssh_user_free(auth->uc, TRUE);
+      auth->uc = NULL;
       /* Reset the possible continuations. */
       ssh_xfree(auth->continuations);
       auth->continuations = (*auth->policy_proc)(user, service,
@@ -557,6 +661,7 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
     {
       auth->requested_user = ssh_xstrdup(user);
       auth->requested_service = ssh_xstrdup(service);
+      auth->uc = ssh_user_initialize(auth->requested_user, TRUE);
     }
 
   /* Find the method. */
@@ -723,10 +828,13 @@ void ssh_auths_up_destroy(void *context)
   if (auth->algorithms_packet)
     ssh_buffer_free(auth->algorithms_packet);
   ssh_xfree(auth->requested_user);
+  if (auth->uc)
+    ssh_user_free(auth->uc, TRUE);
   ssh_xfree(auth->requested_service);
   ssh_xfree(auth->successful_authentications);
   ssh_xfree(auth->continuations);
   ssh_xfree(auth->disabled_authentications);
+  ssh_xfree(auth->banner_msg);
   
   /* Fill with known "garbage" value to ease debugging. */
   memset(auth, 'F', sizeof(*auth));
@@ -821,13 +929,22 @@ char *ssh_auths_default_policy_proc(const char *user, const char *service,
      `methods'              array of supported methods, terminated by
                             an element with NULL name.  This needs to stay
                             valid until the stream is destroyed.
-     `method_context'       context to pass to methods (normally NULL) */
+     `method_context'       context to pass to methods (normally NULL)
+     `banner'               If non-NULL, will be sent in an
+                            SSH_MSG_USERAUTH_BANNER to the client.
+     `uc_return'            if non-NULL, this will be set to the SshUser
+                            object for the authenticated user upon successful
+                            authentication.  If set, it will be the
+                            application's responsibility to free this object
+                            by calling ssh_user_free. */
 
 SshStream ssh_auth_server_wrap(SshStream transport,
                                SshAuthPolicyProc policy_proc,
                                void *policy_context,
-                               SshAuthServerMethod *methods,
-                               void *method_context)
+                               SshAuthServerMethodStruct *methods,
+                               void *method_context,
+                               const char *banner,
+                               SshUser *uc_return)
 {
   SshAuthServer auth;
   int num_methods;
@@ -867,10 +984,16 @@ SshStream ssh_auth_server_wrap(SshStream transport,
   auth->startup_packet = NULL;
   auth->algorithms_packet = NULL;
   auth->requested_user = NULL;
+  auth->uc = NULL;
   auth->requested_service = NULL;
   auth->successful_authentications = NULL;
   auth->disabled_authentications = ssh_xstrdup("");
   auth->continuations = NULL;
+  if (banner)
+    auth->banner_msg = ssh_xstrdup(banner);
+  auth->first_request = TRUE;
+  
+  auth->uc_return = uc_return;
 
   /* Start receiving packets from the down stream. */
   ssh_cross_down_can_receive(auth->down, TRUE);
