@@ -172,7 +172,7 @@ typedef struct
 {
   const char *name;
   SshConnGlobalRequestProc proc;
-} SshConnGlobalRequest;
+} *SshConnGlobalRequest, SshConnGlobalRequestStruct;
 
 /* Data structure for channel open functions.  An array of these is passed
    to the connection protocol.  The last entry in the array should have
@@ -181,7 +181,7 @@ typedef struct
 {
   const char *name;
   SshConnChannelOpenProc proc;
-} SshConnChannelOpen;
+} *SshConnChannelOpen, SshConnChannelOpenStruct;
 
 /* Type to represent the connection protocol. */
 typedef struct SshConnRec *SshConn;
@@ -209,8 +209,8 @@ typedef struct SshConnRec *SshConn;
    the destroy callback of some channel. */
 SshConn ssh_conn_wrap(SshStream auth_stream,
                       const char *service_name,
-                      SshConnGlobalRequest *requests,
-                      SshConnChannelOpen *opens,
+                      SshConnGlobalRequest requests,
+                      SshConnChannelOpen opens,
                       SshConnDisconnectProc disconnect,
                       SshConnDebugProc debug,
                       SshConnSpecialProc special,
@@ -387,4 +387,17 @@ void ssh_conn_channel_close(SshConn conn, int channel_id);
      `...'         varialble length SSH_FORMAT_* list. */
 void ssh_conn_send_encode(SshConn conn, SshCrossPacketType type, ...);
 
+/* Returns the number of current channels and the total number of payload
+   bytes that has been sent to and received from the other end through
+   all channels. Only payload bytes are counted, protocol-messages like rekey
+   packets are not counted in. The byte counters include bytes from current
+   active channels plus bytes from channels that have already been destroyed.
+   In other words, byte counters are cumulative through SshConn lifetime.
+
+   This function is useful for detecting connection idleness.   
+   Any of the integer pointers can be NULL.
+*/
+void ssh_conn_channel_statistic(SshConn conn, int *num_channels,
+                                SshUInt64 *sent_bytes,
+                                SshUInt64 *received_bytes);
 #endif /* SSHCONN_H */
