@@ -363,7 +363,7 @@ void ssh_cross_up_destroy(void *context)
 
 /* Methods table for the stream. */
 
-const SshStreamMethodsTable ssh_cross_up_methods =
+const SshStreamMethodsStruct ssh_cross_up_methods =
 {
   ssh_cross_up_read,
   ssh_cross_up_write,
