@@ -13,7 +13,7 @@
 */
 
 /*
- * $Id: sshmsgs.h,v 1.19 2000/05/22 10:38:07 sjl Exp $
+ * $Id: sshmsgs.h,v 1.21.2.1 2000/11/30 12:52:56 gahokas Exp $
  * $Log: sshmsgs.h,v $
  * $EndLog$
  */
@@ -76,12 +76,31 @@
 #define SSH_MSG_USERAUTH_CHALLENGE              60
 
 /* SecurID authentication */
-#define SSH_MSG_USERAUTH_SECURID_PINREQ         60
-#define SSH_MSG_USERAUTH_SECURID_PINREPLY       61
+#define SSH_MSG_USERAUTH_SECURID_CHALLENGE      60
+#define SSH_MSG_USERAUTH_SECURID_NEW_PIN_REQD   61
+
+/* SecurID new pin modes */
+#define SSH_SECURID_CANNOT_CHOOSE_PIN           0
+#define SSH_SECURID_MUST_CHOOSE_PIN             1
+#define SSH_SECURID_USER_SELECTABLE_PIN         2
 
 /* Public key authentication */
 #define SSH_MSG_USERAUTH_PK_OK                  60
 
+/* PAM authentication */
+#define SSH_MSG_USERAUTH_PAM_MSG                60
+
+/* PAM message styles. */
+#define SSH_PAM_PROMPT_ECHO_OFF                 65
+#define SSH_PAM_PROMPT_ECHO_ON                  66
+#define SSH_PAM_ERROR_MSG                       67
+#define SSH_PAM_TEXT_INFO                       68
+#define SSH_PAM_BINARY_PROMPT                   69
+#define SSH_PAM_BINARY_MSG                      70
+
+/* PAM response retcodes. */
+#define SSH_PAM_DEFAULT_RESP_RETCODE            0
+
 /* #define SSH_FIRST_USERAUTH_SERVICE_PACKET  30 */
 
 /* Packet numbers for the SSH connection protocol. */
