@@ -12,7 +12,7 @@ Test program for the SSH2 transport layer protocol.
 */
 
 /*
- * $Id: t-tr.c,v 1.25 2000/05/28 16:21:07 kivinen Exp $
+ * $Id: t-tr.c,v 1.27 2000/09/27 02:21:22 kivinen Exp $
  * $Log: t-tr.c,v $
  * $EndLog$
  */
@@ -642,7 +642,7 @@ out:
 void handler_callback(SshStreamNotification notification, void *context)
 {
   Handler c = context;
-  SshTransportStatistics stats;
+  SshTransportStatisticsStruct stats;
   Boolean cont = FALSE;
 
 #ifdef DEBUG
@@ -896,15 +896,16 @@ int main(int ac, char **av)
           ssh_debug("Making listener, port %s...", port);
 #endif
           listener = ssh_tcp_make_listener("127.0.0.1", port,
-                                              listener_callback,
-                                              (void *)testcase);
+                                           NULL, 
+                                           listener_callback,
+                                           (void *)testcase);
           if (!listener)
             ssh_fatal("making listener failed");
 #ifdef DEBUG
           ssh_debug("Making connect...");
 #endif
-          ssh_tcp_connect_with_socks("127.0.0.1", port, NULL, 2,
-                             connect_callback, (void *)testcase);
+          ssh_tcp_connect("127.0.0.1", port, NULL, 
+                          connect_callback, (void *)testcase);
 
 #ifdef DEBUG
           ssh_debug("Event loop running...");
