@@ -463,7 +463,7 @@ void clientonly_client(SshAuthClientOperation op,
     }
 }
 
-SshAuthServerMethod simple_server_methods[] =
+SshAuthServerMethodStruct simple_server_methods[] =
 {
   { "password1", simple_password_server },
   { "password2", simple_password_server },
@@ -471,7 +471,7 @@ SshAuthServerMethod simple_server_methods[] =
   { NULL }
 };
 
-SshAuthClientMethod simple_client_methods[] =
+SshAuthClientMethodStruct simple_client_methods[] =
 {
   { "password1", simple_password_client },
   { "password2", simple_password_client },
