@@ -16,7 +16,7 @@ trkex.c
 */
 
 /*
- * $Id: trkex.c,v 1.39 2000/05/28 16:21:05 kivinen Exp $
+ * $Id: trkex.c,v 1.42 2000/11/21 20:10:39 sjl Exp $
  * $Log: trkex.c,v $
  * $EndLog$
  */
@@ -166,7 +166,7 @@ SshCryptoStatus ssh_kexdh_make_group(SshTransportCommon tr,
 
 Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
 {
-  SshIntC t;
+  SshMPIntC t;
   SshBuffer buf;
 
   /* check that the public value is within the range */
@@ -239,32 +239,6 @@ Boolean ssh_kexdh_compute_h(SshTransportCommon tr)
       tr->session_identifier_len = tr->exchange_hash_len;
     }
 
-
-
-#if 0
-  ssh_debug("ssh_kexdh_compute_h (%s)", tr->server ? "server" : "client");
-  printf(" e = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_e);
-  printf("\n f = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_f);
-  printf("\n secret = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_secret);
-  printf("\n k = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_k);
-  printf("\n g = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_g);
-  printf("\n p = ");
-  ssh_mp_out_str(stdout, 16, tr->dh_p);
-
-  printf("\n -- exchange hash computed over -- \n");
-  buffer_dump(buf);
-  printf("\n h = \n");
-  ssh_debug_hexdump(0, tr->exchange_hash, tr->exchange_hash_len);
-  printf(" session_id = \n");
-  ssh_debug_hexdump(0, tr->session_identifier, tr->session_identifier_len);
-  printf("\n");
-#endif
-
   ssh_buffer_free(buf);
 
   return FALSE;
@@ -416,7 +390,6 @@ void ssh_kex_keycheck_callback(Boolean result, void *ctx)
 
   callback_context = tr->key_check_callback_context;
 
-  tr->key_check_returned = TRUE;
   tr->key_check_result = result;
 
   if (tr->key_check_result == FALSE)
@@ -596,7 +569,6 @@ void ssh_kexdh_client_input_kex2(SshTransportCommon tr, SshBuffer input,
 
 
   /* Check the host key */
-  tr->key_check_returned = FALSE;
   if (tr->key_check)
     {
       callback_context =
@@ -611,6 +583,8 @@ void ssh_kexdh_client_input_kex2(SshTransportCommon tr, SshBuffer input,
       callback_context->completion = finalize_callback;
       callback_context->input = input;
 
+      tr->received_state = RECEIVED_KEY_CHECK;
+      
       (*tr->key_check)(tr->server_host_name, pubkey, pubkey_len,
                        ssh_kex_keycheck_callback, tr, tr->key_check_context);
 
