@@ -6,12 +6,12 @@
 /*
  *        Program: sshreadline
  *        $Source: /ssh/CVS/src/lib/sshreadline/sshreadline.c,v $
- *        $Author: kivinen $
+ *        $Author: sjl $
  *
  *        Creation          : 19:47 Mar 12 1997 kivinen
  *        Last Modification : 18:05 May 28 2000 kivinen
- *        Last check in     : $Date: 2000/05/28 16:21:08 $
- *        Revision number   : $Revision: 1.29 $
+ *        Last check in     : $Date: 2000/09/14 23:39:25 $
+ *        Revision number   : $Revision: 1.30 $
  *        State             : $State: Exp $
  *        Version           : 1.668
  *
@@ -1121,7 +1121,7 @@ int ssh_rl_loop(ReadLine *rl)
                           {
                             rl->line[rl->end] = '\0';
                             /* Add the line if different */
-                            if (strcmp(rl->line, ssh_dllist_current(rl->cmd_history)))
+                            if (strcmp((char *)rl->line, ssh_dllist_current(rl->cmd_history)))
                               {
                                 if (rl->cmd_history_line_entered == FALSE)
                                   {
