@@ -20,36 +20,43 @@ Copyright (c) 1997 SSH Communications Security, Finland
 #include <libutil.h>
 #endif /* HAVE_LIBUTIL_H */
 
-#ifdef HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif /* HAVE_SYS_IOCTL_H */
-
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #define USING_TERMIOS
 #endif /* HAVE_TERMIOS_H */
 
+#ifdef HAVE_SYS_TERMIOS_H
+#include <sys/termios.h>
+#define USING_TERMIOS
+#endif /* HAVE_SYS_TERMIOS_H */
+
+#ifdef HAVE_SYS_IOCTL_H
+#include <sys/ioctl.h>
+#endif /* HAVE_SYS_IOCTL_H */
+
 #if defined(HAVE_SGTTY_H) && !defined(USING_TERMIOS)
 #include <sgtty.h>
 #define USING_SGTTY
-#endif
+#endif /* HAVE_SGTTY_H && !USING_TERMIOS */
 
+#ifndef WIN32
 #if !defined(USING_SGTTY) && !defined(USING_TERMIOS)
-  ERROR NO TERMIOS OR SGTTY
-#endif
+#error ERROR NO TERMIOS OR SGTTY;
+#endif /* !USING_SGTTY && !USING_TERMIOS */
+#endif /* WIN32 */
 
 /* Define UID_ROOT to be the user id for root (normally zero, but different
    e.g. on Amiga). */
 #ifndef UID_ROOT
 #define UID_ROOT 0
 #endif
- 
+
 #if HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
-#endif
+#endif /* HAVE_SYS_RESOURCE_H */
 
 #ifdef _HPUX_SOURCE
 #define seteuid(uid) setresuid(-1,(uid),-1)
-#endif
+#endif /* _HPUX_SOURCE */
 
 #endif /* SSHSESSIONINCLUDES_H */
