@@ -16,7 +16,7 @@ and return streams for them in the parent.
 */
 
 /*
- * $Id: sshunixpipestream.c,v 1.5 1999/06/28 13:41:52 tmo Exp $
+ * $Id: sshunixpipestream.c,v 1.6 2000/06/14 00:43:03 kivinen Exp $
  * $Log: sshunixpipestream.c,v $
  * $EndLog$
  */
@@ -38,7 +38,7 @@ typedef struct {
   int exit_status;
 } *SshPipeStream;
 
-extern const SshStreamMethodsTable ssh_pipe_methods;
+extern const SshStreamMethodsStruct ssh_pipe_methods;
 
 /* This callback is called after the parent has exited; this will
    call the application callback so that it gets notified of the EOF
@@ -310,7 +310,7 @@ void ssh_pipe_stream_destroy(void *context)
   ssh_xfree(pipes);
 }
 
-const SshStreamMethodsTable ssh_pipe_methods = {
+const SshStreamMethodsStruct ssh_pipe_methods = {
   ssh_pipe_stream_read,
   ssh_pipe_stream_write,
   ssh_pipe_stream_output_eof,
