@@ -10,7 +10,7 @@ Copyright (c) 1997 SSH Communications Security, Finland
 */
 
 /*
- * $Id: sshunixptystream.c,v 1.8 2000/03/28 18:24:48 sjl Exp $
+ * $Id: sshunixptystream.c,v 1.9 2000/06/14 00:43:04 kivinen Exp $
  * $Log: sshunixptystream.c,v $
  * $EndLog$
  */
@@ -45,7 +45,7 @@ typedef struct {
   int exit_status;
 } *PtyStream;
 
-extern const SshStreamMethodsTable ssh_pty_methods;
+extern const SshStreamMethodsStruct ssh_pty_methods;
 
 /* This callback is called after the parent has exited; this will
    call the application callback so that it gets notified of the EOF
@@ -541,7 +541,7 @@ void ssh_pty_stream_destroy(void *context)
   ssh_xfree(pty);
 }
 
-const SshStreamMethodsTable ssh_pty_methods = {
+const SshStreamMethodsStruct ssh_pty_methods = {
   ssh_pty_stream_read,
   ssh_pty_stream_write,
   ssh_pty_stream_output_eof,
