@@ -16,6 +16,8 @@ Manipulating user information in SSH server (mostly password validation).
 
 typedef struct SshUserRec *SshUser;
 
+typedef struct SshGroupRec *SshGroup;
+
 /* Allocates and initializes a context for the user.  The context is used
    to cache information about the particular user.  Returns NULL if the
    user does not exist. If `user' is NULL, use getuid() to get the current
@@ -24,10 +26,11 @@ typedef struct SshUserRec *SshUser;
    try to look for shadow passwords etc.*/
 SshUser ssh_user_initialize(const char *user, Boolean privileged);
 
-#ifndef WINDOWS
+
 /* As above, but we explicitely want to use uid (instead of name). */
 SshUser ssh_user_initialize_with_uid(uid_t uid, Boolean privileged);
-#endif /* WINDOWS */
+
+
 /* Frees information about the user.  If ``undo'' is TRUE, undoes any
    cached state related to e.g. Kerberos and Secure RPC.  Returns
    FALSE if undo was requested, but was unable to undo everything; otherwise
@@ -38,6 +41,7 @@ Boolean ssh_user_free(SshUser uc, Boolean undo);
 Boolean ssh_user_login_is_allowed(SshUser uc);
 
 /* Returns TRUE if login is allowed with the given local password. */
+#ifdef SSHDIST_SESSION_SIA
 #ifdef HAVE_SIA
 Boolean ssh_user_validate_local_password(SshUser uc,
                                          const char *password,
@@ -48,6 +52,9 @@ Boolean ssh_user_validate_local_password(SshUser uc,
 #endif /* HAVE_SIA */
 
 
+
+#endif /* SSHDIST_SESSION_SIA */
+
 /* Returns TRUE if the user's password needs to be changed. */
 Boolean ssh_user_password_must_be_changed(SshUser uc,
                                           char **prompt_return);
@@ -77,6 +84,7 @@ Boolean ssh_user_validate_secure_rpc_password(SshUser uc,
    The return value of this function MUST BE CHECKED! */
 Boolean ssh_user_become(SshUser uc);
 
+#ifdef SSHDIST_SESSION_SIA
 #ifdef HAVE_SIA
 /* Last chance to finish anything that ssh_user_become() left undone.  The
    difference between the two functions is that ssh_user_become() is called
@@ -90,12 +98,11 @@ Boolean ssh_user_become_real(SshUser uc,
                              const char *remote_host,
                              const char *ttyname);
 #endif /* HAVE_SIA */
+#endif /* SSHDIST_SESSION_SIA */
 
 /* Returns the login name of the user. */
 const char *ssh_user_name(SshUser uc);
 
-#ifndef WINDOWS
-
 /* Returns the uid of the user.  This is unix-specific. */
 uid_t ssh_user_uid(SshUser uc);
 
@@ -105,10 +112,40 @@ gid_t ssh_user_gid(SshUser uc);
 /* Returns the user's home directory.  This is unix-specific. */
 const char *ssh_user_dir(SshUser uc);
 
+
+
+
+/* Returns the group name of the user. */
+const char *ssh_group_name(SshUser uc);
+
 /* Returns the shell of the user.  This is unix-specific. */
 const char *ssh_user_shell(SshUser uc);
 
-#endif /* WINDOWS */
+/* Returns the kerberos name of the user.  If kerberos is not supported,
+   this returns the user's login name. */
+const char *ssh_user_kerberos_name(SshUser uc);
+
+/* Get the groups the user is in. Array is terminated with a NULL
+   pointer. */
+SshGroup *ssh_user_get_groups(SshUser uc);
+
+/* Return the gid. */
+gid_t ssh_group_get_gid(SshGroup group);
+
+/* Return the name. */
+const char *ssh_group_get_name(SshGroup group);
+
+#ifdef KERBEROS
+/* Sets Kerberos credentials to be saved for the user.  The argument should
+   be of type "krb5_creds *"; this will copy the creds into internal
+   data structures. */
+void ssh_user_kerberos_set_creds(SshUser uc, void *creds);
+#endif /* KERBEROS */
+
+#ifdef DAEMON_WITH_PAM
+/* Set PAM handle to be used when logging user out. */
+void ssh_user_pam_set_handle(SshUser uc, void *pam_handle);
+#endif /* DAEMON_WITH_PAM */
 
 /* Returns the time when the user last logged in, and name of the host
    from which the user logged in from.  Returns 0 if the information
@@ -127,11 +164,49 @@ SshTime ssh_user_get_last_login_time(SshUser user,
       ttyname name of the user's tty (slave side)
       host    name of the host the user logged in from (ip if host not known)
       ip      ip address of the host the user logged in from. */
-
 void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
                            const char *host, const char *ip);
 
 /* Records that the user on the tty has logged out. */
 void ssh_user_record_logout(pid_t pid, const char *ttyname);
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* SSHUSER_H */
