@@ -17,7 +17,7 @@
 
 #define SSH_DEBUG_MODULE "SshADT"
 
-/********************************************************** Default methods. */
+/******************************************************** Default callbacks. */
 
 int ssh_adt_default_compare(const void *obj1, const void *obj2,
                             void *context)
@@ -73,24 +73,19 @@ static void set_default_values(SshADTStandardFields *f)
   f->app_methods.context = NULL;
 }
 
-static void init_toplevel_container(SshADTContainer c,
-                                    SshADTContainerType type,
-                                    va_list args)
+static Boolean init_toplevel_container(SshADTContainer c,
+                                       SshADTContainerType type,
+                                       va_list args)
 {
   SshADTArgumentType t;
-  SshADTParameters pars;
-  SshADTContainerPars *ptr;
+  SshADTContainerPars pars, *ptr;
   SshADTStaticData *static_data;
 
   SSH_PRECOND(type != NULL);
 
   memset(&pars, 0, sizeof(pars));
-
-  set_default_values(&(pars.top.f));
-  set_default_values(&(pars.domain.f));
-  set_default_values(&(pars.range.f));
-
-  ptr = &(pars.top);
+  set_default_values(&(pars.f));
+  ptr = &(pars);
 
   ptr->type = type;
 
@@ -134,13 +129,13 @@ static void init_toplevel_container(SshADTContainer c,
         case SSH_ADT_FASTALLOC:
           (void)va_arg(args, int);
           break;
-          
+
         case SSH_ADT_SIZE_HINT:
           (void)va_arg(args, long);
           break;
-          
+
         case SSH_ADT_HEADER:
-          ptr->flags |= SSH_ADT_FLAG_CONTAINED_HEADER;     
+          ptr->flags |= SSH_ADT_FLAG_CONTAINED_HEADER;
           ptr->f.header_offset = va_arg(args, SshUInt32);
           break;
 
@@ -151,11 +146,12 @@ static void init_toplevel_container(SshADTContainer c,
 
 #ifdef _KERNEL
   SSH_ASSERT(ptr->flags & SSH_ADT_FLAG_CONTAINED_HEADER);
+  SSH_ASSERT(!(ptr->flags & SSH_ADT_FLAG_ALLOCATE));
 #endif
 
-  static_data = pars.top.type;
-  
-  (*(static_data->methods.container_init))(c, ptr);
+  static_data = pars.type;
+
+  return ((*(static_data->methods.container_init))(c, ptr));
 }
 
 SshADTContainer ssh_adt_create_generic(SshADTContainerType type, ...)
@@ -163,15 +159,12 @@ SshADTContainer ssh_adt_create_generic(SshADTContainerType type, ...)
   va_list args;
   SshADTContainer c;
 
-#ifdef _KERNEL
-  if (!(c = ssh_kmalloc(sizeof(*c))))
+  if (!(c = SSH_MEM_ALLOC(sizeof(*c))))
     return NULL;
-#else /* _KERNEL */
-  c = ssh_xmalloc(sizeof(*c));
-#endif /* _KERNEL */
 
   va_start(args, type);
-  init_toplevel_container(c, type, args);
+  if (init_toplevel_container(c, type, args) == FALSE)
+    return NULL;
   va_end(args);
 
   return c;
@@ -179,12 +172,9 @@ SshADTContainer ssh_adt_create_generic(SshADTContainerType type, ...)
 
 void ssh_adt_destroy(SshADTContainer container)
 {
+  SSH_PRECOND(container);
   (*(container->static_data->methods.destr))(container);
-#ifdef _KERNEL
-  ssh_kfree(container);
-#else /* _KERNEL */
-  ssh_xfree(container);
-#endif /* _KERNEL */
+  SSH_MEM_FREE(container);
 }
 
 #if 0
@@ -202,7 +192,7 @@ void ssh_adt_uninit(SshADTContainer container)
 {
   SSH_ADT_CALL_DESTROY_HOOK(container, destr);
   SSH_ADT_CALL(container, FALSE, container_uninit, (container));
-  ssh_xfree(container->hooks);
+  SSH_MEM_FREE(container->hooks);
 }
 #endif
 
@@ -210,26 +200,12 @@ void ssh_adt_uninit(SshADTContainer container)
 
 void *ssh_adt_duplicate_object(SshADTContainer container, void *object)
 {
-  return
-    SSH_ADT_CALL_APP_MANDATORY(container, duplicate,
-                               (object, SSH_ADT_APPCTX(container)));
+  void *result;
+  SSH_ADT_CALL_APP_MANDATORY(container, duplicate,
+                             (object, SSH_ADT_APPCTX(container)), result);
+  return result;
 }
 
-/* Should be included in the shortcuts. */
-#if 0
-void *ssh_adt_detach_object(SshADTContainer container, void *object)
-{
-  SshADTHandle handle = ssh_adt_get_handle_to(container, object);
-  return ssh_adt_detach(container, handle);
-}
-
-void ssh_adt_delete_object(SshADTContainer container, void *object)
-{
-  SshADTHandle handle = ssh_adt_get_handle_to(container, object);
-  ssh_adt_delete(container, handle);
-}
-#endif
-
 /************************* Genlist functions, for some internal use perhaps. */
 
 void ssh_adt_genlist_merge(void *from_list, void **to_list,
@@ -254,26 +230,29 @@ void ssh_adt_genlist_remove(void **listptr, void *object, int next_ptr_offset)
     {
       listptr = (void **)((unsigned char *)(*listptr) + next_ptr_offset);
     }
-  *listptr = *((void **)((unsigned char *)object + next_ptr_offset));  
+  *listptr = *((void **)((unsigned char *)object + next_ptr_offset));
 }
 
 /******************************************************************** Hooks. */
 
-void ssh_adt_initialize_hooks(SshADTContainer container)
+int ssh_adt_initialize_hooks(SshADTContainer container)
 {
-  ssh_xfree(container->hooks);
-  container->hooks = ssh_xmalloc(sizeof(*container->hooks));
+  SSH_MEM_FREE(container->hooks);
+  if (!(container->hooks = SSH_MEM_ALLOC(sizeof(*container->hooks))))
+    return 1;
+
   container->hooks->insert      = NULL;
   container->hooks->detach      = NULL;
   container->hooks->map         = NULL;
   container->hooks->unmap       = NULL;
   container->hooks->reallocated = NULL;
   container->hooks->destr       = NULL;
+  return 0;
 }
 
 void ssh_adt_uninitialize_hooks(SshADTContainer container)
 {
-  ssh_xfree(container->hooks);
+  SSH_MEM_FREE(container->hooks);
   container->hooks = NULL;
 }
 
@@ -314,7 +293,9 @@ SshADTHandle ssh_adt_insert_at(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_at);
   return ssh_adt_insert_at__(container, location, handle, object);
 }
 
@@ -324,7 +305,9 @@ SshADTHandle ssh_adt_insert_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_to);
   return ssh_adt_insert_to__(container, location, object);
 }
 
@@ -333,7 +316,9 @@ SshADTHandle ssh_adt_insert(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_to);
   return ssh_adt_insert_to(container, SSH_ADT_DEFAULT, object);
 }
 
@@ -344,7 +329,9 @@ SshADTHandle ssh_adt_duplicate_at(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_at);
   return ssh_adt_insert_at(container, location, handle,
                            ssh_adt_duplicate_object(container, object));
 }
@@ -355,7 +342,9 @@ SshADTHandle ssh_adt_duplicate_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_to);
   return ssh_adt_insert_to(container, location,
                            ssh_adt_duplicate_object(container, object));
 }
@@ -365,7 +354,9 @@ SshADTHandle ssh_adt_duplicate(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.insert_to);
   return ssh_adt_insert(container,
                         ssh_adt_duplicate_object(container, object));
 }
@@ -380,6 +371,7 @@ SshADTHandle ssh_adt_alloc_n_at(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.alloc_n_at);
   return ssh_adt_alloc_n_at__(container, location, handle, size);
 }
 
@@ -389,6 +381,7 @@ SshADTHandle ssh_adt_alloc_n_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
+  SSH_ASSERT(container->static_data->methods.alloc_n_to);
   return ssh_adt_alloc_n_to__(container, location, size);
 }
 
@@ -399,6 +392,7 @@ SshADTHandle ssh_adt_alloc_at(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.alloc_n_at);
   return ssh_adt_alloc_n_at(container, location,
                             handle, SSH_ADT_DEFAULT_SIZE(container));
 }
@@ -408,6 +402,7 @@ SshADTHandle ssh_adt_alloc_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
+  SSH_ASSERT(container->static_data->methods.alloc_n_to);
   return ssh_adt_alloc_n_to(container, location,
                             SSH_ADT_DEFAULT_SIZE(container));
 }
@@ -417,6 +412,7 @@ SshADTHandle ssh_adt_alloc_n(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
+  SSH_ASSERT(container->static_data->methods.alloc_n_to);
   return ssh_adt_alloc_n_to(container, SSH_ADT_DEFAULT, size);
 }
 
@@ -424,8 +420,10 @@ SshADTHandle ssh_adt_alloc(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
+  SSH_ASSERT(container->static_data->methods.alloc_n_to);
   return ssh_adt_alloc_n(container, SSH_ADT_DEFAULT_SIZE(container));
 }
+
 SshADTHandle ssh_adt_put_n_at(SshADTContainer container,
                               SshADTRelativeLocation location,
                               SshADTHandle handle,
@@ -436,6 +434,7 @@ SshADTHandle ssh_adt_put_n_at(SshADTContainer container,
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_at);
   return ssh_adt_put_n_at__(container, location, handle, size, object);
 }
 
@@ -447,6 +446,7 @@ SshADTHandle ssh_adt_put_n_to(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_to);
   return ssh_adt_put_n_to__(container, location, size, object);
 }
 
@@ -459,6 +459,7 @@ SshADTHandle ssh_adt_put_at(SshADTContainer container,
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_at);
   return ssh_adt_put_n_at(container, location, handle,
                           SSH_ADT_DEFAULT_SIZE(container), object);
 }
@@ -470,6 +471,7 @@ SshADTHandle ssh_adt_put_to(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_to);
   return ssh_adt_put_n_to(container, location,
                           SSH_ADT_DEFAULT_SIZE(container), object);
 }
@@ -481,6 +483,7 @@ SshADTHandle ssh_adt_put_n(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_to);
   return ssh_adt_put_n_to(container, SSH_ADT_DEFAULT, size, object);
 }
 
@@ -490,6 +493,7 @@ SshADTHandle ssh_adt_put(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.put_n_to);
   return ssh_adt_put_n(container, SSH_ADT_DEFAULT_SIZE(container), object);
 }
 
@@ -497,12 +501,14 @@ void *ssh_adt_get(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.get);
   return ssh_adt_get__(container, handle);
 }
 
 size_t ssh_adt_num_objects(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
+  SSH_ASSERT(container->static_data->methods.num_objects);
   return ssh_adt_num_objects__(container);
 }
 
@@ -510,14 +516,21 @@ SshADTHandle ssh_adt_get_handle_to(SshADTContainer container,
                                    void *object)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
+                          SSH_ADT_FLAG_ALLOCATE))
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.get_handle_to);
   return ssh_adt_get_handle_to__(container, object);
 }
 
-SshADTHandle ssh_adt_get_handle_to_equal(SshADTContainer container, SshADTHandle object)
+SshADTHandle ssh_adt_get_handle_to_equal(SshADTContainer container,
+                                         SshADTHandle object)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
+                          SSH_ADT_FLAG_ALLOCATE))
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.get_handle_to_equal);
   return ssh_adt_get_handle_to_equal__(container, object);
 }
 
@@ -525,6 +538,7 @@ SshADTHandle ssh_adt_next(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.next);
   return ssh_adt_next__(container, handle);
 }
 
@@ -532,6 +546,7 @@ SshADTHandle ssh_adt_previous(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.previous);
   return ssh_adt_previous__(container, handle);
 }
 
@@ -539,14 +554,25 @@ SshADTHandle ssh_adt_get_handle_to_location(SshADTContainer container,
                                             SshADTAbsoluteLocation location)
 {
   SSH_ADT_ASSERT_CONTAINER;
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
   return ssh_adt_get_handle_to_location__(container, location);
 }
 
+void *ssh_adt_get_object_from_location(SshADTContainer container,
+                                       SshADTAbsoluteLocation location)
+{
+  SSH_ADT_ASSERT_CONTAINER;
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
+  return ssh_adt_get(container,
+                     ssh_adt_get_handle_to_location(container, location));
+}
+
 void *ssh_adt_detach(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.detach);
   return ssh_adt_detach__(container, handle);
 }
 
@@ -566,6 +592,8 @@ void *ssh_adt_detach_from(SshADTContainer container,
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
+  SSH_ASSERT(container->static_data->methods.detach);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
   handle = ssh_adt_get_handle_to_location(container, location);
   SSH_ADT_ASSERT_HANDLE;
   return ssh_adt_detach(container, handle);
@@ -577,7 +605,11 @@ void *ssh_adt_detach_object(SshADTContainer container,
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
+                          SSH_ADT_FLAG_ALLOCATE))
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.detach);
+  SSH_ASSERT(container->static_data->methods.get_handle_to);
   handle = ssh_adt_get_handle_to(container, object);
   SSH_ADT_ASSERT_HANDLE;
   return ssh_adt_detach(container, handle);
@@ -587,6 +619,7 @@ void ssh_adt_delete(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.delet);
   ssh_adt_delete__(container, handle);
 }
 
@@ -595,6 +628,8 @@ void ssh_adt_delete_from(SshADTContainer container,
 {
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
+  SSH_ASSERT(container->static_data->methods.delet);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
   handle = ssh_adt_get_handle_to_location(container, location);
   SSH_ADT_ASSERT_HANDLE;
   ssh_adt_delete(container, handle);
@@ -605,7 +640,11 @@ void ssh_adt_delete_object(SshADTContainer container,
 {
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ADT_ASSERT_OBJECT;
+  if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
+                          SSH_ADT_FLAG_ALLOCATE))
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.delet);
+  SSH_ASSERT(container->static_data->methods.get_handle_to);
   handle = ssh_adt_get_handle_to(container, object);
   SSH_ADT_ASSERT_HANDLE;
   ssh_adt_delete(container, handle);
@@ -614,6 +653,7 @@ void ssh_adt_delete_object(SshADTContainer container,
 SshADTHandle ssh_adt_enumerate_start(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
+  SSH_ASSERT(container->static_data->methods.enumerate_start);
   return ssh_adt_enumerate_start__(container);
 }
 
@@ -622,7 +662,90 @@ SshADTHandle ssh_adt_enumerate_next(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.enumerate_next);
   return ssh_adt_enumerate_next__(container, handle);
 }
 
-#endif
+#endif /* !SSH_ADT_WITH_MACRO_INTERFACE */
+
+/************************************************* duplication of containers */
+
+/* This creates a new container from an old one, independently of the
+   latters class.  It is not implemented as a method, so it should
+   work on all new container classes immediately.  It simply walks
+   through an enumeration loop and inserts all elements to the default
+   location.
+
+   The hooks and callbacks are copied as function pointers, so doing
+   weird stuff with static variables might lead to unexpected results.
+   Callback context is set to NULL, hook contexts are shared.  */
+
+
+#define FAIL(a)                         \
+do                                      \
+{                                       \
+ SSH_DEBUG(3, ("failed: %s.", a));      \
+ SSH_MEM_FREE(c2);                      \
+ return NULL;                           \
+}                                       \
+while(0)
+
+
+SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
+{
+  SshADTContainer c2;
+  SshADTContainerPars pars;
+  SshADTHandle h;
+
+  /* init a new container */
+  if (!(c2 = SSH_MEM_ALLOC(sizeof(*c2))))
+    return NULL;
+
+  /* init pars (the structure is defined in sshadt_i.h) */
+
+  /* class type & information (including methods) */
+  pars.type = (SshADTContainerType) c1->static_data;
+
+  /* the instance flags */
+  pars.flags = c1->flags;
+
+  /* callbacks and memory management setting (context is always set to NULL) */
+  pars.f = c1->f;
+  pars.f.app_methods.context = NULL;
+
+  /* call the init function of the appropriate type */
+  if ((*(((SshADTStaticData *) pars.type)->methods.container_init))(c2, &pars)
+      == FALSE)
+    FAIL("create");
+
+  /* hooks */
+
+  c2->hooks = c1->hooks;
+  /* c2->hooks->reallocated_ctx = NULL; */
+  /* c2->hooks->destr_ctx = NULL; */
+
+  /* copy elements */
+
+  if ((h = ssh_adt_enumerate_start(c1)) != SSH_ADT_INVALID)
+    {
+      do {
+        if (c1->flags & SSH_ADT_FLAG_ALLOCATE)      /* liballoc */
+          {
+            void *o = ssh_adt_get(c1, h);
+            SshADTHandle b = ssh_adt_put(c2, o);
+            if (b == SSH_ADT_INVALID) FAIL("insert_liballoc");
+          }
+        else                                        /* voidptr */
+          {
+            void *o = ssh_adt_get(c1, h);
+            SshADTHandle h = ssh_adt_duplicate(c2, o);
+            if (h == SSH_ADT_INVALID) FAIL("insert_voidptr");
+          }
+      }
+      while ((h = ssh_adt_enumerate_next(c1, h)) != SSH_ADT_INVALID);
+    }
+
+  return c2;
+}
+
+#undef FAIL
