@@ -27,6 +27,6 @@ void ssh_adt_list_sort(SshADTContainer c);
 /* Type for inlined list headers. */
 typedef struct {
   void *a, *b;
-} SshADTListHeaderRec;
+} SshADTListHeaderStruct;
 
 #endif /* SSH_ADT_LIST_H_INCLUDED */
