@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999-2000 SSH Communications Security, Finland
   All rights reserved.
 
   Created Tue Oct 19 16:02:07 1999.
@@ -14,14 +14,15 @@
 #ifndef SSHADT_STD_I_H_INCLUDED
 #define SSHADT_STD_I_H_INCLUDED
 
+
 #define SSH_ADT_STD_COMPARE(container, h1, h2, result)                        \
 do                                                                            \
 {                                                                             \
   void *o1, *o2;                                                              \
   SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(container, (SshADTHandle)h1, o1);        \
   SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(container, (SshADTHandle)h2, o2);        \
-  result = SSH_ADT_CALL_APP_MANDATORY(container, compare,                     \
-                                      (o1, o2, SSH_ADT_APPCTX(container)));   \
+  SSH_ADT_CALL_APP_MANDATORY(container, compare,                              \
+                             (o1, o2, SSH_ADT_APPCTX(container)), result);    \
 }                                                                             \
 while(0)
 
@@ -30,8 +31,8 @@ do                                                                            \
 {                                                                             \
   void *o1;                                                                   \
   SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(container, (SshADTHandle)h1, o1);        \
-  result = SSH_ADT_CALL_APP_MANDATORY(container, compare,                     \
-                                      (o1, o2, SSH_ADT_APPCTX(container)));   \
+  SSH_ADT_CALL_APP_MANDATORY(container, compare,                              \
+                             (o1, o2, SSH_ADT_APPCTX(container)), result);    \
 }                                                                             \
 while(0)
 
@@ -40,8 +41,8 @@ do                                                                            \
 {                                                                             \
   void *o1;                                                                   \
   SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(container, (SshADTHandle)h1, o1);        \
-  result = SSH_ADT_CALL_APP_MANDATORY(container, hash,                        \
-                                      (o1, SSH_ADT_APPCTX(container)));       \
+  SSH_ADT_CALL_APP_MANDATORY(container, hash,                                 \
+                             (o1, SSH_ADT_APPCTX(container)), result);        \
 }                                                                             \
 while(0)
 
@@ -50,14 +51,14 @@ do                                                                            \
 {                                                                             \
   if ((container)->flags & SSH_ADT_FLAG_NEED_EXTRA_NODES)                     \
     {                                                                         \
-      unsigned char *__ptr = ((SshADTHandle)(handle));                        \
-      __ptr -= sizeof(void *);                                                \
-      result = *((void **)__ptr);                                             \
+      unsigned char *__ptr2 = ((SshADTHandle)(handle));                       \
+      __ptr2 -= sizeof(void *);                                               \
+      result = *((void **)__ptr2);                                            \
     }                                                                         \
   else                                                                        \
     {                                                                         \
       result = ((unsigned char *)((SshADTHandle)(handle))) -                  \
-        (container)->ssh_adt_stdf.header_offset;                              \
+        (container)->f.header_offset;                                         \
     }                                                                         \
 }                                                                             \
 while(0)
@@ -69,31 +70,41 @@ do                                                                            \
     {                                                                         \
       unsigned char *__ptr;                                                   \
       SSH_ADT_STD_GET_OBJECT_FROM_HANDLE((container), (handle), __ptr);       \
-      ssh_xfree(__ptr);                                                       \
+      SSH_MEM_FREE(__ptr);                                                    \
     }                                                                         \
   else                                                                        \
     {                                                                         \
-      ssh_xfree(handle);                                                      \
+      SSH_MEM_FREE(handle);                                                   \
     }                                                                         \
 }                                                                             \
 while(0)
 
+#ifdef _KERNEL
+#define SSH_ADT_STD_ALLOC_I(container, size, result)                          \
+do                                                                            \
+{                                                                             \
+  result = NULL;                                                              \
+  ssh_fatal("SSH_ADT_STD_ALLOC_I(): memory mode not supported in _KERNEL");   \
+}                                                                             \
+while(0)
+#else /* _KERNEL */
 #define SSH_ADT_STD_ALLOC_I(container, size, result)                          \
 do                                                                            \
 {                                                                             \
   unsigned char *__ptr;                                                       \
   if (!((container)->flags & SSH_ADT_FLAG_CONTAINED_HEADER))                  \
     {                                                                         \
-      __ptr = ssh_xmalloc((size) +                                            \
+      __ptr = SSH_MEM_ALLOC((size) +                                          \
                           (container)->static_data->internal_header_size);    \
-      result = __ptr - (container)->ssh_adt_stdf.header_offset;               \
+      result = __ptr - (container)->f.header_offset;                          \
     }                                                                         \
   else                                                                        \
     {                                                                         \
-      result = ssh_xmalloc(size);                                             \
+      result = SSH_MEM_ALLOC(size);                                           \
     }                                                                         \
 }                                                                             \
 while(0)
+#endif /* _KERNEL */
 
 #define SSH_ADT_STD_MAKE_HANDLE(container, object, block, result)             \
 do                                                                            \
@@ -102,8 +113,15 @@ do                                                                            \
     {                                                                         \
       SshADTHandle __handle;                                                  \
       SSH_ADT_PROTECT(block);                                                 \
-      *(((void **)__handle)) = object;                                        \
-      result = (unsigned char *)__handle + sizeof(void *);                    \
+      if (__handle != NULL)                                                   \
+        {                                                                     \
+          *(((void **)__handle)) = object;                                    \
+          result = (unsigned char *)__handle + sizeof(void *);                \
+        }                                                                     \
+      else                                                                    \
+        {                                                                     \
+          result = NULL;                                                      \
+        }                                                                     \
     }                                                                         \
   else                                                                        \
     {                                                                         \
@@ -115,13 +133,13 @@ while(0)
 /******************************************************** Function skeletons */
 
 #define SSH_ADT_STD_INIT(name, block)                                         \
-static void name(SshADTContainer c, SshADTContainerPars *pars)                \
+static Boolean name(SshADTContainer c, SshADTContainerPars *pars)             \
 {                                                                             \
   c->static_data = pars->type;                                                \
   c->flags       = pars->flags;                                               \
   c->hooks       = NULL;                                                      \
                                                                               \
-  memcpy(&(c->ssh_adt_stdf), &(pars->f), sizeof(c->ssh_adt_stdf));            \
+  memcpy(&(c->f), &(pars->f), sizeof(c->f));                                  \
                                                                               \
   if (!(c->flags & (SSH_ADT_FLAG_ALLOCATE                                     \
                     | SSH_ADT_FLAG_CONTAINED_HEADER)))                        \
@@ -130,12 +148,12 @@ static void name(SshADTContainer c, SshADTContainerPars *pars)                \
   if ((c->flags & SSH_ADT_FLAG_ALLOCATE) &&                                   \
       !(c->flags & SSH_ADT_FLAG_CONTAINED_HEADER))                            \
     {                                                                         \
-      c->ssh_adt_stdf.header_offset =                                         \
+      c->f.header_offset =                                                    \
         -((SshInt32)(c->static_data->internal_header_size));                  \
     }                                                                         \
                                                                               \
-  c->ssh_adt_aux.num_objects = 0;                                             \
-  SSH_ADT_PROTECT(block);                                                     \
+  c->aux.num_objects = 0;                                                     \
+  return (block);                                                             \
 }
 
 #define SSH_ADT_STD_DESTROY(name, block)                                      \
@@ -143,7 +161,8 @@ static void name(SshADTContainer c)                                           \
 {                                                                             \
   SSH_ADT_CALL_DESTROY_HOOK(c, destr);                                        \
   SSH_ADT_PROTECT(block);                                                     \
-  ssh_xfree(c->hooks);                                                        \
+  if (c->flags & (SSH_ADT_FLAG_CONTAINED_HEADER | SSH_ADT_FLAG_ALLOCATE))     \
+    SSH_MEM_FREE(c->hooks);                                                   \
 }
 
 #define SSH_ADT_STD_INSERT_AT(name, block, make_node_block)                   \
@@ -154,8 +173,17 @@ static SshADTHandle name(SshADTContainer c,                                   \
 {                                                                             \
   SshADTHandle h;                                                             \
   SSH_ADT_STD_MAKE_HANDLE(c, object, make_node_block, h);                     \
-  SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  if (h == SSH_ADT_INVALID)                                                   \
+    {                                                                         \
+      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      return SSH_ADT_INVALID;                                                 \
+    }                                                                         \
+  if (block == FALSE)                                                         \
+    {                                                                         \
+      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      return SSH_ADT_INVALID;                                                 \
+    }                                                                         \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -167,8 +195,17 @@ static SshADTHandle name(SshADTContainer c,                                   \
 {                                                                             \
   SshADTHandle h;                                                             \
   SSH_ADT_STD_MAKE_HANDLE(c, object, make_node_block, h);                     \
-  SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  if (h == SSH_ADT_INVALID)                                                   \
+    {                                                                         \
+      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      return SSH_ADT_INVALID;                                                 \
+    }                                                                         \
+  if (block == FALSE)                                                         \
+    {                                                                         \
+      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      return SSH_ADT_INVALID;                                                 \
+    }                                                                         \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -180,13 +217,13 @@ static SshADTHandle name(SshADTContainer c,                                   \
                          size_t size)                                         \
 {                                                                             \
   SshADTHandle h;                                                             \
-  void *newp;                                                                  \
+  void *newp;                                                                 \
                                                                               \
-  SSH_ADT_STD_ALLOC_I(c, size, newp);                                          \
-  SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                  \
-  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                     \
+  SSH_ADT_STD_ALLOC_I(c, size, newp);                                         \
+  SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                 \
+  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -197,13 +234,13 @@ static SshADTHandle name(SshADTContainer c,                                   \
                   size_t size)                                                \
 {                                                                             \
   SshADTHandle h;                                                             \
-  void *newp;                                                                  \
+  void *newp;                                                                 \
                                                                               \
-  SSH_ADT_STD_ALLOC_I(c, size, newp);                                          \
-  SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                  \
-  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                     \
+  SSH_ADT_STD_ALLOC_I(c, size, newp);                                         \
+  SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                 \
+  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -216,14 +253,14 @@ static SshADTHandle name(SshADTContainer c,                                   \
                          void *object)                                        \
 {                                                                             \
   SshADTHandle h;                                                             \
-  void *newp;                                                                  \
+  void *newp;                                                                 \
                                                                               \
-  SSH_ADT_STD_ALLOC_I(c, size, newp);                                          \
-  SSH_ADT_CALL_APP(c, copy, (newp, size, object,                               \
+  SSH_ADT_STD_ALLOC_I(c, size, newp);                                         \
+  SSH_ADT_CALL_APP(c, copy, (newp, size, object,                              \
                              SSH_ADT_APPCTX(c)));                             \
-  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                     \
+  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -235,14 +272,14 @@ static SshADTHandle name(SshADTContainer c,                                   \
                          void *object)                                        \
 {                                                                             \
   SshADTHandle h;                                                             \
-  void *newp;                                                                  \
+  void *newp;                                                                 \
                                                                               \
-  SSH_ADT_STD_ALLOC_I(c, size, newp);                                          \
-  SSH_ADT_CALL_APP(c, copy, (newp, size, object,                               \
+  SSH_ADT_STD_ALLOC_I(c, size, newp);                                         \
+  SSH_ADT_CALL_APP(c, copy, (newp, size, object,                              \
                                      SSH_ADT_APPCTX(c)));                     \
-  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                     \
+  h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->ssh_adt_aux.num_objects++;                                               \
+  c->aux.num_objects++;                                                       \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -258,7 +295,7 @@ static void *name(SshADTContainer c, SshADTHandle handle)                     \
 #define SSH_ADT_STD_NUM_OBJECTS(name)                                         \
 static size_t name(SshADTContainer c)                                         \
 {                                                                             \
-  return c->ssh_adt_aux.num_objects;                                          \
+  return c->aux.num_objects;                                                  \
 }
 
 #define SSH_ADT_STD_GET_HANDLE_TO(name, block)                                \
@@ -274,6 +311,14 @@ static SshADTHandle name(SshADTContainer c,                                   \
   return SSH_ADT_HANDLE_FROM_OBJECT(c, object);                               \
 }
 
+#ifdef _KERNEL
+#define SSH_ADT_STD_REALLOC(name, block1, block2)                             \
+static void *name(SshADTContainer c, void *object, size_t new_size)           \
+{                                                                             \
+  ssh_fatal("SSH_ADT_STD_REALLOC(): memory mode not supported in _KERNEL");   \
+  return NULL;                                                                \
+}
+#else /* _KERNEL */
 #define SSH_ADT_STD_REALLOC(name, block1, block2)                             \
 static void *name(SshADTContainer c, void *object, size_t new_size)           \
 {                                                                             \
@@ -284,21 +329,22 @@ static void *name(SshADTContainer c, void *object, size_t new_size)           \
   SSH_ADT_PROTECT(block1);                                                    \
   if (c->flags & SSH_ADT_FLAG_CONTAINED_HEADER)                               \
     {                                                                         \
-      ptr = ssh_xrealloc(object, new_size);                                   \
+      ptr = SSH_MEM_REALLOC(object, NULL, new_size);                          \
       newh = SSH_ADT_HANDLE_FROM_OBJECT(c, (void *)ptr);                      \
     }                                                                         \
   else                                                                        \
     {                                                                         \
       ptr = object;                                                           \
       ptr -= c->static_data->internal_header_size;                            \
-      ptr = ssh_xrealloc(ptr, new_size +                                      \
-                         c->static_data->internal_header_size);               \
+      ptr = SSH_MEM_REALLOC(ptr, NULL, new_size +                             \
+                            c->static_data->internal_header_size);            \
       newh = ptr;                                                             \
     }                                                                         \
   SSH_ADT_PROTECT(block2);                                                    \
   SSH_ADT_CALL_REALLOC_HOOK(c, reallocated, oldh, newh);                      \
   return ptr;                                                                 \
 }
+#endif /* _KERNEL */
 
 #define SSH_ADT_STD_DETACH(name, block, destroy_node_block)                   \
 static void *name(SshADTContainer c, SshADTHandle handle)                     \
@@ -306,7 +352,7 @@ static void *name(SshADTContainer c, SshADTHandle handle)                     \
   unsigned char *object;                                                      \
                                                                               \
   SSH_ADT_CALL_HOOK(c, detach, handle);                                       \
-  SSH_DEBUG(5, ("Detach: handle=%p", handle));                                \
+  SSH_DEBUG(6, ("Detach: handle=%p", handle));                                \
                                                                               \
   SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(c, handle, object);                      \
   SSH_ADT_PROTECT(block);                                                     \
@@ -317,14 +363,14 @@ static void *name(SshADTContainer c, SshADTHandle handle)                     \
       SSH_ADT_PROTECT(destroy_node_block);                                    \
     }                                                                         \
                                                                               \
-  c->ssh_adt_aux.num_objects--;                                               \
+  c->aux.num_objects--;                                                       \
   return object;                                                              \
 }
 
 #define SSH_ADT_STD_DELETE(name)                                              \
 static void name(SshADTContainer container, SshADTHandle handle)              \
 {                                                                             \
-  void *object;                                                               \
+  void *object = NULL;                                                        \
                                                                               \
   object = ssh_adt_detach_i(container, handle);                               \
   SSH_ADT_CALL_APP(container, destr, (object, SSH_ADT_APPCTX(container)));    \
@@ -332,7 +378,7 @@ static void name(SshADTContainer container, SshADTHandle handle)              \
     {                                                                         \
       SSH_ADT_STD_FREE_I(container, handle);                                  \
     }                                                                         \
-}
+}                                                                             \
 
 #define SSH_ADT_STD_NOTHING do { } while(0);
 
