@@ -20,6 +20,7 @@
 #include "sshadt_priority_queue.h"
 #include "sshdebug.h"
 #include "sshregression.h"
+#include "sshadt_strmap.h"
 
 #define SSH_DEBUG_MODULE "SshADTTest"
 
@@ -92,7 +93,7 @@ static int str_cmp(const void *ptr1, const void *ptr2, void *ctx)
   return strcmp(ptr1, ptr2);
 }
 
-/* Tests for lists. */
+/********************************************************** Tests for lists. */
 
 /* List container creation. */
 
@@ -136,8 +137,8 @@ SshADTContainer create_list_liballoc_with_header(void)
 
 static void add_voidptr(SshADTContainer c, int i)
 {
-  int *ptr = ssh_xmalloc(sizeof(i));
-  memcpy(ptr, &i, sizeof(i));
+  int *ptr = ssh_xmalloc(sizeof(*ptr));
+  memcpy(ptr, &i, sizeof(*ptr));
   ssh_adt_insert(c, ptr);
 }
 
@@ -230,7 +231,8 @@ static Boolean list_check(SshADTContainer (* create)(void),
   return TRUE;
 }
 
-/* Tests for maps. */
+
+/*********************************************************** Tests for maps. */
 
 static Boolean map_check(void)
 {
@@ -476,6 +478,9 @@ static Boolean bimap_check(void)
   return TRUE;
 }
 
+
+/********************************************************* Tests for arrays. */
+
 static SshADTContainer create_array_voidptr(void)
 {
   return ssh_adt_create_generic(SSH_ADT_ARRAY,
@@ -556,6 +561,9 @@ static Boolean array_check(SshADTContainer (* create)(void),
   return TRUE;
 }
 
+
+/************************************************* Tests for priority queues. */
+
 static Boolean pq_check(void)
 {
   SshADTContainer pq, check_list;
@@ -617,6 +625,42 @@ static Boolean pq_check(void)
   return TRUE;
 }
 
+/****************************************************** Test string mappings */
+
+Boolean conv_strmap_check(void)
+{
+  SshADTContainer c = ssh_adt_create_strmap();
+  char buf[100];
+
+  int i;
+
+  for (i = 0; i < 100; i++)
+    {
+      snprintf(buf, sizeof(buf), "foo-%d", i);
+      ssh_adt_strmap_add(c, buf, (void *)i);
+    }
+
+  for (i = 0; i < 100; i++)
+    {
+      snprintf(buf, sizeof(buf), "foo-%d", i);
+      if (ssh_adt_strmap_get(c, buf) != (void *)i)
+        {
+          return FALSE;
+        }
+      ssh_adt_strmap_remove(c, buf);
+    }
+
+  for (i = 0; i < 100; i++)
+    {
+      snprintf(buf, sizeof(buf), "foo-%d", i);
+      if (ssh_adt_strmap_get(c, buf) != NULL)
+        return FALSE;
+    }
+
+  ssh_adt_destroy(c);
+  return TRUE;
+}
+
 static void run_tests(void)
 {
   ssh_regression_section("Lists");
@@ -658,12 +702,19 @@ static void run_tests(void)
 
   T("Priority Queues, lib allocated",
     pq_check, ());
+
+  ssh_regression_section("Convenience containers");
+
+  T("String map", conv_strmap_check, ());
 }
 
 int main(int argc, char **argv)
 {
   ssh_regression_init(&argc, &argv, "ADT Library",
                       "huima@ssh.fi");
+
+  ssh_debug_set_level_string("*=3");
+
   run_tests();          
   ssh_regression_finish();
   /* Not reached. */
