@@ -10,7 +10,7 @@
   */
 
 /*
- * $Id: snprintf.c,v 1.8 2000/05/15 14:25:38 santtu Exp $
+ * $Id: snprintf.c,v 1.13 2000/09/06 07:25:01 mpu Exp $
  * $Log: snprintf.c,v $
  * $EndLog$
  */
@@ -35,23 +35,34 @@
 #undef isdigit
 #define isdigit(ch) ((ch) >= '0' && (ch) <= '9')
 
+/* XXX should add SSH_ prefix to these defines. */
 #define MINUS_FLAG 0x1
 #define PLUS_FLAG 0x2
 #define SPACE_FLAG 0x4
 #define HASH_FLAG 0x8
 #define CONV_TO_SHORT 0x10
 #define IS_LONG_INT 0x20
-#define IS_LONG_DOUBLE 0x40
-#define X_UPCASE 0x80
-#define IS_NEGATIVE 0x100
-#define UNSIGNED_DEC 0x200
-#define ZERO_PADDING 0x400
+#define IS_LONG_LONG_INT 0x40
+#define IS_LONG_DOUBLE 0x80
+#define X_UPCASE 0x100
+#define IS_NEGATIVE 0x200
+#define UNSIGNED_DEC 0x400
+#define SSH_ZERO_PADDING 0x800
 
 #undef sprintf
 
+#if defined(HAVE_LONG_LONG)
+#define LLTYPE        long long
+#define ULLTYPE        unsigned long long
+#elif defined(HAVE_LONG)
+#define LLTYPE        long
+#define ULLTYPE       unsigned long
+#else
+#error "No long long or long defined."
+#endif
 
-/* Convert a integer from unsigned long int representation 
-   to string representation. This will insert prefixes if needed 
+/* Convert a integer from unsigned long int representation
+   to string representation. This will insert prefixes if needed
    (leading zero for octal and 0x or 0X for hexadecimal) and
    will write at most buf_size characters to buffer.
    tmp_buf is used because we want to get correctly truncated
@@ -59,9 +70,9 @@
    */
 
 static int
-snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
-                       unsigned long int ulong_val, int flags, int width,
-                       int precision)
+snprintf_convert_unumber(char *buffer, size_t buf_size, int base, char *digits,
+                           ULLTYPE ulong_val, int flags, int width,
+                           int precision)
 {
   int tmp_buf_len = 100 + width, len, written = 0;
   char *tmp_buf, *tmp_buf_ptr, prefix[2];
@@ -74,7 +85,7 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
 
   prefix[0] = '\0';
   prefix[1] = '\0';
-  
+
   /* Make tmp_buf_ptr point just past the last char of buffer */
   tmp_buf_ptr = tmp_buf + tmp_buf_len;
 
@@ -82,14 +93,14 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
     precision = 0;
 
   /* Main conversion loop */
-  do 
+  do
     {
       *--tmp_buf_ptr = digits[ulong_val % base];
       ulong_val /= base;
       precision--;
-    } 
+    }
   while ((ulong_val != 0 || precision > 0) && tmp_buf_ptr > tmp_buf);
- 
+
   /* Get the prefix */
   if (!(flags & IS_NEGATIVE))
     {
@@ -106,10 +117,10 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
               prefix[1] = '0';
             }
         }
-      
+
       if (base == 8 && (flags & HASH_FLAG))
         prefix[0] = '0';
-      
+
       if (base == 10 && !(flags & UNSIGNED_DEC) && (flags & PLUS_FLAG))
         prefix[0] = '+';
       else
@@ -121,13 +132,25 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
   else
       prefix[0] = '-';
 
-  if (prefix[0] != '\0' && tmp_buf_ptr > tmp_buf)
+  if ((flags & MINUS_FLAG) || !(flags & SSH_ZERO_PADDING))
     {
-      *--tmp_buf_ptr = prefix[0];
-      if (prefix[1] != '\0' && tmp_buf_ptr > tmp_buf)
-        *--tmp_buf_ptr = prefix[1];
+      /* Left-justified */
+      if (prefix[0] != '\0' && tmp_buf_ptr > tmp_buf)
+        {
+          *--tmp_buf_ptr = prefix[0];
+          if (prefix[1] != '\0' && tmp_buf_ptr > tmp_buf)
+            *--tmp_buf_ptr = prefix[1];
+        }
     }
-  
+  else
+    {
+      /* Right-justified */
+      if (prefix[1] != '\0' && buf_size - written > 0)
+        buffer[written++] = prefix[1];
+      if (prefix[0] != '\0' && buf_size - written > 0)
+        buffer[written++] = prefix[0];
+    }
+
   len = (tmp_buf + tmp_buf_len) - tmp_buf_ptr;
 
   /* Now:
@@ -160,14 +183,14 @@ snprintf_convert_ulong(char *buffer, size_t buf_size, int base, char *digits,
 #endif /* _KERNEL */
                   return written;
                 }
-              buffer[written] = (flags & ZERO_PADDING) ? '0': ' ';
+              buffer[written] = (flags & SSH_ZERO_PADDING) ? '0': ' ';
             }
           written++;
         }
       else
         {
           if (width > len && written < width - len)
-            buffer[written] = (flags & ZERO_PADDING) ? '0': ' ';
+            buffer[written] = (flags & SSH_ZERO_PADDING) ? '0': ' ';
           else
             {
               if (width > len)
@@ -215,7 +238,7 @@ snprintf_convert_float(char *buffer, size_t buf_size,
 
   /* Construct the formatting string and let system's sprintf
      do the real work. */
-  
+
   *format_str_ptr++ = '%';
 
   if (flags & MINUS_FLAG)
@@ -224,11 +247,11 @@ snprintf_convert_float(char *buffer, size_t buf_size,
     *format_str_ptr++ = '+';
   if (flags & SPACE_FLAG)
     *format_str_ptr++ = ' ';
-  if (flags & ZERO_PADDING)
+  if (flags & SSH_ZERO_PADDING)
     *format_str_ptr++ = '0';
   if (flags & HASH_FLAG)
     *format_str_ptr++ = '#';
-    
+
   sprintf(format_str_ptr, "%d.%d", width, precision);
   format_str_ptr += strlen(format_str_ptr);
 
@@ -280,8 +303,9 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
   int flags, width, precision, i;
   char format_char, *orig_str = str;
   int *int_ptr;
-  long int long_val;
+  LLTYPE long_long_val;
   unsigned long int ulong_val;
+  ULLTYPE ulong_long_val;
   char *str_val;
 #ifndef KERNEL
   double dbl_val;
@@ -313,14 +337,14 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                   int length, value;
                   const char *format_start = format_ptr;
 
-                  flags = 0; 
+                  flags = 0;
                   width = 0;
                   precision = -1;
                   format_char = (char)0;
 
                   /* Get the flags */
                   format_ptr++;
-                  while (*format_ptr == '-' || *format_ptr == '+' || 
+                  while (*format_ptr == '-' || *format_ptr == '+' ||
                          *format_ptr == ' ' || *format_ptr == '#' ||
                          *format_ptr == '0')
                     {
@@ -339,24 +363,24 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                           flags |= HASH_FLAG;
                           break;
                         case '0':
-                          flags |= ZERO_PADDING;
+                          flags |= SSH_ZERO_PADDING;
                           break;
                         }
                       format_ptr++;
                     }
 
                   /* Don't pad left-justified numbers withs zeros */
-                  if ((flags & MINUS_FLAG) && (flags & ZERO_PADDING))
-                    flags &= ~ZERO_PADDING;
-        
+                  if ((flags & MINUS_FLAG) && (flags & SSH_ZERO_PADDING))
+                    flags &= ~SSH_ZERO_PADDING;
+
                   /* Is width field present? */
                   if (isdigit(*format_ptr))
                     {
-                      for (value = 0; 
-                           *format_ptr && isdigit(*format_ptr); 
+                      for (value = 0;
+                           *format_ptr && isdigit(*format_ptr);
                            format_ptr++)
                         value = 10 * value + *format_ptr - '0';
-                      
+
                       width = value;
                     }
                   else
@@ -374,8 +398,8 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                       format_ptr++;
                       if (isdigit(*format_ptr))
                         {
-                          for (value = 0; 
-                               *format_ptr && isdigit(*format_ptr); 
+                          for (value = 0;
+                               *format_ptr && isdigit(*format_ptr);
                                format_ptr++)
                             value = 10 * value + *format_ptr - '0';
 
@@ -400,7 +424,13 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                       format_ptr++;
                       break;
                     case 'l':
-                      flags |= IS_LONG_INT;
+                      if (*(format_ptr + 1) == 'l')
+                        {
+                          format_ptr++;
+                          flags |= IS_LONG_LONG_INT;
+                        }
+                      else
+                        flags |= IS_LONG_INT;
                       format_ptr++;
                       break;
                     case 'L':
@@ -410,18 +440,18 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                     default:
                       break;
                     }
-      
+
                   /* Get and check the formatting character */
                   format_char = *format_ptr;
                   format_ptr++;
                   length = format_ptr - format_start;
-                        
+
                   switch (format_char)
                     {
                     case 'c': case 's': case 'p': case 'n':
-                    case 'd': case 'i': case 'o': 
-                    case 'u': case 'x': case 'X': 
-                    case 'f': case 'e': case 'E': case 'g': case 'G': 
+                    case 'd': case 'i': case 'o':
+                    case 'u': case 'x': case 'X':
+                    case 'f': case 'e': case 'E': case 'g': case 'G':
 #ifdef SSH_MAKING_EPRINTF
                     case '@':
 #endif /* SSH_MAKING_EPRINTF */
@@ -431,7 +461,7 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                         flags |= UNSIGNED_DEC;
                       status = length;
                       break;
-                    
+
                     default:
                       status = 0;
                     }
@@ -450,24 +480,26 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                         case 'i': case 'd':
                           /* Convert to unsigned long int before
                              actual conversion to string */
-                          if (flags & IS_LONG_INT)
-                            long_val = va_arg(ap, long int);
+                          if (flags & IS_LONG_LONG_INT)
+                            long_long_val = va_arg(ap, LLTYPE);
+                          else if (flags & IS_LONG_INT)
+                            long_long_val = (LLTYPE) va_arg(ap, long int);
                           else
-                            long_val = (long int) va_arg(ap, int);
-                          
-                          if (long_val < 0)
+                            long_long_val = (LLTYPE) va_arg(ap, int);
+
+                          if (long_long_val < 0)
                             {
-                              ulong_val = (unsigned long int) -long_val;
+                              ulong_long_val = (ULLTYPE) -long_long_val;
                               flags |= IS_NEGATIVE;
                             }
                           else
                             {
-                              ulong_val = (unsigned long int) long_val;
+                              ulong_long_val = (ULLTYPE) long_long_val;
                             }
-                          status = 
-                            snprintf_convert_ulong(str, left, 10, "0123456789",
-                                                   ulong_val, flags,
-                                                   width, precision);
+                          status =
+                            snprintf_convert_unumber(str, left, 10, "0123456789",
+                                                       ulong_long_val, flags,
+                                                       width, precision);
                           str += status;
                           left -= status;
                           break;
@@ -476,73 +508,84 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                           ulong_val = (unsigned long int)
                             va_arg(ap, void *);
                           status =
-                            snprintf_convert_ulong(str, left, 16,
+                            snprintf_convert_unumber(str, left, 16,
                                                    "0123456789abcdef",
                                                    ulong_val, flags,
                                                    width, precision);
                           str += status;
                           left -= status;
-                          break;                            
+                          break;
 
                         case 'x': case 'X':
 
-                          if (flags & IS_LONG_INT)
-                            ulong_val = va_arg(ap, unsigned long int);
+                          if (flags & IS_LONG_LONG_INT)
+                            ulong_long_val = va_arg(ap, ULLTYPE);
+                          else if (flags & IS_LONG_INT)
+                            ulong_long_val = va_arg(ap, unsigned long int);
                           else
-                            ulong_val = 
-                              (unsigned long int) va_arg(ap, unsigned int);
-                          
-                          status = 
-                            snprintf_convert_ulong(str, left, 16, 
-                                                   (format_char == 'x')   ? 
-                                                       "0123456789abcdef" : 
+                            ulong_long_val =
+                              (ULLTYPE) va_arg(ap, unsigned int);
+
+                          status =
+                            snprintf_convert_unumber(str, left, 16,
+                                                   (format_char == 'x')   ?
+                                                       "0123456789abcdef" :
                                                        "0123456789ABCDEF",
-                                                   ulong_val, flags,
+                                                   ulong_long_val, flags,
                                                    width, precision);
                           str += status;
                           left -= status;
-                          
+
                           break;
 
                         case 'o':
-                          if (flags & IS_LONG_INT)
-                            ulong_val = va_arg(ap, unsigned long int);
+                          if (flags & IS_LONG_LONG_INT)
+                            ulong_long_val = va_arg(ap, ULLTYPE);
+                          else if (flags & IS_LONG_INT)
+                            ulong_long_val =
+                                (ULLTYPE) va_arg(ap, unsigned long int);
                           else
-                            ulong_val = 
-                              (unsigned long int) va_arg(ap, unsigned int);
+                            ulong_long_val =
+                              (ULLTYPE) va_arg(ap, unsigned int);
 
-                          status = snprintf_convert_ulong(str, left, 8,
+                          status = snprintf_convert_unumber(str, left, 8,
                                                           "01234567",
-                                                          ulong_val, flags,
+                                                          ulong_long_val, flags,
                                                           width, precision);
                           str += status;
                           left -= status;
                           break;
 
                         case 'u':
-                          if (flags & IS_LONG_INT)
-                            ulong_val = va_arg(ap, unsigned long int);
+                          if (flags & IS_LONG_LONG_INT)
+                            ulong_long_val = va_arg(ap, ULLTYPE);
+                          else if (flags & IS_LONG_INT)
+                            ulong_long_val = (ULLTYPE)
+                                va_arg(ap, unsigned long int);
                           else
-                            ulong_val = 
+                            ulong_long_val =
                               (unsigned long int) va_arg(ap, unsigned int);
 
-                          status = snprintf_convert_ulong(str, left, 10,
+                          status = snprintf_convert_unumber(str, left, 10,
                                                           "0123456789",
-                                                          ulong_val, flags,
+                                                          ulong_long_val, flags,
                                                           width, precision);
                           str += status;
                           left -= status;
                           break;
 
                         case 'c':
-                          if (flags & IS_LONG_INT)
-                            ulong_val = va_arg(ap, unsigned long int);
-                          else
-                            ulong_val = 
-                              (unsigned long int) va_arg(ap, unsigned int);
-                          *str++ = (unsigned char)ulong_val;
-                          left--;
-                          break;
+                            if (flags & IS_LONG_LONG_INT)
+                                ulong_long_val = va_arg(ap, ULLTYPE);
+                            else if (flags & IS_LONG_INT)
+                                ulong_long_val = (ULLTYPE)
+                                    va_arg(ap, unsigned long int);
+                            else
+                                ulong_long_val =
+                                    (ULLTYPE) va_arg(ap, unsigned int);
+                            *str++ = (unsigned char)ulong_long_val;
+                            left--;
+                            break;
 
 #ifdef SSH_MAKING_EPRINTF
                         case '@':
@@ -604,7 +647,7 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
 
                           if (str_val == NULL)
                             str_val = "(null)";
-                          
+
                           if (precision == -1)
                             precision = strlen(str_val);
                           else
@@ -626,11 +669,11 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                             {
                               strncpy(str, str_val, precision);
                               memset(str + precision,
-                                     (flags & ZERO_PADDING)?'0':' ', i);
+                                     (flags & SSH_ZERO_PADDING)?'0':' ', i);
                             }
                           else
                             {
-                              memset(str, (flags & ZERO_PADDING)?'0':' ', i);
+                              memset(str, (flags & SSH_ZERO_PADDING)?'0':' ', i);
                               strncpy(str + i, str_val, precision);
                             }
                           str += width;
@@ -656,7 +699,7 @@ vsnprintf(char *str, size_t size, const char *format, va_list ap)
                           left -= status;
                           break;
 #endif /* KERNEL */
-                          
+
                         default:
                           break;
                         }
