@@ -67,21 +67,6 @@
 #define WANT_COMPLEX_FORMATS
 #endif /* TINY_DEBUG */
 
-/* Some definitions to make memory allocation more readable between
-   the user- and kernel-mode differences. Yes, when we are in usermode
-   we don't need to check the return value of allocators, but, we need
-   to do that when in kernel mode. So, we opt to lowest common
-   denominator and always check the return value */
-#ifdef _KERNEL
-#define MEM_ALLOC(S)            ssh_kmalloc(S)
-#define MEM_FREE(P)             ssh_kfree(P)
-#define MEM_STRDUP(S)           ssh_kstrdup(S)
-#else /* _KERNEL */
-#define MEM_ALLOC(S)            ssh_xmalloc(S)
-#define MEM_FREE(P)             ssh_xfree(P)
-#define MEM_STRDUP(S)           ssh_xstrdup(S)
-#endif /* _KERNEL */
-
 /* Include function name in the default format only if it is really
    available. */
 
@@ -160,8 +145,8 @@ char *ssh_debug_format(const char *format, ...)
   ssh_evsnprintf(buf, sizeof(buf), format, args);
   va_end(args);
 
-  return 
-    MEM_STRDUP(buf);
+  return
+    SSH_MEM_STRDUP(buf);
 }
 
 /* Returns TRUE if debugging has been enabled for the given module
@@ -210,8 +195,8 @@ void ssh_debug_set_global_level(unsigned int level)
     {
       dl = ssh_debug_module_levels;
       ssh_debug_module_levels = dl->next;
-      MEM_FREE(dl->module);
-      MEM_FREE(dl);
+      SSH_MEM_FREE(dl->module);
+      SSH_MEM_FREE(dl);
     }
 }
 
@@ -225,12 +210,12 @@ void ssh_debug_set_module_level(const char *module, unsigned int level)
 
   /* XXX: How to fail on memory failure? Well, it's only debug
      level. Nothing fatal, I hope? */
-  if (!(dl = MEM_ALLOC(sizeof(*dl))))
+  if (!(dl = SSH_MEM_ALLOC(sizeof(*dl))))
     return;
 
-  if (!(dl->module = MEM_STRDUP(module)))
+  if (!(dl->module = SSH_MEM_STRDUP(module)))
     {
-      MEM_FREE(dl);
+      SSH_MEM_FREE(dl);
       return;
     }
   dl->level = level;
@@ -336,14 +321,14 @@ void ssh_debug_set_level_string(const char *string)
             }
           else
             {
-              name = MEM_ALLOC(name_len + 1);
+              name = SSH_MEM_ALLOC(name_len + 1);
 
-              if (name) 
+              if (name)
                 {
                   memcpy(name, name_start, name_len);
                   name[name_len] = '\0';
                   ssh_debug_set_module_level(name, (unsigned int)level_value);
-                  MEM_FREE(name);
+                  SSH_MEM_FREE(name);
                 }
             }
         }
@@ -456,6 +441,12 @@ void ssh_debug(const char *fmt, ...)
     }
 }
 
+
+
+
+
+
+
 /* Outputs a fatal error message.  This function never returns. */
 
 void ssh_fatal(const char *fmt, ...)
@@ -472,7 +463,7 @@ void ssh_fatal(const char *fmt, ...)
      been specified. */
   if (ssh_debug_fatal_callback)
     {
-    (*ssh_debug_fatal_callback)(buf, ssh_debug_error_context);
+      (*ssh_debug_fatal_callback)(buf, ssh_debug_error_context);
     }
   else
     {
@@ -482,9 +473,19 @@ void ssh_fatal(const char *fmt, ...)
 # endif /* KERNEL */
     }
 
-  /* Exit the current program; this is the fatal error handler, and should
-     never return. */
+
+
+
+
+
+
+
+
+
+  /* Cause a fatal error on the current program; this is the fatal
+     error handler, and should never return. */
 #ifndef KERNEL
+  abort();
   exit(1);
 #endif /* KERNEL */
 }
@@ -857,9 +858,9 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
     case '!':
       c1 = parse_atomic_condition(argptr);
       if (c1 == NULL) return NULL;
-      if (!(c2 = MEM_ALLOC(sizeof(*c2))))
+      if (!(c2 = SSH_MEM_ALLOC(sizeof(*c2))))
         {
-          MEM_FREE(c1);
+          SSH_MEM_FREE(c1);
           return NULL;
         }
       c2->type = SSH_DEBUG_FORMAT_COND_NEG;
@@ -867,7 +868,7 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       return c2;
 
     case '*':
-      if (!(c1 = MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
           return NULL;
       c1->type = SSH_DEBUG_FORMAT_COND_MAGIC;
       c1->arg.number = -2;
@@ -880,12 +881,12 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       if (!ssh_debug_parse_arg(*argptr, argptr, &cond_arg))
         return NULL;
 
-      if (!(c1 = MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
       return NULL;
 
-      if (!(c1->arg.string = MEM_STRDUP(cond_arg)))
+      if (!(c1->arg.string = SSH_MEM_STRDUP(cond_arg)))
       {
-        MEM_FREE(c1);
+        SSH_MEM_FREE(c1);
         return NULL;
       }
       switch (c)
@@ -895,13 +896,14 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
         case 'f': c1->type = SSH_DEBUG_FORMAT_COND_FUNCNAME; break;
         case 's': c1->type = SSH_DEBUG_FORMAT_COND_FILENAME; break;
         }
+      return c1;
 
     case '<':
     case '>':
     case '=':
       if (!ssh_debug_parse_arg(*argptr, argptr, &cond_arg))
         return NULL;
-      if (!(c1 = MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
       return NULL;
 
       c1->type = (c == '<' ? SSH_DEBUG_FORMAT_COND_LEVELLT :
@@ -910,7 +912,7 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       c1->arg.number = atoi(cond_arg);
       if (c1->arg.number < 0 || c1->arg.number > 1000)
       {
-        MEM_FREE(c1);
+        SSH_MEM_FREE(c1);
         return NULL;
       }
       return c1;
@@ -942,16 +944,16 @@ static SshDebugCondition parse_condition(char **argptr)
     {
       (*argptr)++;
       c2 = parse_condition(argptr);
-      if (c2 == NULL) 
+      if (c2 == NULL)
         {
-          MEM_FREE(c2);
+          SSH_MEM_FREE(c2);
           return NULL;
         }
-      
-      if (!(c3 = MEM_ALLOC(sizeof(*c3))))
+
+      if (!(c3 = SSH_MEM_ALLOC(sizeof(*c3))))
         {
-          MEM_FREE(c1);
-          MEM_FREE(c2);
+          SSH_MEM_FREE(c1);
+          SSH_MEM_FREE(c2);
           return NULL;
         }
 
@@ -1006,7 +1008,7 @@ static void ssh_debug_parse_format(void)
 
   /* XXX: How to react on memory allocation failure? It's just debug,
      so we opt just to bail out */
-  if (!(fmt_string = MEM_STRDUP(fmt_string)))
+  if (!(fmt_string = SSH_MEM_STRDUP(fmt_string)))
     return;
 
   ref = fmt_string;
@@ -1015,7 +1017,7 @@ static void ssh_debug_parse_format(void)
     {
       /* XXX: How to handle out-of-memory? We break out of the fmt
          loop here */
-      if (!(item = MEM_ALLOC(sizeof(*item))))
+      if (!(item = SSH_MEM_ALLOC(sizeof(*item))))
         break;
 #ifdef WANT_COMPLEX_FORMATS
       item->align_right = 0;
@@ -1117,7 +1119,7 @@ static void ssh_debug_parse_format(void)
               if (temp == NULL)
 #ifdef HAVE_GETHOSTNAME
                 {
-                  if ((item->arg = MEM_ALLOC(60)) != NULL)
+                  if ((item->arg = SSH_MEM_ALLOC(60)) != NULL)
                     {
                       if (gethostname(item->arg, 60) < 0)
                         {
@@ -1131,11 +1133,11 @@ static void ssh_debug_parse_format(void)
                 }
 #else /* HAVE_GETHOSTNAME */
                 {
-                  item->arg = MEM_STRDUP("<host unknown>");
+                  item->arg = SSH_MEM_STRDUP("<host unknown>");
                 }
 #endif /* HAVE_GETHOSTNAME */
               else
-                item->arg = MEM_STRDUP(temp);
+                item->arg = SSH_MEM_STRDUP(temp);
               break;
 
               /* `%E(var)' expands to the value of the environment
@@ -1152,12 +1154,12 @@ static void ssh_debug_parse_format(void)
               temp = (char *)getenv(arg);
               if (temp == NULL)
                 {
-                  if ((item->arg = MEM_ALLOC(strlen(arg) + 10)) != NULL)
+                  if ((item->arg = SSH_MEM_ALLOC(strlen(arg) + 10)) != NULL)
                     snprintf(item->arg, strlen(arg) + 10, "<%s unset>", arg);
                 }
               else
                 {
-                  item->arg = MEM_STRDUP(temp);
+                  item->arg = SSH_MEM_STRDUP(temp);
                 }
 #else /* HAVE_GETENV */
               ssh_warning("getenv() is not available for debugging output, "
@@ -1215,7 +1217,7 @@ static void ssh_debug_parse_format(void)
                   || ssh_debug_format_wrapindent > 1000)
                 goto failure;
 
-              MEM_FREE(item);
+              SSH_MEM_FREE(item);
               continue;
 
               break;
@@ -1279,7 +1281,7 @@ static void ssh_debug_parse_format(void)
                   goto failure;
               item->type = SSH_DEBUG_FORMAT_TERMCONTROL;
               item->numarg = -1;
-              item->arg = MEM_STRDUP(arg);
+              item->arg = SSH_MEM_STRDUP(arg);
               break;
 
               /* '%c(n)' will cause the debug module to sleep n
@@ -1334,14 +1336,14 @@ static void ssh_debug_parse_format(void)
 
             case '%':
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              item->arg = MEM_STRDUP("%");
+              item->arg = SSH_MEM_STRDUP("%");
               break;
 
             default:
               ssh_warning("Invalid or unsupported "
                           "debug format control character '%c'.", c);
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              if ((item->arg = MEM_ALLOC(2)) != NULL)
+              if ((item->arg = SSH_MEM_ALLOC(2)) != NULL)
                 {
                   item->arg[0] = c;
                   item->arg[1] = '\0';
@@ -1368,7 +1370,7 @@ static void ssh_debug_parse_format(void)
               c = *fmt_string;
               *fmt_string = '\0';
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              item->arg = MEM_STRDUP(temp);
+              item->arg = SSH_MEM_STRDUP(temp);
               *fmt_string = c;
             }
         }
@@ -1384,7 +1386,7 @@ static void ssh_debug_parse_format(void)
       *(pptr) = item;
       pptr = &(item->next);
     }
-  MEM_FREE(ref);
+  SSH_MEM_FREE(ref);
 
  /* succesful end of parsing */
 
@@ -1405,7 +1407,17 @@ failure: /* something went wrong... give a warning and fall
                 "SSH_DEBUG_FMT environment variable.");
 #endif
 
-  ssh_debug_format_items = MEM_ALLOC(sizeof(*ssh_debug_format_items));
+  /* Release if there was anything previously */
+  while (ssh_debug_format_items != NULL)
+    {
+      SshDebugFormatItem item = ssh_debug_format_items;
+      ssh_debug_format_items = item->next;
+
+      SSH_MEM_FREE(item->arg);
+      SSH_MEM_FREE(item);
+    }
+
+  ssh_debug_format_items = SSH_MEM_ALLOC(sizeof(*ssh_debug_format_items));
 
   /* XXX: How to recover from memory allocation failure here? */
   if (!ssh_debug_format_items)
@@ -1423,7 +1435,7 @@ failure: /* something went wrong... give a warning and fall
   ssh_debug_format_wrapcol = 0;
 #endif /* WANT_COMPLEX_FORMATS */
 
-  MEM_FREE(ref);
+  SSH_MEM_FREE(ref);
 
   return;
 }
@@ -1447,7 +1459,7 @@ static void ssh_debug_parse_format(void)
 
   for (i = 8; i >= 0; i--)
     {
-      if (!(item = MEM_ALLOC(sizeof(*item))))
+      if (!(item = SSH_MEM_ALLOC(sizeof(*item))))
         return;
 
 #ifdef WANT_COMPLEX_FORMATS
@@ -1459,6 +1471,8 @@ static void ssh_debug_parse_format(void)
       item->next = ssh_debug_format_items;
       ssh_debug_format_items = item;
 
+      item->arg = NULL;
+
     redo:
       switch (i)
         {
@@ -1467,7 +1481,7 @@ static void ssh_debug_parse_format(void)
 
         case 1:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = MEM_STRDUP("/");
+          item->arg = SSH_MEM_STRDUP("/");
           break;
 
         case 2:
@@ -1475,7 +1489,7 @@ static void ssh_debug_parse_format(void)
 
         case 3:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = MEM_STRDUP(":");
+          item->arg = SSH_MEM_STRDUP(":");
           break;
 
         case 4:
@@ -1485,7 +1499,7 @@ static void ssh_debug_parse_format(void)
 
         case 5:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = MEM_STRDUP("/");
+          item->arg = SSH_MEM_STRDUP("/");
           break;
 
         case 6:
@@ -1494,7 +1508,7 @@ static void ssh_debug_parse_format(void)
 #endif
         case 7:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = MEM_STRDUP(": ");
+          item->arg = SSH_MEM_STRDUP(": ");
           break;
 
         case 8:
@@ -1691,7 +1705,7 @@ void ssh_debug_output(int level,
 #ifdef WANT_COMPLEX_FORMATS
         case SSH_DEBUG_FORMAT_IGNORE:
           {
-            MEM_FREE(msg);
+            SSH_MEM_FREE(msg);
             return;
           }
 
@@ -1862,14 +1876,15 @@ void ssh_debug_output(int level,
           break;
 
         case SSH_DEBUG_FORMAT_NEWLINE:
-          if (total_len < SSH_DEBUG_BUFFER_SIZE - 1)
+          if (total_len < SSH_DEBUG_BUFFER_SIZE - 2)
             {
+              *current++ = '\r';
               *current++ = '\n';
 
 #ifdef WANT_COMPLEX_FORMATS
               column = 0;
 #endif
-              total_len++;
+              total_len += 2;
             }
           fragment = NULL;
           break;
@@ -2043,11 +2058,12 @@ void ssh_debug_output(int level,
                   /* Write newline, indent, and redo */
 
                 newline:
-                  if (total_len < (SSH_DEBUG_BUFFER_SIZE - 1))
+                  if (total_len < (SSH_DEBUG_BUFFER_SIZE - 2))
                     {
+                      *current++ = '\r';
                       *current++ = '\n';
                       column = 0;
-                      total_len++;
+                      total_len += 2;
 
                       while (total_len < (SSH_DEBUG_BUFFER_SIZE - 1) &&
                              column < ssh_debug_format_wrapindent DINDENT)
@@ -2085,7 +2101,7 @@ void ssh_debug_output(int level,
 
   /* Free the message and print the formatted version out */
 
-  MEM_FREE(msg);
+  SSH_MEM_FREE(msg);
   ssh_debug("%s", buf);
 
 #ifdef WANT_COMPLEX_FORMATS
