@@ -40,8 +40,10 @@
 int ssh_esnprintf(char *str, size_t size, const char *format, ...);
 int ssh_evsnprintf(char *str, size_t size, const char *format, va_list ap);
 #else /* SSH_MAKING_EPRINTF */
+#ifndef HAVE_SNPRINTF
 int snprintf(char *str, size_t size, const char *format, ...);
 int vsnprintf(char *str, size_t size, const char *format, va_list ap);
+#endif /* HAVE_SNPRINTF */
 #endif /* SSH_MAKING_EPRINTF */
 
 #endif /* NO_SNPRINTF */
