@@ -13,7 +13,7 @@
   */
 
 /*
- * $Id: sshunixeloop.c,v 1.4 2000/03/28 18:24:50 sjl Exp $
+ * $Id: sshunixeloop.c,v 1.10 2000/09/14 10:34:11 dima Exp $
  * $Log: sshunixeloop.c,v $
  * $EndLog$
  */
@@ -58,7 +58,6 @@ typedef struct time_record {
   SshTimeoutCallback callback;
   void *context;
   struct time_record *next;
-  Boolean killed;
 } TimeRec;
 
 typedef struct io_rec {
@@ -136,6 +135,10 @@ static int ssh_eloop_gettimeofday(struct timeval *tp, void *ignored_tz)
 
 void ssh_event_loop_initialize(void)
 {
+  ssh_eloop_select_timeout_no_wait.tv_sec = 0;
+  ssh_eloop_select_timeout_no_wait.tv_usec = 0;
+  check_time.tv_sec = 0;
+  check_time.tv_usec = 0;
   memset(&ssh_eloop_event_rec, 0, sizeof(ssh_eloop_event_rec));
 #ifdef HAVE_SIGNAL
   sigemptyset(&ssh_eloop_event_rec.used_signals);
@@ -233,6 +236,7 @@ void ssh_event_loop_uninitialize(void)
   ssh_xfree(ssh_eloop_event_rec.signal_records);
 #endif /* HAVE_SIGNAL */
 
+  ssh_eloop_initialized = FALSE;
   SSH_DEBUG(4, ("Uninitialized the event loop."));
 }
 
@@ -325,7 +329,6 @@ void ssh_register_timeout(long seconds,
                                          &created->firing_time);
   created->callback = callback;
   created->context = context;
-  created->killed = FALSE;
 
   /* Insert the new timeout in the sorted list of timeouts. */
   iter = &ssh_eloop_event_rec.time_records;
@@ -358,22 +361,28 @@ void ssh_register_idle_timeout(long seconds,
 
 void ssh_cancel_timeouts(SshTimeoutCallback callback, void *context)
 {
-  TimeRec *iter = ssh_eloop_event_rec.time_records;
+  TimeRec **iter = &ssh_eloop_event_rec.time_records;
+  TimeRec *tmp;
 
   assert(ssh_eloop_initialized);
 
-  while (iter != NULL)
+  while (*iter != NULL)
     {
-      if ((iter->context == context ||
+      if (((*iter)->context == context ||
            context == SSH_ALL_CONTEXTS) &&
-          (iter->callback == callback ||
-           callback == SSH_ALL_CALLBACKS) &&
-          iter->killed == FALSE)
+          ((*iter)->callback == callback ||
+           callback == SSH_ALL_CALLBACKS))
         {
-          SSH_DEBUG(7, ("Removed timeout at %d.", iter->firing_time.tv_sec));
-          iter->killed = TRUE;
+          SSH_DEBUG(7, ("Removed timeout at %d.",
+                        (*iter)->firing_time.tv_sec));
+          tmp = *iter;
+          *iter = (*iter)->next;
+          ssh_xfree(tmp);
+        }
+      else
+        {
+          iter = &((*iter)->next);
         }
-      iter = iter->next;
     }
 }
 
@@ -479,14 +488,25 @@ void ssh_io_register_fd(int fd, SshIoCallback callback, void *context)
   created->fd = fd;
   created->killed = FALSE;
   created->was_nonblocking =
+#ifdef VXWORKS
+    1; /* cannot query if it was or not in VxWorks */
+#else /* VXWORKS */
     (fcntl(fd, F_GETFL, 0) & (O_NONBLOCK|O_NDELAY)) != 0;
+#endif /* VXWORKS */
 
+#ifdef VXWORKS
+      {
+        int tmp = 1;
+        ioctl(fd, FIONBIO, &tmp);
+      }
+#else /* VXWORKS */
   /* Make the file descriptor use non-blocking I/O. */
-#if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
+#  if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
-#else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#  else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NDELAY);
-#endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#  endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#endif  /* VXWORKS */
 
   if (fd >= ssh_eloop_event_rec.requests_array_size)
     {
@@ -532,13 +552,17 @@ void ssh_io_unregister_fd(int fd, Boolean keep_nonblocking)
         {
           if (!iter->was_nonblocking && !keep_nonblocking)
             {
-#if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
+#ifdef VXWORKS
+              /* nothing, we cannot know of it was blocking */
+#else /* VXWORKS */
+#  if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
               fcntl(iter->fd, F_SETFL, 
                     fcntl(iter->fd, F_GETFL, 0) & ~O_NONBLOCK);
-#else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#  else /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
               fcntl(iter->fd, F_SETFL,
                     fcntl(iter->fd, F_GETFL, 0) & ~O_NDELAY);
-#endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#  endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
+#endif  /* VXWORKS */
             }
           iter->killed = TRUE;
           SSH_DEBUG(7, ("Killed the file descriptor %d, waiting for removal",
@@ -654,38 +678,30 @@ void ssh_event_loop_run(void)
           
           while ((time_temp = ssh_eloop_event_rec.time_records) != NULL)
             {
-              if (time_temp->killed == FALSE)
+              modified_time = current_time;
+              if (modified_time.tv_usec >= 1000000)
                 {
-                  modified_time = current_time;
-                  if (modified_time.tv_usec >= 1000000)
-                    {
-                      modified_time.tv_usec -= 1000000;
-                      modified_time.tv_sec++;
-                    }
-                  if (ssh_event_loop_compare_time(&(time_temp->firing_time),
-                                                  &modified_time) > 0)
-                    break;
+                  modified_time.tv_usec -= 1000000;
+                  modified_time.tv_sec++;
+                }
+              if (ssh_event_loop_compare_time(&(time_temp->firing_time),
+                                              &modified_time) > 0)
+                break;
 
-                  ssh_eloop_event_rec.time_records =
-                    ssh_eloop_event_rec.time_records->next;
+              ssh_eloop_event_rec.time_records =
+                ssh_eloop_event_rec.time_records->next;
                   
-                  /* It is safe to add or kill timeouts in the
-                     callback. */
-                  SSH_DEBUG(7, ("Calling a timeout callback."));
-                  (*time_temp->callback)(time_temp->context);
-                  done_something = TRUE;
+              /* It is safe to add or kill timeouts in the
+                 callback. */
+              SSH_DEBUG(7, ("Calling a timeout callback."));
+              (*time_temp->callback)(time_temp->context);
+              done_something = TRUE;
 #if 0
-                  /* Removed this to make event loop fair. After this
-                     modification the select part below also gets some time
-                     //kivinen */ 
-                  ssh_eloop_gettimeofday(&current_time, NULL);
+              /* Removed this to make event loop fair. After this
+                 modification the select part below also gets some time
+                 //kivinen */ 
+              ssh_eloop_gettimeofday(&current_time, NULL);
 #endif
-                }
-              else
-                {
-                  ssh_eloop_event_rec.time_records =
-                    ssh_eloop_event_rec.time_records->next;
-                }
               ssh_xfree(time_temp);
             }
           /* If there are any time records in the queue the first of
