@@ -46,7 +46,7 @@ typedef enum {
 
 typedef struct ssh_fsm_hash_chain {
   void *key_ptr;
-  SshFSMStateMapItem *state;
+  SshFSMStateMapItem state;
   struct ssh_fsm_hash_chain *next;
 } SshFSMHashChain;
 
@@ -58,7 +58,7 @@ struct ssh_fsm_thread {
   SshFSM fsm;                   /* The FSM the thread belongs to. */
   SshFSMThreadDestructor destructor;
                                 /* A destructor for thread-specific data. */
-  SshFSMStateMapItem *current_state;
+  SshFSMStateMapItem current_state;
                                 /* The current (next) state. */
   void *tdata_stack[SSH_FSM_STACK_SIZE]; 
                                 /* Stack of thread-specific data items. */
@@ -92,7 +92,7 @@ typedef struct ssh_fsm_message_blob {
 struct ssh_fsm {
   SshFSMHashChain *hash_table[SSH_FSM_HASH_TABLE_SIZE];
                                 /* The state names hash table. */
-  SshFSMStateMapItem *states;   /* The states array. */
+  SshFSMStateMapItem states;   /* The states array. */
   int num_states;               /* Size of the array. */
   SshFSMDestructor destructor;  /* Destructor for FSM-specific data. */
   void *idata;                  /* The FSM-specific data pointer. */
@@ -204,7 +204,7 @@ static SshUInt32 hash_func(char *name)
   return (((SshUInt32)name) % SSH_FSM_HASH_TABLE_SIZE);
 }
 
-static SshFSMStateMapItem *find_state_by_name(SshFSM fsm,
+static SshFSMStateMapItem find_state_by_name(SshFSM fsm,
                                               char *name)
 {
   int i;
@@ -219,7 +219,7 @@ static SshFSMStateMapItem *find_state_by_name(SshFSM fsm,
   return NULL;
 }
 
-static SshFSMStateMapItem *map_state(SshFSM fsm,
+static SshFSMStateMapItem map_state(SshFSM fsm,
                                      char *name)
 {
   SshUInt32 idx = hash_func(name);
@@ -269,7 +269,7 @@ static void clear_hash_table(SshFSM fsm)
 /** Create an FSM object. **/
 
 SshFSM ssh_fsm_allocate(size_t internal_data_size,
-                        SshFSMStateMapItem *states,
+                        SshFSMStateMapItem states,
                         int num_states,
                         SshFSMDestructor destructor)
 {
