@@ -4,7 +4,7 @@ sshincludes.h
 
 Author: Tatu Ylonen <ylo@cs.hut.fi>
 
-Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
+Copyright (c) 1995-2000 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Mon Jan 15 10:36:06 1996 ylo
@@ -13,71 +13,20 @@ Common include files for various platforms.
 
 */
 
-/*
- * $Id: sshincludes.h,v 1.42 2000/03/03 14:57:31 tmo Exp $
- * $Log: sshincludes.h,v $
- * $EndLog$
- */
-
 #ifndef SSHINCLUDES_H
 #define SSHINCLUDES_H
 
-
 /* */
 #include "sshdistdefs.h"
 
-
-/* Conditionals for various OS & compilation environments */
-
-/* Define c++ keywords, so that the compiler will give errors
-   if somebody uses them. */
-#if !defined(__cplusplus) && !defined(SSH_ALLOW_CPLUSPLUS_KEYWORDS)
-#define catch \
-  CATCH_IS_CPLUSPLUS_RESERVED_WORD(CATCH_IS_CPLUSPLUS_RESERVED_WORD) \
-  CATCH_IS_CPLUSPLUS_RESERVED_WORD
-#define class \
-  CLASS_IS_CPLUSPLUS_RESERVED_WORD(CLASS_IS_CPLUSPLUS_RESERVED_WORD) \
-  CLASS_IS_CPLUSPLUS_RESERVED_WORD
-#define delete \
-  DELETE_IS_CPLUSPLUS_RESERVED_WORD(DELETE_IS_CPLUSPLUS_RESERVED_WORD) \
-  DELETE_IS_CPLUSPLUS_RESERVED_WORD
-#define friend \
-  FRIEND_IS_CPLUSPLUS_RESERVED_WORD(FRIEND_IS_CPLUSPLUS_RESERVED_WORD) \
-  FRIEND_IS_CPLUSPLUS_RESERVED_WORD
-#define new \
-  NEW_IS_CPLUSPLUS_RESERVED_WORD(NEW_IS_CPLUSPLUS_RESERVED_WORD) \
-  NEW_IS_CPLUSPLUS_RESERVED_WORD
-#define operator \
-  OPERATOR_IS_CPLUSPLUS_RESERVED_WORD(OPERATOR_IS_CPLUSPLUS_RESERVED_WORD) \
-  OPERATOR_IS_CPLUSPLUS_RESERVED_WORD
-#define private \
-  PRIVATE_IS_CPLUSPLUS_RESERVED_WORD(PRIVATE_IS_CPLUSPLUS_RESERVED_WORD) \
-  PRIVATE_IS_CPLUSPLUS_RESERVED_WORD
-#define protected \
-  PROTECTED_IS_CPLUSPLUS_RESERVED_WORD(PROTECTED_IS_CPLUSPLUS_RESERVED_WORD) \
-  PROTECTED_IS_CPLUSPLUS_RESERVED_WORD
-#define public \
-  PUBLIC_IS_CPLUSPLUS_RESERVED_WORD(PUBLIC_IS_CPLUSPLUS_RESERVED_WORD) \
-  PUBLIC_IS_CPLUSPLUS_RESERVED_WORD
-#define template \
-  TEMPLATE_IS_CPLUSPLUS_RESERVED_WORD(TEMPLATE_IS_CPLUSPLUS_RESERVED_WORD) \
-  TEMPLATE_IS_CPLUSPLUS_RESERVED_WORD
-#define this \
-  THIS_IS_CPLUSPLUS_RESERVED_WORD(THIS_IS_CPLUSPLUS_RESERVED_WORD) \
-  THIS_IS_CPLUSPLUS_RESERVED_WORD
-#define throw \
-  THROW_IS_CPLUSPLUS_RESERVED_WORD(THROW_IS_CPLUSPLUS_RESERVED_WORD) \
-  THROW_IS_CPLUSPLUS_RESERVED_WORD
-#define try \
-  TRY_IS_CPLUSPLUS_RESERVED_WORD(TRY_IS_CPLUSPLUS_RESERVED_WORD) \
-  TRY_IS_CPLUSPLUS_RESERVED_WORD
-#define virtual \
-  VIRTUAL_IS_CPLUSPLUS_RESERVED_WORD(VIRTUAL_IS_CPLUSPLUS_RESERVED_WORD) \
-  VIRTUAL_IS_CPLUSPLUS_RESERVED_WORD
-#endif
-
 #if defined(KERNEL) || defined(_KERNEL)
 
+/* In the kernel mode, we must use ssh_k*alloc() routines. */
+#define SSH_MEM_ALLOC(s)            ssh_kmalloc((s))
+#define SSH_MEM_FREE(p)             ssh_kfree((p))
+#define SSH_MEM_REALLOC(p, o, s)    ssh_krealloc((p), (o), (s))
+#define SSH_MEM_STRDUP(s)           ssh_kstrdup((s))
+
 #ifdef WINNT
 #include "sshincludes_ntddk.h"
 #elif WIN95
@@ -88,14 +37,30 @@ Common include files for various platforms.
 #include "kernel_includes.h"
 #endif
 
+/* Some internal headers used in almost every file. */
+#include "sshdebug.h"
+#include "engine_alloc.h"
+
 #else /* KERNEL || _KERNEL */
 
+#include "sshmalloc.h"
+
+/* In the user mode, we must use ssh_x*alloc() routines. */
+#define SSH_MEM_ALLOC(s)            ssh_xmalloc((s))
+#define SSH_MEM_FREE(p)             ssh_xfree((p))
+#define SSH_MEM_REALLOC(p, o, s)    ssh_xrealloc((p), (s))
+#define SSH_MEM_STRDUP(s)           ssh_xstrdup((s))
+
 #if defined(WIN32)
-#include "sshincludes_win32.h"
+# include "sshincludes_win32.h"
 #elif macintosh
-#include "mac/sshincludes_macos.h"
+# include "mac/sshincludes_macos.h"
 #else
-#include "sshincludes_unix.h"
+# ifdef VXWORKS
+#   include "sshincludes_vxworks.h"
+# else /* VXWORKS */
+#   include "sshincludes_unix.h"
+# endif /* VXWORKS */
 #endif /* WIN32 */
 
 #endif /* KERNEL || _KERNEL */
@@ -127,13 +92,6 @@ Common include files for various platforms.
 #endif
 #define rindex(A,B) ssh_fatal(RINDEX_IS_BSDISM_USE_STRRCHR_INSTEAD)
 
-#if 0
-#ifdef interface
-# undef interface
-#endif
-#define interface ssh_fatal(INTERFACE_IS_RESERVED_AT_MVC)
-#endif
-
 /* Force library to use ssh- memory allocators (they may be
    implemented using zone mallocs, debug-routines or something
    similar) */
@@ -174,7 +132,8 @@ Common include files for various platforms.
 #ifdef localtime
 # undef localtime
 #endif
-#define localtime ssh_fatal(LOCALTIME_IS_FORBIDDEN_USE_SSH_CALENDAR_TIME_INSTEAD)
+#define localtime \
+        ssh_fatal(LOCALTIME_IS_FORBIDDEN_USE_SSH_CALENDAR_TIME_INSTEAD)
 
 #ifdef gmtime
 # undef gmtime
@@ -195,10 +154,66 @@ Common include files for various platforms.
 # undef mktime
 #endif
 #define mktime ssh_fatal(MKTIME_IS_FORBIDDEN)
+
+/* Conditionals for various OS & compilation environments */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 /* Some internal headers used in almost every file. */
 #include "sshdebug.h"
-#include "sshmalloc.h"
 #include "sshtime.h"
 
 #ifndef SSH_CODE_SEGMENT
