@@ -5,7 +5,7 @@
   Author: Jussi Kukkonen <kukkonen@ssh.fi>
           Tatu Ylnen <ylo@ssh.fi>
 
-  Copyright (c) 1999 SSH Communications Security, Finland.
+  Copyright (c) 1999-2000 SSH Communications Security, Finland.
   All rights reserved.
 
   Common definitions for various Unix platforms, extracted
@@ -13,27 +13,22 @@
 
 */
 
-/*
- * $Id: sshincludes_unix.h,v 1.1 1999/04/28 14:41:09 kukkonen Exp $
- * $Log: sshincludes_unix.h,v $
- * $EndLog$
- */
-
 #ifndef SSHINCLUDES_UNIX_H
 #define SSHINCLUDES_UNIX_H
 
-
-#ifndef macintosh
 #include <sys/types.h>
 #include <sys/stat.h>
-#else
-#include <stat.h>
-#endif
 
 #include "sshconf.h"
 
-#define DLLCALLCONV 
-#define DLLEXPORT 
+#define DLLCALLCONV
+#define DLLEXPORT
+
+/* Set SIZEOF_* defines to point to usermode definitions of those */
+#define SIZEOF_INT      USERMODE_SIZEOF_INT
+#define SIZEOF_LONG     USERMODE_SIZEOF_LONG
+#define SIZEOF_LONG_LONG USERMODE_SIZEOF_LONG_LONG
+#define SIZEOF_SHORT    USERMODE_SIZEOF_SHORT
 
 typedef unsigned char SshUInt8;         /* At least 8 bits. */
 typedef signed char SshInt8;            /* At least 8 bits. */
@@ -58,33 +53,39 @@ typedef short SshInt32;                 /* At least 32 bits. */
 #endif
 #endif
 
+#ifdef HAVE_USERMODE_INT
+#define HAVE_INT
+#endif
+#ifdef HAVE_USERMODE_LONG
+#define HAVE_LONG
+#endif
+#ifdef HAVE_USERMODE_LONG_LONG
+#define HAVE_LONG_LONG
+#endif
+#ifdef HAVE_USERMODE_SHORT
+#define HAVE_SHORT
+#endif
+
 #if SIZEOF_LONG >= 8
 typedef unsigned long SshUInt64;
 typedef long SshInt64;
 #define SSHUINT64_IS_64BITS
+#define SSH_C64(x) (x##u)
 #else
 #if SIZEOF_LONG_LONG >= 8
 typedef unsigned long long SshUInt64;
 typedef long long SshInt64;
 #define SSHUINT64_IS_64BITS
+#define SSH_C64(x) (x##llu)
 #else
 /* No 64 bit type; SshUInt64 and SshInt64 will be 32 bits. */
 typedef unsigned long SshUInt64;
 typedef long SshInt64;
+#define SSH_C64(x) ssh_fatal(ERROR_NO_64_BIT_ON_THIS_SYSTEM)
 #endif
 #endif
 
-#ifndef macintosh
 #include <sys/types.h>
-#else /* macintosh */
-#ifdef __MWERKS__
-#include <types.h>
-#include <OpenTransport.h>
-#else
-#error "Don't know how to compile Mac code with other compilers but CodeWarrior"
-#endif /* __MWERKS__ */
-#endif /* macintosh */
-
 #ifdef HAVE_MACHINE_ENDIAN_H
 #include <sys/param.h>
 #include <machine/endian.h>
@@ -94,8 +95,6 @@ typedef long SshInt64;
 #include <endian.h>
 #endif
 
-
-
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
@@ -172,7 +171,7 @@ char *strchr(), *strrchr();
 #endif /* HAVE_GRP_H */
 
 #ifdef HAVE_DIRENT_H
-#include "dirent.h"
+#include <dirent.h>
 #endif /* HAVE_DIRENT_H */
 
 /* These POSIX macros are not defined in every system. */
@@ -232,7 +231,7 @@ char *strchr(), *strrchr();
 /* Some systems have broken S_ISDIR etc. macros in sys/stat.h.  Please ask
    your vendor to fix them.  You can then remove the line below, but only
    after you have sent a complaint to your vendor. */
-#error "Warning macros in sys stat h are broken on your system read sshincludes.h"
+#error "Warning macros in sys stat h are broken read sshincludes unix h"
 #endif /* STAT_MACROS_BROKEN */
 
 #if USE_STRLEN_FOR_AF_UNIX
@@ -250,9 +249,7 @@ char *strchr(), *strrchr();
 #define FALSE 0
 #endif
 
-#ifndef macintosh
 typedef unsigned int Boolean;
-#endif
 
 #ifndef O_BINARY
 /* Define O_BINARY for compatibility with Windows. */
@@ -264,5 +261,23 @@ typedef unsigned int Boolean;
 #include "sshsnprintf.h"
 #endif /* HAVE_SNPRINTF */
 
+#ifdef HAVE_USLEEP
+#define ssh_sleep(secs, usecs) \
+  do { \
+    if ((secs) > 0) sleep((secs)); \
+    SSH_ASSERT((usecs) < 1000000); \
+    usleep((usecs)); \
+} while (0)
+#else /* HAVE_USLEEP */
+#define ssh_sleep(secs, usecs)    sleep((secs))
+#endif /* HAVE_USLEEP */
+
+#ifdef __SUNPRO_C
+#pragma error_messages (off,E_END_OF_LOOP_CODE_NOT_REACHED)
+#endif /* __SUNPRO_C */
+
+#ifdef __SUNPRO_C
+#pragma error_messages (off,E_END_OF_LOOP_CODE_NOT_REACHED)
+#endif /* __SUNPRO_C */
 
 #endif /* SSHINCLUDES_UNIX_H */
