@@ -21,8 +21,11 @@
 
 #define MAX_IP_ADDR_LEN 16
 
+/* The address string of the SSH_IPADDR_ANY. */
+const char *ssh_ipaddr_any = "*** SSH_IPADDR_ANY ***";
+
 /* Mapping between protocol name and doi protocol number */
-const SshKeyword ssh_ip_protocol_id_keywords[] =
+const SshKeywordStruct ssh_ip_protocol_id_keywords[] =
 {
   { "any", SSH_IPPROTO_ANY },
   { "icmp", SSH_IPPROTO_ICMP },
@@ -288,7 +291,7 @@ int ssh_inet_ip_address_compare(const char *address1, const char *address2)
   size_t len;
   int ret;
 
-  len = MAX_IP_ADDR_LEN; 
+  len = MAX_IP_ADDR_LEN;
   if (!ssh_inet_strtobin(address1, a1, &len))
     return 0;
 
@@ -298,7 +301,7 @@ int ssh_inet_ip_address_compare(const char *address1, const char *address2)
       memset(a1, 0, 12);
     }
 
-  len = MAX_IP_ADDR_LEN; 
+  len = MAX_IP_ADDR_LEN;
   if (!ssh_inet_strtobin(address2, a2, &len))
     return 0;
 
@@ -390,7 +393,9 @@ void ssh_ipaddr_merge_bits(SshIpAddr result, SshIpAddr left_ip,
       (right_ip->addr_data[i / 8] & ~(0xff << (8 - left_bits % 8)));
 
 #if 0
-    fprintf(stderr, "i=%d left_bits=%d left_bytes=%d total_bits=%d shift=%d left=%d right=%d\n",
+    fprintf(stderr,
+            "i=%d left_bits=%d left_bytes=%d total_bits=%d shift=%d left=%d "
+            "right=%d\n",
             i, left_bits, left_bits / 8, total_bits,
             (8 - left_bits % 8),
             (left_ip->addr_data[i / 8] & (0xff << (8 - left_bits % 8))),
@@ -460,7 +465,7 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
   right = 0;
   left_ptr = 0;
   right_ptr = 16;
-  
+
   start = cp = str;
 
   /* Look for next ':' delimiter */
@@ -477,7 +482,7 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
         }
 
       len = cp - start;
-      
+
       if (len == 0)
         {
           if (*next != ':')
@@ -487,7 +492,7 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
             }
           need_bytes = 0;
         }
-      
+
       /* ipv6 'x', 'xx', 'xxx' or 'xxxx' part? */
       else if (len <= 4)
         {
@@ -500,37 +505,41 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
                 }
               tmp = (tmp << 4) | h2i(start[i]);
             }
-          
+
           out_bytes[0] = (unsigned char)((tmp >>  8) & 0xff);
           out_bytes[1] = (unsigned char)((tmp >>  0) & 0xff);
-          
+
           need_bytes = 2;
         }
       else if (memchr(start, '.', len) != NULL && (len <= 15))
         {
           char buf[16];
-          
+
           memcpy(buf, start, len);
           buf[len] = '\0';
-          
+
           if (ssh_ipaddr_ipv4_parse(out_bytes, buf) == FALSE)
             return FALSE;
-          
+
           need_bytes = 4;
         }
-      
+
       else
         {
           /* printf("ERROR: Unrecognized address part\n"); */
           return FALSE;
         }
-      
+
       if ((right_ptr - left_ptr) < need_bytes)
         {
-          /* printf("ERROR: Not enough space in output address (have %d, required %d)\n", right_ptr - left_ptr, need_bytes); */
+#if 0
+          printf("ERROR: Not enough space in output address "
+                 "(have %d, required %d)\n",
+                 right_ptr - left_ptr, need_bytes);
+#endif
           return FALSE;
         }
-      
+
       if (right)
         {
           if (addr)
@@ -548,7 +557,7 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
             memcpy(addr + left_ptr, out_bytes, need_bytes);
           left_ptr += need_bytes;
         }
-      
+
       if (*next == ':')
         {
           if (right)
@@ -556,22 +565,22 @@ static Boolean ssh_ipaddr_ipv6_parse(unsigned char *addr, const char *str)
               /* printf("ERROR: Already seen '::'\n"); */
               return FALSE;
             }
-          
+
           right = 1;
           next++;
         }
-      
+
       /* Move on to next iteration */
       start = next;
     }
-  
+
   if ((right_ptr - left_ptr) > 0 && !right)
     {
       /* printf("ERROR: %d unresolved address bytes\n",
          right_ptr - left_ptr); */
       return FALSE;
     }
-  
+
   return TRUE;
 }
 
@@ -622,10 +631,21 @@ Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip,
 
   if (mask == NULL)
     {
+#ifdef _KERNEL
+      dup = ssh_kstrdup(str);
+#else
       dup = ssh_xstrdup(str);
+#endif
+      if (!dup)
+        return FALSE;
+
       if ((cp = strchr(dup, '/')) == NULL)
         {
+#ifdef _KERNEL
+          ssh_kfree(dup);
+#else
           ssh_xfree(dup);
+#endif
           return FALSE;
         }
       str = dup;
@@ -678,7 +698,11 @@ Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip,
     }
 
   if (dup != NULL)
+#ifdef _KERNEL
+    ssh_kfree(dup);
+#else
     ssh_xfree(dup);
+#endif
 
   return ret;
 }
@@ -701,29 +725,60 @@ static void ssh_ipaddr_ipv4_print(const unsigned char *data,
 static void ssh_ipaddr_ipv6_print(const unsigned char *data,
                                   char *buf, size_t buflen)
 {
-  int         i;
-  char        largebuf[64], * cp;
-  unsigned long       tmp;
+  int i, j;
+  char largebuf[64], *cp;
+  int opt_start = 8;
+  int opt_len = 0;
+  SshUInt16 val;
+
+  /* Optimize the largest zero-block from the address. */
+  for (i = 0; i < 8; i++)
+    if (SSH_GET_16BIT(data + i * 2) == 0)
+      {
+        for (j = i + 1; j < 8; j++)
+          {
+            val = SSH_GET_16BIT(data + j * 2);
+            if (val != 0)
+              break;
+          }
+        if (j - i > opt_len)
+          {
+            opt_start = i;
+            opt_len = j - i;
+          }
+        i = j;
+      }
+
+  if (opt_len <= 1)
+    /* Disable optimization. */
+    opt_start = 8;
 
   cp = largebuf;
 
-  /* First, skip any leading zeros -- the only pretty-printing we do */
-  for (i = 0; i < 16; i += 2)
-    if (data[i] || data[i + 1])
-      break;
-
-  if (i != 0)
+  /* Format the result. */
+  for (i = 0; i < 8; i++)
     {
-      strcpy(largebuf, "::");
-      cp = largebuf + 2;
-    }
+      if (i == opt_start)
+        {
+          if (i == 0)
+            *cp++ = ':';
 
-  for (; i < 16; i += 2)
-    {
-      tmp = (data[i] << 8) | data[i + 1];
-      snprintf(cp, 6, "%x%s", (unsigned int) tmp, (i != 14 ? ":" : ""));
-      cp += strlen(cp);
+          *cp++ = ':';
+          i += opt_len - 1;
+        }
+      else
+        {
+          snprintf(cp, 20, "%x", (unsigned int) SSH_GET_16BIT(data + i * 2));
+          cp += strlen(cp);
+
+          if (i + 1 < 8)
+            {
+              *cp = ':';
+              cp++;
+            }
+        }
     }
+  *cp = '\0';
 
   strncpy(buf, largebuf, buflen);
   buf[buflen - 1] = '\0';
@@ -776,12 +831,12 @@ Boolean ssh_ipaddr_with_mask_equal(SshIpAddr ip1, SshIpAddr ip2,
     memcpy(i2 + 12, ip2->addr_data, 4);
   else
     memcpy(i2, ip2->addr_data, 16);
-  
+
   if (SSH_IP_IS4(mask))
     memcpy(m + 12, mask->addr_data, 4);
   else
     memcpy(m, mask->addr_data, 16);
-  
+
   for (i = 0; i < 16; i++)
     if ((i1[i] & m[i]) != (i2[i] & m[i]))
       return FALSE;
@@ -843,12 +898,14 @@ Boolean ssh_ipaddr_mask_equal(SshIpAddr ip, SshIpAddr masked_ip)
 
 /* Render function to render IP addresses for %@ format string for
    ssh_e*printf */
-int ssh_ipaddr_render(char * buf, int buf_size, int precision, void * datum)
+int ssh_ipaddr_render(char *buf, int buf_size, int precision, void *datum)
 {
   const SshIpAddr ip = (SshIpAddr) datum;
   int len;
 
-  if (SSH_IP_ADDR_LEN(ip) * 8 == SSH_IP_MASK_LEN(ip))
+  if (ip->type == SSH_IP_TYPE_NONE)
+    snprintf(buf, buf_size, "<none>");
+  else if (SSH_IP_ADDR_LEN(ip) * 8 == SSH_IP_MASK_LEN(ip))
     (void) ssh_ipaddr_print(ip, buf, buf_size);
   else
     (void) ssh_ipaddr_print_with_mask(ip, buf, buf_size);
@@ -863,7 +920,7 @@ int ssh_ipaddr_render(char * buf, int buf_size, int precision, void * datum)
 }
 
 /* Renders an IP protocol name */
-int ssh_ipproto_render(char * buf, int buf_size, int precision, void * datum)
+int ssh_ipproto_render(char *buf, int buf_size, int precision, void *datum)
 {
   SshInetIPProtocolID proto = (SshInetIPProtocolID) datum;
   int len, i;
@@ -886,3 +943,33 @@ int ssh_ipproto_render(char * buf, int buf_size, int precision, void * datum)
 
   return len;
 }
+
+/* Renders an IP address mask. */
+int ssh_ipmask_render(char *buf, int buf_size, int precision, void *datum)
+{
+  const SshIpAddr ip = (SshIpAddr) datum;
+  int i, j;
+  int bits = 0;
+  int len;
+
+  /* The non-IPv6 masks are printed just like IP addresses. */
+  if (!SSH_IP_IS6(ip))
+    return ssh_ipaddr_render(buf, buf_size, precision, datum);
+
+  /* The IPv6 masks are rendered as number describing the prefix
+     len. */
+
+  for (i = 0; i < SSH_IP_ADDR_LEN(ip); i++)
+    for (j = 7; j >= 0; j--)
+      if (ip->addr_data[i] & (1 << j))
+        bits++;
+
+  snprintf(buf, buf_size, "%d", bits);
+  len = strlen(buf);
+
+  if (precision >= 0)
+    if (len > precision)
+      len = precision;
+
+  return len;
+}
