@@ -1,19 +1,19 @@
 /*
   File: sshinet.h
 
-  Authors: 
+  Authors:
         Tero T Mononen <tmo@ssh.fi>
         Tero Kivinen <kivinen@ssh.fi>
         Tatu Ylonen <ylo@ssh.fi>
 
-  Description: 
+  Description:
         IP protocol specific definitions.  This file also contains functions
         and macros for manipulating IP addresses in various forms, as well
         as for manipulating IP, TCP, and UDP headers.  This file also
         contains definitions specific to various media types (e.g. ethernet).
 
   Copyright:
-        Copyright (c) 1998-1999 SSH Communications Security, Finland
+        Copyright (c) 1998-2000 SSH Communications Security, Finland
         All rights reserved
 */
 
@@ -23,8 +23,23 @@
 #include "sshgetput.h"
 #include "sshenum.h"
 
+/* Special IP address string that can be given for TCP and UDP
+   listeners to bind to all local IP addresses.  This will bind both
+   to IPv4 and IPv6 addresses, if supported by the system. */
+#define SSH_IPADDR_ANY ssh_ipaddr_any
+
+/* Constant placeholder for the SSH_IPADDR_ANY address string. */
+extern const char *ssh_ipaddr_any;
+
+/* Predicate to check whether the address string `addr' is an
+   SSH_IPADDR_ANY. */
+#define SSH_IS_IPADDR_ANY(addr) \
+  ((addr)                       \
+   && ((addr) == SSH_IPADDR_ANY \
+       || strcmp((addr), SSH_IPADDR_ANY) == 0))
+
 /* IP protocol names to keywords definitions */
-extern const SshKeyword ssh_ip_protocol_id_keywords[];
+extern const SshKeywordStruct ssh_ip_protocol_id_keywords[];
 
 /* IP protocol identifiers */
 typedef enum {
@@ -78,8 +93,8 @@ typedef enum {
   SSH_IPPROTO_GRE = 47,      /* General Routing Encapsulation */
   SSH_IPPROTO_MHRP = 48,     /* Mobile Host Routing */
   SSH_IPPROTO_BNA = 49,      /* BNA [Gary Salamon] */
-  SSH_IPPROTO_ESP = 50,      /* Encap Security Payload [RFC1827] */
-  SSH_IPPROTO_AH = 51,       /* Authentication Header [RFC1826] */
+  SSH_IPPROTO_ESP = 50,      /* Encap Security Payload [RFC2406] */
+  SSH_IPPROTO_AH = 51,       /* Authentication Header [RFC2402] */
   SSH_IPPROTO_INLSP = 52,    /* Integrated Net Layer Sec TUBA */
   SSH_IPPROTO_SWIPE = 53,    /* IP with Encryption [JI6] */
   SSH_IPPROTO_NARP = 54,     /* NBMA Address Resolution [RFC1735] */
@@ -148,7 +163,6 @@ typedef enum {
 
 #define SSH_IPPROTO_MIN (  0)
 #define SSH_IPPROTO_MAX (255)
-#define SSH_IPPROTO_MAX (255)
 
 /* Minimum length of the ICMP header. */
 #define SSH_ICMP_HEADER_MINLEN  8
@@ -202,7 +216,58 @@ typedef enum {
   SSH_ICMP_CODE_TIMXCEED_REASS = 1       /* TTL becomes zero in reassembly */
 } SshInetIPIcmpTimexceedCode;
 
-/* --------------------- auxiliary functions -------------------------*/
+
+/* ICMP6 types and codes */
+typedef enum {
+  /* ICMP6 error types */
+  SSH_ICMP6_TYPE_UNREACH = 1,           /* Destination unreachable */
+  SSH_ICMP6_TYPE_TOOBIG = 2,            /* Packet too big */
+  SSH_ICMP6_TYPE_TIMXCEED = 3,          /* Time exceeded */
+  SSH_ICMP6_TYPE_PARAMPROB = 4,         /* Parameter problem */
+
+  /* ICMP6 informational types */
+  SSH_ICMP6_TYPE_ECHOREQUEST = 128,     /* Echo request */
+  SSH_ICMP6_TYPE_ECHOREPLY = 129,        /* Echo reply */
+
+  /* From RFC2461 */
+  SSH_ICMP6_TYPE_ROUTER_SOLICITATION = 133,      /* Router solicitation */
+  SSH_ICMP6_TYPE_ROUTER_ADVERTISEMENT = 134,     /* Router advertisement */
+  SSH_ICMP6_TYPE_NEIGHBOR_SOLICITATION = 135,   /* Neighbor Solicitation */
+  SSH_ICMP6_TYPE_NEIGHBOR_ADVERTISEMENT = 136 /* Neighbor Advertisement */
+} SshInetIPIcmp6Type;
+
+/* ICMP6 destination unreachable codes */
+typedef enum {
+  SSH_ICMP6_CODE_UNREACH_NOROUTE = 0,   /* No route to destination */
+  SSH_ICMP6_CODE_UNREACH_PROHIBITED = 1, /* Communication with host
+                                            administratively prohibited */
+  SSH_ICMP6_CODE_UNREACH_ADDRESS = 3,   /* Address unreachable */
+  SSH_ICMP6_CODE_UNREACH_PORT = 4       /* Port unreachable */
+} SshInetIPIcmp6UnreachCode;
+
+/* ICMP6 time exceeded codes */
+typedef enum {
+  SSH_ICMP6_CODE_TIMXCEED_HOP = 0,      /* Hop limit exceeded */
+  SSH_ICMP6_CODE_TIMXCEED_REASS = 1     /* Reassembly time exceeded */
+} SshInetIPIcmp6TimexceedCode;
+
+/* ICMP6 parameter problem codes */
+typedef enum {
+  SSH_ICMP6_CODE_PARAMPROB_HEADER = 0,  /* Erroneous header */
+  SSH_ICMP6_CODE_PARAMPROB_NH = 1,      /* Unrecognized NH (next hop) type */
+  SSH_ICMP6_CODE_PARAMPROB_OPTION = 2   /* Unrecognized option */
+} SshInetIPIcmp6ParamprobCode;
+
+/* ICMP6 neighbor discovery option types */
+typedef enum {
+  SSH_ICMP6_NEIGHDISC_OPT_SOURCE_LINK_ADDRESS = 1,
+  SSH_ICMP6_NEIGHDISC_OPT_TARGET_LINK_ADDRESS = 2,
+  SSH_ICMP6_NEIGHDISC_OPT_PREFIX_INFORMATION = 3,
+  SSH_ICMP6_NEIGHDISC_OPT_REDIRECTED_HEADER = 4,
+  SSH_ICMP6_NEIGHDISC_OPT_MTU = 5
+} SshInetIPIcmp6NeighdiscOptionType;
+
+/*************************** Auxiliary functions ****************************/
 
 /* Determines whether the given string is a valid numeric IP address.
    (This currently only works for IPv4 addresses, but might be changed
@@ -232,7 +297,8 @@ Boolean ssh_inet_compare_netmask(const char *nets, const char *ip);
 Boolean ssh_inet_strtobin(const char *ip_address, unsigned char *out_buffer,
                           size_t *out_buffer_len_in_out);
 
-/************************* SshIpAddr stuff **************************/
+
+/***************************** SshIpAddr stuff ******************************/
 
 typedef enum {
     SSH_IP_TYPE_NONE = 0,
@@ -249,96 +315,168 @@ typedef struct SshIpAddrRec
 } *SshIpAddr, SshIpAddrStruct;
 
 #define SSH_IP_DEFINED(ip_addr) ((ip_addr)->type != SSH_IP_TYPE_NONE)
-#define SSH_IP_IS4(ip_addr)             ((ip_addr)->type == SSH_IP_TYPE_IPV4)
-#define SSH_IP_IS6(ip_addr)             ((ip_addr)->type == SSH_IP_TYPE_IPV6)
+#define SSH_IP_IS4(ip_addr)     ((ip_addr)->type == SSH_IP_TYPE_IPV4)
+#define SSH_IP_IS6(ip_addr)     ((ip_addr)->type == SSH_IP_TYPE_IPV6)
 
-#define SSH_IP_ADDR_LEN(ip_addr)                (SSH_IP_IS4(ip_addr) ? (4) : SSH_IP_IS6(ip_addr) ? (16) : 0)
+#define SSH_IP_ADDR_LEN(ip_addr)        \
+  (SSH_IP_IS4(ip_addr)                  \
+   ? (4)                                \
+   : (SSH_IP_IS6(ip_addr)               \
+      ? (16)                            \
+      : 0))
+
+#define SSH_IP_UNDEFINE(IPADDR)                     \
+do {                                            \
+  (IPADDR)->type = SSH_IP_TYPE_NONE;            \
+} while (0)
 
 #define __SSH_IP_MASK_DECODE(IPADDR,TYPE,BYTES,BYTELEN,MASKLEN) \
-        do { (IPADDR)->type = (TYPE); \
-             memmove((IPADDR)->addr_data, (BYTES), (BYTELEN)); \
-             (IPADDR)->mask_len = (MASKLEN); } while (0)
+  do {                                                          \
+    (IPADDR)->type = (TYPE);                                    \
+    memmove((IPADDR)->addr_data, (BYTES), (BYTELEN));           \
+    (IPADDR)->mask_len = (MASKLEN);                             \
+  } while (0)
 
 #define __SSH_IP_MASK_ENCODE(IPADDR,TYPE,BYTES,BYTELEN,MASKPTR) \
-        do { SSH_ASSERT((IPADDR)->type == (TYPE)); \
-             memmove((BYTES), (IPADDR)->addr_data, (BYTELEN)); \
-             if (MASKPTR) *((SshUInt32 *) (MASKPTR)) = (IPADDR)->mask_len; } while (0)
+  do {                                                          \
+    SSH_ASSERT((IPADDR)->type == (TYPE));                       \
+    memmove((BYTES), (IPADDR)->addr_data, (BYTELEN));           \
+    if (MASKPTR)                                                \
+      *((SshUInt32 *) (MASKPTR)) = (IPADDR)->mask_len;          \
+  } while (0)
 
-#define SSH_IP4_DECODE(ip_addr,bytes)           __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,32)
-#define SSH_IP4_MASK_DECODE(ip_addr,bytes,mask) __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,mask)
+#define SSH_IP4_DECODE(ip_addr,bytes) \
+  __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,32)
+#define SSH_IP4_MASK_DECODE(ip_addr,bytes,mask) \
+  __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,mask)
 
-#define SSH_IP6_DECODE(ip_addr,bytes)           __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,128)
-#define SSH_IP6_MASK_DECODE(ip_addr,bytes,mask) __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,mask)
+#define SSH_IP6_DECODE(ip_addr,bytes) \
+  __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,128)
+#define SSH_IP6_MASK_DECODE(ip_addr,bytes,mask) \
+  __SSH_IP_MASK_DECODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,mask)
 
-#define SSH_IP4_ENCODE(ip_addr,bytes)           __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,NULL)
-#define SSH_IP4_MASK_ENCODE(ip_addr,bytes,mask) __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,mask)
+#define SSH_IP4_ENCODE(ip_addr,bytes) \
+  __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,NULL)
+#define SSH_IP4_MASK_ENCODE(ip_addr,bytes,mask) \
+  __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV4,bytes,4,mask)
 
-#define SSH_IP6_ENCODE(ip_addr,bytes)           __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,NULL)
-#define SSH_IP6_MASK_ENCODE(ip_addr,bytes,mask) __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,mask)
+#define SSH_IP6_ENCODE(ip_addr,bytes) \
+  __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,NULL)
+#define SSH_IP6_MASK_ENCODE(ip_addr,bytes,mask) \
+  __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,mask)
 
-#define SSH_IP4_TO_INT(ip_addr) \
-     (((unsigned long)(ip_addr)->addr_data[0] << 24) | \
-      ((unsigned long)(ip_addr)->addr_data[1] << 16) | \
-      ((unsigned long)(ip_addr)->addr_data[2] << 8) | \
-      (unsigned long)(ip_addr)->addr_data[3])
+#define __SSH_IP_TO_INT(addr_data)              \
+  (((unsigned long) (addr_data)[0] << 24)       \
+   | ((unsigned long) (addr_data)[1] << 16)     \
+   | ((unsigned long) (addr_data)[2] << 8)      \
+   | (unsigned long) (addr_data)[3])
 
-#define SSH_INT_TO_IP4(ip_addr, num) \
-        do { (ip_addr)->type = SSH_IP_TYPE_IPV4; (ip_addr)->mask_len = 32; \
-          (ip_addr)->addr_data[0] = (unsigned char) (((num) >> 24) & 0xff); \
-          (ip_addr)->addr_data[1] = (unsigned char) (((num) >> 16) & 0xff); \
-          (ip_addr)->addr_data[2] = (unsigned char) (((num) >>  8) & 0xff); \
-          (ip_addr)->addr_data[3] = (unsigned char) (((num) >>  0) & 0xff); } while (0)
+#define SSH_IP4_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data)
+
+#define SSH_INT_TO_IP4(ip_addr, num)                                    \
+  do {                                                                  \
+    (ip_addr)->type = SSH_IP_TYPE_IPV4;                                 \
+    (ip_addr)->mask_len = 32;                                           \
+    (ip_addr)->addr_data[0] = (unsigned char) (((num) >> 24) & 0xff);   \
+    (ip_addr)->addr_data[1] = (unsigned char) (((num) >> 16) & 0xff);   \
+    (ip_addr)->addr_data[2] = (unsigned char) (((num) >>  8) & 0xff);   \
+    (ip_addr)->addr_data[3] = (unsigned char) (((num) >>  0) & 0xff);   \
+  } while (0)
+
+#define SSH_IP6_WORD0_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data)
+#define SSH_IP6_WORD1_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data + 4)
+#define SSH_IP6_WORD2_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data + 8)
+#define SSH_IP6_WORD3_TO_INT(ip_addr) \
+  __SSH_IP_TO_INT((ip_addr)->addr_data + 12)
 
 #define SSH_IP4_BYTE1(ip_addr) ((ip_addr)->addr_data[0])
 #define SSH_IP4_BYTE2(ip_addr) ((ip_addr)->addr_data[1])
 #define SSH_IP4_BYTE3(ip_addr) ((ip_addr)->addr_data[2])
 #define SSH_IP4_BYTE4(ip_addr) ((ip_addr)->addr_data[3])
 
-#define SSH_IP_MASK_LEN(ip_addr)                ((ip_addr)->mask_len)
+#define SSH_IP6_BYTE1(ip_addr) ((ip_addr)->addr_data[0])
+#define SSH_IP6_BYTE2(ip_addr) ((ip_addr)->addr_data[1])
+#define SSH_IP6_BYTE3(ip_addr) ((ip_addr)->addr_data[2])
+#define SSH_IP6_BYTE4(ip_addr) ((ip_addr)->addr_data[3])
+#define SSH_IP6_BYTE5(ip_addr) ((ip_addr)->addr_data[4])
+#define SSH_IP6_BYTE6(ip_addr) ((ip_addr)->addr_data[5])
+#define SSH_IP6_BYTE7(ip_addr) ((ip_addr)->addr_data[6])
+#define SSH_IP6_BYTE8(ip_addr) ((ip_addr)->addr_data[7])
+#define SSH_IP6_BYTE9(ip_addr) ((ip_addr)->addr_data[8])
+#define SSH_IP6_BYTE10(ip_addr) ((ip_addr)->addr_data[9])
+#define SSH_IP6_BYTE11(ip_addr) ((ip_addr)->addr_data[10])
+#define SSH_IP6_BYTE12(ip_addr) ((ip_addr)->addr_data[11])
+#define SSH_IP6_BYTE13(ip_addr) ((ip_addr)->addr_data[12])
+#define SSH_IP6_BYTE14(ip_addr) ((ip_addr)->addr_data[13])
+#define SSH_IP6_BYTE15(ip_addr) ((ip_addr)->addr_data[14])
+#define SSH_IP6_BYTE16(ip_addr) ((ip_addr)->addr_data[15])
 
-#define SSH_IP_CMP(ip1, ip2) \
-        (((ip1)->type != (ip2)->type) ? \
-                ((ip1)->type - (ip2)->type) : \
-                (memcmp((ip1)->addr_data, (ip2)->addr_data, \
-                        SSH_IP_IS6(ip1) ? 16U : 4U)))
+#define SSH_IP_MASK_LEN(ip_addr) ((ip_addr)->mask_len)
 
-#define SSH_IP_EQUAL(ip1, ip2)                  ((ip1)->type == (ip2)->type && \
-        memcmp((ip1)->addr_data, (ip2)->addr_data, SSH_IP_IS6(ip1) ? 16U : 4U) == 0)
+#define SSH_IP_CMP(ip1, ip2)            \
+  (((ip1)->type != (ip2)->type)         \
+   ? ((ip1)->type - (ip2)->type)        \
+   : (memcmp((ip1)->addr_data, (ip2)->addr_data, SSH_IP_IS6(ip1) ? 16U : 4U)))
 
-#define SSH_IP_MASK_EQUAL(ip1, ip2)     ssh_ipaddr_mask_equal(ip1, ip2)
-#define SSH_IP_WITH_MASK_EQUAL(ip1, ip2,mask) ssh_ipaddr_equal_with_mask(ip1, ip2,mask)
+#define SSH_IP_EQUAL(ip1, ip2)                          \
+  ((ip1)->type == (ip2)->type                           \
+   && memcmp((ip1)->addr_data, (ip2)->addr_data,        \
+             SSH_IP_IS6(ip1) ? 16U : 4U) == 0)
+
+#define SSH_IP_MASK_EQUAL(ip1, ip2) ssh_ipaddr_mask_equal(ip1, ip2)
+#define SSH_IP_WITH_MASK_EQUAL(ip1, ip2,mask) \
+  ssh_ipaddr_equal_with_mask(ip1, ip2,mask)
 
 #define SSH_IP_HASH(ip_addr) ssh_ipaddr_hash(ip_addr)
 
-#define SSH_IP_IS_NULLADDR(ip_addr) \
-     (SSH_IP_IS6(ip_addr) ? \
-      !memcmp((ip_addr)->addr_data, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 16) : \
-      !memcmp((ip_addr)->addr_data, "\0\0\0\0", 4))
+#define SSH_IP4_NULLADDR "0.0.0.0"
+#define SSH_IP6_NULLADDR "::"
+
+#define SSH_IP_IS_NULLADDR(ip_addr)                     \
+  (SSH_IP_IS6(ip_addr)                                  \
+   ? !memcmp((ip_addr)->addr_data,                      \
+             "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 16)    \
+   : !memcmp((ip_addr)->addr_data, "\0\0\0\0", 4))
 
 /* Returns TRUE if the given IP address is the link broadcast address
    255.255.255.255.  This returns FALSE for IPv6, where link local
    addresses are apparently treated like multicast addresses, at least
    for ARP. */
-#define SSH_IP_IS_BROADCAST(ip_addr) \
-     (!SSH_IP_IS6(ip_addr) && (ip_addr)->addr_data[0] == 0xff && \
-      (ip_addr)->addr_data[1] == 0xff && (ip_addr)->addr_data[2] == 0xff && \
-      (ip_addr)->addr_data[3] == 0xff)
+#define SSH_IP_IS_BROADCAST(ip_addr)    \
+  (!SSH_IP_IS6(ip_addr)                 \
+   && (ip_addr)->addr_data[0] == 0xff   \
+   && (ip_addr)->addr_data[1] == 0xff   \
+   && (ip_addr)->addr_data[2] == 0xff   \
+   && (ip_addr)->addr_data[3] == 0xff)
 
 /* Returns TRUE if the given IP address is a multicast address between
    224.0.0.0 and 239.255.255.255, inclusive. */
-#define SSH_IP_IS_MULTICAST(ip_addr) \
-     (SSH_IP_IS6(ip_addr) ? \
-      ((ip_addr)->addr_data[0] == 0xff) : \
-      ((ip_addr)->addr_data[0] >= 0xe0 && (ip_addr)->addr_data[0] <= 0xef))
+#define SSH_IP_IS_MULTICAST(ip_addr)    \
+  (SSH_IP_IS6(ip_addr) ?                \
+   ((ip_addr)->addr_data[0] == 0xff) :  \
+   ((ip_addr)->addr_data[0] >= 0xe0 && (ip_addr)->addr_data[0] <= 0xef))
 
 /* Returns TRUE if the given IP address is a loopback address. */
-#define SSH_IP_IS_LOOPBACK(ip_addr) \
-     (SSH_IP_IS6(ip_addr) ? \
-      (memcmp((ip_addr)->addr_data, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\1", 16) == 0) :\
-      (SSH_IP4_BYTE1(ip_addr) == 127))
+#define SSH_IP_IS_LOOPBACK(ip_addr)                             \
+  (SSH_IP_IS6(ip_addr)                                          \
+   ? (memcmp((ip_addr)->addr_data,                              \
+             "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\1", 16) == 0)      \
+   : (SSH_IP4_BYTE1(ip_addr) == 127))
+
+/* Returns TRUE if the given IP address is an IPv6 link-local
+   address. */
+#define SSH_IP6_IS_LINK_LOCAL(ip_addr)  \
+  (SSH_IP_IS6((ip_addr))                \
+   && (((SSH_IP6_WORD0_TO_INT((ip_addr))) & 0xffc00000) == 0xfe800000))
 
 
-/****************** Definitions for IPv4 packets ***********************/
+/******************** Definitions for IP(4/6) packets ***********************/
+
+#define SSH_IPH_VERSION(ucp) SSH_GET_4BIT_HIGH(ucp)
+#define SSH_IPH_SET_VERSION(ucp, v) SSH_PUT_4BIT_HIGH(ucp, (v))
+
+/*********************** Definitions for IPv4 packets ***********************/
+
 
 /* Minimum length of an IPv4 header. */
 #define SSH_IPH4_HDRLEN 20         /* IPv4 header length */
@@ -357,9 +495,12 @@ typedef struct SshIpAddrRec
 #define SSH_IPH4_OFS_SRC               12
 #define SSH_IPH4_OFS_DST               16
 
+/* Address length */
+#define SSH_IPH4_ADDRLEN                4
+
 /* Macros for accessing IPv4 packet header fields.  Any returned values
    will be in host byte order. */
-#define SSH_IPH4_VERSION(ucp) SSH_GET_4BIT_HIGH(ucp)
+#define SSH_IPH4_VERSION(ucp) SSH_IPH_VERSION(ucp)
 #define SSH_IPH4_HLEN(ucp) SSH_GET_4BIT_LOW(ucp)
 #define SSH_IPH4_TOS(ucp) SSH_GET_8BIT((ucp) + 1)
 #define SSH_IPH4_LEN(ucp) SSH_GET_16BIT((ucp) + 2)
@@ -373,7 +514,7 @@ typedef struct SshIpAddrRec
 
 /* Macros for setting IPv4 packet header fields.  Values are in host
    byte order. */
-#define SSH_IPH4_SET_VERSION(ucp, v) SSH_PUT_4BIT_HIGH(ucp, (v))
+#define SSH_IPH4_SET_VERSION(ucp, v) SSH_IPH_SET_VERSION(ucp, v)
 #define SSH_IPH4_SET_HLEN(ucp, v) SSH_PUT_4BIT_LOW(ucp, (v))
 #define SSH_IPH4_SET_TOS(ucp, v) SSH_PUT_8BIT((ucp) + 1, (v))
 #define SSH_IPH4_SET_LEN(ucp, v) SSH_PUT_16BIT((ucp) + 2, (v))
@@ -390,7 +531,7 @@ typedef struct SshIpAddrRec
 #define SSH_IPH4_FRAGOFF_DF      0x4000 /* dont fragment flag */
 #define SSH_IPH4_FRAGOFF_MF      0x2000 /* more fragments flag */
 #define SSH_IPH4_FRAGOFF_OFFMASK 0x1fff /* mask for fragment offset */
-     
+
 /* Definitions for IPv4 option numbers. */
 #define SSH_IPOPT_EOL           0  /* end of option list */
 #define SSH_IPOPT_NOP           1  /* no operation */
@@ -408,7 +549,10 @@ typedef struct SshIpAddrRec
 /* This evaluates to TRUE if the option should be copied on fragmentation. */
 #define SSH_IPOPT_COPIED(o) (((o) & 0x80) != 0)
 
-/* ICMP header length etc., and offsets to headers etc. */
+/****************** Definitions for IPv4 ICMP packets ***********************/
+
+/* ICMP offsets to headers etc. (no HDRLEN since ICMP headers vary in
+   size */
 #define SSH_ICMPH_OFS_TYPE      0
 #define SSH_ICMPH_OFS_CODE      1
 #define SSH_ICMPH_OFS_CHECKSUM  2
@@ -417,9 +561,38 @@ typedef struct SshIpAddrRec
 #define SSH_ICMPH_CODE(icmp) SSH_GET_8BIT((icmp) + SSH_ICMPH_OFS_CODE)
 #define SSH_ICMPH_CHECKSUM(icmp) SSH_GET_16BIT((icmp) + SSH_ICMPH_OFS_CHECKSUM)
 
-#define SSH_ICMPH_SET_TYPE(icmp,v) SSH_PUT_8BIT((icmp) + SSH_ICMPH_OFS_TYPE, (v))
-#define SSH_ICMPH_SET_CODE(icmp,v) SSH_PUT_8BIT((icmp) + SSH_ICMPH_OFS_CODE, (v))
-#define SSH_ICMPH_SET_CHECKSUM(icmp,v) SSH_PUT_16BIT((icmp) + SSH_ICMPH_OFS_CHECKSUM, (v))
+#define SSH_ICMPH_SET_TYPE(icmp,v) \
+  SSH_PUT_8BIT((icmp) + SSH_ICMPH_OFS_TYPE, (v))
+#define SSH_ICMPH_SET_CODE(icmp,v) \
+  SSH_PUT_8BIT((icmp) + SSH_ICMPH_OFS_CODE, (v))
+#define SSH_ICMPH_SET_CHECKSUM(icmp,v) \
+  SSH_PUT_16BIT((icmp) + SSH_ICMPH_OFS_CHECKSUM, (v))
+
+/******************** Definitions for IPv6 ICMP packets *********************/
+
+/* Header length of an ICMPv6 header. */
+#define SSH_ICMP6H_HDRLEN       4
+
+/* Offsets of various fields in ICMPv6 headers. */
+#define SSH_ICMP6H_OFS_TYPE     0
+#define SSH_ICMP6H_OFS_CODE     1
+#define SSH_ICMP6H_OFS_CHECKSUM 2
+
+/* Macros for accessing ICMPv6 header fields. */
+#define SSH_ICMP6H_TYPE(icmp) SSH_GET_8BIT((icmp) + SSH_ICMP6H_OFS_TYPE)
+#define SSH_ICMP6H_CODE(icmp) SSH_GET_8BIT((icmp) + SSH_ICMP6H_OFS_CODE)
+#define SSH_ICMP6H_CHECKSUM(icmp) SSH_GET_16BIT((icmp) + \
+        SSH_ICMP6H_OFS_CHECKSUM)
+
+/* Macros for setting ICMPv6 header fields. */
+#define SSH_ICMP6H_SET_TYPE(icmp,v) \
+  SSH_PUT_8BIT((icmp) + SSH_ICMP6H_OFS_TYPE, (v))
+#define SSH_ICMP6H_SET_CODE(icmp,v) \
+  SSH_PUT_8BIT((icmp) + SSH_ICMP6H_OFS_CODE, (v))
+#define SSH_ICMP6H_SET_CHECKSUM(icmp,v) \
+  SSH_PUT_16BIT((icmp) + SSH_ICMP6H_OFS_CHECKSUM, (v))
+
+/******************** Upper level protocols TCP and UDP *********************/
 
 /* TCP header length, and offsets to header and pseudo-header */
 #define SSH_TCP_HEADER_LEN 20
@@ -469,6 +642,8 @@ typedef struct SshIpAddrRec
 #define SSH_TCPH_SET_CHECKSUM(ucp, v) SSH_PUT_16BIT((ucp) + 16, (v))
 #define SSH_TCPH_SET_URGENT(ucp, v) SSH_PUT_16BIT((ucp) + 18, (v))
 
+/********************* Definitions for IPv4 UDP packets *********************/
+
 /* UDP header length, field offsets and pseudo-header offsets */
 #define SSH_UDP_HEADER_LEN 8
 
@@ -494,7 +669,173 @@ typedef struct SshIpAddrRec
 #define SSH_UDPH_SET_LEN(ucp, v) SSH_PUT_16BIT((ucp) + 4, (v))
 #define SSH_UDPH_SET_CHECKSUM(ucp, v) SSH_PUT_16BIT((ucp) + 6, (v))
 
-/********* Ethernet definitions ****************************************/
+/*********************** Definitions for IPv6 packets ***********************/
+
+/* IPv6 header length. Extension headers are not counted in IPv6
+   header */
+#define SSH_IPH6_HDRLEN 40
+
+#define SSH_IPH6_OFS_VERSION            0
+#define SSH_IPH6_OFS_CLASS              0
+#define SSH_IPH6_OFS_FLOW               1
+#define SSH_IPH6_OFS_LEN                4
+#define SSH_IPH6_OFS_NH                 6
+#define SSH_IPH6_OFS_HL                 7
+#define SSH_IPH6_OFS_SRC                8
+#define SSH_IPH6_OFS_DST                24
+
+#define SSH_IPH6_ADDRLEN        16
+
+#define SSH_IPH6_VERSION(ucp) SSH_IPH_VERSION(ucp)
+#define SSH_IPH6_CLASS(ucp) ((SSH_GET_32BIT(ucp) & SSH_IPH6_CLASS_MASK) \
+                             >> SSH_IPH6_CLASS_SHIFT)
+#define SSH_IPH6_FLOW(ucp) (SSH_GET_32BIT(ucp) & SSH_IPH6_FLOW_MASK)
+#define SSH_IPH6_LEN(ucp) SSH_GET_16BIT((ucp) + SSH_IPH6_OFS_LEN)
+#define SSH_IPH6_NH(ucp) SSH_GET_8BIT((ucp) + SSH_IPH6_OFS_NH)
+#define SSH_IPH6_HL(ucp) SSH_GET_8BIT((ucp) + SSH_IPH6_OFS_HL)
+#define SSH_IPH6_SRC(ipaddr, ucp) SSH_IP6_DECODE((ipaddr), \
+                                                 (ucp) + SSH_IPH6_OFS_SRC)
+#define SSH_IPH6_DST(ipaddr, ucp) SSH_IP6_DECODE((ipaddr), \
+                                                 (ucp) + SSH_IPH6_OFS_DST)
+
+#define SSH_IPH6_SET_VERSION(ucp, v) SSH_IPH_SET_VERSION(ucp, v)
+#define SSH_IPH6_SET_CLASS(ucp, v)              \
+  do {                                          \
+    SSH_PUT_4BIT_LOW((ucp), ((v) >> 4) & 0xf);  \
+    SSH_PUT_4BIT_HIGH((ucp) + 1, (v) & 0xf);    \
+  } while (0)
+#define SSH_IPH6_SET_FLOW(ucp, v)                       \
+  do {                                                  \
+    SSH_PUT_4BIT_LOW((ucp) + 1, ((v) >> 16) & 0xf);     \
+    SSH_PUT_16BIT((ucp) + 2, (v) & 0xffff);             \
+  } while (0)
+#define SSH_IPH6_SET_LEN(ucp, v) SSH_PUT_16BIT((ucp) + SSH_IPH6_OFS_LEN, (v))
+#define SSH_IPH6_SET_NH(ucp, v) SSH_PUT_8BIT((ucp) + SSH_IPH6_OFS_NH, (v))
+#define SSH_IPH6_SET_HL(ucp, v) SSH_PUT_8BIT((ucp) + SSH_IPH6_OFS_HL, (v))
+#define SSH_IPH6_SET_SRC(ipaddr, ucp) SSH_IP6_ENCODE((ipaddr), \
+                                                     (ucp) + SSH_IPH6_OFS_SRC)
+#define SSH_IPH6_SET_DST(ipaddr, ucp) SSH_IP6_ENCODE((ipaddr), \
+                                                     (ucp) + SSH_IPH6_OFS_DST)
+
+#define SSH_IPH6_CLASS_SHIFT            20
+#define SSH_IPH6_CLASS_MASK             0xff00000
+#define SSH_IPH6_FLOW_MASK              0x00fffff
+
+/****************** Definitions for IPv6 extension headers ******************/
+
+/* The common IPv6 extension header format. */
+
+#define SSH_IP6_EXT_COMMON_HDRLEN       2
+
+#define SSH_IP6_EXT_COMMON_OFS_NH       0
+#define SSH_IP6_EXT_COMMON_OFS_LEN      1
+
+#define SSH_IP6_EXT_COMMON_NH(ucp)      SSH_GET_8BIT((ucp))
+#define SSH_IP6_EXT_COMMON_LEN(ucp)     SSH_GET_8BIT((ucp) + 1)
+#define SSH_IP6_EXT_COMMON_LENB(ucp) \
+  ((SSH_IP6_EXT_COMMON_LEN((ucp)) + 1) << 3)
+
+#define SSH_IP6_EXT_COMMON_SET_NH(ucp, v)       SSH_PUT_8BIT((ucp), (v))
+#define SSH_IP6_EXT_COMMON_SET_LEN(ucp, v)      SSH_PUT_8BIT((ucp) + 1, (v))
+
+/* Predicate to check whether the `Next Header' `nh' can be parsed as
+   common IPv6 extension header */
+#define SSH_IP6_EXT_IS_COMMON(nh) \
+  ((nh) == 0 || (nh) == SSH_IPPROTO_IPV6ROUTE || (nh) == SSH_IPPROTO_IPV6OPTS)
+
+/* Hop-by-Hop */
+
+#define SSH_IP6_EXT_HOP_BY_HOP_HDRLEN           SSH_IP6_EXT_COMMON_HDRLEN
+
+#define SSH_IP6_EXT_HOP_BY_HOP_OFS_NH           SSH_IP6_EXT_COMMON_OFS_NH
+#define SSH_IP6_EXT_HOP_BY_HOP_OFS_LEN          SSH_IP6_EXT_COMMON_OFS_LEN
+
+#define SSH_IP6_EXT_HOP_BY_HOP_NH(ucp)          SSH_IP6_EXT_COMMON_NH((ucp))
+#define SSH_IP6_EXT_HOP_BY_HOP_LEN(ucp)         SSH_IP6_EXT_COMMON_LEN((ucp))
+#define SSH_IP6_EXT_HOP_BY_HOP_LENB(ucp)        SSH_IP6_EXT_COMMON_LENB((ucp))
+
+/* Routing */
+
+#define SSH_IP6_EXT_ROUTING_HDRLEN              4
+
+#define SSH_IP6_EXT_ROUTING_OFS_NH              SSH_IP6_EXT_COMMON_OFS_NH
+#define SSH_IP6_EXT_ROUTING_OFS_LEN             SSH_IP6_EXT_COMMON_OFS_LEN
+#define SSH_IP6_EXT_ROUTING_OFS_TYPE            2
+#define SSH_IP6_EXT_ROUTING_OFS_SEGMENTS        3
+
+#define SSH_IP6_EXT_ROUTING_NH(ucp)             SSH_IP6_EXT_COMMON_NH((ucp))
+#define SSH_IP6_EXT_ROUTING_LEN(ucp)            SSH_IP6_EXT_COMMON_LEN((ucp))
+#define SSH_IP6_EXT_ROUTING_LENB(ucp)           SSH_IP6_EXT_COMMON_LENB((ucp))
+#define SSH_IP6_EXT_ROUTING_TYPE(ucp)           SSH_GET_8BIT((ucp) + 2)
+#define SSH_IP6_EXT_ROUTING_SEGMENTS(ucp)       SSH_GET_8BIT((ucp) + 3)
+
+#define SSH_IP6_EXT_ROUTING_SET_NH(ucp, v) \
+  SSH_IP6_EXT_COMMON_SET_NH((ucp), (v))
+#define SSH_IP6_EXT_ROUTING_SET_LEN(ucp, v) \
+  SSH_IP6_EXT_COMMON_SET_LEN((ucp), (v))
+#define SSH_IP6_EXT_ROUTING_SET_TYPE(ucp, v) \
+  SSH_PUT_8BIT((ucp) + 2, (v))
+#define SSH_IP6_EXT_ROUTING_SET_SEGMENTS(ucp, v) \
+  SSH_PUT_8BIT((ucp) + 3, (v))
+
+/* Fragment */
+
+#define SSH_IP6_EXT_FRAGMENT_HDRLEN             8
+
+#define SSH_IP6_EXT_FRAGMENT_OFS_NH             SSH_IP6_EXT_COMMON_OFS_NH
+#define SSH_IP6_EXT_FRAGMENT_OFS_RESERVED1      1
+#define SSH_IP6_EXT_FRAGMENT_OFS_OFFSET         2
+#define SSH_IP6_EXT_FRAGMENT_OFS_ID             4
+
+#define SSH_IP6_EXT_FRAGMENT_NH(ucp)            SSH_IP6_EXT_COMMON_NH((ucp))
+#define SSH_IP6_EXT_FRAGMENT_RESERVED1(ucp)     SSH_GET_8BIT((ucp) + 1)
+#define SSH_IP6_EXT_FRAGMENT_OFFSET(ucp)        (SSH_GET_16BIT((ucp) + 2) >> 3)
+#define SSH_IP6_EXT_FRAGMENT_RESERVED2(ucp) \
+  ((SSH_GET_8BIT((ucp) + 3) >> 1) & 0x3)
+#define SSH_IP6_EXT_FRAGMENT_M(ucp)             (SSH_GET_8BIT((ucp) + 3) & 0x1)
+#define SSH_IP6_EXT_FRAGMENT_ID(ucp)            SSH_GET_32BIT((ucp) + 4)
+
+/* Destination Options */
+
+#define SSH_IP6_EXT_DSTOPTS_HDRLEN      SSH_IP6_EXT_COMMON_HDRLEN
+
+#define SSH_IP6_EXT_DSTOPTS_OFS_NH      SSH_IP6_EXT_COMMON_OFS_NH
+#define SSH_IP6_EXT_DSTOPTS_OFS_LEN     SSH_IP6_EXT_COMMON_OFS_LEN
+
+#define SSH_IP6_EXT_DSTOPTS_NH(ucp)     SSH_IP6_EXT_COMMON_NH((ucp))
+#define SSH_IP6_EXT_DSTOPTS_LEN(ucp)    SSH_IP6_EXT_COMMON_LEN((ucp))
+#define SSH_IP6_EXT_DSTOPTS_LENB(ucp)   SSH_IP6_EXT_COMMON_LENB((ucp))
+
+/******************** Definitions for IPv6 Pseudo-Header ********************/
+
+#define SSH_IP6_PSEUDOH_HDRLEN          40
+
+#define SSH_IP6_PSEUDOH_OFS_SRC         0
+#define SSH_IP6_PSEUDOH_OFS_DST         16
+#define SSH_IP6_PSEUDOH_OFS_LEN         32
+#define SSH_IP6_PSEUDOH_OFS_NH          39
+
+#define SSH_IP6_PSEUDOH_SET_SRC(ipaddr, ucp)    \
+  SSH_IP6_ENCODE((ipaddr), (ucp) + SSH_IP6_PSEUDOH_OFS_SRC)
+#define SSH_IP6_PSEUDOH_SET_DST(ipaddr, ucp)    \
+  SSH_IP6_ENCODE((ipaddr), (ucp) + SSH_IP6_PSEUDOH_OFS_DST)
+#define SSH_IP6_PSEUDOH_SET_LEN(ucp, v) \
+  SSH_PUT_32BIT((ucp) + SSH_IP6_PSEUDOH_OFS_LEN, (v))
+#define SSH_IP6_PSEUDOH_SET_NH(ucp, v)  \
+  SSH_PUT_8BIT((ucp) + SSH_IP6_PSEUDOH_OFS_NH, (v))
+
+/*************************** Ethernet definitions ***************************/
+
+/* Etherenet header size */
+#define SSH_ETHERH_HDRLEN       14
+
+/* Field offsets for ethernet header */
+#define SSH_ETHERH_OFS_DST      0
+#define SSH_ETHERH_OFS_SRC      6
+#define SSH_ETHERH_OFS_TYPE     12
+
+/* Size of ethernet address */
+#define SSH_ETHERH_ADDRLEN      6
 
 /* Known values for the ethernet type field.  The same values are used for
    both ethernet (rfc894) and IEEE 802 encapsulation (the type will just
@@ -511,13 +852,17 @@ typedef struct SshIpAddrRec
    broadcast address. */
 #define SSH_ETHER_IS_MULTICAST(addr) (*(addr) & 0x01)
 
-/* Field offsets for ethernet header, and total header size. */
-#define SSH_ETHER_OFS_DST       0
-#define SSH_ETHER_OFS_SRC       6
-#define SSH_ETHER_OFS_TYPE      12
-#define SSH_ETHER_HEADERSIZE    14
-     
-/********************** Helper functions ******************************/
+/************************** AH and ESP definitions **************************/
+
+#define SSH_ESPH_OFS_SPI        0
+#define SSH_ESPH_OFS_SEQ        4
+
+#define SSH_AHH_OFS_NH          0
+#define SSH_AHH_OFS_LEN         1
+#define SSH_AHH_OFS_SPI         4
+#define SSH_AHH_OFS_SEQ         8
+
+/***************************** Helper functions *****************************/
 
 /* Sets all rightmost bits after keeping `keep_bits' bits on the left to
    the value specified by `value'. */
@@ -526,10 +871,11 @@ void ssh_ipaddr_set_bits(SshIpAddr result, SshIpAddr ip,
 
 void ssh_ipaddr_merge_bits (SshIpAddr result, SshIpAddr left_ip,
                             unsigned int bits, SshIpAddr right_ip);
-     
+
 /* Parses an IP address from the string to the internal representation. */
 Boolean ssh_ipaddr_parse(SshIpAddr ip, const char *str);
-Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip, const char *str, const char *mask);
+Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip, const char *str,
+                                   const char *mask);
 
 /* Prints the IP address into the buffer in string format.  If the buffer
    is too short, the address is truncated.  This returns `buf'. */
@@ -537,15 +883,22 @@ char *ssh_ipaddr_print(const SshIpAddr ip, char *buf, size_t buflen);
 char *ssh_ipaddr_print_with_mask(const SshIpAddr ip, char *buf, size_t buflen);
 
 /* Rendering function (for ssh_e*printf %@ format) for IP addresses */
-int ssh_ipaddr_render (char *buf, int buf_size, int precision, void * datum);
-int ssh_ipproto_render (char *buf, int buf_size, int precision, void * datum);
+int ssh_ipaddr_render(char *buf, int buf_size, int precision, void * datum);
 
-/********************** Internal definitions ***************************/
+/* Rendering function for IP protocol numbers. */
+int ssh_ipproto_render(char *buf, int buf_size, int precision, void * datum);
+
+/* Rendering function for IP address masks. */
+int ssh_ipmask_render(char *buf, int buf_size, int precision, void * datum);
+
+
+/*************************** Internal definitions ***************************/
 
 /* Some prototypes for internal functions. */
 unsigned long ssh_ipaddr_hash(SshIpAddr ip);
 
 Boolean ssh_ipaddr_mask_equal(SshIpAddr ip1, SshIpAddr masked_ip);
-Boolean ssh_ipaddr_with_mask_equal(SshIpAddr ip1, SshIpAddr ip2, SshIpAddr mask);
+Boolean ssh_ipaddr_with_mask_equal(SshIpAddr ip1, SshIpAddr ip2,
+                                   SshIpAddr mask);
 
 #endif /* SSHINET_H */
