@@ -11,7 +11,7 @@ Interface to sockets.
 */
 
 /*
- * $Id: sshtcp.c,v 1.3 2000/05/28 16:21:20 kivinen Exp $
+ * $Id: sshtcp.c,v 1.4 2000/09/27 02:22:22 kivinen Exp $
  * $Log: sshtcp.c,v $
  * $EndLog$
  */
@@ -22,6 +22,7 @@ Interface to sockets.
 #include "sshbuffer.h"
 #include "sshsocks.h"
 #include "sshurl.h"
+#include "sshtimeouts.h"
 
 typedef enum
 {
@@ -63,47 +64,78 @@ typedef struct {
 
   /* An open stream to either the socks server or the final destination. */
   SshStream stream;
+
+  /* Lower level operation handle. If this is set we are in the middle of the
+     asyncronous call */
+  SshOperationHandle handle;
+  /* Upper level operation handle. If this is set we started any asyncronous
+     call, and the lower level must free this structure. If this is NULL then
+     we are in the syncronous code path and the ssh_tcp_connect will take care
+     of the freeing of this structure */
+  SshOperationHandle upper_handle;
 } *ConnectContext;
 
 
 /* Connects to the given address/port, and makes a stream for it.
    The address to use is the first address from the list.  This
    function is defined in the machine-specific file. */
-void ssh_socket_low_connect(const char *address_list, unsigned int port,
-                            SshTcpCallback callback, void *context);
+SshOperationHandle ssh_socket_low_connect(const char *address_list,
+                                          unsigned int port,
+                                          SshTcpCallback callback,
+                                          void *context);
 
 /* Forward declaration; defined later in this file. */
 void ssh_socket_connect_step(ConnectContext);
 
-/* Opens a TCP/IP connection to the given port on the host, and calls
-   the callback when the connection is either ready or has failed.  The
-   connection is attempted several times before giving up.  The callback
-   will receive the resulting stream as an argument if the connection
-   is successful.  Note that the callback may be called either during this
+/* Destroys the connection context. */
+void ssh_socket_destroy_connect_context(ConnectContext c);
+
+/* Calls the user callback with the given error code and stream.
+   Destroys the context. */
+void ssh_socket_connect_final(ConnectContext c, SshStream stream,
+                              SshIpError error);
+/* Connection timed out */
+void ssh_tcp_connect_timed_out(void *context)
+{
+  ConnectContext c = context;
+  ssh_socket_connect_final(c, c->stream, SSH_IP_TIMEOUT);
+}
+
+/* Connection aborted out */
+void ssh_tcp_connect_aborted(void *context)
+{
+  /* We just destroy the context, that will also automatically abort all the
+     underlaying operations */
+  ssh_socket_destroy_connect_context(context);
+}
+
+/* Opens a connection to the specified host, and calls the callback
+   when the connection has been established or has failed.  If
+   connecting is successful, the callback will be called with error
+   set to SSH_TCP_OK and an SshStream object for the connection passed
+   in in the stream argument.  Otherwise, error will indicate the
+   reason for the connection failing, and the stream will be NULL.
+
+   Note that the callback may be called either during this
    call or some time later.
+   
+   Returns SshOperationHandle that can be used to abort the tcp open.
 
    The `host_name_or_address' argument may be a numeric IP address or a
    host name (domain name), in which case it is looked up from the name
    servers.
 
-   This call supports SOCKS (version 4) for going out through firewalls.
-   `socks_server_name' specifies the SOCKS host, and `socks_port' the port
-   on which the SOCKS server listens.  If `socks_server_name' is NULL or
-   empty, the connection will be made without SOCKS.  If `socks_port' is
-   NULL or empty, the default SOCKS port (1080) will be used.
-
-   `connection_attempts' specifies the number of times to attempt the
-   connection before giving up.  (Some systems appear to spuriously fail
-   connections without apparent reason, and retrying usually succeeds in
-   those cases). */
-void ssh_tcp_connect_with_socks(const char *host_name_or_address,
-                                const char *port_or_service,
-                                const char *socks_server_url,
-                                unsigned int connection_attempts,
-                                SshTcpCallback callback,
-                                void *context)
+   The params structure can either be NULL or memset to zero to get default
+   parameters. All data inside the params is copied during this call, so it can
+   be freed immediately when this function returns. */
+SshOperationHandle ssh_tcp_connect(const char *host_name_or_address,
+                                   const char *port_or_service,
+                                   const SshTcpConnectParams params,
+                                   SshTcpCallback callback,
+                                   void *context)
 {
   ConnectContext c;
+
   c = ssh_xmalloc(sizeof(*c));
   memset(c, 0, sizeof(*c));
 
@@ -114,8 +146,17 @@ void ssh_tcp_connect_with_socks(const char *host_name_or_address,
 
   c->user_callback = callback;
   c->user_context = context;
+  if (params && params->connection_timeout != 0)
+    {
+      ssh_register_timeout(params->connection_timeout, 0,
+                           ssh_tcp_connect_timed_out,
+                           c);
+    }
+
+  c->connection_attempts = 1;
+  if (params && params->connection_attempts != 0)
+    c->connection_attempts = params->connection_attempts;
 
-  c->connection_attempts = connection_attempts;
   c->attempts_done = 0;
 
   c->stream = NULL;
@@ -123,11 +164,13 @@ void ssh_tcp_connect_with_socks(const char *host_name_or_address,
   c->state = CONNECT_STATE_HOST_LOOKUP;
 
   /* Initialize socks-related data. */
-  if (socks_server_url != NULL && strcmp(socks_server_url, "") != 0)
+  if (params &&
+      params->socks_server_url != NULL &&
+      strcmp(params->socks_server_url, "") != 0)
     {
       char *scheme, *port;
 
-      ssh_url_parse_and_decode(socks_server_url, &scheme,
+      ssh_url_parse_and_decode(params->socks_server_url, &scheme,
                                &(c->socks_host), &port,
                                &(c->user_name), NULL, &(c->socks_exceptions));
 
@@ -151,30 +194,32 @@ void ssh_tcp_connect_with_socks(const char *host_name_or_address,
   else
     c->socks_host = NULL;
 
+  c->upper_handle = NULL;
+  c->handle = NULL;
   /* Perform a step. */
   ssh_socket_connect_step(c);
-}
-
-/* Opens a connection to the specified host, and calls the callback
-   when the connection has been established or has failed.  If
-   connecting is successful, the callback will be called with error
-   set to SSH_TCP_OK and an SshStream object for the connection passed
-   in in the stream argument.  Otherwise, error will indicate the
-   reason for the connection failing, and the stream will be NULL. */
-
-void ssh_tcp_connect(const char *host_name_or_address,
-                     const char *port_or_service,
-                     SshTcpCallback callback,
-                     void *context)
-{
-  ssh_tcp_connect_with_socks(host_name_or_address, port_or_service,
-                             NULL, 1, callback, context);
+  if (c->handle)
+    {
+      /* We started some asyncronous operation, allocate handle and return that
+         to upper layer */
+      c->upper_handle = ssh_operation_register(ssh_tcp_connect_aborted, c);
+      return c->upper_handle;
+    }
+  else
+    {
+      /* We managed to do everything syncronously. Now we need to free the
+         context, as it is not yet freed */
+      ssh_socket_destroy_connect_context(c);
+      return NULL;
+    }
 }
 
 /* Destroys the connection context. */
-
 void ssh_socket_destroy_connect_context(ConnectContext c)
 {
+  if (c->handle)
+    ssh_operation_abort(c->handle);
+  ssh_cancel_timeouts(ssh_tcp_connect_timed_out, c);
   if (c->host_name)
     ssh_xfree(c->host_name);
   if (c->host_addresses)
@@ -191,6 +236,8 @@ void ssh_socket_destroy_connect_context(ConnectContext c)
     ssh_buffer_free(c->socks_buf);
   if (c->stream)
     ssh_stream_destroy(c->stream);
+  if (c->upper_handle)
+    ssh_operation_unregister(c->upper_handle);
   ssh_xfree(c);
 }
 
@@ -211,10 +258,15 @@ void ssh_socket_connect_final(ConnectContext c, SshStream stream,
     }
 
   /* Call the user callback. */
-  (*c->user_callback)(error, stream, c->user_context);
+  if (c->user_callback)
+    (*c->user_callback)(error, stream, c->user_context);
 
-  /* Destroy the context. */
-  ssh_socket_destroy_connect_context(c);
+  if (c->upper_handle)
+    {
+      /* Destroy the context, if we are in the asyncronous path. If we are in
+         the syncrouns path then the ssh_tcp_connect will free this context. */
+      ssh_socket_destroy_connect_context(c);
+    }
 }
 
 /* Increments the attempt count, and returns true if we have used all of
@@ -238,6 +290,8 @@ void ssh_socket_connect_host_lookup_done(SshIpError error,
                                          void *context)
 {
   ConnectContext c = (ConnectContext)context;
+
+  c->handle = NULL;
   if (error != SSH_IP_OK)
     {
       if (ssh_socket_failure(c, error))
@@ -268,6 +322,7 @@ void ssh_socket_connect_socks_lookup_done(SshIpError error,
                                           void *context)
 {
   ConnectContext c = (ConnectContext)context;
+  c->handle = NULL;
   if (error != SSH_IP_OK)
     {
       if (ssh_socket_failure(c, error))
@@ -310,6 +365,9 @@ void ssh_socket_host_connect_done(SshIpError error,
                                   void *context)
 {
   ConnectContext c = (ConnectContext)context;
+  SshOperationHandle h;
+
+  c->handle = NULL;
 
   if (error != SSH_IP_OK)
     {
@@ -323,8 +381,10 @@ void ssh_socket_host_connect_done(SshIpError error,
           c->next_address = c->host_addresses;
         }
       /* Try connecting again. */
-      ssh_socket_low_connect(c->next_address, c->host_port,
-                             ssh_socket_host_connect_done, (void *)c);
+      h = ssh_socket_low_connect(c->next_address, c->host_port,
+                                 ssh_socket_host_connect_done, (void *)c);
+      if (h)
+        c->handle = h;
       return;
     }
 
@@ -342,6 +402,8 @@ void ssh_socket_socks_notify(SshStreamNotification notification,
 {
   ConnectContext c = (ConnectContext)context;
 
+  c->handle = NULL;
+
   switch (notification)
     {
     case SSH_STREAM_INPUT_AVAILABLE:
@@ -397,9 +459,12 @@ void ssh_socket_socks_connect_done(SshIpError error,
 {
   ConnectContext c = (ConnectContext)context;
   struct SocksInfoRec socksinfo;
+  SshOperationHandle h;
   SocksError ret;
   char host_port[64];
 
+  c->handle = NULL;
+
   if (error != SSH_IP_OK)
     {
       /* Get next address. */
@@ -412,8 +477,10 @@ void ssh_socket_socks_connect_done(SshIpError error,
           c->socks_next_address = (char *) c->socks_addresses;
         }
       /* Try connecting again. */
-      ssh_socket_low_connect(c->socks_next_address, c->socks_port,
-                             ssh_socket_socks_connect_done, (void *)c);
+      h = ssh_socket_low_connect(c->socks_next_address, c->socks_port,
+                                 ssh_socket_socks_connect_done, (void *)c);
+      if (h)
+        c->handle = h;
       return;
     }
 
@@ -454,30 +521,39 @@ void ssh_socket_socks_connect_done(SshIpError error,
 void ssh_socket_connect_step(ConnectContext c)
 {
   int len;
+  SshOperationHandle h;
 
 restart:
   switch (c->state)
     {
     case CONNECT_STATE_HOST_LOOKUP:
-      ssh_tcp_get_host_addrs_by_name(c->host_name,
-                                     ssh_socket_connect_host_lookup_done,
-                                     (void *)c);
+      h = ssh_tcp_get_host_addrs_by_name(c->host_name,
+                                         ssh_socket_connect_host_lookup_done,
+                                         (void *)c);
+      if (h)
+        c->handle = h;
       break;
 
     case CONNECT_STATE_SOCKS_LOOKUP:
-      ssh_tcp_get_host_addrs_by_name(c->socks_host,
-                                     ssh_socket_connect_socks_lookup_done,
-                                     (void *)c);
+      h = ssh_tcp_get_host_addrs_by_name(c->socks_host,
+                                         ssh_socket_connect_socks_lookup_done,
+                                         (void *)c);
+      if (h)
+        c->handle = h;
       break;
 
     case CONNECT_STATE_HOST_CONNECT:
-      ssh_socket_low_connect(c->next_address, c->host_port,
-                             ssh_socket_host_connect_done, (void *)c);
+      h = ssh_socket_low_connect(c->next_address, c->host_port,
+                                 ssh_socket_host_connect_done, (void *)c);
+      if (h)
+        c->handle = h;
       break;
 
     case CONNECT_STATE_SOCKS_CONNECT:
-      ssh_socket_low_connect(c->socks_next_address, c->socks_port,
-                             ssh_socket_socks_connect_done, (void *)c);
+      h = ssh_socket_low_connect(c->socks_next_address, c->socks_port,
+                                 ssh_socket_socks_connect_done, (void *)c);
+      if (h)
+        c->handle = h;
       break;
 
     case CONNECT_STATE_SOCKS_SEND:
