@@ -11,7 +11,7 @@ Unix-specific code for sockets.
 */
 
 /*
- * $Id: sshunixtcp.c,v 1.4 2000/04/06 08:23:57 tmo Exp $
+ * $Id: sshunixtcp.c,v 1.13.6.1 2000/12/04 14:31:30 sjl Exp $
  * $Log: sshunixtcp.c,v $
  * $EndLog$
  */
@@ -50,57 +50,123 @@ Unix-specific code for sockets.
 typedef struct LowConnectRec
 {
   int sock;
-  char *address;
   unsigned int port;
   SshTcpCallback callback;
   void *context;
+  SshIpAddrStruct ipaddr;
+  SshOperationHandle handle;
 } *LowConnect;
 
+#if !defined(HAVE_GETSERVBYNAME) && defined(WANT_SERVBYNAME)\
+ || !defined(HAVE_GETSERVBYPORT) && defined(WANT_SERVBYPORT)
+
+/* that exports __global_table[] as "root" of data */
+#include "sshgetservbyname_servicetable.c"
+
+/* done a a define here, so comparison can be turned to 
+   case-insignificant if needed, first argument should
+   rather be the statically allocated string to assure
+   that we don't read far past the user string if it
+   somehow happens to be non-terminated.
+  */
+#define GETSERV_STRCMP(x,y) strncmp((x), (y), strlen(x)+1)
+
+static Boolean find_in_aliases(char const * const * const aliases, 
+                               char const * const name)
+{
+  int i;
+  for(i=0; ;i++)
+    {
+      if (aliases[i]==NULL) return FALSE;
+      if (GETSERV_STRCMP(aliases[i], name)==0) return TRUE;
+    }
+}
+
+static struct SshServent const *
+ssh_getserv(const char* name, int port, Boolean byname, const char* protocol)
+{
+  int i;
+  int k;
+
+  for(i=0; ;i++)
+    {
+      if(__global_table[i].protocol==NULL) return NULL;
+      if(protocol!=NULL && 
+         GETSERV_STRCMP(__global_table[i].protocol, protocol)!=0) continue;
+      /* search item in list for this protocol */
+      for(k=0; ;k++)
+        {
+          if ((byname!=FALSE) 
+              ? find_in_aliases(__global_table[i].table[k].s_aliases, name)
+              : __global_table[i].table[k].s_port == port )
+            return &(__global_table[i].table[k]);
+        }
+    }
+}
+
+#endif /* SERVBYNAME */
 
 void ssh_socket_set_reuseaddr(int sock)
 {
+#ifdef SO_REUSEADDR
   int on = 1;
   setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *)&on,
              sizeof(on));
+#endif /* SO_REUSEADDR */
+}
+
+void ssh_socket_set_reuseport(int sock)
+{
+#ifdef SO_REUSEPORT
+  int on = 1;
+  setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, (void *)&on,
+             sizeof(on));
+#endif /* SO_REUSEPORT */
 }
 
 #ifdef NO_NONBLOCKING_CONNECT
 
-void ssh_socket_low_connect_try_once(unsigned int events, void *context)
+SshOperationHandle ssh_socket_low_connect_try_once(unsigned int events,
+                                                   void *context)
 {
   LowConnect c = (LowConnect)context;
-  int ret;
-  struct sockaddr_in sinaddr;
+  int ret = -1;
   SshIpError error;
 
-  memset(&sinaddr, 0, sizeof(sinaddr));
-  sinaddr.sin_family = AF_INET;
-  sinaddr.sin_port = htons(c->port);
-
-#ifdef BROKEN_INET_ADDR
-  sinaddr.sin_addr.s_addr = inet_network(c->address);
-#else /* BROKEN_INET_ADDR */
-  sinaddr.sin_addr.s_addr = inet_addr(c->address);
-#endif /* BROKEN_INET_ADDR */
-  if ((sinaddr.sin_addr.s_addr & 0xffffffff) == 0xffffffff)
+  if (SSH_IP_IS6(&ipaddr))
     {
-      close(c->sock);
-      (*c->callback)(SSH_IP_NO_ADDRESS, NULL, c->context);
-      ssh_xfree(c->address);
-      ssh_xfree(c);
-      return;
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+      struct sockaddr_in6 sinaddr6;
+
+      memset(&sinaddr6, 0, sizeof(sinaddr6));
+      sinaddr6.sin6_family = AF_INET6;
+      sinaddr6.sin6_port = htons(c->port);
+      memcpy(sinaddr6.sin6_addr.s6_addr, c->ipaddr.addr_data, 16);
+
+      /* Make a blocking connect attempt. */
+      ret = connect(c->sock, (struct sockaddr *)&sinaddr6, sizeof(sinaddr6));
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+    }
+  else
+    {
+      struct sockaddr_in sinaddr;
+
+      memset(&sinaddr, 0, sizeof(sinaddr));
+      sinaddr.sin_family = AF_INET;
+      sinaddr.sin_port = htons(c->port);
+      sinaddr.sin_addr.s_addr = htons(SSH_IP4_TO_INT(c->ipaddr));
+
+      /* Make a blocking connect attempt. */
+      ret = connect(c->sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr));
     }
 
-  /* Make a blocking connect attempt. */
-  ret = connect(c->sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr));
   if (ret >= 0 || errno == EISCONN) /* Connection is ready. */
     {
       /* Successful connection. */
       (*c->callback)(SSH_IP_OK, ssh_stream_fd_wrap(c->sock, TRUE),
                      c->context);
-      ssh_xfree(c->address);
       ssh_xfree(c);
-      return;
+      return NULL;
     }
 
   /* Connection failed. */
@@ -128,56 +194,79 @@ void ssh_socket_low_connect_try_once(unsigned int events, void *context)
 #endif
 
   close(c->sock);
-  ssh_xfree(c->address);
   (*c->callback)(error, NULL, c->context);
   ssh_xfree(c);
+  return NULL;
 }
 
 #else /* NO_NONBLOCKING_CONNECT */
 
-void ssh_socket_low_connect_try(unsigned int events, void *context)
+/* Connection aborted out */
+void ssh_tcp_low_connect_aborted(void *context)
 {
   LowConnect c = (LowConnect)context;
-  int ret;
-  struct sockaddr_in sinaddr;
+
+  ssh_io_unregister_fd(c->sock, FALSE);
+  close(c->sock);
+
+  (*c->callback)(SSH_IP_TIMEOUT, NULL, c->context);
+  ssh_xfree(c);
+}
+
+SshOperationHandle ssh_socket_low_connect_try(unsigned int events,
+                                              void *context)
+{
+  LowConnect c = (LowConnect)context;
+  int ret = 1;
   SshIpError error;
 
-  memset(&sinaddr, 0, sizeof(sinaddr));
-  sinaddr.sin_family = AF_INET;
-  sinaddr.sin_port = htons(c->port);
-
-#ifdef BROKEN_INET_ADDR
-  sinaddr.sin_addr.s_addr = inet_network(c->address);
-#else /* BROKEN_INET_ADDR */
-  sinaddr.sin_addr.s_addr = inet_addr(c->address);
-#endif /* BROKEN_INET_ADDR */
-  if ((sinaddr.sin_addr.s_addr & 0xffffffff) == 0xffffffff)
+  if (SSH_IP_IS6(&(c->ipaddr)))
     {
-      ssh_io_unregister_fd(c->sock, FALSE);
-      close(c->sock);
-      (*c->callback)(SSH_IP_NO_ADDRESS, NULL, c->context);
-      ssh_xfree(c->address);
-      ssh_xfree(c);
-      return;
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+      struct sockaddr_in6 sinaddr6;
+
+      memset(&sinaddr6, 0, sizeof(sinaddr6));
+      sinaddr6.sin6_family = AF_INET6;
+      sinaddr6.sin6_port = htons(c->port);
+      memcpy(sinaddr6.sin6_addr.s6_addr, c->ipaddr.addr_data, 16);
+
+      /* Make a non-blocking connect attempt. */
+      ret = connect(c->sock, (struct sockaddr *)&sinaddr6, sizeof(sinaddr6));
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+    }
+  else
+    {
+      struct sockaddr_in sinaddr;
+
+      memset(&sinaddr, 0, sizeof(sinaddr));
+      sinaddr.sin_family = AF_INET;
+      sinaddr.sin_port = htons(c->port);
+      sinaddr.sin_addr.s_addr = htonl(SSH_IP4_TO_INT(&(c->ipaddr)));
+
+      /* Make a non-blocking connect attempt. */
+      ret = connect(c->sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr));
     }
 
-  /* Make a non-blocking connect attempt. */
-  ret = connect(c->sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr));
   if (ret >= 0 || errno == EISCONN) /* Connection is ready. */
     {
       /* Successful connection. */
       ssh_io_unregister_fd(c->sock, FALSE);
       (*c->callback)(SSH_IP_OK, ssh_stream_fd_wrap(c->sock, TRUE),
                      c->context);
-      ssh_xfree(c->address);
+      if (c->handle) 
+        ssh_operation_unregister(c->handle);
       ssh_xfree(c);
-      return;
+      return NULL;
     }
   if (errno == EINPROGRESS || errno == EWOULDBLOCK || errno == EALREADY)
     {
       /* Connection still in progress.  */
       ssh_io_set_fd_request(c->sock, SSH_IO_WRITE);
-      return;
+
+      if (!c->handle)
+        c->handle = ssh_operation_register(ssh_tcp_low_connect_aborted, c);
+      
+      return c->handle;
     }
 
   SSH_DEBUG(5, ("Connect failed: %s", strerror(errno)));
@@ -206,10 +295,12 @@ void ssh_socket_low_connect_try(unsigned int events, void *context)
 
   ssh_io_unregister_fd(c->sock, FALSE);
   close(c->sock);
-  ssh_xfree(c->address);
 
   (*c->callback)(error, NULL, c->context);
+  if (c->handle) 
+    ssh_operation_unregister(c->handle);
   ssh_xfree(c);
+  return NULL;
 }
 
 #endif /* NO_NONBLOCKING_CONNECT */
@@ -217,52 +308,77 @@ void ssh_socket_low_connect_try(unsigned int events, void *context)
 /* Connects to the given address/port, and makes a stream for it.
    The address to use is the first address from the list. */
 
-void ssh_socket_low_connect(const char *address_list, unsigned int port,
-                            SshTcpCallback callback, void *context)
+SshOperationHandle ssh_socket_low_connect(const char *address_list,
+                                          unsigned int port,
+                                          SshTcpCallback callback,
+                                          void *context)
 {
-  int sock, first_len;
+  int sock = -1, first_len;
   LowConnect c;
+  char *tmp;
 
-  /* Create a socket. */
-  sock = socket(AF_INET, SOCK_STREAM, 0);
-  if (sock < 0)
-    {
-      (*callback)(SSH_IP_FAILURE, NULL, context);
-      return;
-    }
-
-  /* Set SO_REUSEADDR. */
-  ssh_socket_set_reuseaddr(sock);
+  /* Save data in a context structure. */
+  c = ssh_xmalloc(sizeof(*c));
 
   /* Compute the length of the first address on the list. */
   if (strchr(address_list, ','))
     first_len = strchr(address_list, ',') - address_list;
   else
     first_len = strlen(address_list);
+  tmp = ssh_xmemdup(address_list, first_len);
+
+  if (!ssh_ipaddr_parse(&(c->ipaddr), tmp))
+    {
+      ssh_xfree(tmp);
+      ssh_xfree(c);
+      (*callback)(SSH_IP_NO_ADDRESS, NULL, context);
+      return NULL;
+    }
+
+  ssh_xfree(tmp);
+
+  /* Create a socket. */
+  if (SSH_IP_IS6(&(c->ipaddr)))
+    {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+      sock = socket(AF_INET6, SOCK_STREAM, 0);
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+    }
+  else
+    {
+      sock = socket(AF_INET, SOCK_STREAM, 0);
+    }
+  if (sock < 0)
+    {
+      ssh_xfree(c);
+      (*callback)(SSH_IP_FAILURE, NULL, context);
+      return NULL;
+    }
+
+  /* Set SO_REUSEADDR. */
+  ssh_socket_set_reuseaddr(sock);
 
-  /* Save data in a context structure. */
-  c = ssh_xmalloc(sizeof(*c));
   c->sock = sock;
-  c->address = ssh_xmalloc(first_len + 1);
-  memcpy(c->address, address_list, first_len);
-  c->address[first_len] = '\0';
   c->port = port;
   c->callback = callback;
   c->context = context;
+  c->handle = NULL;
 
 #ifdef NO_NONBLOCKING_CONNECT
 
   /* Try connect once.  Function calls user callback. */
-  ssh_socket_low_connect_try_once(SSH_IO_WRITE, (void *)c);
+  return ssh_socket_low_connect_try_once(SSH_IO_WRITE, (void *)c);
 
 #else /* NO_NONBLOCKING_CONNECT */
 
   /* Register it and request events. */
-  ssh_io_register_fd(sock, ssh_socket_low_connect_try, (void *)c);
+  ssh_io_register_fd(sock,
+                     (void (*)(unsigned int events, void *context))
+                     ssh_socket_low_connect_try, (void *)c);
   ssh_io_set_fd_request(sock, SSH_IO_WRITE);
 
   /* Fake a callback to start asynchronous connect. */
-  ssh_socket_low_connect_try(SSH_IO_WRITE, (void *)c);
+  return ssh_socket_low_connect_try(SSH_IO_WRITE, (void *)c);
 
 #endif /* NO_NONBLOCKING_CONNECT */
 }
@@ -320,45 +436,130 @@ void ssh_tcp_listen_callback(unsigned int events, void *context)
 
 SshTcpListener ssh_tcp_make_listener(const char *local_address,
                                      const char *port_or_service,
+                                     const SshTcpListenerParams params, 
                                      SshTcpCallback callback,
                                      void *context)
 {
-  int sock, port;
-  struct sockaddr_in sinaddr;
+  int sock = -1, port, listen_backlog, buf_len;
   SshTcpListener listener;
+  SshIpAddrStruct ipaddr;
+
+  if (!ssh_ipaddr_parse(&ipaddr, local_address))
+    return NULL;
 
   /* Parse port and address. */
   port = ssh_tcp_get_port_by_service(port_or_service, "tcp");
-  memset(&sinaddr, 0, sizeof(sinaddr));
-  sinaddr.sin_family = AF_INET;
-  sinaddr.sin_port = htons(port);
-
-#ifdef BROKEN_INET_ADDR
-  sinaddr.sin_addr.s_addr = inet_network(local_address);
-#else /* BROKEN_INET_ADDR */
-  sinaddr.sin_addr.s_addr = inet_addr(local_address);
-#endif /* BROKEN_INET_ADDR */
-  if ((sinaddr.sin_addr.s_addr & 0xffffffff) == 0xffffffff)
-    return NULL;
 
   /* Create a socket. */
-  sock = socket(AF_INET, SOCK_STREAM, 0);
+  if (SSH_IP_IS6(&ipaddr))
+    {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+      sock = socket(AF_INET6, SOCK_STREAM, 0);
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+    }
+  else
+    {
+      sock = socket(AF_INET, SOCK_STREAM, 0);
+    }
   if (sock < 0)
     return NULL;
 
-  ssh_socket_set_reuseaddr(sock);
+  if (!params)
+    {
+      ssh_socket_set_reuseaddr(sock);
+    }
+  else
+    {
+      switch (params->listener_reuseable)
+        {
+        case SSH_TCP_LISTENER_REUSEABLE_PORT:
+          ssh_socket_set_reuseport(sock);
+          break;
+        case SSH_TCP_LISTENER_REUSEABLE_ADDRESS:
+          ssh_socket_set_reuseaddr(sock);
+          break;
+        case SSH_TCP_LISTENER_REUSEABLE_BOTH:
+          ssh_socket_set_reuseport(sock);
+          ssh_socket_set_reuseaddr(sock);
+          break;
+        case SSH_TCP_LISTENER_REUSEABLE_NONE:
+          break;
+        }
+    }
 
-  if (bind(sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr)) < 0)
+  if (SSH_IP_IS6(&ipaddr))
+    {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+      struct sockaddr_in6 sinaddr6;
+
+      memset(&sinaddr6, 0, sizeof(sinaddr6));
+      sinaddr6.sin6_family = AF_INET6;
+      sinaddr6.sin6_port = htons(port);
+      memcpy(sinaddr6.sin6_addr.s6_addr, ipaddr.addr_data, 16);
+
+      if (bind(sock, (struct sockaddr *)&sinaddr6, sizeof(sinaddr6)) < 0)
+        {
+          close(sock);
+          return NULL;
+        }
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+    }
+  else
+    {
+      struct sockaddr_in sinaddr;
+  
+      memset(&sinaddr, 0, sizeof(sinaddr));
+      sinaddr.sin_family = AF_INET;
+      sinaddr.sin_port = htons(port);
+
+#ifdef BROKEN_INET_ADDR
+      sinaddr.sin_addr.s_addr = inet_network(local_address);
+#else /* BROKEN_INET_ADDR */
+      sinaddr.sin_addr.s_addr = inet_addr(local_address);
+#endif /* BROKEN_INET_ADDR */
+      if ((sinaddr.sin_addr.s_addr & 0xffffffff) == 0xffffffff)
+        return NULL;
+
+      if (bind(sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr)) < 0)
+        {
+          close(sock);
+          return NULL;
+        }
+    }
+
+  listen_backlog = 5;
+  if (params && params->listen_backlog != 0)
+    listen_backlog = params->listen_backlog;
+  if (listen(sock, listen_backlog) < 0)
     {
       close(sock);
       return NULL;
     }
 
-  if (listen(sock, 5) < 0)
+#ifdef SO_SNDBUF
+  if (params && params->send_buffer_size != 0)
     {
-      close(sock);
-      return NULL;
+      buf_len = params->send_buffer_size;
+      if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, &buf_len, sizeof(int)) == -1)
+        {
+          SSH_DEBUG(3,
+                    ("ssh_tcp_make_listener: setsockopt SO_SNDBUF failed: %s",
+                     strerror(errno)));
+        }
     }
+#endif /* SO_SNDBUF */
+#ifdef SO_RCVBUF
+  if (params && params->send_buffer_size != 0)
+    {
+      buf_len = params->send_buffer_size;
+      if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &buf_len, sizeof(int)) == -1)
+        {
+          SSH_DEBUG(3,
+                    ("ssh_tcp_make_listener: setsockopt SO_RCVBUF failed: %s",
+                     strerror(errno)));
+        }
+    }
+#endif /* SO_RCVBUF */
 
   listener = ssh_xmalloc(sizeof(*listener));
   listener->sock = sock;
@@ -395,8 +596,8 @@ void ssh_tcp_destroy_listener(SshTcpListener listener)
 
 Boolean ssh_tcp_has_ip_options(SshStream stream)
 {
-  int sock;
-  char options[8192];
+  int sock, ret;
+  char *options;
 #ifdef HAVE_POSIX_STYLE_SOCKET_PROTOTYPES
   size_t option_size;
 #else /* HAVE_POSIX_STYLE_SOCKET_PROTOTYPES */
@@ -406,9 +607,12 @@ Boolean ssh_tcp_has_ip_options(SshStream stream)
   sock = ssh_stream_fd_get_readfd(stream);
   if (sock == -1)
     return FALSE;
-  option_size = sizeof(options);
-  return getsockopt(sock, IPPROTO_IP, IP_OPTIONS, options,
-                    &option_size) >= 0 && option_size != 0;
+  option_size = 8192;
+  options = ssh_xmalloc(option_size);
+  ret = getsockopt(sock, IPPROTO_IP, IP_OPTIONS, options,
+                   &option_size);
+  ssh_xfree(options);
+  return (ret >= 0 && option_size != 0);
 }
 
 /* Returns the ip-address of the remote host, as string.  This returns
@@ -418,7 +622,12 @@ Boolean ssh_tcp_has_ip_options(SshStream stream)
 Boolean ssh_tcp_get_remote_address(SshStream stream, char *buf,
                                    size_t buflen)
 {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  struct sockaddr_in6 saddr;
+  SshIpAddrStruct ip;
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   struct sockaddr_in saddr;
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   int sock;
 #ifdef HAVE_POSIX_STYLE_SOCKET_PROTOTYPES
   size_t saddrlen;
@@ -432,9 +641,17 @@ Boolean ssh_tcp_get_remote_address(SshStream stream, char *buf,
 
   saddrlen = sizeof(saddr);
   if (getpeername(sock, (struct sockaddr *)&saddr, &saddrlen) < 0)
-    return 0;
+    return FALSE;
 
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  if (saddr.sin6_family == AF_INET6)
+    SSH_IP6_DECODE(&ip, saddr.sin6_addr.s6_addr);
+  else
+    SSH_INT_TO_IP4(&ip, htonl(((struct sockaddr_in*)&saddr)->sin_addr.s_addr));
+  ssh_ipaddr_print(&ip, buf, buflen);
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   strncpy(buf, inet_ntoa(saddr.sin_addr), buflen);
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   return TRUE;
 }
 
@@ -444,7 +661,11 @@ Boolean ssh_tcp_get_remote_address(SshStream stream, char *buf,
 Boolean ssh_tcp_get_remote_port(SshStream stream, char *buf,
                                 size_t buflen)
 {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  struct sockaddr_in6 saddr;
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   struct sockaddr_in saddr;
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   int sock;
 #ifdef HAVE_POSIX_STYLE_SOCKET_PROTOTYPES
   size_t saddrlen;
@@ -458,9 +679,13 @@ Boolean ssh_tcp_get_remote_port(SshStream stream, char *buf,
 
   saddrlen = sizeof(saddr);
   if (getpeername(sock, (struct sockaddr *)&saddr, &saddrlen) < 0)
-    return 0;
+    return FALSE;
 
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  snprintf(buf, buflen, "%u", ntohs(saddr.sin6_port));
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   snprintf(buf, buflen, "%u", ntohs(saddr.sin_port));
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   return TRUE;
 }
 
@@ -469,7 +694,12 @@ Boolean ssh_tcp_get_remote_port(SshStream stream, char *buf,
 Boolean ssh_tcp_get_local_address(SshStream stream, char *buf,
                                   size_t buflen)
 {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  struct sockaddr_in6 saddr;
+  SshIpAddrStruct ip;
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   struct sockaddr_in saddr;
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   int sock;
 #ifdef HAVE_POSIX_STYLE_SOCKET_PROTOTYPES
   size_t saddrlen;
@@ -483,9 +713,17 @@ Boolean ssh_tcp_get_local_address(SshStream stream, char *buf,
 
   saddrlen = sizeof(saddr);
   if (getsockname(sock, (struct sockaddr *)&saddr, &saddrlen) < 0)
-    return 0;
+    return FALSE;
 
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  if (saddr.sin6_family == AF_INET6)
+    SSH_IP6_DECODE(&ip, saddr.sin6_addr.s6_addr);
+  else
+    SSH_INT_TO_IP4(&ip, htonl(((struct sockaddr_in*)&saddr)->sin_addr.s_addr));
+  ssh_ipaddr_print(&ip, buf, buflen);
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   strncpy(buf, inet_ntoa(saddr.sin_addr), buflen);
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   return TRUE;
 }
 
@@ -494,7 +732,11 @@ Boolean ssh_tcp_get_local_address(SshStream stream, char *buf,
 Boolean ssh_tcp_get_local_port(SshStream stream, char *buf,
                                size_t buflen)
 {
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  struct sockaddr_in6 saddr;
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   struct sockaddr_in saddr;
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
   int sock;
 #ifdef HAVE_POSIX_STYLE_SOCKET_PROTOTYPES
   size_t saddrlen;
@@ -508,10 +750,18 @@ Boolean ssh_tcp_get_local_port(SshStream stream, char *buf,
 
   saddrlen = sizeof(saddr);
   if (getsockname(sock, (struct sockaddr *)&saddr, &saddrlen) < 0)
-    return 0;
+    return FALSE;
 
+#ifdef HAVE_SOCKADDR_IN6_STRUCT
+  if (saddr.sin6_family == AF_INET6)
+    snprintf(buf, buflen, "%u", ntohs(saddr.sin6_port));
+  else
+    snprintf(buf, buflen, "%u",
+             ntohs(((struct sockaddr_in *) &saddr)->sin_port));
+#else /* HAVE_SOCKADDR_IN6_STRUCT */
   snprintf(buf, buflen, "%u", ntohs(saddr.sin_port));
-  return 1;
+#endif /* HAVE_SOCKADDR_IN6_STRUCT */
+  return TRUE;
 }
 
 /* Sets/resets TCP options TCP_NODELAY for the socket.  */
@@ -616,17 +866,47 @@ void ssh_tcp_get_host_name(char *buf, size_t buflen)
    needed. */
 char *ssh_tcp_get_host_addrs_by_name_sync(const char *name)
 {
-  char addresses[1024], *cp;
-  unsigned char outbuf[16];
-  struct in_addr in_addr;
-  struct hostent *hp;
+#ifdef VXWORKS
+  struct in_addr address;
+  unsigned char outbuf[INET_ADDR_LEN+1];
   size_t outbuflen = 4;
+#else /* VXWORKS */
+  char *addresses;
+  size_t addr_len, addr_ptr;
+  unsigned char outbuf[16];
+  struct hostent *hp;
+  size_t outbuflen = 16;
+  SshIpAddrStruct ip;
   int i;
+#  ifdef HAVE_GETIPNODEBYADDR
+  int error_num;
+#  endif /* HAVE_GETIPNODEBYADDR */
+#endif /* VXWORKS */
 
   /* First check if it is already an ip address. */
   if (ssh_inet_strtobin(name, outbuf, &outbuflen))
     return ssh_xstrdup(name);
+#ifdef VXWORKS
+  address.s_addr = hostGetByName(name);
+  if (address.s_addr == ERROR) return NULL;
+  inet_ntoa_b(address, outbuf);
+  return ssh_xstrdup(outbuf);
+#else /* VXWORKS */
+#  ifdef HAVE_GETIPNODEBYADDR
+  /* XXX Doesn't work with BSD/OS (others?). */
+  hp = getipnodebyname(name, AF_INET6,
+                       AI_V4MAPPED | AI_ADDRCONFIG | AI_ALL,
+                       &error_num);
+  if (!hp)
+    return NULL;
 
+  if (!hp->h_addr_list[0])
+    {
+      freehostent(hp);
+      return NULL;
+    }
+  outbuflen = 16;
+#  else /* HAVE_GETIPNODEBYADDR */
   /* Look up the host from the name servers. */
   hp = gethostbyname(name);
   if (!hp)
@@ -634,29 +914,45 @@ char *ssh_tcp_get_host_addrs_by_name_sync(const char *name)
 
   if (!hp->h_addr_list[0])
     return NULL;
+  outbuflen = 4;
+#  endif /* HAVE_GETIPNODEBYADDR */
 
   /* Format the addresses into a comma-separated string. */
-  strcpy(addresses, "");
+  addr_len = 256;
+  addresses = ssh_xmalloc(addr_len);
+
+  addr_ptr = 0;
   for (i = 0; hp->h_addr_list[i]; i++)
     {
-      memcpy(&in_addr, hp->h_addr_list[i], sizeof(in_addr));
-      cp = inet_ntoa(in_addr);
-      if (strlen(addresses) + strlen(cp) + 2 >= sizeof(addresses))
-        break;
+      if (outbuflen == 4)
+        SSH_IP4_DECODE(&ip, hp->h_addr_list[i]);
+      else
+        SSH_IP6_DECODE(&ip, hp->h_addr_list[i]);
+
+      if (addr_len - addr_ptr < 40)
+        {
+          addr_len *= 2;
+          addresses = ssh_xrealloc(addresses, addr_len);
+        }
+      ssh_ipaddr_print(&ip, addresses + addr_ptr, addr_len - addr_ptr);
+      addr_ptr += strlen(addresses + addr_ptr);
       if (i > 0)
-        strcat(addresses, ",");
-      strcat(addresses, cp);
+        {
+          addresses[addr_ptr++] = ',';
+          addresses[addr_ptr] = '\0';
+        }
     }
-  return ssh_xstrdup(addresses);
+  return addresses;
+#endif /* VXWORKS */
 }
 
 /* Looks up all ip-addresses of the host, returning them as a
    comma-separated list when calling the callback.  The host name may
    already be an ip address, in which case it is returned directly. */
 
-void ssh_tcp_get_host_addrs_by_name(const char *name,
-                                    SshLookupCallback callback,
-                                    void *context)
+SshOperationHandle ssh_tcp_get_host_addrs_by_name(const char *name,
+                                                  SshLookupCallback callback,
+                                                  void *context)
 {
   char *addrs;
 
@@ -668,6 +964,7 @@ void ssh_tcp_get_host_addrs_by_name(const char *name,
     }
   else
     (*callback)(SSH_IP_NO_ADDRESS, NULL, context);
+  return NULL;
 }
 
 
@@ -684,11 +981,90 @@ void ssh_tcp_get_host_addrs_by_name(const char *name,
 
 char *ssh_tcp_get_host_by_addr_sync(const char *addr)
 {
-  char name[1024];
-  size_t outbuflen = 4;
+#ifdef HAVE_GETIPNODEBYADDR
   struct hostent *hp;
-  struct in_addr in_addr;
   unsigned char outbuf[16];
+  size_t outbuflen = 16;
+  int error_num;
+  char *name;
+  int i;
+
+  if (!ssh_inet_strtobin(addr, outbuf, &outbuflen))
+    return NULL;
+
+  hp = getipnodebyaddr(outbuf, outbuflen,
+                       (outbuflen == 16) ? AF_INET6 : AF_INET,
+                       &error_num);
+  if (!hp)
+    return NULL;
+  
+  name = ssh_xstrdup(hp->h_name);
+  freehostent(hp);
+
+  /* Map it back to an IP address and check that the given address
+     actually is an address of this host.  This is necessary because
+     anyone with access to a name server can define arbitrary names
+     for an IP address.  Mapping from name to IP address can be
+     trusted better (but can still be fooled if the intruder has
+     access to the name server of the domain). */
+  hp = getipnodebyname(name, (outbuflen == 16) ? AF_INET6 : AF_INET,
+                       AI_V4MAPPED | AI_ADDRCONFIG | AI_ALL,
+                       &error_num);
+  if (!hp)
+    {
+      ssh_xfree(name);
+      return NULL;
+    }
+  
+  /* Look for the address from the list of addresses. */
+  for (i = 0; hp->h_addr_list[i]; i++)
+    if (memcmp(hp->h_addr_list[i], outbuf, outbuflen) == 0)
+      break;
+  /* If we reached the end of the list, the address was not there. */
+  if (!hp->h_addr_list[i])
+    {
+      freehostent(hp);
+      ssh_xfree(name);
+      return NULL;
+    }
+
+  freehostent(hp);
+  /* Address was found for the host name.  We accept the host name. */
+  return name;
+#else
+#ifdef VXWORKS
+  char name[MAXHOSTNAMELEN+1];
+  size_t outbuflen = 4; /* IPv4 only in VxWorks */
+  unsigned char outbuf[16];
+  int address, address_2;
+
+  if (!ssh_inet_strtobin(addr, outbuf, &outbuflen))
+    return NULL;
+
+  if (outbuflen!=4)
+    return NULL; /* IPv4 only in VxWorks */
+
+  memmove(&address, outbuf, outbuflen);
+  if (hostGetByAddr(address, name) == ERROR) return NULL;
+
+  /* Map it back to an IP address and check that the given address
+     actually is an address of this host.  This is necessary because
+     anyone with access to a name server can define arbitrary names
+     for an IP address.  Mapping from name to IP address can be
+     trusted better (but can still be fooled if the intruder has
+     access to the name server of the domain). */
+  address_2 = hostGetByName(name);
+  if (address != address_2) return NULL;
+
+  /* Address was found for the host name.  We accept the host name. */
+  return ssh_xstrdup(name);
+
+#else /* VXWORKS */
+  unsigned char outbuf[16];
+  size_t outbuflen = 16;
+  struct in_addr in_addr;
+  struct hostent *hp;
+  char *name;
   int i;
 
   if (!ssh_inet_strtobin(addr, outbuf, &outbuflen))
@@ -700,8 +1076,7 @@ char *ssh_tcp_get_host_by_addr_sync(const char *addr)
     return NULL;
 
   /* Got host name. */
-  strncpy(name, hp->h_name, sizeof(name));
-  name[sizeof(name) - 1] = '\0';
+  name = ssh_xstrdup(hp->h_name);
 
   /* Map it back to an IP address and check that the given address
      actually is an address of this host.  This is necessary because
@@ -711,7 +1086,10 @@ char *ssh_tcp_get_host_by_addr_sync(const char *addr)
      access to the name server of the domain). */
   hp = gethostbyname(name);
   if (!hp)
-    return NULL;
+    {
+      ssh_xfree(name);
+      return NULL;
+    }
 
   /* Look for the address from the list of addresses. */
   for (i = 0; hp->h_addr_list[i]; i++)
@@ -719,10 +1097,15 @@ char *ssh_tcp_get_host_by_addr_sync(const char *addr)
       break;
   /* If we reached the end of the list, the address was not there. */
   if (!hp->h_addr_list[i])
-    return NULL;
+    {
+      ssh_xfree(name);
+      return NULL;
+    }
 
   /* Address was found for the host name.  We accept the host name. */
-  return ssh_xstrdup(name);
+  return name;
+#endif /* VXWORKS */
+#endif /* HAVE_GETIPNODEBYADDR */
 }
 
 /* Looks up the name of the host by its ip-address.  Verifies that the
@@ -730,9 +1113,9 @@ char *ssh_tcp_get_host_by_addr_sync(const char *addr)
    Calls the callback with either error or success.  The callback should
    copy the returned name. */
 
-void ssh_tcp_get_host_by_addr(const char *addr,
-                              SshLookupCallback callback,
-                              void *context)
+SshOperationHandle ssh_tcp_get_host_by_addr(const char *addr,
+                                            SshLookupCallback callback,
+                                            void *context)
 {
   char *name;
 
@@ -744,6 +1127,7 @@ void ssh_tcp_get_host_by_addr(const char *addr,
     }
   else
     (*callback)(SSH_IP_NO_ADDRESS, NULL, context);
+  return NULL;
 }
 
 /* Looks up the service (port number) by name and protocol.  `protocol' must
@@ -752,14 +1136,16 @@ void ssh_tcp_get_host_by_addr(const char *addr,
 int ssh_tcp_get_port_by_service(const char *name, const char *proto)
 {
 #ifdef HAVE_GETSERVBYNAME
-  const char *cp;
   struct servent *se;
+#endif /* HAVE_GETSERVBYNAME */
+  const char *cp;
   int port;
 
   for (cp = name; isdigit((int)*cp); cp++)
     ;
   if (!*cp && *name)
     return atoi(name);
+#ifdef HAVE_GETSERVBYNAME
   se = getservbyname(name, proto);
   if (!se)
     return -1;
@@ -767,7 +1153,15 @@ int ssh_tcp_get_port_by_service(const char *name, const char *proto)
   endservent();
   return port;
 #else  /* HAVE_GETSERVBYNAME */
+#  ifdef WANT_SERVBYNAME
+    {
+      struct SshServent const *se;
+      se = ssh_getserv(name, 0, TRUE, proto);
+      return (se == NULL)? -1 : se->s_port;
+    }
+#  else /* WANT_SERVBYNAME */
   return -1;
+#  endif /* WANT_SERVBYNAME */
 #endif /* HAVE_GETSERVBYNAME */
 }
 
@@ -790,7 +1184,21 @@ void ssh_tcp_get_service_by_port(unsigned int port, const char *proto,
     strncpy(buf, se->s_name, buflen);
   endservent();
 #else /* HAVE_GETSERVBYPORT */
+#  ifdef WANT_SERVBYPORT
+    {
+      struct SshServent const *se;
+      se = ssh_getserv(NULL, port, FALSE, proto);
+      if (se)
+        if (se->s_aliases[0] != NULL)
+          {
+            strncpy(buf, se->s_aliases[0], buflen);
+            return;
+          }
+      snprintf(buf, buflen, "%u", port);
+    }
+#  else /* WANT_SERVBY_PORT */
   snprintf(buf, buflen, "%u", port);
+#  endif /* WANT_SERVBY_PORT */
 #endif /* HAVE_GETSERVBYPORT */
 }
 
