@@ -364,7 +364,7 @@ void ssh_packet_impl_destroy(void *context)
 
 /* Methods table for the stream. */
 
-const SshStreamMethodsTable ssh_packet_impl_methods =
+const SshStreamMethodsStruct ssh_packet_impl_methods =
 {
   ssh_packet_impl_read,
   ssh_packet_impl_write,
