@@ -577,6 +577,20 @@ void ssh_packet_wrapper_send(SshPacketWrapper down, SshPacketType type,
                                  SSH_FORMAT_END);
 }
 
+/* Set callbacks (and context) to new values. */
+void ssh_packet_wrapper_set_callbacks(SshPacketWrapper wrapper,
+                                      SshPacketReceiveProc received_packet,
+                                      SshPacketEofProc received_eof,
+                                      SshPacketCanSendProc can_send,
+                                      void *context)
+{
+  SSH_PRECOND(wrapper);
+  wrapper->received_packet = received_packet;
+  wrapper->received_eof = received_eof;
+  wrapper->can_send = can_send;
+  wrapper->context = context;
+}
+
 /* Causes any I/O requests from `packet_stream' (which must be implemented
    using the ssh_impl_* functions in this module) to be shortcircuited to
    the stream inside `wrapper', and vice versa.  The `received_packet',
