@@ -602,7 +602,7 @@ void ssh_stream_filter_set_callback(void *context, SshStreamCallback callback,
 
 /* Stream methods table for filter streams. */
 
-const SshStreamMethodsTable ssh_stream_filter_methods =
+const SshStreamMethodsStruct ssh_stream_filter_methods =
 {
   ssh_stream_filter_read,
   ssh_stream_filter_write,
