@@ -12,7 +12,7 @@ Created: Sat Apr 18 11:57:24 EEST 1998
 */
 
 /*
- * $Id: t-localstream.c,v 1.5 2000/05/28 16:21:27 kivinen Exp $
+ * $Id: t-localstream.c,v 1.6 2000/06/14 00:44:21 kivinen Exp $
  * $Log: t-localstream.c,v $
  * $EndLog$
  */
@@ -31,7 +31,7 @@ Created: Sat Apr 18 11:57:24 EEST 1998
 
 int exited1 = 0, exited2 = 0;
 SshLocalListener listener1, listener2;
-SshStreamStats stats;
+SshStreamStatsStruct stats;
 SshBufferStruct send_buffer, expect_buffer;
 unsigned long send_count = 0, read_count = 0;
 
