@@ -12,7 +12,7 @@ Created: Thu Oct 24 23:10:57 1996 ylo
 */
 
 /*
- * $Id: t-stream.c,v 1.19.2.2 2000/06/07 21:32:11 sjl Exp $
+ * $Id: t-stream.c,v 1.24 2000/09/27 02:23:05 kivinen Exp $
  * $Log: t-stream.c,v $
  * $EndLog$
  */
@@ -27,6 +27,8 @@ Created: Thu Oct 24 23:10:57 1996 ylo
 
 #ifdef SSHDIST_SSH2
 #undef OUTSIDE_TESTS
+
+
 #endif /* SSHDIST_SSH2 */
 
 #ifdef WINDOWS
@@ -49,7 +51,7 @@ Created: Thu Oct 24 23:10:57 1996 ylo
 int exited1 = 0, exited2 = 0;
 SshTcpListener listener1, listener2;
 SshStream connect2;
-SshStreamStats stats;
+SshStreamStatsStruct stats;
 SshBufferStruct send_buffer, expect_buffer;
 unsigned long send_count = 0, read_count = 0;
 
@@ -60,11 +62,16 @@ void connect2_done(SshIpError error, SshStream stream, void *context)
 
 void timeout(void *context)
 {
+  SshTcpConnectParamsStruct params;
   if (context != (void *)1)
     abort();
-  
-  ssh_tcp_connect_with_socks(LOCALHOST, "34513", "", 2,
-                             connect2_done, (void *)2);
+
+  memset(&params, 0, sizeof(params));
+  params.socks_server_url = "";
+  params.connection_attempts = 2;
+  params.connection_timeout = 10;
+  ssh_tcp_connect(LOCALHOST, "34513", &params,
+                  connect2_done, (void *)2);
 }
 
 void listener2_callback(SshIpError status, SshStream stream, void *context)
@@ -303,6 +310,13 @@ int main(int ac, char **av)
   int pass;
   unsigned char buf[1024];
   int exitval = 0;
+  const char *debug_string = "*=2";
+  SshTcpConnectParamsStruct params;
+
+  if (ac == 2)
+    debug_string = av[1];
+
+  ssh_debug_set_level_string(debug_string);
   
   printf("Doing %d iterations of stream test:", PASSES);
   for (pass = 0; pass < PASSES; pass++)
@@ -368,19 +382,21 @@ int main(int ac, char **av)
       ssh_event_loop_initialize();
 
       /* Try creating a failing listener. */
-      listener1 = ssh_tcp_make_listener("0.0.0.0", "34512",
-                                           listenerfail_callback, NULL);
+      listener1 = ssh_tcp_make_listener("0.0.0.0", "34512", NULL, 
+                                        listenerfail_callback, NULL);
       if (!listener1)
         ssh_fatal("Creating listener1 failed");
-      listener2 = ssh_tcp_make_listener("0.0.0.0", "34512",
-                                           listenerfail_callback, NULL);
+      listener2 = ssh_tcp_make_listener("0.0.0.0", "34512", NULL, 
+                                        listenerfail_callback, NULL);
       if (listener2)
         ssh_fatal("Creating listener2 succeeded when it should fail.");
       ssh_tcp_destroy_listener(listener1);
 
       /* Try making a failing connection. */
-      ssh_tcp_connect_with_socks("127.1", "34514", NULL, 2,
-                                 connectfail_done, NULL);
+      memset(&params, 0, sizeof(params));
+      params.connection_attempts = 2;
+      params.connection_timeout = 10;
+      ssh_tcp_connect("127.1", "34514", &params, connectfail_done, NULL);
       /* For the first half, do the tests one at a time to ease debugging. */
 #ifndef WINDOWS /* In Windows we must run the event loop all the time */
       if (pass < PASSES / 2)
@@ -393,30 +409,35 @@ int main(int ac, char **av)
 
       if (pass % 50 == 0)
         {
+          memset(&params, 0, sizeof(params));
+          params.socks_server_url = SOCKSHOST;
+          params.connection_attempts = 2;
+          params.connection_timeout = 10;
           /* Try connecting with socks to a failing address. */
-          ssh_tcp_connect_with_socks(OUTSIDESSHHOST, "34512", SOCKSHOST,
-                                     2, connectfail_done, NULL);
+          ssh_tcp_connect(OUTSIDESSHHOST, "34512", &params,
+                          connectfail_done, NULL);
           /* First time, do the tests one at a time to ease debugging. */
           if (pass < PASSES / 2)
             ssh_event_loop_run();
           
+          params.socks_server_url = SOCKSIP;
           /* Try connecting with socks to a failing address. */
-          ssh_tcp_connect_with_socks(OUTSIDESSHHOST, "34512", SOCKSIP,
-                                     2, connectfail_done, NULL);
+          ssh_tcp_connect(OUTSIDESSHHOST, "34512", &params,
+                          connectfail_done, NULL);
           /* First time, do the tests one at a time to ease debugging. */
           if (pass < PASSES / 2)
             ssh_event_loop_run();
           
+          params.socks_server_url = SOCKSHOST;
           /* Try connecting with socks to a successful address. */
-          ssh_tcp_connect_with_socks(OUTSIDESSHIP, "22", SOCKSHOST,
-                                     2, connectssh_done, NULL);
+          ssh_tcp_connect(OUTSIDESSHIP, "22", &params, connectssh_done, NULL);
           /* First time, do the tests one at a time to ease debugging. */
           if (pass < PASSES / 2)
             ssh_event_loop_run();
 
+          params.socks_server_url = SOCKSIP;
           /* Try connecting with socks to a successful address. */
-          ssh_tcp_connect_with_socks(OUTSIDESSHIP, "22", SOCKSIP,
-                                     2, connectssh_done, NULL);
+          ssh_tcp_connect(OUTSIDESSHIP, "22", &params, connectssh_done, NULL);
           /* First time, do the tests one at a time to ease debugging. */
           if (pass < PASSES / 2)
             ssh_event_loop_run();
@@ -428,9 +449,9 @@ int main(int ac, char **av)
            - that the callbacks get called when set
            - that data can be transmitted
            - that EOF is passed ok */
-      listener1 = ssh_tcp_make_listener("0.0.0.0", "34512",
+      listener1 = ssh_tcp_make_listener("0.0.0.0", "34512", NULL, 
                                         listener1_callback, NULL);
-      listener2 = ssh_tcp_make_listener("127.0.0.1", "34513",
+      listener2 = ssh_tcp_make_listener("127.0.0.1", "34513", NULL, 
                                         listener2_callback, NULL);
       ssh_register_timeout(0L, 50000L, timeout, (void *)1);
       
@@ -441,8 +462,12 @@ int main(int ac, char **av)
       buf[sizeof (buf) - 1] = '\0';
 #endif
       
-      ssh_tcp_connect_with_socks((char *) buf, "34512", SOCKSLOCAL, 2,
-                                 connect1_done, (void *)3);
+      memset(&params, 0, sizeof(params));
+      params.connection_attempts = 2;
+      params.connection_timeout = 10;
+      params.socks_server_url = SOCKSLOCAL;
+      ssh_tcp_connect((char *) buf, "34512",  &params, connect1_done,
+                      (void *)3);
 
       /* This is supposed to exit when all the listeners and connecting streams
          have destroyed themselves. */
