@@ -12,7 +12,7 @@
   */
 
 /*
- * $Id: t-udp.c,v 1.6 1998/10/08 15:43:41 kivinen Exp $
+ * $Id: t-udp.c,v 1.9 2000/09/26 09:50:46 kivinen Exp $
  * $Log: t-udp.c,v $
  * $EndLog$
  */
@@ -20,7 +20,7 @@
 #include "sshincludes.h"
 #include "sshudp.h"
 #include "sshtimeouts.h"
-#include "sshunixeloop.h"
+#include "ssheloop.h"
 
 SshUdpListener c_listener, p_listener;
 unsigned char c_data[256];
@@ -80,6 +80,7 @@ void c(void)
   
   ssh_event_loop_run();
   ssh_debug("child exiting...");
+  ssh_event_loop_uninitialize();
 }
 
 void p(void)
@@ -106,7 +107,7 @@ void usage(void)
 
 int main(int argc, char **argv)
 {
-  pid_t pid;
+  pid_t pid, pid2;
   
   memset(c_data, 0, 256);
   memset(p2_data, 0, 256);
@@ -143,6 +144,7 @@ int main(int argc, char **argv)
   if (pid == 0)
     {
       c();
+      sleep(2);
       exit(0);
     }
   else
@@ -150,9 +152,16 @@ int main(int argc, char **argv)
       int status;
       
       p();
-      if (wait(&status) != pid)
+#ifdef HAVE_WAITPID
+      while ((pid2 = waitpid(pid, &status, 0)) < 0)
+        if (errno != EINTR)
+          break;
+#else /* HAVE_WAITPID */
+      pid2 = wait(&status);
+#endif /* HAVE_WAITPID */
+      if (pid2 != pid)
         {
-          ssh_fatal("Wrong pid returned by wait");
+          ssh_fatal("Wrong pid returned by wait, %d vs %d", pid, pid2);
         }
       if (WIFSIGNALED(status))
         {
